@@ -17,8 +17,8 @@
 package javax.servlet.http;
 
 import java.io.IOException;
-import java.io.PrintWriter;
 import java.io.OutputStreamWriter;
+import java.io.PrintWriter;
 import java.io.UnsupportedEncodingException;
 import java.lang.reflect.Method;
 import java.text.MessageFormat;
@@ -73,8 +73,9 @@ import javax.servlet.ServletResponse;
  * @author  Various
  * @version $Version$
  */
-public abstract class HttpServlet extends GenericServlet
-    implements java.io.Serializable {
+public abstract class HttpServlet extends GenericServlet {
+
+    private static final long serialVersionUID = 1L;
 
     private static final String METHOD_DELETE = "DELETE";
     private static final String METHOD_HEAD = "HEAD";
@@ -96,7 +97,9 @@ public abstract class HttpServlet extends GenericServlet
     /**
      * Does nothing, because this is an abstract class.
      */
-    public HttpServlet() { }
+    public HttpServlet() {
+        // NOOP
+    }
     
     
     /**
@@ -226,7 +229,7 @@ public abstract class HttpServlet extends GenericServlet
      * @param req   the request object that is passed to the servlet
      *
      * @param resp  the response object that the servlet
-     *                  uses to return the headers to the clien
+     *                  uses to return the headers to the client
      *
      * @exception IOException   if an input or output error occurs
      *
@@ -416,7 +419,7 @@ public abstract class HttpServlet extends GenericServlet
     }
     
 
-    private static Method[] getAllDeclaredMethods(Class c) {
+    private static Method[] getAllDeclaredMethods(Class<?> c) {
 
         if (c.equals(javax.servlet.http.HttpServlet.class)) {
             return null;
@@ -471,7 +474,8 @@ public abstract class HttpServlet extends GenericServlet
      * @exception ServletException  if the request for the
      *                                  OPTIONS cannot be handled
      */
-    protected void doOptions(HttpServletRequest req, HttpServletResponse resp)
+    protected void doOptions(HttpServletRequest req,
+            HttpServletResponse resp)
         throws ServletException, IOException {
 
         Method[] methods = getAllDeclaredMethods(this.getClass());
@@ -501,7 +505,7 @@ public abstract class HttpServlet extends GenericServlet
         
         String allow = null;
         if (ALLOW_GET)
-            if (allow==null) allow=METHOD_GET;
+            allow=METHOD_GET;
         if (ALLOW_HEAD)
             if (allow==null) allow=METHOD_HEAD;
             else allow += ", " + METHOD_HEAD;
@@ -555,25 +559,25 @@ public abstract class HttpServlet extends GenericServlet
         int responseLength;
         
         String CRLF = "\r\n";
-        String responseString = "TRACE "+ req.getRequestURI()+
-            " " + req.getProtocol();
+        StringBuilder buffer = new StringBuilder("TRACE ").append(req.getRequestURI())
+            .append(" ").append(req.getProtocol());
         
-        Enumeration reqHeaderEnum = req.getHeaderNames();
+        Enumeration<String> reqHeaderEnum = req.getHeaderNames();
         
         while( reqHeaderEnum.hasMoreElements() ) {
-            String headerName = (String)reqHeaderEnum.nextElement();
-            responseString += CRLF + headerName + ": " +
-                req.getHeader(headerName); 
+            String headerName = reqHeaderEnum.nextElement();
+            buffer.append(CRLF).append(headerName).append(": ")
+                .append(req.getHeader(headerName)); 
         }
         
-        responseString += CRLF;
+        buffer.append(CRLF);
         
-        responseLength = responseString.length();
+        responseLength = buffer.length();
         
         resp.setContentType("message/http");
         resp.setContentLength(responseLength);
         ServletOutputStream out = resp.getOutputStream();
-        out.print(responseString);        
+        out.print(buffer.toString());        
         out.close();
         return;
     }                
@@ -702,6 +706,7 @@ public abstract class HttpServlet extends GenericServlet
      * 
      * @see javax.servlet.Servlet#service
      */
+    @Override
     public void service(ServletRequest req, ServletResponse res)
         throws ServletException, IOException {
 
@@ -746,15 +751,18 @@ class NoBodyResponse extends HttpServletResponseWrapper {
 
     // SERVLET RESPONSE interface methods
 
+    @Override
     public void setContentLength(int len) {
         super.setContentLength(len);
         didSetContentLength = true;
     }
 
+    @Override
     public ServletOutputStream getOutputStream() throws IOException {
         return noBody;
     }
 
+    @Override
     public PrintWriter getWriter() throws UnsupportedEncodingException {
 
         if (writer == null) {
@@ -783,17 +791,21 @@ class NoBodyOutputStream extends ServletOutputStream {
     private int                contentLength = 0;
 
     // file private
-    NoBodyOutputStream() {}
+    NoBodyOutputStream() {
+        // NOOP
+    }
 
     // file private
     int getContentLength() {
         return contentLength;
     }
 
+    @Override
     public void write(int b) {
         contentLength++;
     }
 
+    @Override
     public void write(byte buf[], int offset, int len)
         throws IOException
     {
