@@ -26,18 +26,12 @@ import java.util.EventObject;
  * General event for notifying listeners of significant changes on a Container.
  *
  * @author Craig R. McClanahan
- * @version $Id: ContainerEvent.java 939350 2010-04-29 15:36:29Z kkolinko $
+ * @version $Id: ContainerEvent.java 1072140 2011-02-18 20:44:55Z markt $
  */
 
-public final class ContainerEvent
-    extends EventObject {
-
-
-    /**
-     * The Container on which this event occurred.
-     */
-    private Container container = null;
+public final class ContainerEvent extends EventObject {
 
+    private static final long serialVersionUID = 1L;
 
     /**
      * The event data associated with this event.
@@ -61,7 +55,6 @@ public final class ContainerEvent
     public ContainerEvent(Container container, String type, Object data) {
 
         super(container);
-        this.container = container;
         this.type = type;
         this.data = data;
 
@@ -83,7 +76,7 @@ public final class ContainerEvent
      */
     public Container getContainer() {
 
-        return (this.container);
+        return (Container) getSource();
 
     }
 
@@ -101,6 +94,7 @@ public final class ContainerEvent
     /**
      * Return a string representation of this event.
      */
+    @Override
     public String toString() {
 
         return ("ContainerEvent['" + getContainer() + "','" +
