@@ -19,10 +19,16 @@
 package org.apache.catalina;
 
 
+import java.net.URL;
+import java.util.Set;
+
+import javax.servlet.ServletContainerInitializer;
 import javax.servlet.ServletContext;
+import javax.servlet.ServletRequest;
+import javax.servlet.ServletSecurityElement;
+import javax.servlet.descriptor.JspConfigDescriptor;
 
-import org.apache.tomcat.util.http.mapper.Mapper;
-
+import org.apache.catalina.core.ApplicationServletRegistration;
 import org.apache.catalina.deploy.ApplicationParameter;
 import org.apache.catalina.deploy.ErrorPage;
 import org.apache.catalina.deploy.FilterDef;
@@ -31,6 +37,8 @@ import org.apache.catalina.deploy.LoginConfig;
 import org.apache.catalina.deploy.NamingResources;
 import org.apache.catalina.deploy.SecurityConstraint;
 import org.apache.catalina.util.CharsetMapper;
+import org.apache.tomcat.JarScanner;
+import org.apache.tomcat.util.http.mapper.Mapper;
 
 
 /**
@@ -50,7 +58,7 @@ import org.apache.catalina.util.CharsetMapper;
  * <p>
  *
  * @author Craig R. McClanahan
- * @version $Id: Context.java 1027196 2010-10-25 17:00:43Z markt $
+ * @version $Id: Context.java 1101069 2011-05-09 15:39:34Z markt $
  */
 
 public interface Context extends Container {
@@ -64,9 +72,50 @@ public interface Context extends Container {
      */
     public static final String RELOAD_EVENT = "reload";
 
+    /**
+     * Container event for adding a welcome file.
+     */
+    public static final String ADD_WELCOME_FILE_EVENT = "addWelcomeFile";
+    
+    /**
+     * Container event for removing a wrapper.
+     */
+    public static final String REMOVE_WELCOME_FILE_EVENT = "removeWelcomeFile";
+
+    /**
+     * Container event for clearing welcome files.
+     */
+    public static final String  CLEAR_WELCOME_FILES_EVENT = "clearWelcomeFiles";
+
+    /**
+     * Container event for changing the ID of a session.
+     */
+    public static final String CHANGE_SESSION_ID_EVENT = "changeSessionId";
 
     // ------------------------------------------------------------- Properties
 
+    /**
+     * Returns <code>true</code> if requests mapped to servlets without
+     * "multipart config" to parse multipart/form-data requests anyway.
+     *
+     * @return <code>true</code> if requests mapped to servlets without
+     *    "multipart config" to parse multipart/form-data requests,
+     *    <code>false</code> otherwise.
+     */
+    public boolean getAllowCasualMultipartParsing();
+
+
+   /**
+     * Set to <code>true</code> to allow requests mapped to servlets that
+     * do not explicitly declare @MultipartConfig or have
+     * &lt;multipart-config&gt; specified in web.xml to parse
+     * multipart/form-data requests.
+     *
+     * @param allowCasualMultipartParsing <code>true</code> to allow such
+     *        casual parsing, <code>false</code> otherwise.
+     */
+    public void setAllowCasualMultipartParsing(boolean allowCasualMultipartParsing);
+
 
     /**
      * Return the set of initialized application event listener objects,
@@ -116,14 +165,6 @@ public interface Context extends Container {
     public boolean getAvailable();
 
 
-    /**
-     * Set the application available flag for this Context.
-     *
-     * @param available The new application available flag
-     */
-    public void setAvailable(boolean available);
-
-
     /**
      * Return the Locale to character set mapper for this Context.
      */
@@ -139,17 +180,17 @@ public interface Context extends Container {
 
 
     /**
-     * Return the path to a file to save this Context information.
+     * Return the URL of the XML descriptor for this context.
      */
-    public String getConfigFile();
+    public URL getConfigFile();
 
 
     /**
-     * Set the path to a file to save this Context information.
+     * Set the URL of the XML descriptor for this context.
      *
-     * @param configFile The path to a file to save this Context information.
+     * @param configFile The URL of the XML descriptor for this context.
      */
-    public void setConfigFile(String configFile);
+    public void setConfigFile(URL configFile);
 
 
     /**
@@ -257,6 +298,30 @@ public interface Context extends Container {
     public void setSessionCookiePath(String sessionCookiePath);
 
     
+    /**
+     * Is a / added to the end of the session cookie path to ensure browsers,
+     * particularly IE, don't send a session cookie for context /foo with
+     * requests intended for context /foobar.
+     * 
+     * @return <code>true</code> if the slash is added, otherwise
+     *         <code>false</code>  
+     */
+    public boolean getSessionCookiePathUsesTrailingSlash();
+    
+    
+    /**
+     * Configures if a / is added to the end of the session cookie path to
+     * ensure browsers, particularly IE, don't send a session cookie for context
+     * /foo with requests intended for context /foobar.
+     * 
+     * @param sessionCookiePathUsesTrailingSlash   <code>true</code> if the
+     *                                             slash is should be added,
+     *                                             otherwise <code>false</code>  
+     */
+    public void setSessionCookiePathUsesTrailingSlash(
+            boolean sessionCookiePathUsesTrailingSlash);
+
+    
     /**
      * Return the "allow crossing servlet contexts" flag.
      */
@@ -325,35 +390,6 @@ public interface Context extends Container {
      * @param docBase The new document root
      */
     public void setDocBase(String docBase);
-    
-    
-    /**
-     * Is URL rewriting disabled?
-     * URL rewriting is an optional component of the servlet 2.5 specification.
-     * However if set to true this will be non-compliant with the specification
-     * as the specification requires that there <b>must</b> be a way to retain
-     * sessions if the client doesn't allow session cookies.
-     * 
-     * @return true If URL rewriting is disabled.
-     * 
-     * @see <a href="http://jcp.org/aboutJava/communityprocess/mrel/jsr154/index2.html">Servlet
-     *      2.5 Specification. Sections SRV.7.1.3 and SRV.7.1.4</a>
-     * @see javax.servlet.http.HttpServletResponse#encodeURL(String) encodeURL
-     * @see javax.servlet.http.HttpServletResponse#encodeRedirectURL(String)
-     *      encodeRedirectURL
-     */
-    public boolean isDisableURLRewriting();
-    
-    /**
-     * Is URL rewriting disabled?
-     * URL rewriting is an optional component of the servlet 2.5 specification.
-     * However if set to true this will be non-compliant with the specification
-     * as the specification requires that there <b>must</b> be a way to retain
-     * sessions if the client doesn't allow session cookies.
-     *
-     * @param disable True to disable URL Rewriting. Default <b>false</b>.
-     */
-    public void setDisableURLRewriting(boolean disable);
 
 
     /**
@@ -505,6 +541,25 @@ public interface Context extends Container {
     public void setSessionTimeout(int timeout);
 
 
+    /**
+     * Returns <code>true</code> if remaining request data will be read
+     * (swallowed) even the request violates a data size constraint.
+     *
+     * @return <code>true</code> if data will be swallowed (default),
+     *    <code>false</code> otherwise.
+     */
+    public boolean getSwallowAbortedUploads();
+
+
+    /**
+     * Set to <code>false</code> to disable request data swallowing
+     * after an upload was aborted due to size constraints.
+     *
+     * @param swallowAbortedUploads <code>false</code> to disable
+     *        swallowing, <code>true</code> otherwise (default).
+     */
+    public void setSwallowAbortedUploads(boolean swallowAbortedUploads);
+
     /**
      * Return the value of the swallowOutput flag.
      */
@@ -537,6 +592,103 @@ public interface Context extends Container {
     public void setWrapperClass(String wrapperClass);
 
 
+    /**
+     * Get the server.xml <context> attribute's xmlNamespaceAware.
+     * @return true if namespace awareness is enabled.
+     *
+     */
+    public boolean getXmlNamespaceAware();
+
+
+    /**
+     * Get the server.xml <context> attribute's xmlValidation.
+     * @return true if validation is enabled.
+     *
+     */
+    public boolean getXmlValidation();
+
+
+    /**
+     * Set the validation feature of the XML parser used when
+     * parsing xml instances.
+     * @param xmlValidation true to enable xml instance validation
+     */
+    public void setXmlValidation(boolean xmlValidation);
+
+
+   /**
+     * Set the namespace aware feature of the XML parser used when
+     * parsing xml instances.
+     * @param xmlNamespaceAware true to enable namespace awareness
+     */
+    public void setXmlNamespaceAware(boolean xmlNamespaceAware);
+    /**
+     * Get the server.xml <context> attribute's xmlValidation.
+     * @return true if validation is enabled.
+     */
+     
+
+    /**
+     * Set the validation feature of the XML parser used when
+     * parsing tlds files. 
+     * @param tldValidation true to enable xml instance validation
+     */
+    public void setTldValidation(boolean tldValidation);
+
+
+    /**
+     * Get the server.xml <context> attribute's webXmlValidation.
+     * @return true if validation is enabled.
+     *
+     */
+    public boolean getTldValidation();
+
+
+    /**
+     * Get the server.xml &lt;host&gt; attribute's xmlNamespaceAware.
+     * @return true if namespace awareness is enabled.
+     */
+    public boolean getTldNamespaceAware();
+
+
+    /**
+     * Set the namespace aware feature of the XML parser used when
+     * parsing xml instances.
+     * @param tldNamespaceAware true to enable namespace awareness
+     */
+    public void setTldNamespaceAware(boolean tldNamespaceAware);
+
+    /**
+     * Get the Jar Scanner to be used to scan for JAR resources for this
+     * context.
+     * @return  The Jar Scanner configured for this context.
+     */
+    public JarScanner getJarScanner();
+
+    /**
+     * Set the Jar Scanner to be used to scan for JAR resources for this
+     * context.
+     * @param jarScanner    The Jar Scanner to be used for this context.
+     */
+    public void setJarScanner(JarScanner jarScanner);
+
+    /**
+     * Obtain the {@link Authenticator} that is used by this context or
+     * <code>null</code> if none is used.
+     */
+    public Authenticator getAuthenticator();
+    
+    /**
+     * Set whether or not the effective web.xml for this context should be
+     * logged on context start.
+     */
+    public void setLogEffectiveWebXml(boolean logEffectiveWebXml);
+    
+    /**
+     * Should the effective web.xml for this context be logged on context start?
+     */
+    public boolean getLogEffectiveWebXml();
+    
     // --------------------------------------------------------- Public Methods
 
 
@@ -586,6 +738,17 @@ public interface Context extends Container {
      */
     public void addFilterMap(FilterMap filterMap);
 
+    /**
+     * Add a filter mapping to this Context before the mappings defined in the
+     * deployment descriptor but after any other mappings added via this method.
+     *
+     * @param filterMap The filter mapping to be added
+     *
+     * @exception IllegalArgumentException if the specified filter name
+     *  does not match an existing filter definition, or the filter mapping
+     *  is malformed
+     */
+    public void addFilterMapBefore(FilterMap filterMap);
 
     /**
      * Add the classname of an InstanceListener to be added to each
@@ -596,18 +759,6 @@ public interface Context extends Container {
     public void addInstanceListener(String listener);
 
 
-    /**
-     * Add the given URL pattern as a jsp-property-group.  This maps
-     * resources that match the given pattern so they will be passed
-     * to the JSP container.  Though there are other elements in the
-     * property group, we only care about the URL pattern here.  The
-     * JSP container will parse the rest.
-     *
-     * @param pattern URL pattern to be mapped 
-     */
-    public void addJspMapping(String pattern);
-
-
     /**
      * Add a Locale Encoding Mapping (see Sec 5.4 of Servlet spec 2.4)
      *
@@ -665,14 +816,18 @@ public interface Context extends Container {
 
 
     /**
-     * Add a JSP tag library for the specified URI.
+     * Add a new servlet mapping, replacing any existing mapping for
+     * the specified pattern.
      *
-     * @param uri URI, relative to the web.xml file, of this tag library
-     * @param location Location of the tag library descriptor
+     * @param pattern URL pattern to be mapped
+     * @param name Name of the corresponding servlet to execute
+     * @param jspWildcard true if name identifies the JspServlet
+     * and pattern contains a wildcard; false otherwise
      */
-    public void addTaglib(String uri, String location);
+    public void addServletMapping(String pattern, String name,
+            boolean jspWildcard);
+
 
-    
     /**
      * Add a resource which will be watched for reloading by the host auto
      * deployer. Note: this will not be used in embedded mode.
@@ -883,23 +1038,6 @@ public interface Context extends Container {
     public int[] findStatusPages();
 
 
-    /**
-     * Return the tag library descriptor location for the specified taglib
-     * URI, if any; otherwise, return <code>null</code>.
-     *
-     * @param uri URI, relative to the web.xml file
-     */
-    public String findTaglib(String uri);
-
-
-    /**
-     * Return the URIs of all tag libraries for which a tag library
-     * descriptor location has been specified.  If none are specified,
-     * a zero-length array is returned.
-     */
-    public String[] findTaglibs();
-
-
     /**
      * Return the set of watched resources for this Context. If none are 
      * defined, a zero length array will be returned.
@@ -937,6 +1075,22 @@ public interface Context extends Container {
     public String[] findWrapperListeners();
 
 
+    /**
+     * Notify all {@link javax.servlet.ServletRequestListener}s that a request
+     * has started.
+     * @return <code>true</code> if the listeners fire successfully, else
+     *         <code>false</code>
+     */
+    public boolean fireRequestInitEvent(ServletRequest request);
+
+    /**
+     * Notify all {@link javax.servlet.ServletRequestListener}s that a request
+     * has ended.
+     * @return <code>true</code> if the listeners fire successfully, else
+     *         <code>false</code>
+     */
+    public boolean fireRequestDestroyEvent(ServletRequest request);
+
     /**
      * Reload this web application, if reloading is supported.
      *
@@ -1050,14 +1204,6 @@ public interface Context extends Container {
     public void removeServletMapping(String pattern);
 
 
-    /**
-     * Remove the tag library location forthe specified tag library URI.
-     *
-     * @param uri URI, relative to the web.xml file
-     */
-    public void removeTaglib(String uri);
-
-    
     /**
      * Remove the specified watched resource name from the list associated
      * with this Context.
@@ -1095,65 +1241,155 @@ public interface Context extends Container {
 
 
     /**
-     * Get the server.xml <context> attribute's xmlNamespaceAware.
-     * @return true if namespace awarenes is enabled.
+     * Return the real path for a given virtual path, if possible; otherwise
+     * return <code>null</code>.
      *
+     * @param path The path to the desired resource
      */
-    public boolean getXmlNamespaceAware();
+    public String getRealPath(String path);
+    
+    
+    /**
+     * Return the effective major version of the Servlet spec used by this
+     * context.
+     */
+    public int getEffectiveMajorVersion();
+    
+    
+    /**
+     * Set the effective major version of the Servlet spec used by this
+     * context.
+     */
+    public void setEffectiveMajorVersion(int major);
+    
+    
+    /**
+     * Return the effective minor version of the Servlet spec used by this
+     * context.
+     */
+    public int getEffectiveMinorVersion();
+    
+    
+    /**
+     * Set the effective minor version of the Servlet spec used by this
+     * context.
+     */
+    public void setEffectiveMinorVersion(int minor);
+    
+    
+    /**
+     * Obtain the JSP configuration for this context.
+     */
+    public JspConfigDescriptor getJspConfigDescriptor();
 
+    
+    /**
+     * Add a URL for a JAR that contains static resources in a
+     * META-INF/resources directory that should be included in the static
+     * resources for this context.
+     */
+    public void addResourceJarUrl(URL url);
+    
+    
+    /**
+     * Add a ServletContainerInitializer instance to this web application.
+     * 
+     * @param sci       The instance to add
+     * @param classes   The classes in which the initializer expressed an
+     *                  interest
+     */
+    public void addServletContainerInitializer(
+            ServletContainerInitializer sci, Set<Class<?>> classes);
+    
+    /**
+     * Is this Context paused whilst it is reloaded?
+     */
+    public boolean getPaused();
+
+    
+    /**
+     * Is this context using version 2.2 of the Servlet spec?
+     */
+    boolean isServlet22();
 
     /**
-     * Get the server.xml <context> attribute's xmlValidation.
-     * @return true if validation is enabled.
-     *
+     * Notification that servlet security has been dynamically set in a
+     * {@link javax.servlet.ServletRegistration.Dynamic}
+     * @param registration servlet security was modified for
+     * @param servletSecurityElement new security constraints for this servlet
+     * @return urls currently mapped to this registration that are already
+     *         present in web.xml
      */
-    public boolean getXmlValidation();
-
+    Set<String> addServletSecurity(ApplicationServletRegistration registration,
+            ServletSecurityElement servletSecurityElement);
+    
+    /**
+     * Sets the (comma separated) list of Servlets that expect a resource to be
+     * present. Used to ensure that welcome files associated with Servlets that
+     * expect a resource to be present are not mapped when there is no resource. 
+     */
+    public void setResourceOnlyServlets(String resourceOnlyServlets);
+    
+    /**
+     * Obtains the list of Servlets that expect a resource to be present.
+     * 
+     * @return  A comma separated list of Servlet names as used in web.xml
+     */
+    public String getResourceOnlyServlets();
 
     /**
-     * Set the validation feature of the XML parser used when
-     * parsing xml instances.
-     * @param xmlValidation true to enable xml instance validation
+     * Checks the named Servlet to see if it expects a resource to be present.
+     * 
+     * @param servletName   Name of the Servlet (as per web.xml) to check
+     * @return              <code>true</code> if the Servlet expects a resource,
+     *                      otherwise <code>false</code>
      */
-    public void setXmlValidation(boolean xmlValidation);
-
-
-   /**
-     * Set the namespace aware feature of the XML parser used when
-     * parsing xml instances.
-     * @param xmlNamespaceAware true to enable namespace awareness
+    public boolean isResourceOnlyServlet(String servletName);
+    
+    /**
+     * Return the base name to use for WARs, directories or context.xml files
+     * for this context.
      */
-    public void setXmlNamespaceAware(boolean xmlNamespaceAware);
+    public String getBaseName();
      
+    /**
+     * Set the version of this web application - used to differentiate
+     * different versions of the same web application when using parallel
+     * deployment.
+     */
+    public void setWebappVersion(String webappVersion);
 
     /**
-     * Set the validation feature of the XML parser used when
-     * parsing tlds files. 
-     * @param tldValidation true to enable xml instance validation
+     * Set the version of this web application - used to differentiate
+     * different versions of the same web application when using parallel
+     * deployment. If not specified, defaults to the empty string.
      */
-    public void setTldValidation(boolean tldValidation);
-
+    public String getWebappVersion();
+    
+    /**
+     * Configure whether or not requests listeners will be fired on forwards for
+     * this Context.
+     */
+    public void setFireRequestListenersOnForwards(boolean enable);
 
     /**
-     * Get the validation feature of the XML parser used when
-     * parsing tlds files. 
-     * @return true if validation is enabled.
+     * Determine whether or not requests listeners will be fired on forwards for
+     * this Context.
      */
-    public boolean getTldValidation();
-
+    public boolean getFireRequestListenersOnForwards();
+    
+    /**
+     * Configures if a user presents authentication credentials, whether the
+     * context will process them when the request is for a non-protected
+     * resource.
+     */
+    public void setPreemptiveAuthentication(boolean enable);
 
     /**
-     * Get the server.xml &lt;host&gt; attribute's xmlNamespaceAware.
-     * @return true if namespace awarenes is enabled.
+     * Determines if a user presents authentication credentials, will the
+     * context will process them when the request is for a non-protected
+     * resource.
      */
-    public boolean getTldNamespaceAware();
-
-
-    /**
-     * Set the namespace aware feature of the XML parser used when
-     * parsing xml instances.
-     * @param tldNamespaceAware true to enable namespace awareness
-     */
-    public void setTldNamespaceAware(boolean tldNamespaceAware);
+    public boolean getPreemptiveAuthentication();
 }
 
