@@ -40,7 +40,7 @@ import java.io.IOException;
  * </ul>
  *
  * @author Craig R. McClanahan
- * @version $Id: Manager.java 939350 2010-04-29 15:36:29Z kkolinko $
+ * @version $Id: Manager.java 1036595 2010-11-18 19:59:11Z markt $
  */
 
 public interface Manager {
@@ -127,7 +127,7 @@ public interface Manager {
      *
      * @return Total number of sessions created by this manager.
      */
-    public int getSessionCounter();
+    public long getSessionCounter();
 
 
     /** 
@@ -135,7 +135,7 @@ public interface Manager {
      *
      * @param sessionCounter Total number of sessions created by this manager.
      */
-    public void setSessionCounter(int sessionCounter);
+    public void setSessionCounter(long sessionCounter);
 
 
     /**
@@ -171,7 +171,7 @@ public interface Manager {
      *
      * @return Number of sessions that have expired
      */
-    public int getExpiredSessions();
+    public long getExpiredSessions();
 
 
     /**
@@ -179,7 +179,7 @@ public interface Manager {
      *
      * @param expiredSessions Number of sessions that have expired
      */
-    public void setExpiredSessions(int expiredSessions);
+    public void setExpiredSessions(long expiredSessions);
 
 
     /**
@@ -191,15 +191,6 @@ public interface Manager {
     public int getRejectedSessions();
 
 
-    /**
-     * Sets the number of sessions that were not created because the maximum
-     * number of active sessions was reached.
-     *
-     * @param rejectedSessions Number of rejected sessions
-     */
-    public void setRejectedSessions(int rejectedSessions);
-
-
     /**
      * Gets the longest time (in seconds) that an expired session had been
      * alive.
@@ -222,24 +213,30 @@ public interface Manager {
 
     /**
      * Gets the average time (in seconds) that expired sessions had been
-     * alive.
-     *
+     * alive. This may be based on sample data.
+     * 
      * @return Average time (in seconds) that expired sessions had been
      * alive.
      */
     public int getSessionAverageAliveTime();
 
+    
+    /**
+     * Gets the current rate of session creation (in session per minute). This
+     * may be based on sample data.
+     * 
+     * @return  The current rate (in sessions per minute) of session creation
+     */
+    public int getSessionCreateRate();
+    
 
     /**
-     * Sets the average time (in seconds) that expired sessions had been
-     * alive.
-     *
-     * @param sessionAverageAliveTime Average time (in seconds) that expired
-     * sessions had been alive.
+     * Gets the current rate of session expiration (in session per minute). This
+     * may be based on sample data
+     * 
+     * @return  The current rate (in sessions per minute) of session expiration
      */
-    public void setSessionAverageAliveTime(int sessionAverageAliveTime);
-
-
+    public int getSessionExpireRate();
     // --------------------------------------------------------- Public Methods
 
 
@@ -276,20 +273,6 @@ public interface Manager {
     public Session createEmptySession();
 
 
-    /**
-     * Construct and return a new session object, based on the default
-     * settings specified by this Manager's properties.  The session
-     * id will be assigned by this method, and available via the getId()
-     * method of the returned session.  If a new session cannot be created
-     * for any reason, return <code>null</code>.
-     * 
-     * @exception IllegalStateException if a new session cannot be
-     *  instantiated for any reason
-     * @deprecated
-     */
-    public Session createSession();
-
-
     /**
      * Construct and return a new session object, based on the default
      * settings specified by this Manager's properties.  The session
@@ -348,6 +331,15 @@ public interface Manager {
     public void remove(Session session);
 
 
+    /**
+     * Remove this Session from the active Sessions for this Manager.
+     *
+     * @param session   Session to be removed
+     * @param update    Should the expiration statistics be updated
+     */
+    public void remove(Session session, boolean update);
+
+
     /**
      * Remove a property change listener from this component.
      *
