@@ -20,6 +20,7 @@ package org.apache.catalina.authenticator;
 
 
 import java.io.IOException;
+import java.nio.charset.Charset;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.Principal;
@@ -27,13 +28,12 @@ import java.util.LinkedHashMap;
 import java.util.Map;
 import java.util.StringTokenizer;
 
+import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
-
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.Realm;
 import org.apache.catalina.connector.Request;
-import org.apache.catalina.connector.Response;
 import org.apache.catalina.deploy.LoginConfig;
 import org.apache.catalina.util.MD5Encoder;
 import org.apache.juli.logging.Log;
@@ -47,12 +47,12 @@ import org.apache.juli.logging.LogFactory;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: DigestAuthenticator.java 1158180 2011-08-16 10:11:43Z markt $
+ * @version $Id: DigestAuthenticator.java 1158177 2011-08-16 10:04:00Z markt $
  */
 
 public class DigestAuthenticator extends AuthenticatorBase {
 
-    private static Log log = LogFactory.getLog(DigestAuthenticator.class);
+    private static final Log log = LogFactory.getLog(DigestAuthenticator.class);
 
 
     // -------------------------------------------------------------- Constants
@@ -96,7 +96,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
     /**
      * MD5 message digest provider.
      */
-    protected static MessageDigest md5Helper;
+    protected static volatile MessageDigest md5Helper;
 
 
     /**
@@ -217,7 +217,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
      */
     @Override
     public boolean authenticate(Request request,
-                                Response response,
+                                HttpServletResponse response,
                                 LoginConfig config)
         throws IOException {
 
@@ -293,6 +293,12 @@ public class DigestAuthenticator extends AuthenticatorBase {
     }
 
 
+    @Override
+    protected String getAuthMethod() {
+        return Constants.DIGEST_METHOD;
+    }
+
+
     // ------------------------------------------------------ Protected Methods
 
 
@@ -345,7 +351,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
         } else if (quotedString.length() > 2) {
             return quotedString.substring(1, quotedString.length() - 1);
         } else {
-            return new String();
+            return "";
         }
     }
 
@@ -373,7 +379,8 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
         byte[] buffer;
         synchronized (md5Helper) {
-            buffer = md5Helper.digest(ipTimeKey.getBytes());
+            buffer = md5Helper.digest(
+                    ipTimeKey.getBytes(Charset.defaultCharset()));
         }
 
         return currentTime + ":" + md5Encoder.encode(buffer);
@@ -407,8 +414,8 @@ public class DigestAuthenticator extends AuthenticatorBase {
      *              should be performed
      * @param nonce nonce token
      */
-    protected void setAuthenticateHeader(Request request,
-                                         Response response,
+    protected void setAuthenticateHeader(HttpServletRequest request,
+                                         HttpServletResponse response,
                                          LoginConfig config,
                                          String nonce,
                                          boolean isNonceStale) {
@@ -429,7 +436,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
             getOpaque() + "\"";
         }
 
-        response.setHeader("WWW-Authenticate", authenticateHeader);
+        response.setHeader(AUTH_HEADER_NAME, authenticateHeader);
 
     }
 
@@ -437,17 +444,17 @@ public class DigestAuthenticator extends AuthenticatorBase {
     // ------------------------------------------------------- Lifecycle Methods
     
     @Override
-    public void start() throws LifecycleException {
-        super.start();
+    protected synchronized void startInternal() throws LifecycleException {
+        super.startInternal();
         
         // Generate a random secret key
         if (getKey() == null) {
-            setKey(generateSessionId());
+            setKey(sessionIdGenerator.generateSessionId());
         }
         
         // Generate the opaque string the same way
         if (getOpaque() == null) {
-            setOpaque(generateSessionId());
+            setOpaque(sessionIdGenerator.generateSessionId());
         }
         
         cnonces = new LinkedHashMap<String, DigestAuthenticator.NonceInfo>() {
@@ -612,7 +619,8 @@ public class DigestAuthenticator extends AuthenticatorBase {
                 request.getRemoteAddr() + ":" + nonceTime + ":" + key;
             byte[] buffer = null;
             synchronized (md5Helper) {
-                buffer = md5Helper.digest(serverIpTimeKey.getBytes());
+                buffer = md5Helper.digest(
+                        serverIpTimeKey.getBytes(Charset.defaultCharset()));
             }
             String md5ServerIpTimeKey = md5Encoder.encode(buffer);
             if (!md5ServerIpTimeKey.equals(md5clientIpTimeKey)) {
@@ -674,7 +682,7 @@ public class DigestAuthenticator extends AuthenticatorBase {
 
             byte[] buffer;
             synchronized (md5Helper) {
-                buffer = md5Helper.digest(a2.getBytes());
+                buffer = md5Helper.digest(a2.getBytes(Charset.defaultCharset()));
             }
             String md5a2 = md5Encoder.encode(buffer);
 
