@@ -18,44 +18,39 @@
 
 package org.apache.catalina.connector;
 
+import java.util.Arrays;
 import java.util.HashMap;
+import java.util.HashSet;
 
-import javax.management.MBeanRegistration;
-import javax.management.MBeanServer;
-import javax.management.MalformedObjectNameException;
 import javax.management.ObjectName;
 
-import org.apache.catalina.Container;
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Service;
 import org.apache.catalina.core.AprLifecycleListener;
-import org.apache.catalina.core.StandardEngine;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.mbeans.MBeanUtils;
+import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.coyote.Adapter;
 import org.apache.coyote.ProtocolHandler;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.http.mapper.Mapper;
-import org.apache.tomcat.util.modeler.Registry;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
- * Implementation of a Coyote connector for Tomcat 5.x.
+ * Implementation of a Coyote connector.
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: Connector.java 1200601 2011-11-10 22:24:17Z kkolinko $
+ * @version $Id: Connector.java 1161340 2011-08-25 00:28:41Z kkolinko $
  */
 
 
-public class Connector
-    implements Lifecycle, MBeanRegistration
-{
-    private static Log log = LogFactory.getLog(Connector.class);
+public class Connector extends LifecycleMBeanBase  {
+
+    private static final Log log = LogFactory.getLog(Connector.class);
 
 
     /**
@@ -68,22 +63,19 @@ public class Connector
     // ------------------------------------------------------------ Constructor
 
 
-    public Connector()
-        throws Exception {
+    public Connector() {
         this(null);
     }
 
-    public Connector(String protocol)
-        throws Exception {
+    public Connector(String protocol) {
         setProtocol(protocol);
         // Instantiate protocol handler
         try {
-            Class clazz = Class.forName(protocolHandlerClassName);
+            Class<?> clazz = Class.forName(protocolHandlerClassName);
             this.protocolHandler = (ProtocolHandler) clazz.newInstance();
         } catch (Exception e) {
-            log.error
-                (sm.getString
-                 ("coyoteConnector.protocolHandlerInstantiationFailed", e));
+            log.error(sm.getString(
+                    "coyoteConnector.protocolHandlerInstantiationFailed"), e);
         }
     }
 
@@ -104,15 +96,9 @@ public class Connector
 
 
     /**
-     * The Container used for processing requests received by this Connector.
+     * Default timeout for asynchronous requests (ms).
      */
-    protected Container container = null;
-
-
-    /**
-     * Use "/" as path for session cookies ?
-     */
-    protected boolean emptySessionPath = false;
+    protected  long asyncTimeout = 10000;
 
 
     /**
@@ -134,12 +120,6 @@ public class Connector
         "org.apache.catalina.connector.Connector/2.1";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
     /**
      * The port number on which we listen for requests.
      */
@@ -156,7 +136,7 @@ public class Connector
 
 
     /**
-     * The server port to which we should pretent requests to this Connector
+     * The server port to which we should pretend requests to this Connector
      * were directed.  This is useful when operating Tomcat behind a proxy
      * server, so that redirects get constructed accurately.  If not specified,
      * the port number specified by the <code>port</code> property is used.
@@ -187,17 +167,10 @@ public class Connector
     /**
      * The string manager for this package.
      */
-    protected StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * The maximum number of parameters (GET plus POST) which will be
-     * automatically parsed by the container. 10000 by default. A value of less
-     * than 0 means no limit.
-     */
-    protected int maxParameterCount = 10000;
-
     /**
      * Maximum size of a POST which will be automatically parsed by the
      * container. 2MB by default.
@@ -211,34 +184,23 @@ public class Connector
      */
     protected int maxSavePostSize = 4 * 1024;
 
+    /**
+     * Comma-separated list of HTTP methods that will be parsed according
+     * to POST-style rules for application/x-www-form-urlencoded request bodies.
+     */
+    protected String parseBodyMethods = "POST";
 
     /**
-     * Has this component been initialized yet?
+     * A Set of methods determined by {@link #parseBodyMethods}.
      */
-    protected boolean initialized = false;
+    protected HashSet<String> parseBodyMethodsSet;
 
 
-    /**
-     * Has this component been started yet?
-     */
-    protected boolean started = false;
-
-
-    /**
-     * The shutdown signal to our background thread
-     */
-    protected boolean stopped = false;
-
     /**
      * Flag to use IP-based virtual hosting.
      */
     protected boolean useIPVHosts = false;
 
-    /**
-     * The background thread.
-     */
-    protected Thread thread = null;
-
 
     /**
      * Coyote Protocol handler class name.
@@ -284,20 +246,14 @@ public class Connector
      protected boolean useBodyEncodingForURI = false;
 
 
-     protected static HashMap replacements = new HashMap();
+     protected static HashMap<String,String> replacements =
+         new HashMap<String,String>();
      static {
          replacements.put("acceptCount", "backlog");
          replacements.put("connectionLinger", "soLinger");
          replacements.put("connectionTimeout", "soTimeout");
-         replacements.put("connectionUploadTimeout", "timeout");
-         replacements.put("clientAuth", "clientauth");
-         replacements.put("keystoreFile", "keystore");
          replacements.put("randomFile", "randomfile");
          replacements.put("rootFile", "rootfile");
-         replacements.put("keystorePass", "keypass");
-         replacements.put("keystoreType", "keytype");
-         replacements.put("sslProtocol", "protocol");
-         replacements.put("sslProtocols", "protocols");
      }
 
 
@@ -310,7 +266,7 @@ public class Connector
     public Object getProperty(String name) {
         String repl = name;
         if (replacements.get(name) != null) {
-            repl = (String) replacements.get(name);
+            repl = replacements.get(name);
         }
         return IntrospectionUtils.getProperty(protocolHandler, repl);
     }
@@ -322,7 +278,7 @@ public class Connector
     public boolean setProperty(String name, String value) {
         String repl = name;
         if (replacements.get(name) != null) {
-            repl = (String) replacements.get(name);
+            repl = replacements.get(name);
         }
         return IntrospectionUtils.setProperty(protocolHandler, repl, value);
     }
@@ -343,15 +299,6 @@ public class Connector
     }
 
 
-    /**
-     * remove a configured property.
-     */
-    public void removeProperty(String name) {
-        // FIXME !
-        //protocolHandler.removeAttribute(name);
-    }
-
-
     /**
      * Return the <code>Service</code> with which we are associated (if any).
      */
@@ -370,7 +317,6 @@ public class Connector
     public void setService(Service service) {
 
         this.service = service;
-        // FIXME: setProperty("service", service);
 
     }
 
@@ -397,85 +343,30 @@ public class Connector
 
     }
 
+    
     /**
-     * Is this connector available for processing requests?
+     * Return the default timeout for async requests in ms.
      */
-    public boolean isAvailable() {
+    public long getAsyncTimeout() {
 
-        return (started);
+        return asyncTimeout;
 
     }
 
 
     /**
-     * Return the input buffer size for this Connector.
+     * Set the default timeout for async requests.
      *
-     * @deprecated
+     * @param asyncTimeout The new timeout in ms.
      */
-    public int getBufferSize() {
-        return 2048;
-    }
+    public void setAsyncTimeout(long asyncTimeout) {
 
-    /**
-     * Set the input buffer size for this Connector.
-     *
-     * @param bufferSize The new input buffer size.
-     * @deprecated
-     */
-    public void setBufferSize(int bufferSize) {
-    }
-
-
-    /**
-     * Return the Container used for processing requests received by this
-     * Connector.
-     */
-    public Container getContainer() {
-        if( container==null ) {
-            // Lazy - maybe it was added later
-            findContainer();
-        }
-        return (container);
+        this.asyncTimeout= asyncTimeout;
+        setProperty("asyncTimeout", String.valueOf(asyncTimeout));
 
     }
 
-
-    /**
-     * Set the Container used for processing requests received by this
-     * Connector.
-     *
-     * @param container The new Container to use
-     */
-    public void setContainer(Container container) {
-
-        this.container = container;
-
-    }
-
-
-    /**
-     * Return the "empty session path" flag.
-     */
-    public boolean getEmptySessionPath() {
-
-        return (this.emptySessionPath);
-
-    }
-
-
-    /**
-     * Set the "empty session path" flag.
-     *
-     * @param emptySessionPath The new "empty session path" flag value
-     */
-    public void setEmptySessionPath(boolean emptySessionPath) {
-
-        this.emptySessionPath = emptySessionPath;
-        setProperty("emptySessionPath", String.valueOf(emptySessionPath));
-
-    }
-
-
+    
     /**
      * Return the "enable DNS lookups" flag.
      */
@@ -509,34 +400,14 @@ public class Connector
     }
 
 
-    /**
-     * Return the mapper.
-     */
-    public Mapper getMapper() {
-        return (mapper);
-    }
+     /**
+      * Return the mapper.
+      */
+     public Mapper getMapper() {
 
+         return (mapper);
 
-    /**
-     * Return the maximum number of parameters (GET plus POST) that will be
-     * automatically parsed by the container. A value of less than 0 means no
-     * limit.
-     */
-    public int getMaxParameterCount() {
-        return maxParameterCount;
-    }
-
-
-    /**
-     * Set the maximum number of parameters (GET plus POST) that will be
-     * automatically parsed by the container. A value of less than 0 means no
-     * limit.
-     *
-     * @param maxParameterCount The new setting
-     */
-    public void setMaxParameterCount(int maxParameterCount) {
-        this.maxParameterCount = maxParameterCount;
-    }
+     }
 
 
     /**
@@ -588,6 +459,33 @@ public class Connector
     }
 
 
+    public String getParseBodyMethods() {
+
+        return this.parseBodyMethods;
+
+    }
+
+    public void setParseBodyMethods(String methods) {
+
+        HashSet<String> methodSet = new HashSet<String>();
+
+        if( null != methods )
+            methodSet.addAll(Arrays.asList(methods.split("\\s*,\\s*")));
+
+        if( methodSet.contains("TRACE") )
+            throw new IllegalArgumentException(sm.getString("coyoteConnector.parseBodyMethodNoTrace"));
+
+        this.parseBodyMethods = methods;
+        this.parseBodyMethodsSet = methodSet;
+
+    }
+
+    protected boolean isParseBodyMethod(String method) {
+
+        return parseBodyMethodsSet.contains(method);
+
+    }
+
     /**
      * Return the port number on which we listen for requests.
      */
@@ -621,7 +519,7 @@ public class Connector
             || "org.apache.coyote.http11.Http11AprProtocol".equals
             (getProtocolHandlerClassName())) {
             return "HTTP/1.1";
-        } else if ("org.apache.jk.server.JkCoyoteHandler".equals
+        } else if ("org.apache.coyote.ajp.AjpProtocol".equals
                    (getProtocolHandlerClassName())
                    || "org.apache.coyote.ajp.AjpAprProtocol".equals
                    (getProtocolHandlerClassName())) {
@@ -658,7 +556,7 @@ public class Connector
                     ("org.apache.coyote.http11.Http11Protocol");
             } else if ("AJP/1.3".equals(protocol)) {
                 setProtocolHandlerClassName
-                    ("org.apache.jk.server.JkCoyoteHandler");
+                    ("org.apache.coyote.ajp.AjpProtocol");
             } else if (protocol != null) {
                 setProtocolHandlerClassName(protocol);
             }
@@ -722,7 +620,6 @@ public class Connector
             setProperty("proxyName", proxyName);
         } else {
             this.proxyName = null;
-            removeProperty("proxyName");
         }
 
     }
@@ -869,7 +766,6 @@ public class Connector
 
      }
 
-
     /**
      * Indicates whether the generation of an X-Powered-By response header for
      * servlet-generated responses is enabled or disabled for this Connector.
@@ -915,9 +811,9 @@ public class Connector
 
 
     public String getExecutorName() {
-        Object ex = IntrospectionUtils.getProperty(protocolHandler, "executor");
-        if (ex instanceof org.apache.catalina.Executor) {
-            return ((org.apache.catalina.Executor) ex).getName();
+        Object obj = protocolHandler.getExecutor();
+        if (obj instanceof org.apache.catalina.Executor) {
+            return ((org.apache.catalina.Executor) obj).getName();
         }
         return "Internal";
     }
@@ -951,50 +847,11 @@ public class Connector
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
-
-
-    /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to add
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-
-    protected ObjectName createObjectName(String domain, String type)
-            throws MalformedObjectNameException {
+    protected String createObjectNameKeyProperties(String type) {
+        
         Object addressObj = getProperty("address");
 
-        StringBuilder sb = new StringBuilder(domain);
-        sb.append(":type=");
+        StringBuilder sb = new StringBuilder("type=");
         sb.append(type);
         sb.append(",port=");
         sb.append(getPort());
@@ -1005,61 +862,14 @@ public class Connector
                 sb.append(ObjectName.quote(address));
             }
         }
-        ObjectName _oname = new ObjectName(sb.toString());
-        return _oname;
-    }
-
-    /**
-     * Initialize this connector (create ServerSocket here!)
-     */
-    public void initialize()
-        throws LifecycleException
-    {
-        if (initialized) {
-            if(log.isInfoEnabled())
-                log.info(sm.getString("coyoteConnector.alreadyInitialized"));
-           return;
-        }
-
-        this.initialized = true;
-
-        if( oname == null && (container instanceof StandardEngine)) {
-            try {
-                // we are loaded directly, via API - and no name was given to us
-                StandardEngine cb=(StandardEngine)container;
-                oname = createObjectName(cb.getName(), "Connector");
-                Registry.getRegistry(null, null)
-                    .registerComponent(this, oname, null);
-                controller=oname;
-            } catch (Exception e) {
-                log.error( "Error registering connector ", e);
-            }
-            if(log.isDebugEnabled())
-                log.debug("Creating name for connector " + oname);
-        }
-
-        // Initializa adapter
-        adapter = new CoyoteAdapter(this);
-        protocolHandler.setAdapter(adapter);
-
-        IntrospectionUtils.setProperty(protocolHandler, "jkHome",
-                                       System.getProperty("catalina.base"));
-
-        try {
-            protocolHandler.init();
-        } catch (Exception e) {
-            throw new LifecycleException
-                (sm.getString
-                 ("coyoteConnector.protocolHandlerInitializationFailed", e));
-        }
+        return sb.toString();
     }
 
 
     /**
      * Pause the connector.
      */
-    public void pause()
-        throws LifecycleException {
+    public void pause() {
         try {
             protocolHandler.pause();
         } catch (Exception e) {
@@ -1072,8 +882,7 @@ public class Connector
     /**
      * Pause the connector.
      */
-    public void resume()
-        throws LifecycleException {
+    public void resume() {
         try {
             protocolHandler.resume();
         } catch (Exception e) {
@@ -1083,40 +892,47 @@ public class Connector
     }
 
 
+    @Override
+    protected void initInternal() throws LifecycleException {
+
+        super.initInternal();
+        
+        // Initialize adapter
+        adapter = new CoyoteAdapter(this);
+        protocolHandler.setAdapter(adapter);
+
+        // Make sure parseBodyMethodsSet has a default
+        if( null == parseBodyMethodsSet )
+            setParseBodyMethods(getParseBodyMethods());
+
+        try {
+            protocolHandler.init();
+        } catch (Exception e) {
+            throw new LifecycleException
+                (sm.getString
+                 ("coyoteConnector.protocolHandlerInitializationFailed"), e);
+        }
+
+        // Initialize mapper listener
+        mapperListener.init();
+    }
+
+
     /**
      * Begin processing requests via this Connector.
      *
      * @exception LifecycleException if a fatal startup error occurs
      */
-    public void start() throws LifecycleException {
-        if( !initialized )
-            initialize();
+    @Override
+    protected void startInternal() throws LifecycleException {
 
-        // Validate and update our current state
-        if (started ) {
-            if(log.isInfoEnabled())
-                log.info(sm.getString("coyoteConnector.alreadyStarted"));
-            return;
+        // Validate settings before starting
+        if (getPort() < 1) {
+            throw new LifecycleException(sm.getString(
+                    "coyoteConnector.invalidPort", Integer.valueOf(getPort())));
         }
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
 
-        // We can't register earlier - the JMX registration of this happens
-        // in Server.start callback
-        if ( this.oname != null ) {
-            // We are registred - register the adapter as well.
-            try {
-                Registry.getRegistry(null, null).registerComponent
-                    (protocolHandler, createObjectName(this.domain,"ProtocolHandler"), null);
-            } catch (Exception ex) {
-                log.error(sm.getString
-                          ("coyoteConnector.protocolRegistrationFailed"), ex);
-            }
-        } else {
-            if(log.isInfoEnabled())
-                log.info(sm.getString
-                     ("coyoteConnector.cannotRegisterProtocol"));
-        }
+        setState(LifecycleState.STARTING);
 
         try {
             protocolHandler.start();
@@ -1128,25 +944,10 @@ public class Connector
 
             throw new LifecycleException
                 (errPrefix + " " + sm.getString
-                 ("coyoteConnector.protocolHandlerStartFailed", e));
+                 ("coyoteConnector.protocolHandlerStartFailed"), e);
         }
 
-        if( this.domain != null ) {
-            mapperListener.setDomain( domain );
-            //mapperListener.setEngine( service.getContainer().getName() );
-            mapperListener.init();
-            try {
-                ObjectName mapperOname = createObjectName(this.domain,"Mapper");
-                if (log.isDebugEnabled())
-                    log.debug(sm.getString(
-                            "coyoteConnector.MapperRegistration", mapperOname));
-                Registry.getRegistry(null, null).registerComponent
-                    (mapper, mapperOname, "Mapper");
-            } catch (Exception ex) {
-                log.error(sm.getString
-                        ("coyoteConnector.protocolRegistrationFailed"), ex);
-            }
-        }
+        mapperListener.start();
     }
 
 
@@ -1155,144 +956,42 @@ public class Connector
      *
      * @exception LifecycleException if a fatal shutdown error occurs
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
-        // Validate and update our current state
-        if (!started) {
-            log.error(sm.getString("coyoteConnector.notStarted"));
-            return;
-
-        }
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
+        setState(LifecycleState.STOPPING);
 
         try {
-            mapperListener.destroy();
-            Registry.getRegistry(null, null).unregisterComponent
-                (createObjectName(this.domain,"Mapper"));
-            Registry.getRegistry(null, null).unregisterComponent
-                (createObjectName(this.domain,"ProtocolHandler"));
-        } catch (MalformedObjectNameException e) {
-            log.error( sm.getString
-                    ("coyoteConnector.protocolUnregistrationFailed"), e);
+            protocolHandler.stop();
+        } catch (Exception e) {
+            throw new LifecycleException
+                (sm.getString
+                 ("coyoteConnector.protocolHandlerStopFailed"), e);
         }
+
+        mapperListener.stop();
+    }
+
+
+    @Override
+    protected void destroyInternal() throws LifecycleException {
+        mapperListener.destroy();
+        
         try {
             protocolHandler.destroy();
         } catch (Exception e) {
             throw new LifecycleException
                 (sm.getString
-                 ("coyoteConnector.protocolHandlerDestroyFailed", e));
+                 ("coyoteConnector.protocolHandlerDestroyFailed"), e);
         }
 
-    }
-
-
-    // -------------------- JMX registration  --------------------
-    protected String domain;
-    protected ObjectName oname;
-    protected MBeanServer mserver;
-    ObjectName controller;
-
-    public ObjectName getController() {
-        return controller;
-    }
-
-    public void setController(ObjectName controller) {
-        this.controller = controller;
-    }
-
-    public ObjectName getObjectName() {
-        return oname;
-    }
-
-    public String getDomain() {
-        return domain;
-    }
-
-    public ObjectName preRegister(MBeanServer server,
-                                  ObjectName name) throws Exception {
-        oname=name;
-        mserver=server;
-        domain=name.getDomain();
-        return name;
-    }
-
-    public void postRegister(Boolean registrationDone) {
-    }
-
-    public void preDeregister() throws Exception {
-    }
-
-    public void postDeregister() {
-        try {
-            if( started ) {
-                stop();
-            }
-        } catch( Throwable t ) {
-            log.error( "Unregistering - can't stop", t);
+        if (getService() != null) {
+            getService().removeConnector(this);
         }
+        
+        super.destroyInternal();
     }
 
-    protected void findContainer() {
-        try {
-            // Register to the service
-            ObjectName parentName=new ObjectName( domain + ":" +
-                    "type=Service");
-
-            if(log.isDebugEnabled())
-                log.debug("Adding to " + parentName );
-            if( mserver.isRegistered(parentName )) {
-                mserver.invoke(parentName, "addConnector", new Object[] { this },
-                        new String[] {"org.apache.catalina.connector.Connector"});
-                // As a side effect we'll get the container field set
-                // Also initialize will be called
-                //return;
-            }
-            // XXX Go directly to the Engine
-            // initialize(); - is called by addConnector
-            ObjectName engName=new ObjectName( domain + ":" + "type=Engine");
-            if( mserver.isRegistered(engName )) {
-                Object obj=mserver.getAttribute(engName, "managedResource");
-                if(log.isDebugEnabled())
-                      log.debug("Found engine " + obj + " " + obj.getClass());
-                container=(Container)obj;
-
-                // Internal initialize - we now have the Engine
-                initialize();
-
-                if(log.isDebugEnabled())
-                    log.debug("Initialized");
-                // As a side effect we'll get the container field set
-                // Also initialize will be called
-                return;
-            }
-        } catch( Exception ex ) {
-            log.error( "Error finding container " + ex);
-        }
-    }
-
-    public void init() throws Exception {
-
-        if( this.getService() != null ) {
-            if(log.isDebugEnabled())
-                 log.debug( "Already configured" );
-            return;
-        }
-        if( container==null ) {
-            findContainer();
-        }
-    }
-
-    public void destroy() throws Exception {
-        if( oname!=null && controller==oname ) {
-            if(log.isDebugEnabled())
-                 log.debug("Unregister itself " + oname );
-            Registry.getRegistry(null, null).unregisterComponent(oname);
-        }
-        if( getService() == null)
-            return;
-        getService().removeConnector(this);
-    }
 
     /**
      * Provide a useful toString() implementation as it may be used when logging
@@ -1308,4 +1007,17 @@ public class Connector
         sb.append(']');
         return sb.toString();
     }
+
+    // -------------------- JMX registration  --------------------
+
+    @Override
+    protected String getDomainInternal() {
+        return MBeanUtils.getDomain(getService());
+    }
+
+    @Override
+    protected String getObjectNameKeyProperties() {
+        return createObjectNameKeyProperties("Connector");
+    }
+
 }
