@@ -24,6 +24,7 @@ import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
+import java.util.Collection;
 import java.util.Locale;
 
 import javax.servlet.ServletOutputStream;
@@ -31,8 +32,8 @@ import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.Globals;
-import org.apache.catalina.util.StringManager;
 import org.apache.catalina.security.SecurityUtil;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Facade class that wraps a Coyote response object. 
@@ -40,7 +41,7 @@ import org.apache.catalina.security.SecurityUtil;
  *
  * @author Remy Maucherat
  * @author Jean-Francois Arcand
- * @version $Id: ResponseFacade.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: ResponseFacade.java 1149678 2011-07-22 18:09:54Z markt $
  */
 @SuppressWarnings("deprecation")
 public class ResponseFacade 
@@ -50,7 +51,7 @@ public class ResponseFacade
     // ----------------------------------------------------------- DoPrivileged
     
     private final class SetContentTypePrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<Void> {
 
         private String contentType;
 
@@ -58,14 +59,15 @@ public class ResponseFacade
             this.contentType = contentType;
         }
         
-        public Object run() {
+        @Override
+        public Void run() {
             response.setContentType(contentType);
             return null;
         }            
     }
 
     private final class DateHeaderPrivilegedAction
-            implements PrivilegedAction {
+            implements PrivilegedAction<Void> {
 
         private String name;
         private long value;
@@ -77,7 +79,8 @@ public class ResponseFacade
             this.add = add;
         }
 
-        public Object run() {
+        @Override
+        public Void run() {
             if(add) {
                 response.addDateHeader(name, value);
             } else {
@@ -107,7 +110,7 @@ public class ResponseFacade
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
@@ -131,6 +134,7 @@ public class ResponseFacade
     /**
      * Prevent cloning the facade.
      */
+    @Override
     protected Object clone()
         throws CloneNotSupportedException {
         throw new CloneNotSupportedException();
@@ -159,9 +163,20 @@ public class ResponseFacade
     }
 
 
+    public long getContentWritten() {
+
+        if (response == null) {
+            throw new IllegalStateException(
+                            sm.getString("responseFacade.nullResponse"));
+        }
+
+        return response.getContentWritten();
+    }
+
     // ------------------------------------------------ ServletResponse Methods
 
 
+    @Override
     public String getCharacterEncoding() {
 
         if (response == null) {
@@ -173,6 +188,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public ServletOutputStream getOutputStream()
         throws IOException {
 
@@ -188,6 +204,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public PrintWriter getWriter()
         throws IOException {
 
@@ -203,6 +220,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setContentLength(int len) {
 
         if (isCommitted())
@@ -213,6 +231,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setContentType(String type) {
 
         if (isCommitted())
@@ -226,17 +245,19 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setBufferSize(int size) {
 
         if (isCommitted())
             throw new IllegalStateException
-                (/*sm.getString("responseBase.reset.ise")*/);
+                (sm.getString("coyoteResponse.setBufferSize.ise"));
 
         response.setBufferSize(size);
 
     }
 
 
+    @Override
     public int getBufferSize() {
 
         if (response == null) {
@@ -248,6 +269,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void flushBuffer()
         throws IOException {
 
@@ -258,9 +280,11 @@ public class ResponseFacade
 
         if (SecurityUtil.isPackageProtectionEnabled()){
             try{
-                AccessController.doPrivileged(new PrivilegedExceptionAction(){
+                AccessController.doPrivileged(
+                        new PrivilegedExceptionAction<Void>(){
 
-                    public Object run() throws IOException{
+                    @Override
+                    public Void run() throws IOException{
                         response.setAppCommitted(true);
 
                         response.flushBuffer();
@@ -282,17 +306,19 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void resetBuffer() {
 
         if (isCommitted())
             throw new IllegalStateException
-                (/*sm.getString("responseBase.reset.ise")*/);
+                (sm.getString("coyoteResponse.resetBuffer.ise"));
 
         response.resetBuffer();
 
     }
 
 
+    @Override
     public boolean isCommitted() {
 
         if (response == null) {
@@ -304,17 +330,19 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void reset() {
 
         if (isCommitted())
             throw new IllegalStateException
-                (/*sm.getString("responseBase.reset.ise")*/);
+                (sm.getString("coyoteResponse.reset.ise"));
 
         response.reset();
 
     }
 
 
+    @Override
     public void setLocale(Locale loc) {
 
         if (isCommitted())
@@ -324,6 +352,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public Locale getLocale() {
 
         if (response == null) {
@@ -335,6 +364,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void addCookie(Cookie cookie) {
 
         if (isCommitted())
@@ -345,6 +375,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public boolean containsHeader(String name) {
 
         if (response == null) {
@@ -356,6 +387,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public String encodeURL(String url) {
 
         if (response == null) {
@@ -367,6 +399,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public String encodeRedirectURL(String url) {
 
         if (response == null) {
@@ -378,6 +411,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public String encodeUrl(String url) {
 
         if (response == null) {
@@ -389,6 +423,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public String encodeRedirectUrl(String url) {
 
         if (response == null) {
@@ -400,12 +435,13 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void sendError(int sc, String msg)
         throws IOException {
 
         if (isCommitted())
             throw new IllegalStateException
-                (/*sm.getString("responseBase.reset.ise")*/);
+                (sm.getString("coyoteResponse.sendError.ise"));
 
         response.setAppCommitted(true);
 
@@ -414,12 +450,13 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void sendError(int sc)
         throws IOException {
 
         if (isCommitted())
             throw new IllegalStateException
-                (/*sm.getString("responseBase.reset.ise")*/);
+                (sm.getString("coyoteResponse.sendError.ise"));
 
         response.setAppCommitted(true);
 
@@ -428,12 +465,13 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void sendRedirect(String location)
         throws IOException {
 
         if (isCommitted())
             throw new IllegalStateException
-                (/*sm.getString("responseBase.reset.ise")*/);
+                (sm.getString("coyoteResponse.sendRedirect.ise"));
 
         response.setAppCommitted(true);
 
@@ -442,6 +480,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setDateHeader(String name, long date) {
 
         if (isCommitted())
@@ -457,6 +496,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void addDateHeader(String name, long date) {
 
         if (isCommitted())
@@ -472,6 +512,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setHeader(String name, String value) {
 
         if (isCommitted())
@@ -482,6 +523,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void addHeader(String name, String value) {
 
         if (isCommitted())
@@ -492,6 +534,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setIntHeader(String name, int value) {
 
         if (isCommitted())
@@ -502,6 +545,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void addIntHeader(String name, int value) {
 
         if (isCommitted())
@@ -512,6 +556,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setStatus(int sc) {
 
         if (isCommitted())
@@ -522,6 +567,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setStatus(int sc, String sm) {
 
         if (isCommitted())
@@ -531,6 +577,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public String getContentType() {
 
         if (response == null) {
@@ -542,6 +589,7 @@ public class ResponseFacade
     }
 
 
+    @Override
     public void setCharacterEncoding(String arg0) {
 
         if (response == null) {
@@ -552,4 +600,23 @@ public class ResponseFacade
         response.setCharacterEncoding(arg0);
     }
 
+    @Override
+    public int getStatus() {
+        return response.getStatus();
+    }
+    
+    @Override
+    public String getHeader(String name) {
+        return response.getHeader(name);
+    }
+    
+    @Override
+    public Collection<String> getHeaderNames() {
+        return response.getHeaderNames();
+    }
+    
+    @Override
+    public Collection<String> getHeaders(String name) {
+        return response.getHeaders(name);
+    }
 }
