@@ -32,15 +32,16 @@ import javax.servlet.ServletResponse;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
-import org.apache.catalina.CometEvent;
-import org.apache.catalina.CometFilter;
-import org.apache.catalina.CometFilterChain;
-import org.apache.catalina.CometProcessor;
 import org.apache.catalina.Globals;
 import org.apache.catalina.InstanceEvent;
+import org.apache.catalina.comet.CometEvent;
+import org.apache.catalina.comet.CometFilter;
+import org.apache.catalina.comet.CometFilterChain;
+import org.apache.catalina.comet.CometProcessor;
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.catalina.util.InstanceSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Implementation of <code>javax.servlet.FilterChain</code> used to manage
@@ -50,19 +51,19 @@ import org.apache.catalina.util.StringManager;
  * method itself.
  *
  * @author Craig R. McClanahan
- * @version $Revision: 734559 $ $Date: 2009-01-15 00:39:18 +0100 (Do, 15. Jan 2009) $
+ * @version $Id: ApplicationFilterChain.java 1078022 2011-03-04 15:52:01Z markt $
  */
 
 final class ApplicationFilterChain implements FilterChain, CometFilterChain {
 
     // Used to enforce requirements of SRV.8.2 / SRV.14.2.5.1
-    private final static ThreadLocal lastServicedRequest;
-    private final static ThreadLocal lastServicedResponse;
+    private static final ThreadLocal<ServletRequest> lastServicedRequest;
+    private static final ThreadLocal<ServletResponse> lastServicedResponse;
 
     static {
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
-            lastServicedRequest = new ThreadLocal();
-            lastServicedResponse = new ThreadLocal();
+        if (ApplicationDispatcher.WRAP_SAME_OBJECT) {
+            lastServicedRequest = new ThreadLocal<ServletRequest>();
+            lastServicedResponse = new ThreadLocal<ServletResponse>();
         } else {
             lastServicedRequest = null;
             lastServicedResponse = null;
@@ -135,15 +136,15 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
      * Static class array used when the SecurityManager is turned on and 
      * <code>doFilter</code> is invoked.
      */
-    private static Class[] classType = new Class[]{ServletRequest.class, 
-                                                   ServletResponse.class,
-                                                   FilterChain.class};
+    private static Class<?>[] classType = new Class[]{ServletRequest.class, 
+                                                      ServletResponse.class,
+                                                      FilterChain.class};
                                                    
     /**
      * Static class array used when the SecurityManager is turned on and 
      * <code>service</code> is invoked.
      */                                                 
-    private static Class[] classTypeUsedInService = new Class[]{
+    private static Class<?>[] classTypeUsedInService = new Class[]{
                                                          ServletRequest.class,
                                                          ServletResponse.class};
 
@@ -151,16 +152,17 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
      * Static class array used when the SecurityManager is turned on and 
      * <code>doFilterEvent</code> is invoked.
      */
-    private static Class[] cometClassType = 
+    private static Class<?>[] cometClassType = 
         new Class[]{ CometEvent.class, CometFilterChain.class};
                                                    
     /**
      * Static class array used when the SecurityManager is turned on and 
      * <code>event</code> is invoked.
      */                                                 
-    private static Class[] classTypeUsedInEvent = 
+    private static Class<?>[] classTypeUsedInEvent = 
         new Class[] { CometEvent.class };
 
+
     // ---------------------------------------------------- FilterChain Methods
 
 
@@ -175,6 +177,7 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet exception occurs
      */
+    @Override
     public void doFilter(ServletRequest request, ServletResponse response)
         throws IOException, ServletException {
 
@@ -183,8 +186,9 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
             final ServletResponse res = response;
             try {
                 java.security.AccessController.doPrivileged(
-                    new java.security.PrivilegedExceptionAction() {
-                        public Object run() 
+                    new java.security.PrivilegedExceptionAction<Void>() {
+                        @Override
+                        public Void run() 
                             throws ServletException, IOException {
                             internalDoFilter(req,res);
                             return null;
@@ -220,6 +224,11 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                 support.fireInstanceEvent(InstanceEvent.BEFORE_FILTER_EVENT,
                                           filter, request, response);
                 
+                if (request.isAsyncSupported() && "false".equalsIgnoreCase(
+                        filterConfig.getFilterDef().getAsyncSupported())) {
+                    request.setAttribute(Globals.ASYNC_SUPPORTED_ATTR,
+                            Boolean.FALSE);
+                }
                 if( Globals.IS_SECURITY_ENABLED ) {
                     final ServletRequest req = request;
                     final ServletResponse res = response;
@@ -230,7 +239,6 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                     SecurityUtil.doAsPrivilege
                         ("doFilter", filter, classType, args, principal);
                     
-                    args = null;
                 } else {  
                     filter.doFilter(request, response, this);
                 }
@@ -253,6 +261,7 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                                               filter, request, response, e);
                 throw e;
             } catch (Throwable e) {
+                ExceptionUtils.handleThrowable(e);
                 if (filter != null)
                     support.fireInstanceEvent(InstanceEvent.AFTER_FILTER_EVENT,
                                               filter, request, response, e);
@@ -264,13 +273,19 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
 
         // We fell off the end of the chain -- call the servlet instance
         try {
-            if (Globals.STRICT_SERVLET_COMPLIANCE) {
+            if (ApplicationDispatcher.WRAP_SAME_OBJECT) {
                 lastServicedRequest.set(request);
                 lastServicedResponse.set(response);
             }
 
             support.fireInstanceEvent(InstanceEvent.BEFORE_SERVICE_EVENT,
                                       servlet, request, response);
+            if (request.isAsyncSupported()
+                    && !support.getWrapper().isAsyncSupported()) {
+                request.setAttribute(Globals.ASYNC_SUPPORTED_ATTR,
+                        Boolean.FALSE);
+            }
+            // Use potentially wrapped request from this point
             if ((request instanceof HttpServletRequest) &&
                 (response instanceof HttpServletResponse)) {
                     
@@ -285,10 +300,8 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                                                classTypeUsedInService, 
                                                args,
                                                principal);   
-                    args = null;
                 } else {  
-                    servlet.service((HttpServletRequest) request,
-                                    (HttpServletResponse) response);
+                    servlet.service(request, response);
                 }
             } else {
                 servlet.service(request, response);
@@ -308,12 +321,13 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                                       servlet, request, response, e);
             throw e;
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             support.fireInstanceEvent(InstanceEvent.AFTER_SERVICE_EVENT,
                                       servlet, request, response, e);
             throw new ServletException
               (sm.getString("filterChain.servlet"), e);
         } finally {
-            if (Globals.STRICT_SERVLET_COMPLIANCE) {
+            if (ApplicationDispatcher.WRAP_SAME_OBJECT) {
                 lastServicedRequest.set(null);
                 lastServicedResponse.set(null);
             }
@@ -323,16 +337,14 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
 
 
     /**
-     * Invoke the next filter in this chain, passing the specified request
-     * and response.  If there are no more filters in this chain, invoke
-     * the <code>service()</code> method of the servlet itself.
-     *
-     * @param request The servlet request we are processing
-     * @param response The servlet response we are creating
-     *
+     * Process the event, using the security manager if the option is enabled.
+     * 
+     * @param event the event to process
+     * 
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet exception occurs
      */
+    @Override
     public void doFilterEvent(CometEvent event)
         throws IOException, ServletException {
 
@@ -340,8 +352,9 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
             final CometEvent ev = event;
             try {
                 java.security.AccessController.doPrivileged(
-                    new java.security.PrivilegedExceptionAction() {
-                        public Object run() 
+                    new java.security.PrivilegedExceptionAction<Void>() {
+                        @Override
+                        public Void run() 
                             throws ServletException, IOException {
                             internalDoFilterEvent(ev);
                             return null;
@@ -372,7 +385,7 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
      * @return The last request to be serviced. 
      */
     public static ServletRequest getLastServicedRequest() {
-        return (ServletRequest) lastServicedRequest.get();
+        return lastServicedRequest.get();
     }
 
     
@@ -383,7 +396,7 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
      * @return The last response to be serviced. 
      */
     public static ServletResponse getLastServicedResponse() {
-        return (ServletResponse) lastServicedResponse.get();
+        return lastServicedResponse.get();
     }
     
     
@@ -411,7 +424,6 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                     SecurityUtil.doAsPrivilege("doFilterEvent", filter,
                             cometClassType, args, principal);
 
-                    args = null;
                 } else {  
                     filter.doFilterEvent(event, this);
                 }
@@ -440,6 +452,7 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                             */
                 throw e;
             } catch (Throwable e) {
+                ExceptionUtils.handleThrowable(e);
                 /*if (filter != null)
                     support.fireInstanceEvent(InstanceEvent.AFTER_FILTER_EVENT,
                             filter, event, e);*/
@@ -465,7 +478,6 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                         classTypeUsedInEvent, 
                         args,
                         principal);
-                args = null;
             } else {  
                 ((CometProcessor) servlet).event(event);
             }
@@ -491,6 +503,7 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
                     */
             throw e;
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             /*
             support.fireInstanceEvent(InstanceEvent.AFTER_SERVICE_EVENT,
                     servlet, request, response, e);
@@ -505,7 +518,6 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
     // -------------------------------------------------------- Package Methods
 
 
-
     /**
      * Add a filter to the set of filters that will be executed in this chain.
      *
@@ -513,6 +525,11 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
      */
     void addFilter(ApplicationFilterConfig filterConfig) {
 
+        // Prevent the same filter being added multiple times
+        for(ApplicationFilterConfig filter:filters)
+            if(filter==filterConfig)
+                return;
+
         if (n == filters.length) {
             ApplicationFilterConfig[] newFilters =
                 new ApplicationFilterConfig[n + INCREMENT];
@@ -571,6 +588,4 @@ final class ApplicationFilterChain implements FilterChain, CometFilterChain {
         this.support = support;
 
     }
-
-
 }
