@@ -14,11 +14,8 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
-
 import java.io.IOException;
 import java.util.Locale;
 
@@ -26,8 +23,6 @@ import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpServletResponseWrapper;
 
-import org.apache.catalina.util.StringManager;
-
 
 /**
  * Wrapper around a <code>javax.servlet.http.HttpServletResponse</code>
@@ -42,7 +37,7 @@ import org.apache.catalina.util.StringManager;
  * keep these two classes in synchronization when making changes!
  *
  * @author Craig R. McClanahan
- * @version $Id: ApplicationHttpResponse.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: ApplicationHttpResponse.java 1104557 2011-05-17 20:57:46Z kkolinko $
  */
 
 class ApplicationHttpResponse extends HttpServletResponseWrapper {
@@ -96,13 +91,6 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
         "org.apache.catalina.core.ApplicationHttpResponse/1.0";
 
 
-    /**
-     * The string manager for this package.
-     */
-    protected static StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
     // ------------------------------------------------ ServletResponse Methods
 
 
@@ -112,6 +100,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @exception IllegalStateException if the response has already
      *  been committed
      */
+    @Override
     public void reset() {
 
         // If already committed, the wrapped response will throw ISE
@@ -126,6 +115,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param len The new content length
      */
+    @Override
     public void setContentLength(int len) {
 
         if (!included)
@@ -139,6 +129,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param type The new content type
      */
+    @Override
     public void setContentType(String type) {
 
         if (!included)
@@ -152,6 +143,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param loc The new locale
      */
+    @Override
     public void setLocale(Locale loc) {
 
         if (!included)
@@ -165,6 +157,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param size The buffer size
      */
+    @Override
     public void setBufferSize(int size) {
         if (!included)
             getResponse().setBufferSize(size);
@@ -179,6 +172,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param cookie The new cookie
      */
+    @Override
     public void addCookie(Cookie cookie) {
 
         if (!included)
@@ -193,6 +187,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @param name The new header name
      * @param value The new header value
      */
+    @Override
     public void addDateHeader(String name, long value) {
 
         if (!included)
@@ -207,6 +202,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @param name The new header name
      * @param value The new header value
      */
+    @Override
     public void addHeader(String name, String value) {
 
         if (!included)
@@ -221,6 +217,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @param name The new header name
      * @param value The new header value
      */
+    @Override
     public void addIntHeader(String name, int value) {
 
         if (!included)
@@ -236,6 +233,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void sendError(int sc) throws IOException {
 
         if (!included)
@@ -252,6 +250,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void sendError(int sc, String msg) throws IOException {
 
         if (!included)
@@ -267,6 +266,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @exception IOException if an input/output error occurs
      */
+    @Override
     public void sendRedirect(String location) throws IOException {
 
         if (!included)
@@ -281,6 +281,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @param name The new header name
      * @param value The new header value
      */
+    @Override
     public void setDateHeader(String name, long value) {
 
         if (!included)
@@ -295,6 +296,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @param name The new header name
      * @param value The new header value
      */
+    @Override
     public void setHeader(String name, String value) {
 
         if (!included)
@@ -309,6 +311,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      * @param name The new header name
      * @param value The new header value
      */
+    @Override
     public void setIntHeader(String name, int value) {
 
         if (!included)
@@ -322,6 +325,7 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param sc The new status code
      */
+    @Override
     public void setStatus(int sc) {
 
         if (!included)
@@ -335,7 +339,13 @@ class ApplicationHttpResponse extends HttpServletResponseWrapper {
      *
      * @param sc The new status code
      * @param msg The new message
+     * @deprecated As of version 2.1, due to ambiguous meaning of the message
+     *             parameter. To set a status code use
+     *             <code>setStatus(int)</code>, to send an error with a
+     *             description use <code>sendError(int, String)</code>.
      */
+    @Deprecated
+    @Override
     public void setStatus(int sc, String msg) {
 
         if (!included)
