@@ -24,7 +24,6 @@ import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
 import java.sql.DriverManager;
-import java.util.StringTokenizer;
 
 import javax.imageio.ImageIO;
 import javax.xml.parsers.DocumentBuilderFactory;
@@ -80,7 +79,7 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     public void setAWTThreadProtection(boolean awtThreadProtection) {
       this.awtThreadProtection = awtThreadProtection;
     }
-    
+
     /**
      * Protect against the memory leak caused when the first call to
      * <code>sun.misc.GC.requestLatency(long)</code> is triggered by a web
@@ -191,22 +190,10 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
         this.driverManagerProtection = driverManagerProtection;
     }
 
-    /**
-     * List of comma-separated fully qualified class names to load and initialize during
-     * the startup of this Listener. This allows to pre-load classes that are known to 
-     * provoke classloader leaks if they are loaded during a request processing.
-     */
-    private String classesToInitialize = null;
-    public String getClassesToInitialize() {
-        return classesToInitialize;
-    }
-    public void setClassesToInitialize(String classesToInitialize) {
-        this.classesToInitialize = classesToInitialize;
-    }
-    
+    @Override
     public void lifecycleEvent(LifecycleEvent event) {
         // Initialise these classes when Tomcat starts
-        if (Lifecycle.INIT_EVENT.equals(event.getType())) {
+        if (Lifecycle.BEFORE_INIT_EVENT.equals(event.getType())) {
 
             ClassLoader loader = Thread.currentThread().getContextClassLoader();
 
@@ -246,10 +233,10 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
 
                 // Trigger the creation of the AWT (AWT-Windows, AWT-XAWT,
                 // etc.) thread
-                if(awtThreadProtection) {
+                if (awtThreadProtection) {
                   java.awt.Toolkit.getDefaultToolkit();
                 }
-                
+
                 /*
                  * Several components end up calling:
                  * sun.misc.GC.requestLatency(long)
@@ -407,22 +394,6 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                         }
                     }
                 }
-                
-                if (classesToInitialize != null) {
-                    StringTokenizer strTok =
-                        new StringTokenizer(classesToInitialize, ", \r\n\t");
-                    while (strTok.hasMoreTokens()) {
-                        String classNameToLoad = strTok.nextToken();
-                        try {
-                            Class.forName(classNameToLoad);
-                        } catch (ClassNotFoundException e) {
-                            log.error(
-                                sm.getString("jreLeakListener.classToInitializeFail",
-                                    classNameToLoad), e);
-                            // continue with next class to load
-                        }
-                    }
-                }
 
             } finally {
                 Thread.currentThread().setContextClassLoader(loader);
