@@ -14,27 +14,20 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
-
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
-import java.io.File;
-import java.util.List;
+import java.util.Locale;
 import java.util.concurrent.atomic.AtomicReference;
 
-import javax.management.MBeanServer;
-import javax.management.MalformedObjectNameException;
-import javax.management.ObjectName;
-
 import org.apache.catalina.AccessLog;
 import org.apache.catalina.Container;
 import org.apache.catalina.ContainerEvent;
 import org.apache.catalina.ContainerListener;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
+import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
@@ -48,8 +41,6 @@ import org.apache.catalina.realm.JAASRealm;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import org.apache.tomcat.util.modeler.Registry;
-import org.apache.tomcat.util.modeler.modules.MbeansSource;
 
 /**
  * Standard implementation of the <b>Engine</b> interface.  Each
@@ -58,14 +49,12 @@ import org.apache.tomcat.util.modeler.modules.MbeansSource;
  * You can set the jvmRoute direct or with the System.property <b>jvmRoute</b>.
  *
  * @author Craig R. McClanahan
- * @version $Id: StandardEngine.java 1056442 2011-01-07 18:26:09Z markt $
+ * @version $Id: StandardEngine.java 1067725 2011-02-06 18:46:42Z markt $
  */
 
-public class StandardEngine
-    extends ContainerBase
-    implements Engine {
+public class StandardEngine extends ContainerBase implements Engine {
 
-    private static Log log = LogFactory.getLog(StandardEngine.class);
+    private static final Log log = LogFactory.getLog(StandardEngine.class);
 
     // ----------------------------------------------------------- Constructors
 
@@ -81,6 +70,7 @@ public class StandardEngine
         try {
             setJvmRoute(System.getProperty("jvmRoute"));
         } catch(Exception ex) {
+            log.warn(sm.getString("standardEngine.jvmRouteFail"));
         }
         // By default, the engine will hold the reloading thread
         backgroundProcessorDelay = 10;
@@ -110,26 +100,12 @@ public class StandardEngine
      */
     private Service service = null;
 
-    /** Allow the base dir to be specified explicitely for
+    /** Allow the base dir to be specified explicitly for
      * each engine. In time we should stop using catalina.base property -
      * otherwise we loose some flexibility.
      */
     private String baseDir = null;
 
-    /** Optional mbeans config file. This will replace the "hacks" in
-     * jk and ServerListener. The mbeans file will support (transparent) 
-     * persistence - soon. It'll probably replace jk2.properties and could
-     * replace server.xml. Of course - the same beans could be loaded and 
-     * managed by an external entity - like the embedding app - which
-     *  can use a different persistence mechanism.
-     */ 
-    private String mbeansFile = null;
-    
-    /** Mbeans loaded by the engine.  
-     */ 
-    private List mbeans;
-    
-
     /**
      * The JVM Route ID for this Tomcat instance. All Route ID's must be unique
      * across the cluster.
@@ -149,10 +125,11 @@ public class StandardEngine
      *
      * @return configured realm, or a JAAS realm by default
      */
+    @Override
     public Realm getRealm() {
         Realm configured=super.getRealm();
         // If no set realm has been called - default to JAAS
-        // This can be overriden at engine, context and host level  
+        // This can be overridden at engine, context and host level  
         if( configured==null ) {
             configured=new JAASRealm();
             this.setRealm( configured );
@@ -164,6 +141,7 @@ public class StandardEngine
     /**
      * Return the default host.
      */
+    @Override
     public String getDefaultHost() {
 
         return (defaultHost);
@@ -176,31 +154,20 @@ public class StandardEngine
      *
      * @param host The new default host
      */
+    @Override
     public void setDefaultHost(String host) {
 
         String oldDefaultHost = this.defaultHost;
         if (host == null) {
             this.defaultHost = null;
         } else {
-            this.defaultHost = host.toLowerCase();
+            this.defaultHost = host.toLowerCase(Locale.ENGLISH);
         }
         support.firePropertyChange("defaultHost", oldDefaultHost,
                                    this.defaultHost);
 
     }
     
-    public void setName(String name ) {
-        if( domain != null ) {
-            // keep name==domain, ignore override
-            // we are already registered
-            super.setName( domain );
-            return;
-        }
-        // The engine name is used as domain
-        domain=name; // XXX should we set it in init() ? It shouldn't matter
-        super.setName( name );
-    }
-
 
     /**
      * Set the cluster-wide unique identifier for this Engine.
@@ -208,6 +175,7 @@ public class StandardEngine
      * <p>
      * This property should not be changed once it is set.
      */
+    @Override
     public void setJvmRoute(String routeId) {
         jvmRouteId = routeId;
     }
@@ -217,6 +185,7 @@ public class StandardEngine
      * Retrieve the cluster-wide unique identifier for this Engine.
      * This value is only useful in a load-balancing scenario.
      */
+    @Override
     public String getJvmRoute() {
         return jvmRouteId;
     }
@@ -225,6 +194,7 @@ public class StandardEngine
     /**
      * Return the <code>Service</code> with which we are associated (if any).
      */
+    @Override
     public Service getService() {
 
         return (this.service);
@@ -237,24 +207,17 @@ public class StandardEngine
      *
      * @param service The service that owns this Engine
      */
+    @Override
     public void setService(Service service) {
         this.service = service;
     }
 
-    public String getMbeansFile() {
-        return mbeansFile;
-    }
-
-    public void setMbeansFile(String mbeansFile) {
-        this.mbeansFile = mbeansFile;
-    }
-
     public String getBaseDir() {
         if( baseDir==null ) {
-            baseDir=System.getProperty("catalina.base");
+            baseDir=System.getProperty(Globals.CATALINA_BASE_PROP);
         }
         if( baseDir==null ) {
-            baseDir=System.getProperty("catalina.home");
+            baseDir=System.getProperty(Globals.CATALINA_HOME_PROP);
         }
         return baseDir;
     }
@@ -272,6 +235,7 @@ public class StandardEngine
      *
      * @param child Child container to be added
      */
+    @Override
     public void addChild(Container child) {
 
         if (!(child instanceof Host))
@@ -287,6 +251,7 @@ public class StandardEngine
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -299,6 +264,7 @@ public class StandardEngine
      *
      * @param container Proposed parent Container
      */
+    @Override
     public void setParent(Container container) {
 
         throw new IllegalArgumentException
@@ -307,181 +273,32 @@ public class StandardEngine
     }
 
 
-    private boolean initialized=false;
-    
-    public void init() {
-        if( initialized ) return;
-        initialized=true;
-
-        if( oname==null ) {
-            // not registered in JMX yet - standalone mode
-            try {
-                if (domain==null) {
-                    domain=getName();
-                }
-                if(log.isDebugEnabled())
-                    log.debug( "Register " + domain );
-                oname=new ObjectName(domain + ":type=Engine");
-                controller=oname;
-                Registry.getRegistry(null, null)
-                    .registerComponent(this, oname, null);
-            } catch( Throwable t ) {
-                log.info("Error registering ", t );
-            }
-        }
-
-        if( mbeansFile == null ) {
-            String defaultMBeansFile=getBaseDir() + "/conf/tomcat5-mbeans.xml";
-            File f=new File( defaultMBeansFile );
-            if( f.exists() ) mbeansFile=f.getAbsolutePath();
-        }
-        if( mbeansFile != null ) {
-            readEngineMbeans();
-        }
-        if( mbeans != null ) {
-            try {
-                Registry.getRegistry(null, null).invoke(mbeans, "init", false);
-            } catch (Exception e) {
-                log.error("Error in init() for " + mbeansFile, e);
-            }
-        }
-        
-        // not needed since the following if statement does the same thing the right way
-        // remove later after checking
-        //if( service==null ) {
-        //    try {
-        //        ObjectName serviceName=getParentName();        
-        //        if( mserver.isRegistered( serviceName )) {
-        //            log.info("Registering with the service ");
-        //            try {
-        //                mserver.invoke( serviceName, "setContainer",
-        //                        new Object[] { this },
-        //                        new String[] { "org.apache.catalina.Container" } );
-        //            } catch( Exception ex ) {
-        //               ex.printStackTrace();
-        //            }
-        //        }
-        //    } catch( Exception ex ) {
-        //        log.error("Error registering with service ");
-        //    }
-        //}
-        
-        if( service==null ) {
-            // for consistency...: we are probably in embeded mode
-            try {
-                service=new StandardService();
-                service.setContainer( this );
-                service.initialize();
-                // Use same name for Service
-                service.setName(getName());
-            } catch( Throwable t ) {
-                log.error(t);
-            }
-        }
-        
-    }
-    
-    public void destroy() throws LifecycleException {
-        if( ! initialized ) return;
-        initialized=false;
-        
-        // if we created it, make sure it's also destroyed
-        // this call implizit this.stop()
-        ((StandardService)service).destroy();
-
-        if( mbeans != null ) {
-            try {
-                Registry.getRegistry(null, null)
-                    .invoke(mbeans, "destroy", false);
-            } catch (Exception e) {
-                log.error(sm.getString("standardEngine.unregister.mbeans.failed" ,mbeansFile), e);
-            }
-        }
-        // 
-        if( mbeans != null ) {
-            try {
-                for( int i=0; i<mbeans.size() ; i++ ) {
-                    Registry.getRegistry(null, null)
-                        .unregisterComponent((ObjectName)mbeans.get(i));
-                }
-            } catch (Exception e) {
-                log.error(sm.getString("standardEngine.unregister.mbeans.failed", mbeansFile), e);
-            }
-        }
-        
-        // force all metadata to be reloaded.
-        // That doesn't affect existing beans. We should make it per
-        // registry - and stop using the static.
-        Registry.getRegistry(null, null).resetMetadata();
-        
-    }
-    
     /**
-     * Start this Engine component.
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
-     * @exception LifecycleException if a startup error occurs
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-        if( started ) {
-            return;
-        }
-        if( !initialized ) {
-            init();
-        }
-
-        // Look for a realm - that may have been configured earlier. 
-        // If the realm is added after context - it'll set itself.
-        if( realm == null ) {
-            ObjectName realmName=null;
-            try {
-                realmName=new ObjectName( domain + ":type=Realm");
-                if( mserver.isRegistered(realmName ) ) {
-                    mserver.invoke(realmName, "init", 
-                            new Object[] {},
-                            new String[] {}
-                    );            
-                }
-            } catch( Throwable t ) {
-                log.debug("No realm for this engine " + realmName);
-            }
-        }
-            
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
+        
         // Log our server identification information
-        //System.out.println(ServerInfo.getServerInfo());
         if(log.isInfoEnabled())
             log.info( "Starting Servlet Engine: " + ServerInfo.getServerInfo());
-        if( mbeans != null ) {
-            try {
-                Registry.getRegistry(null, null)
-                    .invoke(mbeans, "start", false);
-            } catch (Exception e) {
-                log.error("Error in start() for " + mbeansFile, e);
-            }
-        }
 
         // Standard container startup
-        super.start();
-
+        super.startInternal();
     }
+
     
-    public void stop() throws LifecycleException {
-        super.stop();
-        if( mbeans != null ) {
-            try {
-                Registry.getRegistry(null, null).invoke(mbeans, "stop", false);
-            } catch (Exception e) {
-                log.error("Error in stop() for " + mbeansFile, e);
-            }
-        }
-    }
-
-
     /**
      * Return a String representation of this component.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("StandardEngine[");
+        StringBuilder sb = new StringBuilder("StandardEngine[");
         sb.append(getName());
         sb.append("]");
         return (sb.toString());
@@ -494,6 +311,7 @@ public class StandardEngine
      * host's ROOT context. If still none is found, return the default NoOp
      * access log.
      */
+    @Override
     public void logAccess(Request request, Response response, long time,
             boolean useDefault) {
 
@@ -511,11 +329,7 @@ public class StandardEngine
                 // Look in the defaultHost
                 Host host = (Host) findChild(getDefaultHost());
                 Context context = null;
-                boolean checkHost = (host != null);
-                if (checkHost && host instanceof ContainerBase) {
-                    checkHost = ((ContainerBase) host).started;
-                }
-                if (checkHost) {
+                if (host != null && host.getState().isAvailable()) {
                     newDefaultAccessLog = host.getAccessLog();
 
                     if (newDefaultAccessLog != null) {
@@ -528,11 +342,8 @@ public class StandardEngine
                     } else {
                         // Try the ROOT context of default host
                         context = (Context) host.findChild("");
-                        boolean checkContext = (context != null);
-                        if (checkContext && context instanceof ContainerBase) {
-                            checkContext = ((ContainerBase) context).started;
-                        }
-                        if (checkContext) {
+                        if (context != null &&
+                                context.getState().isAvailable()) {
                             newDefaultAccessLog = context.getAccessLog();
                             if (newDefaultAccessLog != null) {
                                 if (defaultAccessLog.compareAndSet(null,
@@ -562,69 +373,48 @@ public class StandardEngine
     }
 
 
+    /**
+     * Return the parent class loader for this component.
+     */
+    @Override
+    public ClassLoader getParentClassLoader() {
+        if (parentClassLoader != null)
+            return (parentClassLoader);
+        if (service != null) {
+            return (service.getParentClassLoader());
+        }
+        return (ClassLoader.getSystemClassLoader());
+    }
+
     // -------------------- JMX registration  --------------------
 
-    public ObjectName preRegister(MBeanServer server,
-                                  ObjectName name) throws Exception
-    {
-        super.preRegister(server,name);
-
-        this.setName( name.getDomain());
-
-        return name;
+    @Override
+    protected String getObjectNameKeyProperties() {
+        return "type=Engine";
     }
 
-    // FIXME Remove -- not used 
-    public ObjectName getParentName() throws MalformedObjectNameException {
-        if (getService()==null) {
-            return null;
+    // ----------------------------------------------------------- Inner classes
+    protected static final class NoopAccessLog implements AccessLog {
+
+        @Override
+        public void log(Request request, Response response, long time) {
+            // NOOP
         }
-        String name = getService().getName();
-        ObjectName serviceName=new ObjectName(domain +
-                        ":type=Service,serviceName="+name);
-        return serviceName;                
-    }
-    
-    public ObjectName createObjectName(String domain, ObjectName parent)
-        throws Exception
-    {
-        if( log.isDebugEnabled())
-            log.debug("Create ObjectName " + domain + " " + parent );
-        return new ObjectName( domain + ":type=Engine");
-    }
 
-    
-    private void readEngineMbeans() {
-        try {
-            MbeansSource mbeansMB=new MbeansSource();
-            File mbeansF=new File( mbeansFile );
-            mbeansMB.setSource(mbeansF);
+        @Override
+        public void setRequestAttributesEnabled(
+                boolean requestAttributesEnabled) {
+            // NOOP
             
-            Registry.getRegistry(null, null).registerComponent
-                (mbeansMB, domain + ":type=MbeansFile", null);
-            mbeansMB.load();
-            mbeansMB.init();
-            mbeansMB.setRegistry(Registry.getRegistry(null, null));
-            mbeans=mbeansMB.getMBeans();
-            
-        } catch( Throwable t ) {
-            log.error( "Error loading " + mbeansFile, t );
         }
-        
-    }
-    
-    public String getDomain() {
-        if (domain!=null) {
-            return domain;
-        } else { 
-            return getName();
+
+        @Override
+        public boolean getRequestAttributesEnabled() {
+            // NOOP
+            return false;
         }
     }
     
-    public void setDomain(String domain) {
-        this.domain = domain;
-    }
- 
     protected static final class AccessLogListener
             implements PropertyChangeListener, LifecycleListener,
             ContainerListener {
@@ -645,36 +435,33 @@ public class StandardEngine
             engine.addPropertyChangeListener(this);
             if (host != null) {
                 host.addContainerListener(this);
-                if (host instanceof Lifecycle) {
-                    ((Lifecycle) host).addLifecycleListener(this);
-                }
+                host.addLifecycleListener(this);
             }
-            if (context instanceof Lifecycle) {
-                ((Lifecycle) context).addLifecycleListener(this);
+            if (context != null) {
+                context.addLifecycleListener(this);
             }
         }
 
         private void uninstall() {
             disabled = true;
-            if (context instanceof Lifecycle) {
-                ((Lifecycle) context).removeLifecycleListener(this);
+            if (context != null) {
+                context.removeLifecycleListener(this);
             }
             if (host != null) {
-                if (host instanceof Lifecycle) {
-                    ((Lifecycle) host).removeLifecycleListener(this);
-                }
+                host.removeLifecycleListener(this);
                 host.removeContainerListener(this);
             }
             engine.removePropertyChangeListener(this);
         }
 
+        @Override
         public void lifecycleEvent(LifecycleEvent event) {
             if (disabled) return;
 
             String type = event.getType();
             if (Lifecycle.AFTER_START_EVENT.equals(type) ||
                     Lifecycle.BEFORE_STOP_EVENT.equals(type) ||
-                    Lifecycle.DESTROY_EVENT.equals(type)) {
+                    Lifecycle.BEFORE_DESTROY_EVENT.equals(type)) {
                 // Container is being started/stopped/removed
                 // Force re-calculation and disable listener since it won't
                 // be re-used
@@ -683,6 +470,7 @@ public class StandardEngine
             }
         }
 
+        @Override
         public void propertyChange(PropertyChangeEvent evt) {
             if (disabled) return;
             if ("defaultHost".equals(evt.getPropertyName())) {
@@ -693,13 +481,13 @@ public class StandardEngine
             }
         }
 
+        @Override
         public void containerEvent(ContainerEvent event) {
             // Only useful for hosts
             if (disabled) return;
             if (Container.ADD_CHILD_EVENT.equals(event.getType())) {
                 Context context = (Context) event.getData();
                 if ("".equals(context.getPath())) {
-                    // New ROOT context in default host
                     // Force re-calculation and disable listener since it won't
                     // be re-used
                     engine.defaultAccessLog.set(null);
