@@ -19,30 +19,23 @@ package org.apache.catalina.ha.session;
 import java.io.IOException;
 
 import javax.servlet.ServletException;
-import javax.servlet.http.Cookie;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
-import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
 import org.apache.catalina.Manager;
 import org.apache.catalina.Session;
-import org.apache.catalina.ha.CatalinaCluster;
-import org.apache.catalina.ha.ClusterManager;
-import org.apache.catalina.ha.ClusterMessage;
-import org.apache.catalina.ha.ClusterValve;
-import org.apache.catalina.ha.session.DeltaSession;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
+import org.apache.catalina.ha.CatalinaCluster;
+import org.apache.catalina.ha.ClusterManager;
+import org.apache.catalina.ha.ClusterValve;
 import org.apache.catalina.session.ManagerBase;
 import org.apache.catalina.session.PersistentManager;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
 import org.apache.catalina.valves.ValveBase;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Valve to handle Tomcat jvmRoute takeover using mod_jk module after node
@@ -53,7 +46,7 @@ import org.apache.catalina.valves.ValveBase;
  * backup cluster node, that answered the request. After the response is
  * delivered to the client, all subsequent client requests will go directly to
  * the backup node. The change of sessionid is also sent to all other cluster
- * nodes. After all that, the session stickyness will work directly to the
+ * nodes. After all that, the session stickiness will work directly to the
  * backup node and the traffic will not go back to the failed node after it is
  * restarted!
  * 
@@ -94,12 +87,12 @@ import org.apache.catalina.valves.ValveBase;
  * requested sessions are migrated.
  * 
  * @author Peter Rossbach
- * @version $Id: JvmRouteBinderValve.java 953025 2010-06-09 14:30:30Z markt $
+ * @version $Id: JvmRouteBinderValve.java 1133177 2011-06-07 22:14:52Z markt $
  */
-public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Lifecycle {
+public class JvmRouteBinderValve extends ValveBase implements ClusterValve {
 
     /*--Static Variables----------------------------------------*/
-    public static org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory
+    public static final org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory
             .getLog(JvmRouteBinderValve.class);
 
     /**
@@ -107,6 +100,11 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
      */
     protected static final String info = "org.apache.catalina.ha.session.JvmRouteBinderValve/1.2";
 
+    //------------------------------------------------------ Constructor
+    public JvmRouteBinderValve() {
+        super(true);
+    }
+
     /*--Instance Variables--------------------------------------*/
 
     /**
@@ -117,12 +115,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     /**
      * The string manager for this package.
      */
-    protected StringManager sm = StringManager.getManager(Constants.Package);
-
-    /**
-     * Has this component been started yet?
-     */
-    protected boolean started = false;
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     /**
      * enabled this component
@@ -136,16 +129,13 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
 
     protected String sessionIdAttribute = "org.apache.catalina.ha.session.JvmRouteOrignalSessionID";
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
 
     /*--Logic---------------------------------------------------*/
 
     /**
      * Return descriptive information about this implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -162,7 +152,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     }
 
     /**
-     * get name of failed reqeust session attribute
+     * get name of failed request session attribute
      * 
      * @param sessionIdAttribute
      *            The sessionIdAttribute to set.
@@ -205,12 +195,14 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
      * @exception ServletException
      *                if a servlet error has occurred
      */
+    @Override
     public void invoke(Request request, Response response) throws IOException,
             ServletException {
 
-         if (getEnabled() 
-             && request.getContext() != null
-             && request.getContext().getDistributable() ) {
+         if (getEnabled() &&
+                 request.getContext() != null &&
+                 request.getContext().getDistributable() &&
+                 !request.isAsyncDispatching()) {
              // valve cluster can access manager - other cluster handle turnover 
              // at host level - hopefully!
              Manager manager = request.getContext().getManager();
@@ -221,7 +213,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
                        && getCluster().getManager(((ClusterManager)manager).getName()) != null)
                      ||
                      (manager instanceof PersistentManager)))
-                 handlePossibleTurnover(request, response);
+                 handlePossibleTurnover(request);
         }
         // Pass this request on to the next valve in our pipeline
         getNext().invoke(request, response);
@@ -230,11 +222,10 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     /**
      * handle possible session turn over.
      * 
-     * @see JvmRouteBinderValve#handleJvmRoute(Request, Response, String, String)
+     * @see JvmRouteBinderValve#handleJvmRoute(Request, String, String)
      * @param request current request
-     * @param response current response
      */
-    protected void handlePossibleTurnover(Request request, Response response) {
+    protected void handlePossibleTurnover(Request request) {
         String sessionID = request.getRequestedSessionId() ;
         if (sessionID != null) {
             long t1 = System.currentTimeMillis();
@@ -244,11 +235,11 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
                     log.debug(sm.getString("jvmRoute.missingJvmRouteAttribute"));
                 return;
             }
-            handleJvmRoute( request, response, sessionID, jvmRoute);
+            handleJvmRoute( request, sessionID, jvmRoute);
             if (log.isDebugEnabled()) {
                 long t2 = System.currentTimeMillis();
                 long time = t2 - t1;
-                log.debug(sm.getString("jvmRoute.turnoverInfo", new Long(time)));
+                log.debug(sm.getString("jvmRoute.turnoverInfo", Long.valueOf(time)));
             }
         }
     }
@@ -278,7 +269,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
             if(manager != null)
                 log.debug(sm.getString("jvmRoute.foundManager", manager,  request.getContext().getName()));
             else 
-                log.debug(sm.getString("jvmRoute.notFoundManager", manager,  request.getContext().getName()));
+                log.debug(sm.getString("jvmRoute.notFoundManager", request.getContext().getName()));
         }
         return manager;
     }
@@ -286,6 +277,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     /**
      * @return Returns the cluster.
      */
+    @Override
     public CatalinaCluster getCluster() {
         return cluster;
     }
@@ -293,25 +285,24 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     /**
      * @param cluster The cluster to set.
      */
+    @Override
     public void setCluster(CatalinaCluster cluster) {
         this.cluster = cluster;
     }
     
     /**
-     * Handle jvmRoute stickyness after tomcat instance failed. After this
+     * Handle jvmRoute stickiness after tomcat instance failed. After this
      * correction a new Cookie send to client with new jvmRoute and the
-     * SessionID change propage to the other cluster nodes.
+     * SessionID change propagate to the other cluster nodes.
      * 
      * @param request current request
-     * @param response
-     *            Tomcat Response
      * @param sessionId
      *            request SessionID from Cookie
      * @param localJvmRoute
      *            local jvmRoute
      */
     protected void handleJvmRoute(
-            Request request, Response response,String sessionId, String localJvmRoute) {
+            Request request, String sessionId, String localJvmRoute) {
         // get requested jvmRoute.
         String requestJvmRoute = null;
         int index = sessionId.indexOf(".");
@@ -334,7 +325,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
             String newSessionID = id + "." + localJvmRoute;
             // OK - turnover the session and inform other cluster nodes
             if (catalinaSession != null) {
-                changeSessionID(request, response, sessionId, newSessionID,
+                changeSessionID(request, sessionId, newSessionID,
                         catalinaSession);
                 numberOfSessions++;
             } else {
@@ -345,7 +336,7 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
                 }
                 if (catalinaSession != null) {
                     // session is rewrite at other request, rewrite this also
-                    changeRequestSessionID(request, response, sessionId, newSessionID);
+                    changeRequestSessionID(request, sessionId, newSessionID);
                 } else {
                     if (log.isDebugEnabled()) {
                         log.debug(sm.getString("jvmRoute.cannotFindSession",sessionId));
@@ -359,7 +350,6 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
      * change session id and send to all cluster nodes
      * 
      * @param request current request
-     * @param response current response
      * @param sessionId
      *            original session id
      * @param newSessionID
@@ -367,25 +357,20 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
      * @param catalinaSession
      *            current session with original session id
      */
-    protected void changeSessionID(Request request,
-            Response response, String sessionId, String newSessionID, Session catalinaSession) {
-        lifecycle.fireLifecycleEvent("Before session migration",
-                catalinaSession);
-        // FIXME: setId trigger session Listener, but only chance to registiert manager with correct id!
-        catalinaSession.setId(newSessionID);
+    protected void changeSessionID(Request request, String sessionId,
+            String newSessionID, Session catalinaSession) {
+        fireLifecycleEvent("Before session migration", catalinaSession);
+        catalinaSession.setId(newSessionID, false);
         // FIXME: Why we remove change data from other running request?
         // setId also trigger resetDeltaRequest!!
         if (catalinaSession instanceof DeltaSession)
             ((DeltaSession) catalinaSession).resetDeltaRequest();
-        changeRequestSessionID(request, response, sessionId, newSessionID);
+        changeRequestSessionID(request, sessionId, newSessionID);
 
-        if (getCluster() != null) {
-            // now sending the change to all other clusternode!
-            ClusterManager manager = (ClusterManager)catalinaSession.getManager();
-            sendSessionIDClusterBackup(manager,request,sessionId, newSessionID);
-        }
+        // now sending the change to all other clusternodes!
+        sendSessionIDClusterBackup(request,sessionId, newSessionID);
 
-        lifecycle.fireLifecycleEvent("After session migration", catalinaSession);
+        fireLifecycleEvent("After session migration", catalinaSession);
         if (log.isDebugEnabled()) {
             log.debug(sm.getString("jvmRoute.changeSession", sessionId,
                     newSessionID));
@@ -395,16 +380,15 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     /**
      * Change Request Session id
      * @param request current request
-     * @param response current response
      * @param sessionId
      *            original session id
      * @param newSessionID
      *            new session id for node migration
      */
-    protected void changeRequestSessionID(Request request, Response response, String sessionId, String newSessionID) {
+    protected void changeRequestSessionID(Request request, String sessionId, String newSessionID) {
         request.changeSessionId(newSessionID);
 
-        // set orginal sessionid at request, to allow application detect the
+        // set original sessionid at request, to allow application detect the
         // change
         if (sessionIdAttribute != null && !"".equals(sessionIdAttribute)) {
             if (log.isDebugEnabled()) {
@@ -417,141 +401,37 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
     /**
      * Send the changed Sessionid to all clusternodes.
      * 
-     * @see JvmRouteSessionIDBinderListener#messageReceived(ClusterMessage)
-     * @param manager
-     *            ClusterManager
+     * @see JvmRouteSessionIDBinderListener#messageReceived(
+     *            org.apache.catalina.ha.ClusterMessage)
      * @param sessionId
      *            current failed sessionid
      * @param newSessionID
      *            new session id, bind to the new cluster node
      */
-    protected void sendSessionIDClusterBackup(ClusterManager manager,Request request,String sessionId,
+    protected void sendSessionIDClusterBackup(Request request, String sessionId,
             String newSessionID) {
-        SessionIDMessage msg = new SessionIDMessage();
-        msg.setOrignalSessionID(sessionId);
-        msg.setBackupSessionID(newSessionID);
-        Context context = request.getContext();
-        msg.setContextPath(context.getPath());
-        msg.setHost(context.getParent().getName());
-        if(manager.doDomainReplication())
-            cluster.sendClusterDomain(msg);
-        else
-            cluster.send(msg);
-    }
-
-    /**
-     * Sets a new cookie for the given session id and response and see
-     * {@link org.apache.catalina.connector.Request#configureSessionCookie(javax.servlet.http.Cookie)}
-     * 
-     * @param request current request
-     * @param response Tomcat Response
-     * @param sessionId The session id
-     * 
-     * @deprecated Use {@link Request#changeSessionId(String)}
-     */
-    protected void setNewSessionCookie(Request request,
-                                       Response response, String sessionId) {
-        if (response != null) {
+        CatalinaCluster c = getCluster();
+        if (c != null) {
+            SessionIDMessage msg = new SessionIDMessage();
+            msg.setOrignalSessionID(sessionId);
+            msg.setBackupSessionID(newSessionID);
             Context context = request.getContext();
-            if (context.getCookies()) {
-                // set a new session cookie
-                String scName = context.getSessionCookieName();
-                if (scName == null) {
-                    scName = Globals.SESSION_COOKIE_NAME;
-                }
-                Cookie newCookie = new Cookie(scName, sessionId);
-                
-                newCookie.setMaxAge(-1);
-                
-                String contextPath = null;
-                if (!response.getConnector().getEmptySessionPath() &&
-                        (context != null)) {
-                    if (context.getSessionCookiePath() != null) {
-                        contextPath = context.getSessionCookiePath();
-                    } else {
-                        contextPath = context.getEncodedPath();
-                    }
-                }
-                if ((contextPath != null) && (contextPath.length() > 0)) {
-                    newCookie.setPath(contextPath);
-                } else {
-                    newCookie.setPath("/");
-                }
-                
-                if (context.getSessionCookieDomain() != null) {
-                    newCookie.setDomain(context.getSessionCookieDomain());
-                }
-
-                if (request.isSecure()) {
-                    newCookie.setSecure(true);
-                }
-
-                if (log.isDebugEnabled()) {
-                    Object[] args = new Object[] {sessionId,
-                            newCookie.getName(),
-                            newCookie.getPath(),
-                            new Boolean(newCookie.getSecure()),
-                            new Boolean(context.getUseHttpOnly())};
-                    log.debug(sm.getString("jvmRoute.newSessionCookie", args));
-                }
-                response.addCookieInternal(newCookie, context.getUseHttpOnly());
-            }
+            msg.setContextName(context.getName());
+            msg.setHost(context.getParent().getName());
+            c.send(msg);
         }
     }
 
-    // ------------------------------------------------------ Lifecycle Methods
-
     /**
-     * Add a lifecycle event listener to this component.
-     * 
-     * @param listener
-     *            The listener to add
+     * Start this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     * 
-     * @param listener
-     *            The listener to add
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component. This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
-     * 
-     * @exception LifecycleException
-     *                if this component detects a fatal error that prevents this
-     *                component from being used
-     */
-    public void start() throws LifecycleException {
-
-        // Validate and update our current component state
-        if (started)
-            throw new LifecycleException(sm
-                    .getString("jvmRoute.valve.alreadyStarted"));
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
+    @Override
+    protected synchronized void startInternal() throws LifecycleException {
+        
         if (cluster == null) {
             Container hostContainer = getContainer();
             // compatibility with JvmRouteBinderValve version 1.1
@@ -579,25 +459,22 @@ public class JvmRouteBinderValve extends ValveBase implements ClusterValve, Life
                 log.info(sm.getString("jvmRoute.noCluster"));
         }
 
+        super.startInternal();
     }
 
+    
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component. This method should be the last one called on a given instance
-     * of this component.
-     * 
-     * @exception LifecycleException
-     *                if this component detects a fatal error that needs to be
-     *                reported
+     * Stop this component and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
+     *
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected synchronized void stopInternal() throws LifecycleException {
+
+        super.stopInternal();
 
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException(sm
-                    .getString("jvmRoute.valve.notStarted"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
         cluster = null;
         numberOfSessions = 0;
         if (log.isInfoEnabled())
