@@ -18,18 +18,14 @@
 package org.apache.catalina.mbeans;
 
 import java.io.File;
-import java.util.Vector;
 
-import javax.management.MBeanException;
 import javax.management.MBeanServer;
 import javax.management.ObjectName;
-import javax.management.RuntimeOperationsException;
 
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
 import org.apache.catalina.Host;
 import org.apache.catalina.Server;
-import org.apache.catalina.ServerFactory;
 import org.apache.catalina.Service;
 import org.apache.catalina.Valve;
 import org.apache.catalina.authenticator.SingleSignOn;
@@ -51,10 +47,7 @@ import org.apache.catalina.startup.HostConfig;
 import org.apache.catalina.valves.AccessLogValve;
 import org.apache.catalina.valves.RemoteAddrValve;
 import org.apache.catalina.valves.RemoteHostValve;
-import org.apache.catalina.valves.RequestDumperValve;
 import org.apache.catalina.valves.ValveBase;
-import org.apache.tomcat.util.modeler.BaseModelMBean;
-import org.apache.tomcat.util.modeler.Registry;
 
 
 /**
@@ -62,12 +55,12 @@ import org.apache.tomcat.util.modeler.Registry;
  * <code>org.apache.catalina.core.StandardServer</code> component.</p>
  *
  * @author Amy Roh
- * @version $Revision: 467222 $ $Date: 2006-10-24 05:17:11 +0200 (Di, 24. Okt 2006) $
+ * @version $Id: MBeanFactory.java 1029527 2010-11-01 02:04:53Z markt $
  */
 
-public class MBeanFactory extends BaseModelMBean {
+public class MBeanFactory {
 
-    private static org.apache.juli.logging.Log log = 
+    private static final org.apache.juli.logging.Log log = 
         org.apache.juli.logging.LogFactory.getLog(MBeanFactory.class);
 
     /**
@@ -75,11 +68,6 @@ public class MBeanFactory extends BaseModelMBean {
      */
     private static MBeanServer mserver = MBeanUtils.createServer();
 
-    /**
-     * The configuration information registry for our managed beans.
-     */
-    private static Registry registry = MBeanUtils.createRegistry();
-
 
     // ----------------------------------------------------------- Constructors
 
@@ -88,26 +76,34 @@ public class MBeanFactory extends BaseModelMBean {
      * Construct a <code>ModelMBean</code> with default
      * <code>ModelMBeanInfo</code> information.
      *
-     * @exception MBeanException if the initializer of an object
-     *  throws an exception
-     * @exception RuntimeOperationsException if an IllegalArgumentException
-     *  occurs
+     * @exception javax.management.MBeanException if the initializer of an
+     *  object throws an exception
+     * @exception javax.management.RuntimeOperationsException if an
+     *  IllegalArgumentException occurs
      */
-    public MBeanFactory()
-        throws MBeanException, RuntimeOperationsException {
+    public MBeanFactory() {
 
         super();
-
+        
     }
 
 
     // ------------------------------------------------------------- Attributes
 
-
+    /**
+     * The container (Server/Service) for which this factory was created.
+     */
+    private Object container;
 
 
     // ------------------------------------------------------------- Operations
 
+    /**
+     * Set the container that this factory was created for.
+     */
+    public void setContainer(Object container) {
+        this.container = container;
+    }
 
     /**
      * Return the managed bean definition for the specified bean type
@@ -208,17 +204,24 @@ public class MBeanFactory extends BaseModelMBean {
     
     private Service getService(ObjectName oname) throws Exception {
     
-        String domain = oname.getDomain();
-        Server server = ServerFactory.getServer();
-        Service[] services = server.findServices();
+        if (container instanceof Service) {
+            // Don't bother checking the domain - this is the only option
+            return (Service) container;
+        }
+
         StandardService service = null;
-        for (int i = 0; i < services.length; i++) {
-            service = (StandardService) services[i];
-            if (domain.equals(service.getObjectName().getDomain())) {
-                break;
+        String domain = oname.getDomain();
+        if (container instanceof Server) {
+            Service[] services = ((Server)container).findServices();
+            for (int i = 0; i < services.length; i++) {
+                service = (StandardService) services[i];
+                if (domain.equals(service.getObjectName().getDomain())) {
+                    break;
+                }
             }
         }
-        if (!service.getObjectName().getDomain().equals(domain)) {
+        if (service == null ||
+                !service.getObjectName().getDomain().equals(domain)) {
             throw new Exception("Service with the domain is not found");
         }        
         return service;
@@ -241,7 +244,7 @@ public class MBeanFactory extends BaseModelMBean {
         AccessLogValve accessLogger = new AccessLogValve();
         ContainerBase containerBase = getParentContainerFromParent(pname);
         // Add the new instance to its parent component
-        containerBase.addValve(accessLogger);
+        containerBase.getPipeline().addValve(accessLogger);
         ObjectName oname = accessLogger.getObjectName();
         return (oname.toString());
 
@@ -308,8 +311,8 @@ public class MBeanFactory extends BaseModelMBean {
      * @exception Exception if an MBean cannot be created or registered
      */
     public String createHttpConnector(String parent, String address, int port)
-        throws Exception {
-	return createConnector(parent, address, port, false, false);
+            throws Exception {
+        return createConnector(parent, address, port, false, false);
     }
 
     /**
@@ -371,7 +374,7 @@ public class MBeanFactory extends BaseModelMBean {
      * @exception Exception if an MBean cannot be created or registered
      */
     public String createJDBCRealm(String parent, String driverName, 
-    	String connectionName, String connectionPassword, String connectionURL)
+        String connectionName, String connectionPassword, String connectionURL)
         throws Exception {
 
         // Create a new JDBCRealm instance
@@ -474,7 +477,7 @@ public class MBeanFactory extends BaseModelMBean {
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
         ContainerBase containerBase = getParentContainerFromParent(pname);
-        containerBase.addValve(valve);
+        containerBase.getPipeline().addValve(valve);
         ObjectName oname = valve.getObjectName();
         return (oname.toString());
 
@@ -497,36 +500,13 @@ public class MBeanFactory extends BaseModelMBean {
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
         ContainerBase containerBase = getParentContainerFromParent(pname);
-        containerBase.addValve(valve);
+        containerBase.getPipeline().addValve(valve);
         ObjectName oname = valve.getObjectName();
         return (oname.toString());
         
     }
 
 
-    /**
-     * Create a new Request Dumper Valve.
-     *
-     * @param parent MBean Name of the associated parent component
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    public String createRequestDumperValve(String parent)
-        throws Exception {
-
-        // Create a new RequestDumperValve instance
-        RequestDumperValve valve = new RequestDumperValve();
-
-        // Add the new instance to its parent component
-        ObjectName pname = new ObjectName(parent);
-        ContainerBase containerBase = getParentContainerFromParent(pname);
-        containerBase.addValve(valve);
-        ObjectName oname = valve.getObjectName();
-        return (oname.toString());
-
-    }
-
-
     /**
      * Create a new Single Sign On Valve.
      *
@@ -543,7 +523,7 @@ public class MBeanFactory extends BaseModelMBean {
         // Add the new instance to its parent component
         ObjectName pname = new ObjectName(parent);
         ContainerBase containerBase = getParentContainerFromParent(pname);
-        containerBase.addValve(valve);
+        containerBase.getPipeline().addValve(valve);
         ObjectName oname = valve.getObjectName();
         return (oname.toString());
 
@@ -564,23 +544,10 @@ public class MBeanFactory extends BaseModelMBean {
                                         String docBase)
         throws Exception {
                                             
-        // XXX for backward compatibility. Remove it once supported by the admin
-        return 
-            createStandardContext(parent,path,docBase,false,false,false,false);                                  
+        return createStandardContext(parent, path, docBase, false, false,
+                false, false);                                  
     }
 
-    /**
-     * Given a context path, get the config file name.
-     */
-    private String getConfigFile(String path) {
-        String basename = null;
-        if (path.equals("")) {
-            basename = "ROOT";
-        } else {
-            basename = path.substring(1).replace('/', '#');
-        }
-        return (basename);
-    }
 
    /**
      * Create a new StandardContext.
@@ -619,20 +586,20 @@ public class MBeanFactory extends BaseModelMBean {
                                              ":type=Deployer,host="+
                                              pname.getKeyProperty("host"));
         if(mserver.isRegistered(deployer)) {
-            String contextPath = context.getPath();
+            String contextName = context.getName();
             mserver.invoke(deployer, "addServiced",
-                           new Object [] {contextPath},
+                           new Object [] {contextName},
                            new String [] {"java.lang.String"});
             String configPath = (String)mserver.getAttribute(deployer,
                                                              "configBaseName");
-            String baseName = getConfigFile(contextPath);
+            String baseName = context.getBaseName();
             File configFile = new File(new File(configPath), baseName+".xml");
-            context.setConfigFile(configFile.getAbsolutePath());
+            context.setConfigFile(configFile.toURI().toURL());
             mserver.invoke(deployer, "manageApp",
                            new Object[] {context},
                            new String[] {"org.apache.catalina.Context"});
             mserver.invoke(deployer, "removeServiced",
-                           new Object [] {contextPath},
+                           new Object [] {contextName},
                            new String [] {"java.lang.String"});
         } else {
             log.warn("Deployer not found for "+pname.getKeyProperty("host"));
@@ -643,51 +610,7 @@ public class MBeanFactory extends BaseModelMBean {
         }
 
         // Return the corresponding MBean name
-        ObjectName oname = context.getJmxName();
-
-        return (oname.toString());
-
-    }
-
-
-   /**
-     * Create a new StandardEngine.
-     *
-     * @param parent MBean Name of the associated parent component
-     * @param engineName Unique name of this Engine
-     * @param defaultHost Default hostname of this Engine
-     * @param serviceName Unique name of this Service
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-
-    public Vector createStandardEngineService(String parent, 
-            String engineName, String defaultHost, String serviceName)
-        throws Exception {
-
-        // Create a new StandardService instance
-        StandardService service = new StandardService();
-        service.setName(serviceName);
-        // Create a new StandardEngine instance
-        StandardEngine engine = new StandardEngine();
-        engine.setName(engineName);
-        engine.setDefaultHost(defaultHost);
-        // Need to set engine before adding it to server in order to set domain
-        service.setContainer(engine);
-        // Add the new instance to its parent component
-        Server server = ServerFactory.getServer();
-        server.addService(service);
-        Vector onames = new Vector();
-        // FIXME service & engine.getObjectName
-        //ObjectName oname = engine.getObjectName();
-        ObjectName oname = 
-            MBeanUtils.createObjectName(engineName, engine);
-        onames.add(0, oname);
-        //oname = service.getObjectName();
-        oname = 
-            MBeanUtils.createObjectName(engineName, service);
-        onames.add(1, oname);
-        return (onames);
+        return context.getObjectName().toString();
 
     }
 
@@ -702,8 +625,6 @@ public class MBeanFactory extends BaseModelMBean {
      * @param deployOnStartup Deploy on server startup?
      * @param deployXML Should we deploy Context XML config files property?
      * @param unpackWARs Should we unpack WARs when auto deploying?
-     * @param xmlNamespaceAware Should we turn on/off XML namespace awareness?
-     * @param xmlValidation Should we turn on/off XML validation?        
      *
      * @exception Exception if an MBean cannot be created or registered
      */
@@ -712,9 +633,7 @@ public class MBeanFactory extends BaseModelMBean {
                                      boolean autoDeploy,
                                      boolean deployOnStartup,
                                      boolean deployXML,                                       
-                                     boolean unpackWARs,
-                                     boolean xmlNamespaceAware,
-                                     boolean xmlValidation)
+                                     boolean unpackWARs)
         throws Exception {
 
         // Create a new StandardHost instance
@@ -725,9 +644,7 @@ public class MBeanFactory extends BaseModelMBean {
         host.setDeployOnStartup(deployOnStartup);
         host.setDeployXML(deployXML);
         host.setUnpackWARs(unpackWARs);
-        host.setXmlNamespaceAware(xmlNamespaceAware);
-        host.setXmlValidation(xmlValidation);
-	
+    
         // add HostConfig for active reloading
         HostConfig hostConfig = new HostConfig();
         host.addLifecycleListener(hostConfig);
@@ -744,6 +661,38 @@ public class MBeanFactory extends BaseModelMBean {
     }
 
 
+    /**
+     * Creates a new StandardService and StandardEngine.
+     *
+     * @param domain       Domain name for the container instance
+     * @param defaultHost  Name of the default host to be used in the Engine
+     * @param baseDir      Base directory value for Engine 
+     *
+     * @exception Exception if an MBean cannot be created or registered
+     */
+    public String createStandardServiceEngine(String domain,
+            String defaultHost, String baseDir) throws Exception{
+
+        if (!(container instanceof Server)) {
+            throw new Exception("Container not Server");
+        }
+        
+        StandardEngine engine = new StandardEngine();
+        engine.setDomain(domain);
+        engine.setName(domain);
+        engine.setDefaultHost(defaultHost);
+        engine.setBaseDir(baseDir);
+
+        Service service = new StandardService();
+        service.setContainer(engine);
+        service.setName(domain);
+        
+        ((Server) container).addService(service);
+        
+        return engine.getObjectName().toString();
+    }
+    
+    
     /**
      * Create a new StandardManager.
      *
@@ -773,32 +722,6 @@ public class MBeanFactory extends BaseModelMBean {
     }
 
 
-    /**
-     * Create a new StandardService.
-     *
-     * @param parent MBean Name of the associated parent component
-     * @param name Unique name of this StandardService
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    public String createStandardService(String parent, String name, String domain)
-        throws Exception {
-
-        // Create a new StandardService instance
-        StandardService service = new StandardService();
-        service.setName(name);
-
-        // Add the new instance to its parent component
-        Server server = ServerFactory.getServer();
-        server.addService(service);
-
-        // Return the corresponding MBean name
-        return (service.getObjectName().toString());
-
-    }
-
-
-
     /**
      * Create a new  UserDatabaseRealm.
      *
@@ -873,12 +796,11 @@ public class MBeanFactory extends BaseModelMBean {
 
         // Acquire a reference to the component to be removed
         ObjectName oname = new ObjectName(name);
-        Server server = ServerFactory.getServer();
         Service service = getService(oname);
         String port = oname.getKeyProperty("port");
         //String address = oname.getKeyProperty("address");
 
-        Connector conns[] = (Connector[]) service.findConnectors();
+        Connector conns[] = service.findConnectors();
 
         for (int i = 0; i < conns.length; i++) {
             String connAddress = String.valueOf(conns[i].getProperty("address"));
@@ -905,7 +827,7 @@ public class MBeanFactory extends BaseModelMBean {
     /**
      * Remove an existing Context.
      *
-     * @param contextName MBean Name of the comonent to remove
+     * @param contextName MBean Name of the component to remove
      *
      * @exception Exception if a component cannot be removed
      */
@@ -956,7 +878,7 @@ public class MBeanFactory extends BaseModelMBean {
     /**
      * Remove an existing Host.
      *
-     * @param name MBean Name of the comonent to remove
+     * @param name MBean Name of the component to remove
      *
      * @exception Exception if a component cannot be removed
      */
@@ -971,19 +893,15 @@ public class MBeanFactory extends BaseModelMBean {
 
         // Remove this component from its parent component
         if(host!=null) {
-            if(host instanceof StandardHost)
-                ((StandardHost)host).destroy();
-            else
-                engine.removeChild(host);
+            engine.removeChild(host);
         }
-
     }
 
 
     /**
      * Remove an existing Loader.
      *
-     * @param name MBean Name of the comonent to remove
+     * @param name MBean Name of the component to remove
      *
      * @exception Exception if a component cannot be removed
      */
@@ -1000,7 +918,7 @@ public class MBeanFactory extends BaseModelMBean {
     /**
      * Remove an existing Manager.
      *
-     * @param name MBean Name of the comonent to remove
+     * @param name MBean Name of the component to remove
      *
      * @exception Exception if a component cannot be removed
      */
@@ -1017,7 +935,7 @@ public class MBeanFactory extends BaseModelMBean {
     /**
      * Remove an existing Realm.
      *
-     * @param name MBean Name of the comonent to remove
+     * @param name MBean Name of the component to remove
      *
      * @exception Exception if a component cannot be removed
      */
@@ -1039,22 +957,21 @@ public class MBeanFactory extends BaseModelMBean {
      */
     public void removeService(String name) throws Exception {
 
+        if (!(container instanceof Server)) {
+            throw new Exception();
+        }
+        
         // Acquire a reference to the component to be removed
         ObjectName oname = new ObjectName(name);
-        String serviceName = oname.getKeyProperty("serviceName");
-        Server server = ServerFactory.getServer();
-        Service service = server.findService(serviceName);
-
-        // Remove this component from its parent component
-        server.removeService(service);
-
+        Service service = getService(oname); 
+        ((Server) container).removeService(service);
     }
 
 
     /**
      * Remove an existing Valve.
      *
-     * @param name MBean Name of the comonent to remove
+     * @param name MBean Name of the component to remove
      *
      * @exception Exception if a component cannot be removed
      */
@@ -1063,12 +980,11 @@ public class MBeanFactory extends BaseModelMBean {
         // Acquire a reference to the component to be removed
         ObjectName oname = new ObjectName(name);
         ContainerBase container = getParentContainerFromChild(oname);
-        String sequence = oname.getKeyProperty("seq");
-        Valve[] valves = (Valve[])container.getValves();
+        Valve[] valves = container.getPipeline().getValves();
         for (int i = 0; i < valves.length; i++) {
             ObjectName voname = ((ValveBase) valves[i]).getObjectName();
             if (voname.equals(oname)) {
-                container.removeValve(valves[i]);
+                container.getPipeline().removeValve(valves[i]);
             }
         }
     }
