@@ -20,6 +20,7 @@ package org.apache.catalina.realm;
 import java.io.IOException;
 import java.net.URI;
 import java.net.URISyntaxException;
+import java.nio.charset.Charset;
 import java.security.Principal;
 import java.text.MessageFormat;
 import java.util.ArrayList;
@@ -30,19 +31,19 @@ import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Set;
 import java.util.Map.Entry;
+import java.util.Set;
 
-import javax.naming.Context;
+import javax.naming.AuthenticationException;
 import javax.naming.CommunicationException;
 import javax.naming.CompositeName;
+import javax.naming.Context;
 import javax.naming.InvalidNameException;
+import javax.naming.Name;
 import javax.naming.NameNotFoundException;
+import javax.naming.NameParser;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
-import javax.naming.NameParser;
-import javax.naming.Name;
-import javax.naming.AuthenticationException;
 import javax.naming.PartialResultException;
 import javax.naming.ServiceUnavailableException;
 import javax.naming.directory.Attribute;
@@ -51,10 +52,12 @@ import javax.naming.directory.DirContext;
 import javax.naming.directory.InitialDirContext;
 import javax.naming.directory.SearchControls;
 import javax.naming.directory.SearchResult;
+
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.util.Base64;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.CharChunk;
+import org.ietf.jgss.GSSCredential;
 
 /**
  * <p>Implementation of <strong>Realm</strong> that works with a directory
@@ -169,7 +172,7 @@ import org.apache.tomcat.util.buf.CharChunk;
  *
  * @author John Holman
  * @author Craig R. McClanahan
- * @version $Id: JNDIRealm.java 1090775 2011-04-10 11:02:22Z markt $
+ * @version $Id: JNDIRealm.java 1138019 2011-06-21 14:29:49Z markt $
  */
 
 public class JNDIRealm extends RealmBase {
@@ -223,7 +226,7 @@ public class JNDIRealm extends RealmBase {
      * Constant that holds the name of the environment property for specifying
      * the manner in which aliases should be dereferenced.
      */
-    public final static String DEREF_ALIASES = "java.naming.ldap.derefAliases";
+    public static final String DEREF_ALIASES = "java.naming.ldap.derefAliases";
 
     /**
      * Descriptive information about this Realm implementation.
@@ -327,6 +330,13 @@ public class JNDIRealm extends RealmBase {
     protected String roleBase = "";
 
 
+    /**
+     * The MessageFormat object associated with the current
+     * <code>roleBase</code>.
+     */
+    protected MessageFormat roleBaseFormat = null;
+
+
     /**
      * The MessageFormat object associated with the current
      * <code>roleSearch</code>.
@@ -364,7 +374,14 @@ public class JNDIRealm extends RealmBase {
      */
     protected boolean roleNested = false;
 
-
+    /**
+     * When searching for user roles, should the search be performed as the user
+     * currently being authenticated? If false, {@link #connectionName} and
+     * {@link #connectionPassword} will be used if specified, else an anonymous
+     * connection will be used. 
+     */
+    protected boolean roleSearchAsUser = false;
+    
     /**
      * An alternate URL, to which, we should connect if connectionURL fails.
      */
@@ -388,6 +405,26 @@ public class JNDIRealm extends RealmBase {
      */
     protected String connectionTimeout = "5000";
     
+    /**
+     * The sizeLimit (also known as the countLimit) to use when the realm is
+     * configured with {@link #userSearch}. Zero for no limit.
+     */
+    protected long sizeLimit = 0;
+
+    /**
+     * The timeLimit (in milliseconds) to use when the realm is configured with
+     * {@link #userSearch}. Zero for no limit.
+     */
+    protected int timeLimit = 0;
+
+    
+    /**
+     * Should delegated credentials from the SPNEGO authenticator be used if
+     * available
+     */
+    protected boolean useDelegatedCredential = true;
+
+
     // ------------------------------------------------------------- Properties
 
     /**
@@ -677,6 +714,10 @@ public class JNDIRealm extends RealmBase {
     public void setRoleBase(String roleBase) {
 
         this.roleBase = roleBase;
+        if (roleBase == null)
+            roleBaseFormat = null;
+        else
+            roleBaseFormat = new MessageFormat(roleBase);
 
     }
 
@@ -770,7 +811,6 @@ public class JNDIRealm extends RealmBase {
         this.roleNested = roleNested;
 
     }
-     
 
 
     /**
@@ -900,13 +940,42 @@ public class JNDIRealm extends RealmBase {
     }
 
 
+    public long getSizeLimit() {
+        return sizeLimit;
+    }
+
+
+    public void setSizeLimit(long sizeLimit) {
+        this.sizeLimit = sizeLimit;
+    }
+
+
+    public int getTimeLimit() {
+        return timeLimit;
+    }
+
+
+    public void setTimeLimit(int timeLimit) {
+        this.timeLimit = timeLimit;
+    }
+
+
+    
+    public boolean isUseDelegatedCredential() {
+        return useDelegatedCredential;
+    }
+
+    public void setUseDelegatedCredential(boolean useDelegatedCredential) {
+        this.useDelegatedCredential = useDelegatedCredential;
+    }
+
     /**
      * Return descriptive information about this Realm implementation and
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
     @Override
-	public String getInfo() {
+    public String getInfo() {
 
         return info;
 
@@ -929,6 +998,7 @@ public class JNDIRealm extends RealmBase {
      * @param credentials Password or other credentials to use in
      *  authenticating this username
      */
+    @Override
     public Principal authenticate(String username, String credentials) {
 
         DirContext context = null;
@@ -1064,12 +1134,12 @@ public class JNDIRealm extends RealmBase {
                             List<String> roles = getRoles(context, user);
                             if (containerLog.isDebugEnabled()) {
                                 Iterator<String> it = roles.iterator();
+                                // TODO: Use a single log message
                                 while (it.hasNext()) {
                                     containerLog.debug("Found role: " + it.next());
                                 }
                             }
-                            return (new GenericPrincipal(this,
-                                                         username,
+                            return (new GenericPrincipal(username,
                                                          credentials,
                                                          roles));
                         }
@@ -1098,13 +1168,14 @@ public class JNDIRealm extends RealmBase {
             List<String> roles = getRoles(context, user);
             if (containerLog.isDebugEnabled()) {
                 Iterator<String> it = roles.iterator();
+                // TODO: Use a single log message
                 while (it.hasNext()) {
                     containerLog.debug("Found role: " + it.next());
                 }
             }
 
             // Create and return a suitable Principal for this user
-            return (new GenericPrincipal(this, username, credentials, roles));
+            return (new GenericPrincipal(username, credentials, roles));
         }
     }
 
@@ -1139,7 +1210,7 @@ public class JNDIRealm extends RealmBase {
      *
      * @exception NamingException if a directory server error occurs
      *
-     * @see #getUser(DirContext, String, int)
+     * @see #getUser(DirContext, String, String, int)
      */
     protected User getUser(DirContext context, String username, String credentials)
         throws NamingException {
@@ -1278,27 +1349,11 @@ public class JNDIRealm extends RealmBase {
             // If the getUserByPattern() call fails, try it again with the
             // credentials of the user that we're searching for
             try {
-                // Set up security environment to bind as the user
-                context.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
-                context.addToEnvironment(Context.SECURITY_CREDENTIALS, credentials);
+                userCredentialsAdd(context, dn, credentials);
 
                 user = getUserByPattern(context, username, attrIds, dn);
             } finally {
-                // Restore the original security environment
-                if (connectionName != null) {
-                    context.addToEnvironment(Context.SECURITY_PRINCIPAL,
-                                             connectionName);
-                } else {
-                    context.removeFromEnvironment(Context.SECURITY_PRINCIPAL);
-                }
-
-                if (connectionPassword != null) {
-                    context.addToEnvironment(Context.SECURITY_CREDENTIALS,
-                                             connectionPassword);
-                }
-                else {
-                    context.removeFromEnvironment(Context.SECURITY_CREDENTIALS);
-                }
+                userCredentialsRemove(context);
             }
         }
         return user;
@@ -1337,6 +1392,9 @@ public class JNDIRealm extends RealmBase {
             constraints.setSearchScope(SearchControls.ONELEVEL_SCOPE);
         }
 
+        constraints.setCountLimit(sizeLimit);
+        constraints.setTimeLimit(timeLimit);
+
         // Specify the attributes to be retrieved
         if (attrIds == null)
             attrIds = new String[0];
@@ -1438,7 +1496,6 @@ public class JNDIRealm extends RealmBase {
      }
 
 
-
     /**
      * Check whether the credentials presented by the user match those
      * retrieved from the directory.
@@ -1475,9 +1532,8 @@ public class JNDIRealm extends RealmBase {
                 synchronized (this) {
                     password = password.substring(5);
                     md.reset();
-                    md.update(credentials.getBytes());
-                    String digestedPassword =
-                        new String(Base64.encode(md.digest()));
+                    md.update(credentials.getBytes(Charset.defaultCharset()));
+                    String digestedPassword = Base64.encode(md.digest());
                     validated = password.equals(digestedPassword);
                 }
             } else if (password.startsWith("{SSHA}")) {
@@ -1487,12 +1543,13 @@ public class JNDIRealm extends RealmBase {
                     password = password.substring(6);
 
                     md.reset();
-                    md.update(credentials.getBytes());
+                    md.update(credentials.getBytes(Charset.defaultCharset()));
 
                     // Decode stored password.
                     ByteChunk pwbc = new ByteChunk(password.length());
                     try {
-                        pwbc.append(password.getBytes(), 0, password.length());
+                        pwbc.append(password.getBytes(Charset.defaultCharset()),
+                                0, password.length());
                     } catch (IOException e) {
                         // Should never happen
                         containerLog.error("Could not append password bytes to chunk: ", e);
@@ -1529,7 +1586,6 @@ public class JNDIRealm extends RealmBase {
     }
 
 
-
     /**
      * Check credentials by binding to the directory as the user
      *
@@ -1556,9 +1612,7 @@ public class JNDIRealm extends RealmBase {
              containerLog.trace("  validating credentials by binding as the user");
         }
 
-        // Set up security environment to bind as the user
-        context.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
-        context.addToEnvironment(Context.SECURITY_CREDENTIALS, credentials);
+        userCredentialsAdd(context, dn, credentials);
 
         // Elicit an LDAP bind operation
         boolean validated = false;
@@ -1575,6 +1629,35 @@ public class JNDIRealm extends RealmBase {
             }
         }
 
+        userCredentialsRemove(context);
+
+        return (validated);
+    }
+
+     /**
+      * Configure the context to use the provided credentials for
+      * authentication.
+      *
+      * @param context      DirContext to configure
+      * @param dn           Distinguished name of user
+      * @param credentials  Credentials of user
+      */
+    private void userCredentialsAdd(DirContext context, String dn,
+            String credentials) throws NamingException {
+        // Set up security environment to bind as the user
+        context.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
+        context.addToEnvironment(Context.SECURITY_CREDENTIALS, credentials);
+    }
+
+    /**
+     * Configure the context to use {@link #connectionName} and
+     * {@link #connectionPassword} if specified or an anonymous connection if
+     * those attributes are not specified.
+     * 
+      * @param context      DirContext to configure
+     */
+    private void userCredentialsRemove(DirContext context)
+            throws NamingException {
         // Restore the original security environment
         if (connectionName != null) {
             context.addToEnvironment(Context.SECURITY_PRINCIPAL,
@@ -1590,9 +1673,7 @@ public class JNDIRealm extends RealmBase {
         else {
             context.removeFromEnvironment(Context.SECURITY_CREDENTIALS);
         }
-
-        return (validated);
-     }
+    }
 
     /**
      * Return a List of roles associated with the given User.  Any
@@ -1648,9 +1729,30 @@ public class JNDIRealm extends RealmBase {
             controls.setSearchScope(SearchControls.ONELEVEL_SCOPE);
         controls.setReturningAttributes(new String[] {roleName});
 
+        String base = null;
+        if (roleBaseFormat != null) {
+            NameParser np = context.getNameParser("");
+            Name name = np.parse(dn);
+            String nameParts[] = new String[name.size()];
+            for (int i = 0; i < name.size(); i++) {
+                nameParts[i] = name.get(i);
+            }
+            base = roleBaseFormat.format(nameParts);
+        }
+
         // Perform the configured search and process the results
-        NamingEnumeration<SearchResult> results =
-            context.search(roleBase, filter, controls);
+        NamingEnumeration<SearchResult> results = null;
+        try {
+            if (roleSearchAsUser) {
+                userCredentialsAdd(context, dn, user.getPassword());
+            }
+            results = context.search(base, filter, controls);
+        } finally {
+            if (roleSearchAsUser) {
+                userCredentialsRemove(context);
+            }
+        }
+
         if (results == null)
             return (list);  // Should never happen, but just in case ...
 
@@ -1766,7 +1868,6 @@ public class JNDIRealm extends RealmBase {
     }
 
 
-
     /**
      * Add values of a specified attribute to a list
      *
@@ -1831,6 +1932,7 @@ public class JNDIRealm extends RealmBase {
     /**
      * Return a short name for this Realm implementation.
      */
+    @Override
     protected String getName() {
 
         return (name);
@@ -1841,6 +1943,7 @@ public class JNDIRealm extends RealmBase {
     /**
      * Return the password associated with the given principal's user name.
      */
+    @Override
     protected String getPassword(String username) {
 
         return (null);
@@ -1850,7 +1953,14 @@ public class JNDIRealm extends RealmBase {
     /**
      * Return the Principal associated with the given user name.
      */
+    @Override
     protected Principal getPrincipal(String username) {
+        return getPrincipal(username, null);
+    }
+    
+    @Override
+    protected Principal getPrincipal(String username,
+            GSSCredential gssCredential) {
 
         DirContext context = null;
         Principal principal = null;
@@ -1860,12 +1970,12 @@ public class JNDIRealm extends RealmBase {
             // Ensure that we have a directory context available
             context = open();
 
-            // Occassionally the directory context will timeout.  Try one more
+            // Occasionally the directory context will timeout.  Try one more
             // time before giving up.
             try {
 
                 // Authenticate the specified username if possible
-                principal = getPrincipal(context, username);
+                principal = getPrincipal(context, username, gssCredential);
 
             } catch (CommunicationException e) {
 
@@ -1880,7 +1990,7 @@ public class JNDIRealm extends RealmBase {
                 context = open();
 
                 // Try the authentication again.
-                principal = getPrincipal(context, username);
+                principal = getPrincipal(context, username, gssCredential);
 
             } catch (ServiceUnavailableException e) {
 
@@ -1895,7 +2005,7 @@ public class JNDIRealm extends RealmBase {
                 context = open();
 
                 // Try the authentication again.
-                principal = getPrincipal(context, username);
+                principal = getPrincipal(context, username, gssCredential);
 
             }
 
@@ -1928,14 +2038,52 @@ public class JNDIRealm extends RealmBase {
      * Return the Principal associated with the given user name.
      */
     protected synchronized Principal getPrincipal(DirContext context,
-                                                  String username)
+            String username, GSSCredential gssCredential)
         throws NamingException {
 
-        User user = getUser(context, username);
+        User user = null;
+        List<String> roles = null;
+
+        try {
+            if (gssCredential != null && isUseDelegatedCredential()) {
+                // Set up context
+                context.addToEnvironment(
+                        Context.SECURITY_AUTHENTICATION, "GSSAPI");
+                context.addToEnvironment(
+                        "javax.security.sasl.server.authentication", "true");
+                context.addToEnvironment(
+                        "javax.security.sasl.qop", "auth-conf");
+                // Note: Subject already set in SPNEGO authenticator so no need
+                //       for Subject.doAs() here
+            }
+            user = getUser(context, username);
+            if (user != null) {
+                roles = getRoles(context, user);
+            }
+        } finally {
+            try {
+                context.removeFromEnvironment(
+                        Context.SECURITY_AUTHENTICATION);
+            } catch (NamingException e) {
+                // Ignore
+            }
+            try {
+                context.removeFromEnvironment(
+                        "javax.security.sasl.server.authentication");
+            } catch (NamingException e) {
+                // Ignore
+            }
+            try {
+                context.removeFromEnvironment(
+                        "javax.security.sasl.qop");
+            } catch (NamingException e) {
+                // Ignore
+            }
+        }
 
         if (user != null) {
-            return new GenericPrincipal(this, user.getUserName(),
-                    user.getPassword(), getRoles(context, user));
+            return new GenericPrincipal(user.getUserName(), user.getPassword(),
+                    roles, null, null, gssCredential);
         }
         
         return null;
@@ -2012,7 +2160,7 @@ public class JNDIRealm extends RealmBase {
         if (derefAliases != null)
             env.put(JNDIRealm.DEREF_ALIASES, derefAliases);
         if (connectionTimeout != null)
-        	env.put("com.sun.jndi.ldap.connect.timeout", connectionTimeout);
+            env.put("com.sun.jndi.ldap.connect.timeout", connectionTimeout);
 
         return env;
 
@@ -2035,15 +2183,15 @@ public class JNDIRealm extends RealmBase {
 
 
     /**
-     * Prepare for active use of the public methods of this Component.
+     * Prepare for the beginning of active use of the public methods of this
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
-     *  that prevents it from being started
+     *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
-
-        // Perform normal superclass initialization
-        super.start();
+    @Override
+    protected void startInternal() throws LifecycleException {
 
         // Validate that we can open our connection
         try {
@@ -2052,19 +2200,22 @@ public class JNDIRealm extends RealmBase {
             throw new LifecycleException(sm.getString("jndiRealm.open"), e);
         }
 
+        super.startInternal();
     }
 
 
     /**
-     * Gracefully shut down active use of the public methods of this Component.
+     * Gracefully terminate the active use of the public methods of this
+     * component and implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that needs to be reported
      */
-    public void stop() throws LifecycleException {
+     @Override
+    protected void stopInternal() throws LifecycleException {
 
-        // Perform normal superclass finalization
-        super.stop();
+        super.stopInternal();
 
         // Close any open directory server connection
         close(this.context);
@@ -2132,7 +2283,7 @@ public class JNDIRealm extends RealmBase {
      * @return String the escaped/encoded result
      */
     protected String doRFC2254Encoding(String inString) {
-        StringBuffer buf = new StringBuffer(inString.length());
+        StringBuilder buf = new StringBuilder(inString.length());
         for (int i = 0; i < inString.length(); i++) {
             char c = inString.charAt(i);
             switch (c) {
@@ -2216,45 +2367,45 @@ public class JNDIRealm extends RealmBase {
     }
 
 
-     // ------------------------------------------------------ Private Classes
-    
-     /**
-      * A protected class representing a User
-      */
-     protected static class User {
-         
-         final private String username;
-         final private String dn;
-         final private String password;
-         final private List<String> roles;
+    // ------------------------------------------------------ Private Classes
 
-         public User(String username, String dn, String password,
-                 List<String> roles) {
-             this.username = username;
-             this.dn = dn;
-             this.password = password;
-             if (roles == null) {
-                 this.roles = Collections.emptyList();
-             } else {
-                 this.roles = Collections.unmodifiableList(roles);
-             }
-         }
+    /**
+     * A protected class representing a User
+     */
+    protected static class User {
+         
+        private final String username;
+        private final String dn;
+        private final String password;
+        private final List<String> roles;
+
+        public User(String username, String dn, String password,
+                List<String> roles) {
+            this.username = username;
+            this.dn = dn;
+            this.password = password;
+            if (roles == null) {
+                this.roles = Collections.emptyList();
+            } else {
+                this.roles = Collections.unmodifiableList(roles);
+            }
+        }
     
-         public String getUserName() {
-             return username;
-         }
+        public String getUserName() {
+            return username;
+        }
          
-         public String getDN() {
-             return dn;
-         }
+        public String getDN() {
+            return dn;
+        }
+        
+        public String getPassword() {
+            return password;
+        }
          
-         public String getPassword() {
-             return password;
-         }
-         
-         public List<String> getRoles() {
-             return roles;
-         }
-     }
+        public List<String> getRoles() {
+            return roles;
+        }
+    }
 }
 
