@@ -37,18 +37,17 @@ import java.util.Locale;
 import java.util.StringTokenizer;
 import java.util.Vector;
 
+import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletConfig;
 import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import javax.servlet.ServletOutputStream;
-import javax.servlet.UnavailableException;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import javax.servlet.http.HttpSession;
 
-import org.apache.catalina.Globals;
 import org.apache.catalina.util.IOTools;
 
 
@@ -180,7 +179,7 @@ import org.apache.catalina.util.IOTools;
  * </p>
  * <p>
  *
- * <b>Metavariable Values</b>: According to the CGI specificion,
+ * <b>Metavariable Values</b>: According to the CGI specification,
  * implementations may choose to represent both null or missing values in an
  * implementation-specific manner, but must define that manner.  This
  * implementation chooses to always define all required metavariables, but
@@ -193,7 +192,7 @@ import org.apache.catalina.util.IOTools;
  *
  * <b>NPH --  Non-parsed-header implementation</b>:  This implementation does
  * not support the CGI NPH concept, whereby server ensures that the data
- * supplied to the script are preceisely as supplied by the client and
+ * supplied to the script are precisely as supplied by the client and
  * unaltered by the server.
  * </p>
  * <p>
@@ -228,15 +227,13 @@ import org.apache.catalina.util.IOTools;
  * <li> Better documentation
  * <li> Confirm use of ServletInputStream.available() in CGIRunner.run() is
  *      not needed
- * <li> Make checking for "." and ".." in servlet & cgi PATH_INFO less
- *      draconian
  * <li> [add more to this TODO list]
  * </ul>
  * </p>
  *
  * @author Martin T Dengler [root@martindengler.com]
  * @author Amy Roh
- * @version $Id: CGIServlet.java 1061536 2011-01-20 21:44:08Z markt $
+ * @version $Id: CGIServlet.java 1162172 2011-08-26 17:12:33Z markt $
  * @since Tomcat 4.0
  *
  */
@@ -246,6 +243,8 @@ public final class CGIServlet extends HttpServlet {
 
     /* some vars below copied from Craig R. McClanahan's InvokerServlet */
 
+    private static final long serialVersionUID = 1L;
+
     /** the debugging detail level for this servlet. */
     private int debug = 0;
 
@@ -294,18 +293,11 @@ public final class CGIServlet extends HttpServlet {
      *                                  interferes with the servlet's normal
      *                                  operation
      */
+    @Override
     public void init(ServletConfig config) throws ServletException {
 
         super.init(config);
 
-        // Verify that we were not accessed using the invoker servlet
-        String servletName = getServletConfig().getServletName();
-        if (servletName == null)
-            servletName = "";
-        if (servletName.startsWith("org.apache.catalina.INVOKER."))
-            throw new UnavailableException
-                ("Cannot invoke CGIServlet through the invoker");
-        
         // Set our properties from the initialization parameters
         if (getServletConfig().getInitParameter("debug") != null)
             debug = Integer.parseInt(getServletConfig().getInitParameter("debug"));
@@ -346,7 +338,6 @@ public final class CGIServlet extends HttpServlet {
     }
 
 
-
     /**
      * Prints out important Servlet API and container information
      *
@@ -363,14 +354,15 @@ public final class CGIServlet extends HttpServlet {
      *
      */
     protected void printServletEnvironment(ServletOutputStream out,
-        HttpServletRequest req, HttpServletResponse res) throws IOException {
+        HttpServletRequest req, HttpServletResponse res)
+    throws IOException {
 
         // Document the properties from ServletRequest
         out.println("<h1>ServletRequest Properties</h1>");
         out.println("<ul>");
-        Enumeration attrs = req.getAttributeNames();
+        Enumeration<String> attrs = req.getAttributeNames();
         while (attrs.hasMoreElements()) {
-            String attr = (String) attrs.nextElement();
+            String attr = attrs.nextElement();
             out.println("<li><b>attribute</b> " + attr + " = " +
                            req.getAttribute(attr));
         }
@@ -380,14 +372,14 @@ public final class CGIServlet extends HttpServlet {
                        req.getContentLength());
         out.println("<li><b>contentType</b> = " +
                        req.getContentType());
-        Enumeration locales = req.getLocales();
+        Enumeration<Locale> locales = req.getLocales();
         while (locales.hasMoreElements()) {
-            Locale locale = (Locale) locales.nextElement();
+            Locale locale = locales.nextElement();
             out.println("<li><b>locale</b> = " + locale);
         }
-        Enumeration params = req.getParameterNames();
+        Enumeration<String> params = req.getParameterNames();
         while (params.hasMoreElements()) {
-            String param = (String) params.nextElement();
+            String param = params.nextElement();
             String values[] = req.getParameterValues(param);
             for (int i = 0; i < values.length; i++)
                 out.println("<li><b>parameter</b> " + param + " = " +
@@ -414,9 +406,9 @@ public final class CGIServlet extends HttpServlet {
             for (int i = 0; i < cookies.length; i++)
                 out.println("<li><b>cookie</b> " + cookies[i].getName() +" = " +cookies[i].getValue());
         }
-        Enumeration headers = req.getHeaderNames();
+        Enumeration<String> headers = req.getHeaderNames();
         while (headers.hasMoreElements()) {
-            String header = (String) headers.nextElement();
+            String header = headers.nextElement();
             out.println("<li><b>header</b> " + header + " = " +
                            req.getHeader(header));
         }
@@ -451,7 +443,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         attrs = req.getAttributeNames();
         while (attrs.hasMoreElements()) {
-            String attr = (String) attrs.nextElement();
+            String attr = attrs.nextElement();
             out.println("<li><b>" + attr + "</b> = " +
                            req.getAttribute(attr));
         }
@@ -481,7 +473,7 @@ public final class CGIServlet extends HttpServlet {
             out.println("<ul>");
             attrs = session.getAttributeNames();
             while (attrs.hasMoreElements()) {
-                String attr = (String) attrs.nextElement();
+                String attr = attrs.nextElement();
                 out.println("<li><b>" + attr + "</b> = " +
                                session.getAttribute(attr));
             }
@@ -503,7 +495,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         params = getServletConfig().getInitParameterNames();
         while (params.hasMoreElements()) {
-            String param = (String) params.nextElement();
+            String param = params.nextElement();
             String value = getServletConfig().getInitParameter(param);
             out.println("<li><b>" + param + "</b> = " + value);
         }
@@ -529,7 +521,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         params = getServletContext().getInitParameterNames();
         while (params.hasMoreElements()) {
-            String param = (String) params.nextElement();
+            String param = params.nextElement();
             String value = getServletContext().getInitParameter(param);
             out.println("<li><b>" + param + "</b> = " + value);
         }
@@ -541,7 +533,7 @@ public final class CGIServlet extends HttpServlet {
         out.println("<ul>");
         attrs = getServletContext().getAttributeNames();
         while (attrs.hasMoreElements()) {
-            String attr = (String) attrs.nextElement();
+            String attr = attrs.nextElement();
             out.println("<li><b>" + attr + "</b> = " +
                            getServletContext().getAttribute(attr));
         }
@@ -549,11 +541,9 @@ public final class CGIServlet extends HttpServlet {
         out.println("<hr>");
 
 
-
     }
 
 
-
     /**
      * Provides CGI Gateway service -- delegates to <code>doGet</code>
      *
@@ -566,13 +556,13 @@ public final class CGIServlet extends HttpServlet {
      * @see javax.servlet.http.HttpServlet
      *
      */
+    @Override
     protected void doPost(HttpServletRequest req, HttpServletResponse res)
         throws IOException, ServletException {
         doGet(req, res);
     }
 
 
-
     /**
      * Provides CGI Gateway service
      *
@@ -585,14 +575,10 @@ public final class CGIServlet extends HttpServlet {
      * @see javax.servlet.http.HttpServlet
      *
      */
+    @Override
     protected void doGet(HttpServletRequest req, HttpServletResponse res)
         throws ServletException, IOException {
 
-        // Verify that we were not accessed using the invoker servlet
-        if (req.getAttribute(Globals.INVOKED_ATTR) != null)
-            throw new UnavailableException
-                ("Cannot invoke CGIServlet through the invoker");
-
         CGIEnvironment cgiEnv = new CGIEnvironment(req, getServletContext());
 
         if (cgiEnv.isValid()) {
@@ -653,13 +639,6 @@ public final class CGIServlet extends HttpServlet {
     } //doGet
 
 
-
-    /** For future testing use only; does nothing right now */
-    public static void main(String[] args) {
-        System.out.println("$Header$");
-    }
-
-
     /**
      * Encapsulates the CGI environment and rules to derive
      * that environment from the servlet container and request information.
@@ -667,7 +646,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      * </p>
      *
-     * @version  $Id: CGIServlet.java 1061536 2011-01-20 21:44:08Z markt $
+     * @version  $Id: CGIServlet.java 1162172 2011-08-26 17:12:33Z markt $
      * @since    Tomcat 4.0
      *
      */
@@ -693,7 +672,7 @@ public final class CGIServlet extends HttpServlet {
         private File tmpDir = null;
 
         /** derived cgi environment */
-        private Hashtable env = null;
+        private Hashtable<String, String> env = null;
 
         /** cgi command to be invoked */
         private String command = null;
@@ -733,7 +712,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Uses the ServletContext to set some CGI variables
          *
@@ -743,11 +721,10 @@ public final class CGIServlet extends HttpServlet {
         protected void setupFromContext(ServletContext context) {
             this.context = context;
             this.webAppRootDir = context.getRealPath("/");
-            this.tmpDir = (File) context.getAttribute(Globals.WORK_DIR_ATTR);
+            this.tmpDir = (File) context.getAttribute(ServletContext.TEMPDIR);
         }
 
 
-
         /**
          * Uses the HttpServletRequest to set most CGI variables
          *
@@ -761,16 +738,17 @@ public final class CGIServlet extends HttpServlet {
             boolean isIncluded = false;
 
             // Look to see if this request is an include
-            if (req.getAttribute(Globals.INCLUDE_REQUEST_URI_ATTR) != null) {
+            if (req.getAttribute(
+                    RequestDispatcher.INCLUDE_REQUEST_URI) != null) {
                 isIncluded = true;
             }
             if (isIncluded) {
                 this.contextPath = (String) req.getAttribute(
-                        Globals.INCLUDE_CONTEXT_PATH_ATTR);
+                        RequestDispatcher.INCLUDE_CONTEXT_PATH);
                 this.servletPath = (String) req.getAttribute(
-                        Globals.INCLUDE_SERVLET_PATH_ATTR);
+                        RequestDispatcher.INCLUDE_SERVLET_PATH);
                 this.pathInfo = (String) req.getAttribute(
-                        Globals.INCLUDE_PATH_INFO_ATTR);
+                        RequestDispatcher.INCLUDE_PATH_INFO);
             } else {
                 this.contextPath = req.getContextPath();
                 this.servletPath = req.getServletPath();
@@ -792,7 +770,7 @@ public final class CGIServlet extends HttpServlet {
                 String qs;
                 if (isIncluded) {
                     qs = (String) req.getAttribute(
-                            Globals.INCLUDE_QUERY_STRING_ATTR);
+                            RequestDispatcher.INCLUDE_QUERY_STRING);
                 } else {
                     qs = req.getQueryString();
                 }
@@ -913,21 +891,21 @@ public final class CGIServlet extends HttpServlet {
             }
             if (!currentLocation.isFile()) {
                 return new String[] { null, null, null, null };
-            } else {
-                if (debug >= 2) {
-                    log("findCGI: FOUND cgi at " + currentLocation);
-                }
-                path = currentLocation.getAbsolutePath();
-                name = currentLocation.getName();
+            }
 
-                if (".".equals(contextPath)) {
-                    scriptname = servletPath;
-                } else {
-                    scriptname = contextPath + servletPath;
-                }
-                if (!servletPath.equals(cginame)) {
-                    scriptname = scriptname + cginame;
-                }
+            if (debug >= 2) {
+                log("findCGI: FOUND cgi at " + currentLocation);
+            }
+            path = currentLocation.getAbsolutePath();
+            name = currentLocation.getName();
+
+            if (".".equals(contextPath)) {
+                scriptname = servletPath;
+            } else {
+                scriptname = contextPath + servletPath;
+            }
+            if (!servletPath.equals(cginame)) {
+                scriptname = scriptname + cginame;
             }
 
             if (debug >= 1) {
@@ -939,10 +917,10 @@ public final class CGIServlet extends HttpServlet {
 
         /**
          * Constructs the CGI environment to be supplied to the invoked CGI
-         * script; relies heavliy on Servlet API methods and findCGI
+         * script; relies heavily on Servlet API methods and findCGI
          *
          * @param    req request associated with the CGI
-         *           invokation
+         *           Invocation
          *
          * @return   true if environment was set OK, false if there
          *           was a problem and no environment was set
@@ -962,7 +940,6 @@ public final class CGIServlet extends HttpServlet {
 
             // Add the CGI environment variables
             String sPathInfoOrig = null;
-            String sPathTranslatedOrig = null;
             String sPathInfoCGI = null;
             String sPathTranslatedCGI = null;
             String sCGIFullPath = null;
@@ -975,10 +952,6 @@ public final class CGIServlet extends HttpServlet {
             sPathInfoOrig = this.pathInfo;
             sPathInfoOrig = sPathInfoOrig == null ? "" : sPathInfoOrig;
 
-            sPathTranslatedOrig = req.getPathTranslated();
-            sPathTranslatedOrig =
-                sPathTranslatedOrig == null ? "" : sPathTranslatedOrig;
-
             if (webAppRootDir == null ) {
                 // The app has not been deployed in exploded form
                 webAppRootDir = tmpDir.toString();
@@ -1012,7 +985,8 @@ public final class CGIServlet extends HttpServlet {
             envp.put("SERVER_PROTOCOL", nullsToBlanks(req.getProtocol()));
 
             int port = req.getServerPort();
-            Integer iPort = (port == 0 ? new Integer(-1) : new Integer(port));
+            Integer iPort =
+                (port == 0 ? Integer.valueOf(-1) : Integer.valueOf(port));
             envp.put("SERVER_PORT", iPort.toString());
 
             envp.put("REQUEST_METHOD", nullsToBlanks(req.getMethod()));
@@ -1061,8 +1035,6 @@ public final class CGIServlet extends HttpServlet {
              */
             if (sPathInfoCGI != null && !("".equals(sPathInfoCGI))) {
                 sPathTranslatedCGI = context.getRealPath(sPathInfoCGI);
-            } else {
-                sPathTranslatedCGI = null;
             }
             if (sPathTranslatedCGI == null || "".equals(sPathTranslatedCGI)) {
                 //NOOP
@@ -1094,15 +1066,15 @@ public final class CGIServlet extends HttpServlet {
              */
             int contentLength = req.getContentLength();
             String sContentLength = (contentLength <= 0 ? "" :
-                                     (new Integer(contentLength)).toString());
+                (Integer.valueOf(contentLength)).toString());
             envp.put("CONTENT_LENGTH", sContentLength);
 
 
-            Enumeration headers = req.getHeaderNames();
+            Enumeration<String> headers = req.getHeaderNames();
             String header = null;
             while (headers.hasMoreElements()) {
                 header = null;
-                header = ((String) headers.nextElement()).toUpperCase();
+                header = headers.nextElement().toUpperCase(Locale.ENGLISH);
                 //REMIND: rewrite multiple headers as if received as single
                 //REMIND: change character set
                 //REMIND: I forgot what the previous REMIND means
@@ -1133,8 +1105,8 @@ public final class CGIServlet extends HttpServlet {
          * directory to enable CGI script to be executed.
          */
         protected void expandCGIScript() {
-            StringBuffer srcPath = new StringBuffer();
-            StringBuffer destPath = new StringBuffer();
+            StringBuilder srcPath = new StringBuilder();
+            StringBuilder destPath = new StringBuilder();
             InputStream is = null;
 
             // paths depend on mapping
@@ -1174,10 +1146,16 @@ public final class CGIServlet extends HttpServlet {
             } 
 
             // create directories
-            String dirPath = new String (destPath.toString().substring(
-                    0,destPath.toString().lastIndexOf("/")));
+            String dirPath = destPath.toString().substring(
+                    0,destPath.toString().lastIndexOf("/"));
             File dir = new File(dirPath);
-            dir.mkdirs();
+            if (!dir.mkdirs() && !dir.isDirectory()) {
+                if (debug >= 2) {
+                    log("expandCGIScript: failed to create directories for '" +
+                            dir.getAbsolutePath() + "'");
+                }
+                return;
+            }
 
             try {
                 synchronized (expandFileLock) {
@@ -1203,7 +1181,10 @@ public final class CGIServlet extends HttpServlet {
             } catch (IOException ioe) {
                 // delete in case file is corrupted 
                 if (f.exists()) {
-                    f.delete();
+                    if (!f.delete() && debug >= 2) {
+                        log("expandCGIScript: failed to delete '" +
+                                f.getAbsolutePath() + "'");
+                    }
                 }
             }
         }
@@ -1216,9 +1197,10 @@ public final class CGIServlet extends HttpServlet {
          * @return  HTML string containing CGI environment info
          *
          */
+        @Override
         public String toString() {
 
-            StringBuffer sb = new StringBuffer();
+            StringBuilder sb = new StringBuilder();
 
             sb.append("<TABLE border=2>");
 
@@ -1234,13 +1216,13 @@ public final class CGIServlet extends HttpServlet {
             sb.append("</td></tr>");
 
             if (isValid()) {
-                Enumeration envk = env.keys();
+                Enumeration<String> envk = env.keys();
                 while (envk.hasMoreElements()) {
-                    String s = (String) envk.nextElement();
+                    String s = envk.nextElement();
                     sb.append("<tr><td>");
                     sb.append(s);
                     sb.append("</td><td>");
-                    sb.append(blanksToString((String) env.get(s),
+                    sb.append(blanksToString(env.get(s),
                                              "[will be set to blank]"));
                     sb.append("</td></tr>");
                 }
@@ -1260,7 +1242,7 @@ public final class CGIServlet extends HttpServlet {
 
             sb.append("<tr><td>Command Line Params</td><td>");
             for (int i=0; i < cmdLineParameters.size(); i++) {
-                String param = (String) cmdLineParameters.get(i);
+                String param = cmdLineParameters.get(i);
                 sb.append("<p>");
                 sb.append(param);
                 sb.append("</p>");
@@ -1273,7 +1255,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets derived command string
          *
@@ -1285,7 +1266,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets derived CGI working directory
          *
@@ -1297,31 +1277,28 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets derived CGI environment
          *
          * @return   CGI environment
          *
          */
-        protected Hashtable getEnvironment() {
+        protected Hashtable<String,String> getEnvironment() {
             return env;
         }
 
 
-
         /**
          * Gets derived CGI query parameters
          *
          * @return   CGI query parameters
          *
          */
-        protected ArrayList getParameters() {
+        protected ArrayList<String> getParameters() {
             return cmdLineParameters;
         }
 
 
-
         /**
          * Gets validity status
          *
@@ -1334,7 +1311,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts null strings to blank strings ("")
          *
@@ -1347,7 +1323,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts null strings to another string
          *
@@ -1362,7 +1337,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts blank strings to another string
          *
@@ -1379,14 +1353,9 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
     } //class CGIEnvironment
 
 
-
-
-
-
     /**
      * Encapsulates the knowledge of how to run a CGI script, given the
      * script's desired environment and (optionally) input/output streams
@@ -1400,7 +1369,7 @@ public final class CGIServlet extends HttpServlet {
      * <p>
      *
      * The CGI environment and settings are derived from the information
-     * passed to the constuctor.
+     * passed to the constructor.
      *
      * </p>
      * <p>
@@ -1409,7 +1378,7 @@ public final class CGIServlet extends HttpServlet {
      * and <code>setResponse</code> methods, respectively.
      * </p>
      *
-     * @version $Id: CGIServlet.java 1061536 2011-01-20 21:44:08Z markt $
+     * @version $Id: CGIServlet.java 1162172 2011-08-26 17:12:33Z markt $
      */
 
     protected class CGIRunner {
@@ -1418,13 +1387,13 @@ public final class CGIServlet extends HttpServlet {
         private String command = null;
 
         /** environment used when invoking the cgi script */
-        private Hashtable env = null;
+        private Hashtable<String,String> env = null;
 
         /** working directory used when invoking the cgi script */
         private File wd = null;
 
         /** command line parameters to be passed to the invoked script */
-        private ArrayList params = null;
+        private ArrayList<String> params = null;
 
         /** stdin to be passed to cgi script */
         private InputStream stdin = null;
@@ -1436,8 +1405,6 @@ public final class CGIServlet extends HttpServlet {
         private boolean readyToRun = false;
 
 
-
-
         /**
          *  Creates a CGIRunner and initializes its environment, working
          *  directory, and query parameters.
@@ -1450,10 +1417,10 @@ public final class CGIServlet extends HttpServlet {
          * @param  env      Hashtable with the desired script environment
          * @param  wd       File with the script's desired working directory
          * @param  params   ArrayList with the script's query command line
-         *                  paramters as strings
+         *                  parameters as strings
          */
-        protected CGIRunner(String command, Hashtable env, File wd,
-                            ArrayList params) {
+        protected CGIRunner(String command, Hashtable<String,String> env,
+                            File wd, ArrayList<String> params) {
             this.command = command;
             this.env = env;
             this.wd = wd;
@@ -1462,7 +1429,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Checks & sets ready status
          */
@@ -1479,7 +1445,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Gets ready status
          *
@@ -1491,7 +1456,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Sets HttpServletResponse object used to set headers and send
          * output to
@@ -1505,7 +1469,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Sets standard input to be passed on to the invoked cgi script
          *
@@ -1518,7 +1481,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Converts a Hashtable to a String array by converting each
          * key/value pair in the Hashtable to a String in the form
@@ -1531,13 +1493,13 @@ public final class CGIServlet extends HttpServlet {
          * @exception  NullPointerException   if a hash key has a null value
          *
          */
-        protected String[] hashToStringArray(Hashtable h)
+        protected String[] hashToStringArray(Hashtable<String,?> h)
             throws NullPointerException {
             Vector<String> v = new Vector<String>();
-            Enumeration e = h.keys();
+            Enumeration<String> e = h.keys();
             while (e.hasMoreElements()) {
-                String k = e.nextElement().toString();
-                v.add(k + "=" + h.get(k));
+                String k = e.nextElement();
+                v.add(k + "=" + h.get(k).toString());
             }
             String[] strArr = new String[v.size()];
             v.copyInto(strArr);
@@ -1545,7 +1507,6 @@ public final class CGIServlet extends HttpServlet {
         }
 
 
-
         /**
          * Executes a CGI script with the desired environment, current working
          * directory, and input/output streams
@@ -1578,7 +1539,10 @@ public final class CGIServlet extends HttpServlet {
          *             segments</u>:
          *             This implementation does not allow "<code>.</code>" and
          *             "<code>..</code>" in the the path, and such characters
-         *             will result in an IOException being thrown;
+         *             will result in an IOException being thrown (this should
+         *             never happen since Tomcat normalises the requestURI
+         *             before determining the contextPath, servletPath and
+         *             pathInfo);
          *     <LI> <u>Implementation limitations</u>: This implementation
          *             does not impose any limitations except as documented
          *             above.  This implementation may be limited by the
@@ -1620,11 +1584,11 @@ public final class CGIServlet extends HttpServlet {
             }
 
             /* original content/structure of this section taken from
-             * http://developer.java.sun.com/developer/
-             *                               bugParade/bugs/4216884.html
+             * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4216884
              * with major modifications by Martin Dengler
              */
             Runtime rt = null;
+            BufferedReader cgiHeaderReader = null;
             InputStream cgiOutput = null;
             BufferedReader commandsStdErr = null;
             Thread errReaderThread = null;
@@ -1648,7 +1612,7 @@ public final class CGIServlet extends HttpServlet {
                         cmdAndArgs.toArray(new String[cmdAndArgs.size()]),
                         hashToStringArray(env), wd);
     
-                String sContentLength = (String) env.get("CONTENT_LENGTH");
+                String sContentLength = env.get("CONTENT_LENGTH");
 
                 if(!"".equals(sContentLength)) {
                     commandsStdIn = new BufferedOutputStream(proc.getOutputStream());
@@ -1659,8 +1623,7 @@ public final class CGIServlet extends HttpServlet {
 
                 /* we want to wait for the process to exit,  Process.waitFor()
                  * is useless in our situation; see
-                 * http://developer.java.sun.com/developer/
-                 *                               bugParade/bugs/4223650.html
+                 * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4223650
                  */
 
                 boolean isRunning = true;
@@ -1669,15 +1632,16 @@ public final class CGIServlet extends HttpServlet {
                 final BufferedReader stdErrRdr = commandsStdErr ;
 
                 errReaderThread = new Thread() {
+                    @Override
                     public void run () {
                         sendToLog(stdErrRdr) ;
-                    } ;
+                    }
                 };
                 errReaderThread.start();
 
                 InputStream cgiHeaderStream =
                     new HTTPHeaderInputStream(proc.getInputStream());
-                BufferedReader cgiHeaderReader =
+                cgiHeaderReader =
                     new BufferedReader(new InputStreamReader(cgiHeaderStream));
             
                 while (isRunning) {
@@ -1725,7 +1689,9 @@ public final class CGIServlet extends HttpServlet {
                             // such as a socket disconnect on the servlet side; otherwise, the
                             // external process could hang
                             if (bufRead != -1) {
-                                while ((bufRead = cgiOutput.read(bBuf)) != -1) {}
+                                while ((bufRead = cgiOutput.read(bBuf)) != -1) {
+                                    // NOOP - just read the data
+                                }
                             }
                         }
         
@@ -1737,15 +1703,25 @@ public final class CGIServlet extends HttpServlet {
                         try {
                             Thread.sleep(500);
                         } catch (InterruptedException ignored) {
+                            // Ignore
                         }
                     }
                 } //replacement for Process.waitFor()
+    
             }
             catch (IOException e){
                 log ("Caught exception " + e);
                 throw e;
             }
             finally{
+                // Close the header reader
+                if (cgiHeaderReader != null) {
+                    try {
+                        cgiHeaderReader.close();
+                    } catch (IOException ioe) {
+                        log ("Exception closing header reader " + ioe);
+                    }
+                }
                 // Close the output stream if used
                 if (cgiOutput != null) {
                     try {
@@ -1846,11 +1822,11 @@ public final class CGIServlet extends HttpServlet {
                     rdr.close() ;
                 } catch (IOException ce) {
                     log("sendToLog error", ce) ;
-                } ;
-            } ;
+                }
+            }
             if ( lineCount > 0 && debug > 2) {
                 log("runCGI: " + lineCount + " lines received on stderr") ;
-            } ;
+            }
         }
     } //class CGIRunner
 
@@ -1859,7 +1835,7 @@ public final class CGIServlet extends HttpServlet {
      * upto and including the two blank lines terminating the headers. It
      * allows the content to be read using bytes or characters as appropriate.
      */
-    protected class HTTPHeaderInputStream extends InputStream {
+    protected static class HTTPHeaderInputStream extends InputStream {
         private static final int STATE_CHARACTER = 0;
         private static final int STATE_FIRST_CR = 1;
         private static final int STATE_FIRST_LF = 2;
@@ -1877,6 +1853,7 @@ public final class CGIServlet extends HttpServlet {
         /**
          * @see java.io.InputStream#read()
          */
+        @Override
         public int read() throws IOException {
             if (state == STATE_HEADER_END) {
                 return -1;
