@@ -24,6 +24,7 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
@@ -42,8 +43,12 @@ import javax.naming.NameClassPair;
 import javax.naming.NamingEnumeration;
 import javax.naming.NamingException;
 import javax.naming.directory.DirContext;
+import javax.servlet.RequestDispatcher;
+import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import javax.servlet.ServletOutputStream;
+import javax.servlet.ServletResponse;
+import javax.servlet.ServletResponseWrapper;
 import javax.servlet.UnavailableException;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
@@ -57,14 +62,15 @@ import javax.xml.transform.stream.StreamSource;
 
 import org.apache.catalina.Globals;
 import org.apache.catalina.connector.RequestFacade;
+import org.apache.catalina.connector.ResponseFacade;
 import org.apache.catalina.util.RequestUtil;
 import org.apache.catalina.util.ServerInfo;
-import org.apache.catalina.util.StringManager;
 import org.apache.catalina.util.URLEncoder;
 import org.apache.naming.resources.CacheEntry;
 import org.apache.naming.resources.ProxyDirContext;
 import org.apache.naming.resources.Resource;
 import org.apache.naming.resources.ResourceAttributes;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -108,12 +114,14 @@ import org.apache.naming.resources.ResourceAttributes;
  * </p>
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: DefaultServlet.java 1146703 2011-07-14 13:38:40Z markt $
+ * @version $Id: DefaultServlet.java 1145380 2011-07-11 22:08:06Z markt $
  */
 
 public class DefaultServlet
     extends HttpServlet {
-    
+
+    private static final long serialVersionUID = 1L;
+
     // ----------------------------------------------------- Instance Variables
 
 
@@ -180,7 +188,7 @@ public class DefaultServlet
     /**
      * Proxy directory context.
      */
-    protected ProxyDirContext resources = null;
+    protected transient ProxyDirContext resources = null;
 
 
     /**
@@ -203,7 +211,7 @@ public class DefaultServlet
     /**
      * Full range marker.
      */
-    protected static ArrayList FULL = new ArrayList();
+    protected static ArrayList<Range> FULL = new ArrayList<Range>();
     
     
     // ----------------------------------------------------- Static Initializer
@@ -237,7 +245,7 @@ public class DefaultServlet
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
@@ -253,13 +261,16 @@ public class DefaultServlet
     /**
      * Finalize this servlet.
      */
+    @Override
     public void destroy() {
+        // NOOP
     }
 
 
     /**
      * Initialize this servlet.
      */
+    @Override
     public void init() throws ServletException {
 
         if (getServletConfig().getInitParameter("debug") != null)
@@ -338,15 +349,16 @@ public class DefaultServlet
         // subclasses can change this behaviour.
 
         // Are we being processed by a RequestDispatcher.include()?
-        if (request.getAttribute(Globals.INCLUDE_REQUEST_URI_ATTR) != null) {
+        if (request.getAttribute(
+                RequestDispatcher.INCLUDE_REQUEST_URI) != null) {
             String result = (String) request.getAttribute(
-                                            Globals.INCLUDE_PATH_INFO_ATTR);
+                    RequestDispatcher.INCLUDE_PATH_INFO);
             if (result == null) {
                 result = (String) request.getAttribute(
-                                            Globals.INCLUDE_SERVLET_PATH_ATTR);
+                        RequestDispatcher.INCLUDE_SERVLET_PATH);
             } else {
                 result = (String) request.getAttribute(
-                                  Globals.INCLUDE_SERVLET_PATH_ATTR) + result;
+                        RequestDispatcher.INCLUDE_SERVLET_PATH) + result;
             }
             if ((result == null) || (result.equals(""))) {
                 result = "/";
@@ -390,6 +402,7 @@ public class DefaultServlet
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     protected void doGet(HttpServletRequest request,
                          HttpServletResponse response)
         throws IOException, ServletException {
@@ -409,6 +422,7 @@ public class DefaultServlet
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     protected void doHead(HttpServletRequest request,
                           HttpServletResponse response)
         throws IOException, ServletException {
@@ -438,10 +452,11 @@ public class DefaultServlet
      * @exception ServletException  if the request for the
      *                                  OPTIONS cannot be handled
      */
+    @Override
     protected void doOptions(HttpServletRequest req, HttpServletResponse resp)
         throws ServletException, IOException {
 
-        StringBuffer allow = new StringBuffer();
+        StringBuilder allow = new StringBuilder();
         // There is a doGet method
         allow.append("GET, HEAD");
         // There is a doPost
@@ -471,6 +486,7 @@ public class DefaultServlet
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     protected void doPost(HttpServletRequest request,
                           HttpServletResponse response)
         throws IOException, ServletException {
@@ -479,7 +495,7 @@ public class DefaultServlet
 
 
     /**
-     * Process a POST request for the specified resource.
+     * Process a PUT request for the specified resource.
      *
      * @param req The servlet request we are processing
      * @param resp The servlet response we are creating
@@ -487,6 +503,7 @@ public class DefaultServlet
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     protected void doPut(HttpServletRequest req, HttpServletResponse resp)
         throws ServletException, IOException {
 
@@ -562,7 +579,7 @@ public class DefaultServlet
         // resource - create a temp. file on the local filesystem to
         // perform this operation
         File tempDir = (File) getServletContext().getAttribute
-            ("javax.servlet.context.tempdir");
+            (ServletContext.TEMPDIR);
         // Convert all '/' characters to '.' in resourcePath
         String convertedResourcePath = path.replace('/', '.');
         File contentFile = new File(tempDir, convertedResourcePath);
@@ -580,7 +597,7 @@ public class DefaultServlet
             if (obj instanceof Resource)
                 oldResource = (Resource) obj;
         } catch (NamingException e) {
-            ;
+            // Ignore
         }
 
         // Copy data in oldRevisionContent to contentFile
@@ -618,7 +635,7 @@ public class DefaultServlet
 
 
     /**
-     * Process a POST request for the specified resource.
+     * Process a DELETE request for the specified resource.
      *
      * @param req The servlet request we are processing
      * @param resp The servlet response we are creating
@@ -626,6 +643,7 @@ public class DefaultServlet
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet-specified error occurs
      */
+    @Override
     protected void doDelete(HttpServletRequest req, HttpServletResponse resp)
         throws ServletException, IOException {
 
@@ -689,7 +707,7 @@ public class DefaultServlet
     /**
      * URL rewriter.
      *
-     * @param path Path which has to be rewiten
+     * @param path Path which has to be rewritten
      */
     protected String rewriteUrl(String path) {
         return urlEncoder.encode( path );
@@ -699,7 +717,7 @@ public class DefaultServlet
     /**
      * Display the size of a file.
      */
-    protected void displaySize(StringBuffer buf, int filesize) {
+    protected void displaySize(StringBuilder buf, int filesize) {
 
         int leftside = filesize / 1024;
         int rightside = (filesize % 1024) / 103;  // makes 1 digit
@@ -727,10 +745,12 @@ public class DefaultServlet
                                  boolean content)
         throws IOException, ServletException {
 
+        boolean serveContent = content;
+        
         // Identify the requested resource path
         String path = getRelativePath(request);
         if (debug > 0) {
-            if (content)
+            if (serveContent)
                 log("DefaultServlet.serveResource:  Serving resource '" +
                     path + "' headers and data");
             else
@@ -744,17 +764,15 @@ public class DefaultServlet
             // Check if we're included so we can return the appropriate 
             // missing resource name in the error
             String requestUri = (String) request.getAttribute(
-                                            Globals.INCLUDE_REQUEST_URI_ATTR);
+                    RequestDispatcher.INCLUDE_REQUEST_URI);
             if (requestUri == null) {
                 requestUri = request.getRequestURI();
             } else {
-                // We're included, and the response.sendError() below is going
-                // to be ignored by the resource that is including us.
-                // Therefore, the only way we can let the including resource
-                // know is by including warning message in response
-                response.getWriter().write(RequestUtil.filter(
-                    sm.getString("defaultServlet.missingResource",
-                    requestUri)));
+                // We're included
+                // SRV.9.3 says we must throw a FNFE
+                throw new FileNotFoundException(
+                        sm.getString("defaultServlet.missingResource",
+                    requestUri));
             }
 
             response.sendError(HttpServletResponse.SC_NOT_FOUND,
@@ -769,7 +787,7 @@ public class DefaultServlet
                 // Check if we're included so we can return the appropriate 
                 // missing resource name in the error
                 String requestUri = (String) request.getAttribute(
-                                            Globals.INCLUDE_REQUEST_URI_ATTR);
+                        RequestDispatcher.INCLUDE_REQUEST_URI);
                 if (requestUri == null) {
                     requestUri = request.getRequestURI();
                 }
@@ -779,20 +797,16 @@ public class DefaultServlet
             }
         }
 
-        boolean isError = false;
-        Integer status =
-            (Integer) request.getAttribute("javax.servlet.error.status_code");
-        if (status != null) {
-            isError = status.intValue() >= HttpServletResponse.SC_BAD_REQUEST;
-        }
+        boolean isError =
+            response.getStatus() >= HttpServletResponse.SC_BAD_REQUEST;
 
         // Check if the conditions specified in the optional If headers are
         // satisfied.
         if (cacheEntry.context == null) {
 
             // Checking If headers
-            boolean included =
-                (request.getAttribute(Globals.INCLUDE_CONTEXT_PATH_ATTR) != null);
+            boolean included = (request.getAttribute(
+                    RequestDispatcher.INCLUDE_CONTEXT_PATH) != null);
             if (!included && !isError &&
                     !checkIfHeaders(request, response, cacheEntry.attributes)) {
                 return;
@@ -807,7 +821,7 @@ public class DefaultServlet
             cacheEntry.attributes.setMimeType(contentType);
         }
 
-        ArrayList ranges = null;
+        ArrayList<Range> ranges = null;
         long contentLength = -1L;
 
         if (cacheEntry.context != null) {
@@ -844,7 +858,7 @@ public class DefaultServlet
             // Special case for zero length files, which would cause a
             // (silent) ISE when setting the output buffer size
             if (contentLength == 0L) {
-                content = false;
+                serveContent = false;
             }
 
         }
@@ -852,7 +866,7 @@ public class DefaultServlet
         ServletOutputStream ostream = null;
         PrintWriter writer = null;
 
-        if (content) {
+        if (serveContent) {
 
             // Trying to retrieve the servlet output stream
 
@@ -863,8 +877,11 @@ public class DefaultServlet
                 // trying to serve a text file
                 if ( (contentType == null)
                         || (contentType.startsWith("text"))
-                        || (contentType.endsWith("xml")) ) {
+                        || (contentType.endsWith("xml"))
+                        || (contentType.contains("/javascript")) ) {
                     writer = response.getWriter();
+                    // Cannot reliably serve partial content with a Writer
+                    ranges = FULL;
                 } else {
                     throw e;
                 }
@@ -872,6 +889,21 @@ public class DefaultServlet
 
         }
 
+        // Check to see if a Filter, Valve of wrapper has written some content.
+        // If it has, disable range requests and setting of a content length
+        // since neither can be done reliably.
+        ServletResponse r = response;
+        long contentWritten = 0;
+        while (r instanceof ServletResponseWrapper) {
+            r = ((ServletResponseWrapper) r).getResponse();
+        }
+        if (r instanceof ResponseFacade) {
+            contentWritten = ((ResponseFacade) r).getContentWritten();
+        }
+        if (contentWritten > 0) {
+            ranges = FULL;
+        }
+        
         if ( (cacheEntry.context != null)
                 || isError
                 || ( ((ranges == null) || (ranges.isEmpty()))
@@ -885,22 +917,29 @@ public class DefaultServlet
                         contentType + "'");
                 response.setContentType(contentType);
             }
-            if ((cacheEntry.resource != null) && (contentLength >= 0)) {
+            if ((cacheEntry.resource != null) && (contentLength >= 0)
+                    && (!serveContent || ostream != null)) {
                 if (debug > 0)
                     log("DefaultServlet.serveFile:  contentLength=" +
                         contentLength);
-                if (contentLength < Integer.MAX_VALUE) {
-                    response.setContentLength((int) contentLength);
-                } else {
-                    // Set the content-length as String to be able to use a long
-                    response.setHeader("content-length", "" + contentLength);
+                // Don't set a content length if something else has already
+                // written to the response.
+                if (contentWritten == 0) {
+                    if (contentLength < Integer.MAX_VALUE) {
+                        response.setContentLength((int) contentLength);
+                    } else {
+                        // Set the content-length as String to be able to use a
+                        // long
+                        response.setHeader("content-length",
+                                "" + contentLength);
+                    }
                 }
             }
 
             InputStream renderResult = null;
             if (cacheEntry.context != null) {
 
-                if (content) {
+                if (serveContent) {
                     // Serve the directory browser
                     renderResult = render(getPathPrefix(request), cacheEntry);
                 }
@@ -908,7 +947,7 @@ public class DefaultServlet
             }
 
             // Copy the input stream to our output stream (if requested)
-            if (content) {
+            if (serveContent) {
                 try {
                     response.setBufferSize(output);
                 } catch (IllegalStateException e) {
@@ -933,7 +972,7 @@ public class DefaultServlet
 
             if (ranges.size() == 1) {
 
-                Range range = (Range) ranges.get(0);
+                Range range = ranges.get(0);
                 response.addHeader("Content-Range", "bytes "
                                    + range.start
                                    + "-" + range.end + "/"
@@ -953,7 +992,7 @@ public class DefaultServlet
                     response.setContentType(contentType);
                 }
 
-                if (content) {
+                if (serveContent) {
                     try {
                         response.setBufferSize(output);
                     } catch (IllegalStateException e) {
@@ -963,7 +1002,8 @@ public class DefaultServlet
                         if (!checkSendfile(request, response, cacheEntry, range.end - range.start + 1, range))
                             copy(cacheEntry, ostream, range);
                     } else {
-                        copy(cacheEntry, writer, range);
+                        // we should not get here
+                        throw new IllegalStateException();
                     }
                 }
 
@@ -972,7 +1012,7 @@ public class DefaultServlet
                 response.setContentType("multipart/byteranges; boundary="
                                         + mimeSeparation);
 
-                if (content) {
+                if (serveContent) {
                     try {
                         response.setBufferSize(output);
                     } catch (IllegalStateException e) {
@@ -982,8 +1022,8 @@ public class DefaultServlet
                         copy(cacheEntry, ostream, ranges.iterator(),
                              contentType);
                     } else {
-                        copy(cacheEntry, writer, ranges.iterator(),
-                             contentType);
+                        // we should not get here
+                        throw new IllegalStateException();
                     }
                 }
 
@@ -1062,10 +1102,9 @@ public class DefaultServlet
      * @param response The servlet response we are creating
      * @return Vector of ranges
      */
-    protected ArrayList parseRange(HttpServletRequest request,
-                                HttpServletResponse response,
-                                ResourceAttributes resourceAttributes)
-        throws IOException {
+    protected ArrayList<Range> parseRange(HttpServletRequest request,
+            HttpServletResponse response,
+            ResourceAttributes resourceAttributes) throws IOException {
 
         // Checking If-Range
         String headerValue = request.getHeader("If-Range");
@@ -1076,7 +1115,7 @@ public class DefaultServlet
             try {
                 headerValueTime = request.getDateHeader("If-Range");
             } catch (IllegalArgumentException e) {
-                ;
+                // Ignore
             }
 
             String eTag = resourceAttributes.getETag();
@@ -1206,10 +1245,9 @@ public class DefaultServlet
 
         if (xsltInputStream==null) {
             return renderHtml(contextPath, cacheEntry);
-        } else {
-            return renderXml(contextPath, cacheEntry, xsltInputStream);
         }
-
+        return renderXml(contextPath, cacheEntry, xsltInputStream);
+        
     }
 
     /**
@@ -1224,7 +1262,7 @@ public class DefaultServlet
                                     InputStream xsltInputStream)
         throws IOException, ServletException {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
 
         sb.append("<?xml version=\"1.0\"?>");
         sb.append("<listing ");
@@ -1242,14 +1280,15 @@ public class DefaultServlet
         try {
 
             // Render the directory entries within this directory
-            NamingEnumeration enumeration = resources.list(cacheEntry.name);
+            NamingEnumeration<NameClassPair> enumeration =
+                resources.list(cacheEntry.name);
             
             // rewriteUrl(contextPath) is expensive. cache result for later reuse
             String rewrittenContextPath =  rewriteUrl(contextPath);
 
             while (enumeration.hasMoreElements()) {
 
-                NameClassPair ncPair = (NameClassPair) enumeration.nextElement();
+                NameClassPair ncPair = enumeration.nextElement();
                 String resourceName = ncPair.getName();
                 String trimmed = resourceName/*.substring(trim)*/;
                 if (trimmed.equalsIgnoreCase("WEB-INF") ||
@@ -1352,7 +1391,7 @@ public class DefaultServlet
         OutputStreamWriter osWriter = new OutputStreamWriter(stream, "UTF8");
         PrintWriter writer = new PrintWriter(osWriter);
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         
         // rewriteUrl(contextPath) is expensive. cache result for later reuse
         String rewrittenContextPath =  rewriteUrl(contextPath);
@@ -1416,11 +1455,12 @@ public class DefaultServlet
         try {
 
             // Render the directory entries within this directory
-            NamingEnumeration enumeration = resources.list(cacheEntry.name);
+            NamingEnumeration<NameClassPair> enumeration =
+                resources.list(cacheEntry.name);
             boolean shade = false;
             while (enumeration.hasMoreElements()) {
 
-                NameClassPair ncPair = (NameClassPair) enumeration.nextElement();
+                NameClassPair ncPair = enumeration.nextElement();
                 String resourceName = ncPair.getName();
                 String trimmed = resourceName/*.substring(trim)*/;
                 if (trimmed.equalsIgnoreCase("WEB-INF") ||
@@ -1515,7 +1555,7 @@ public class DefaultServlet
      * Get the readme file as a string.
      */
     protected String getReadme(DirContext directory)
-        throws IOException, ServletException {
+        throws IOException {
 
         if (readmeFile != null) {
             try {
@@ -1543,7 +1583,7 @@ public class DefaultServlet
      * Return the xsl template inputstream (if possible)
      */
     protected InputStream findXsltInputStream(DirContext directory)
-        throws IOException, ServletException {
+        throws IOException {
 
         if (localXsltFile != null) {
             try {
@@ -1613,16 +1653,15 @@ public class DefaultServlet
             && (response.getClass().getName().equals("org.apache.catalina.connector.ResponseFacade"))) {
             request.setAttribute("org.apache.tomcat.sendfile.filename", entry.attributes.getCanonicalPath());
             if (range == null) {
-                request.setAttribute("org.apache.tomcat.sendfile.start", new Long(0L));
-                request.setAttribute("org.apache.tomcat.sendfile.end", new Long(length));
+                request.setAttribute("org.apache.tomcat.sendfile.start", Long.valueOf(0L));
+                request.setAttribute("org.apache.tomcat.sendfile.end", Long.valueOf(length));
             } else {
-                request.setAttribute("org.apache.tomcat.sendfile.start", new Long(range.start));
-                request.setAttribute("org.apache.tomcat.sendfile.end", new Long(range.end + 1));
+                request.setAttribute("org.apache.tomcat.sendfile.start", Long.valueOf(range.start));
+                request.setAttribute("org.apache.tomcat.sendfile.end", Long.valueOf(range.end + 1));
             }
             return true;
-        } else {
-            return false;
         }
+        return false;
     }
     
     
@@ -1631,7 +1670,7 @@ public class DefaultServlet
      *
      * @param request The servlet request we are processing
      * @param response The servlet response we are creating
-     * @param resourceInfo File object
+     * @param resourceAttributes File object
      * @return boolean true if the resource meets the specified condition,
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
@@ -1676,15 +1715,14 @@ public class DefaultServlet
      *
      * @param request The servlet request we are processing
      * @param response The servlet response we are creating
-     * @param resourceInfo File object
+     * @param resourceAttributes File object
      * @return boolean true if the resource meets the specified condition,
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
      */
     protected boolean checkIfModifiedSince(HttpServletRequest request,
-                                         HttpServletResponse response,
-                                         ResourceAttributes resourceAttributes)
-        throws IOException {
+            HttpServletResponse response,
+            ResourceAttributes resourceAttributes) {
         try {
             long headerValue = request.getDateHeader("If-Modified-Since");
             long lastModified = resourceAttributes.getLastModified();
@@ -1715,7 +1753,7 @@ public class DefaultServlet
      *
      * @param request The servlet request we are processing
      * @param response The servlet response we are creating
-     * @param resourceInfo File object
+     * @param resourceAttributes File object
      * @return boolean true if the resource meets the specified condition,
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
@@ -1758,11 +1796,9 @@ public class DefaultServlet
                     response.setHeader("ETag", eTag);
 
                     return false;
-                } else {
-                    response.sendError
-                        (HttpServletResponse.SC_PRECONDITION_FAILED);
-                    return false;
                 }
+                response.sendError(HttpServletResponse.SC_PRECONDITION_FAILED);
+                return false;
             }
         }
         return true;
@@ -1775,7 +1811,7 @@ public class DefaultServlet
      *
      * @param request The servlet request we are processing
      * @param response The servlet response we are creating
-     * @param resourceInfo File object
+     * @param resourceAttributes File object
      * @return boolean true if the resource meets the specified condition,
      * and false if the condition is not satisfied, in which case request
      * processing is stopped
@@ -1808,7 +1844,8 @@ public class DefaultServlet
      * output stream, and ensure that both streams are closed before returning
      * (even in the face of an exception).
      *
-     * @param resourceInfo The resource information
+     * @param cacheEntry The cache entry for the source resource
+     * @param is The input stream to read the source resource from   
      * @param ostream The output stream to write to
      *
      * @exception IOException if an input/output error occurs
@@ -1854,7 +1891,8 @@ public class DefaultServlet
      * output stream, and ensure that both streams are closed before returning
      * (even in the face of an exception).
      *
-     * @param resourceInfo The resource info
+     * @param cacheEntry The cache entry for the source resource
+     * @param is The input stream to read the source resource from   
      * @param writer The writer to write to
      *
      * @exception IOException if an input/output error occurs
@@ -1897,7 +1935,7 @@ public class DefaultServlet
      * output stream, and ensure that both streams are closed before returning
      * (even in the face of an exception).
      *
-     * @param resourceInfo The ResourceInfo object
+     * @param cacheEntry The cache entry for the source resource
      * @param ostream The output stream to write to
      * @param range Range the client wanted to retrieve
      * @exception IOException if an input/output error occurs
@@ -1928,52 +1966,14 @@ public class DefaultServlet
      * output stream, and ensure that both streams are closed before returning
      * (even in the face of an exception).
      *
-     * @param resourceInfo The ResourceInfo object
-     * @param writer The writer to write to
-     * @param range Range the client wanted to retrieve
-     * @exception IOException if an input/output error occurs
-     */
-    protected void copy(CacheEntry cacheEntry, PrintWriter writer,
-                      Range range)
-        throws IOException {
-
-        IOException exception = null;
-
-        InputStream resourceInputStream = cacheEntry.resource.streamContent();
-
-        Reader reader;
-        if (fileEncoding == null) {
-            reader = new InputStreamReader(resourceInputStream);
-        } else {
-            reader = new InputStreamReader(resourceInputStream,
-                                           fileEncoding);
-        }
-
-        exception = copyRange(reader, writer, range.start, range.end);
-
-        // Clean up the input stream
-        reader.close();
-
-        // Rethrow any exception that has occurred
-        if (exception != null)
-            throw exception;
-
-    }
-
-
-    /**
-     * Copy the contents of the specified input stream to the specified
-     * output stream, and ensure that both streams are closed before returning
-     * (even in the face of an exception).
-     *
-     * @param resourceInfo The ResourceInfo object
+     * @param cacheEntry The cache entry for the source resource
      * @param ostream The output stream to write to
      * @param ranges Enumeration of the ranges the client wanted to retrieve
      * @param contentType Content type of the resource
      * @exception IOException if an input/output error occurs
      */
     protected void copy(CacheEntry cacheEntry, ServletOutputStream ostream,
-                      Iterator ranges, String contentType)
+                      Iterator<Range> ranges, String contentType)
         throws IOException {
 
         IOException exception = null;
@@ -1984,7 +1984,7 @@ public class DefaultServlet
             InputStream istream =
                 new BufferedInputStream(resourceInputStream, input);
 
-            Range currentRange = (Range) ranges.next();
+            Range currentRange = ranges.next();
 
             // Writing MIME header.
             ostream.println();
@@ -2014,65 +2014,6 @@ public class DefaultServlet
     }
 
 
-    /**
-     * Copy the contents of the specified input stream to the specified
-     * output stream, and ensure that both streams are closed before returning
-     * (even in the face of an exception).
-     *
-     * @param resourceInfo The ResourceInfo object
-     * @param writer The writer to write to
-     * @param ranges Enumeration of the ranges the client wanted to retrieve
-     * @param contentType Content type of the resource
-     * @exception IOException if an input/output error occurs
-     */
-    protected void copy(CacheEntry cacheEntry, PrintWriter writer,
-                      Iterator ranges, String contentType)
-        throws IOException {
-
-        IOException exception = null;
-
-        while ( (exception == null) && (ranges.hasNext()) ) {
-
-            InputStream resourceInputStream = cacheEntry.resource.streamContent();
-            
-            Reader reader;
-            if (fileEncoding == null) {
-                reader = new InputStreamReader(resourceInputStream);
-            } else {
-                reader = new InputStreamReader(resourceInputStream,
-                                               fileEncoding);
-            }
-
-            Range currentRange = (Range) ranges.next();
-
-            // Writing MIME header.
-            writer.println();
-            writer.println("--" + mimeSeparation);
-            if (contentType != null)
-                writer.println("Content-Type: " + contentType);
-            writer.println("Content-Range: bytes " + currentRange.start
-                           + "-" + currentRange.end + "/"
-                           + currentRange.length);
-            writer.println();
-
-            // Printing content
-            exception = copyRange(reader, writer, currentRange.start,
-                                  currentRange.end);
-
-            reader.close();
-
-        }
-
-        writer.println();
-        writer.print("--" + mimeSeparation + "--");
-
-        // Rethrow any exception that has occurred
-        if (exception != null)
-            throw exception;
-
-    }
-
-
     /**
      * Copy the contents of the specified input stream to the specified
      * output stream, and ensure that both streams are closed before returning
@@ -2156,12 +2097,17 @@ public class DefaultServlet
         if (debug > 10)
             log("Serving bytes:" + start + "-" + end);
 
+        long skipped = 0;
         try {
-            istream.skip(start);
+            skipped = istream.skip(start);
         } catch (IOException e) {
             return e;
         }
-
+        if (skipped < start) {
+            return new IOException(sm.getString("defaultservlet.skipfail",
+                    Long.valueOf(skipped), Long.valueOf(start)));
+        }
+        
         IOException exception = null;
         long bytesToRead = end - start + 1;
 
@@ -2190,59 +2136,10 @@ public class DefaultServlet
     }
 
 
-    /**
-     * Copy the contents of the specified input stream to the specified
-     * output stream, and ensure that both streams are closed before returning
-     * (even in the face of an exception).
-     *
-     * @param reader The reader to read from
-     * @param writer The writer to write to
-     * @param start Start of the range which will be copied
-     * @param end End of the range which will be copied
-     * @return Exception which occurred during processing
-     */
-    protected IOException copyRange(Reader reader, PrintWriter writer,
-                                  long start, long end) {
-
-        try {
-            reader.skip(start);
-        } catch (IOException e) {
-            return e;
-        }
-
-        IOException exception = null;
-        long bytesToRead = end - start + 1;
-
-        char buffer[] = new char[input];
-        int len = buffer.length;
-        while ( (bytesToRead > 0) && (len >= buffer.length)) {
-            try {
-                len = reader.read(buffer);
-                if (bytesToRead >= len) {
-                    writer.write(buffer, 0, len);
-                    bytesToRead -= len;
-                } else {
-                    writer.write(buffer, 0, (int) bytesToRead);
-                    bytesToRead = 0;
-                }
-            } catch (IOException e) {
-                exception = e;
-                len = -1;
-            }
-            if (len < buffer.length)
-                break;
-        }
-
-        return exception;
-
-    }
-
-
-
     // ------------------------------------------------------ Range Inner Class
 
 
-    protected class Range {
+    protected static class Range {
 
         public long start;
         public long end;
@@ -2254,17 +2151,7 @@ public class DefaultServlet
         public boolean validate() {
             if (end >= length)
                 end = length - 1;
-            return ( (start >= 0) && (end >= 0) && (start <= end)
-                     && (length > 0) );
+            return (start >= 0) && (end >= 0) && (start <= end) && (length > 0);
         }
-
-        public void recycle() {
-            start = 0;
-            end = 0;
-            length = 0;
-        }
-
     }
-
-
 }
