@@ -43,7 +43,6 @@ import javax.servlet.http.HttpSessionActivationListener;
 import javax.servlet.http.HttpSessionAttributeListener;
 import javax.servlet.http.HttpSessionBindingEvent;
 import javax.servlet.http.HttpSessionBindingListener;
-import javax.servlet.http.HttpSessionContext;
 import javax.servlet.http.HttpSessionEvent;
 import javax.servlet.http.HttpSessionListener;
 
@@ -53,13 +52,12 @@ import org.apache.catalina.Manager;
 import org.apache.catalina.Session;
 import org.apache.catalina.SessionEvent;
 import org.apache.catalina.SessionListener;
-import org.apache.catalina.util.Enumerator;
-import org.apache.catalina.util.StringManager;
-
 import org.apache.catalina.core.StandardContext;
 import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.security.SecurityUtil;
-import org.apache.catalina.session.ManagerBase.SessionTiming;
+import org.apache.catalina.util.Enumerator;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Standard implementation of the <b>Session</b> interface.  This object is
@@ -79,17 +77,41 @@ import org.apache.catalina.session.ManagerBase.SessionTiming;
  * @author Craig R. McClanahan
  * @author Sean Legassick
  * @author <a href="mailto:jon@latchkey.com">Jon S. Stevens</a>
- * @version $Id: StandardSession.java 1061433 2011-01-20 18:10:16Z markt $
+ * @version $Id: StandardSession.java 1094069 2011-04-16 22:25:28Z markt $
  */
 
-public class StandardSession
-    implements HttpSession, Session, Serializable {
+public class StandardSession implements HttpSession, Session, Serializable {
 
+    private static final long serialVersionUID = 1L;
 
-    protected static final boolean ACTIVITY_CHECK = 
-        Globals.STRICT_SERVLET_COMPLIANCE
-        || Boolean.valueOf(System.getProperty("org.apache.catalina.session.StandardSession.ACTIVITY_CHECK", "false")).booleanValue();
+    protected static final boolean STRICT_SERVLET_COMPLIANCE;
 
+    protected static final boolean ACTIVITY_CHECK;
+
+    protected static final boolean LAST_ACCESS_AT_START;
+
+    static {
+        STRICT_SERVLET_COMPLIANCE = Globals.STRICT_SERVLET_COMPLIANCE;
+        
+        String activityCheck = System.getProperty(
+                "org.apache.catalina.session.StandardSession.ACTIVITY_CHECK");
+        if (activityCheck == null) {
+            ACTIVITY_CHECK = STRICT_SERVLET_COMPLIANCE;
+        } else {
+            ACTIVITY_CHECK =
+                Boolean.valueOf(activityCheck).booleanValue();
+        }
+
+        String lastAccessAtStart = System.getProperty(
+                "org.apache.catalina.session.StandardSession.LAST_ACCESS_AT_START");
+        if (lastAccessAtStart == null) {
+            LAST_ACCESS_AT_START = STRICT_SERVLET_COMPLIANCE;
+        } else {
+            LAST_ACCESS_AT_START =
+                Boolean.valueOf(lastAccessAtStart).booleanValue();
+        }
+    }
+    
 
     // ----------------------------------------------------------- Constructors
 
@@ -132,7 +154,7 @@ public class StandardSession
     /**
      * The collection of user data attributes associated with this Session.
      */
-    protected Map attributes = new ConcurrentHashMap();
+    protected Map<String, Object> attributes = new ConcurrentHashMap<String, Object>();
 
 
     /**
@@ -154,7 +176,8 @@ public class StandardSession
      * Set of attribute names which are not allowed to be persisted.
      */
     protected static final String[] excludedAttributes = {
-        Globals.SUBJECT_ATTR
+        Globals.SUBJECT_ATTR,
+        Globals.GSS_CREDENTIAL_ATTR
     };
 
 
@@ -194,7 +217,8 @@ public class StandardSession
     /**
      * The session event listeners for this Session.
      */
-    protected transient ArrayList listeners = new ArrayList();
+    protected transient ArrayList<SessionListener> listeners =
+        new ArrayList<SessionListener>();
 
 
     /**
@@ -228,7 +252,7 @@ public class StandardSession
      * and event listeners.  <b>IMPLEMENTATION NOTE:</b> This object is
      * <em>not</em> saved and restored across session serializations!
      */
-    protected transient Map notes = new Hashtable();
+    protected transient Map<String, Object> notes = new Hashtable<String, Object>();
 
 
     /**
@@ -242,14 +266,16 @@ public class StandardSession
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
     /**
      * The HTTP session context associated with this session.
      */
-    protected static HttpSessionContext sessionContext = null;
+    @Deprecated
+    protected static volatile
+            javax.servlet.http.HttpSessionContext sessionContext = null;
 
 
     /**
@@ -279,6 +305,7 @@ public class StandardSession
      * Return the authentication type used to authenticate our cached
      * Principal, if any.
      */
+    @Override
     public String getAuthType() {
 
         return (this.authType);
@@ -292,6 +319,7 @@ public class StandardSession
      *
      * @param authType The new cached authentication type
      */
+    @Override
     public void setAuthType(String authType) {
 
         String oldAuthType = this.authType;
@@ -307,6 +335,7 @@ public class StandardSession
      *
      * @param time The new creation time
      */
+    @Override
     public void setCreationTime(long time) {
 
         this.creationTime = time;
@@ -319,6 +348,7 @@ public class StandardSession
     /**
      * Return the session identifier for this session.
      */
+    @Override
     public String getId() {
 
         return (this.id);
@@ -329,6 +359,7 @@ public class StandardSession
     /**
      * Return the session identifier for this session.
      */
+    @Override
     public String getIdInternal() {
 
         return (this.id);
@@ -341,7 +372,17 @@ public class StandardSession
      *
      * @param id The new session identifier
      */
+    @Override
     public void setId(String id) {
+        setId(id, true);
+    }
+
+
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setId(String id, boolean notify) {
 
         if ((this.id != null) && (manager != null))
             manager.remove(this);
@@ -350,7 +391,10 @@ public class StandardSession
 
         if (manager != null)
             manager.add(this);
-        tellNew();
+        
+        if (notify) {
+            tellNew();
+        }
     }
 
 
@@ -383,12 +427,13 @@ public class StandardSession
                                        "afterSessionCreated",
                                        listener);
                 } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
                     try {
                         fireContainerEvent(context,
                                            "afterSessionCreated",
                                            listener);
                     } catch (Exception e) {
-                        ;
+                        // Ignore
                     }
                     manager.getContainer().getLogger().error
                         (sm.getString("standardSession.sessionEvent"), t);
@@ -404,6 +449,7 @@ public class StandardSession
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -416,7 +462,36 @@ public class StandardSession
      * session, as the number of milliseconds since midnight, January 1, 1970
      * GMT.  Actions that your application takes, such as getting or setting
      * a value associated with the session, do not affect the access time.
+     * This one gets updated whenever a request starts.
      */
+    @Override
+    public long getThisAccessedTime() {
+
+        if (!isValidInternal()) {
+            throw new IllegalStateException
+                (sm.getString("standardSession.getThisAccessedTime.ise"));
+        }
+
+        return (this.thisAccessedTime);
+    }
+
+    /**
+     * Return the last client access time without invalidation check
+     * @see #getThisAccessedTime()
+     */
+    @Override
+    public long getThisAccessedTimeInternal() {
+        return (this.thisAccessedTime);
+    }
+
+    /**
+     * Return the last time the client sent a request associated with this
+     * session, as the number of milliseconds since midnight, January 1, 1970
+     * GMT.  Actions that your application takes, such as getting or setting
+     * a value associated with the session, do not affect the access time.
+     * This one gets updated whenever a request finishes.
+     */
+    @Override
     public long getLastAccessedTime() {
 
         if (!isValidInternal()) {
@@ -429,8 +504,9 @@ public class StandardSession
 
     /**
      * Return the last client access time without invalidation check
-     * @see #getLastAccessedTime().
+     * @see #getLastAccessedTime()
      */
+    @Override
     public long getLastAccessedTimeInternal() {
         return (this.lastAccessedTime);
     }
@@ -438,6 +514,7 @@ public class StandardSession
     /**
      * Return the Manager within which this Session is valid.
      */
+    @Override
     public Manager getManager() {
 
         return (this.manager);
@@ -450,6 +527,7 @@ public class StandardSession
      *
      * @param manager The new Manager
      */
+    @Override
     public void setManager(Manager manager) {
 
         this.manager = manager;
@@ -462,6 +540,7 @@ public class StandardSession
      * before the servlet container will invalidate the session.  A negative
      * time indicates that the session should never time out.
      */
+    @Override
     public int getMaxInactiveInterval() {
 
         return (this.maxInactiveInterval);
@@ -476,6 +555,7 @@ public class StandardSession
      *
      * @param interval The new maximum interval
      */
+    @Override
     public void setMaxInactiveInterval(int interval) {
 
         this.maxInactiveInterval = interval;
@@ -491,6 +571,7 @@ public class StandardSession
      *
      * @param isNew The new value for the <code>isNew</code> flag
      */
+    @Override
     public void setNew(boolean isNew) {
 
         this.isNew = isNew;
@@ -505,6 +586,7 @@ public class StandardSession
      * <code>Realm.authenticate()</code> calls on every request.  If there
      * is no current associated Principal, return <code>null</code>.
      */
+    @Override
     public Principal getPrincipal() {
 
         return (this.principal);
@@ -520,6 +602,7 @@ public class StandardSession
      *
      * @param principal The new Principal, or <code>null</code> if none
      */
+    @Override
     public void setPrincipal(Principal principal) {
 
         Principal oldPrincipal = this.principal;
@@ -533,13 +616,16 @@ public class StandardSession
      * Return the <code>HttpSession</code> for which this object
      * is the facade.
      */
+    @Override
     public HttpSession getSession() {
 
         if (facade == null){
             if (SecurityUtil.isPackageProtectionEnabled()){
                 final StandardSession fsession = this;
-                facade = (StandardSessionFacade)AccessController.doPrivileged(new PrivilegedAction(){
-                    public Object run(){
+                facade = AccessController.doPrivileged(
+                        new PrivilegedAction<StandardSessionFacade>(){
+                    @Override
+                    public StandardSessionFacade run(){
                         return new StandardSessionFacade(fsession);
                     }
                 });
@@ -555,6 +641,7 @@ public class StandardSession
     /**
      * Return the <code>isValid</code> flag for this session.
      */
+    @Override
     public boolean isValid() {
 
         if (this.expiring) {
@@ -571,7 +658,12 @@ public class StandardSession
 
         if (maxInactiveInterval >= 0) { 
             long timeNow = System.currentTimeMillis();
-            int timeIdle = (int) ((timeNow - thisAccessedTime) / 1000L);
+            int timeIdle;
+            if (LAST_ACCESS_AT_START) {
+                timeIdle = (int) ((timeNow - lastAccessedTime) / 1000L);
+            } else {
+                timeIdle = (int) ((timeNow - thisAccessedTime) / 1000L);
+            }
             if (timeIdle >= maxInactiveInterval) {
                 expire(true);
             }
@@ -586,6 +678,7 @@ public class StandardSession
      *
      * @param isValid The new value for the <code>isValid</code> flag
      */
+    @Override
     public void setValid(boolean isValid) {
         this.isValid = isValid;
     }
@@ -599,11 +692,11 @@ public class StandardSession
      * should be called by the context when a request comes in for a particular
      * session, even if the application does not reference it.
      */
+    @Override
     public void access() {
 
-        this.lastAccessedTime = this.thisAccessedTime;
         this.thisAccessedTime = System.currentTimeMillis();
-        
+
         if (ACTIVITY_CHECK) {
             accessCount.incrementAndGet();
         }
@@ -614,10 +707,23 @@ public class StandardSession
     /**
      * End the access.
      */
+    @Override
     public void endAccess() {
 
         isNew = false;
 
+        /**
+         * The servlet spec mandates to ignore request handling time
+         * in lastAccessedTime.
+         */
+        if (LAST_ACCESS_AT_START) {
+            this.lastAccessedTime = this.thisAccessedTime;
+            this.thisAccessedTime = System.currentTimeMillis();
+        } else {
+            this.thisAccessedTime = System.currentTimeMillis();
+            this.lastAccessedTime = this.thisAccessedTime;
+        }
+
         if (ACTIVITY_CHECK) {
             accessCount.decrementAndGet();
         }
@@ -628,6 +734,7 @@ public class StandardSession
     /**
      * Add a session event listener to this component.
      */
+    @Override
     public void addSessionListener(SessionListener listener) {
 
         listeners.add(listener);
@@ -639,6 +746,7 @@ public class StandardSession
      * Perform the internal processing required to invalidate this session,
      * without triggering an exception if the session has already expired.
      */
+    @Override
     public void expire() {
 
         expire(true);
@@ -711,6 +819,7 @@ public class StandardSession
                                                "afterSessionDestroyed",
                                                listener);
                         } catch (Throwable t) {
+                            ExceptionUtils.handleThrowable(t);
                             try {
                                 fireContainerEvent(context,
                                                    "afterSessionDestroyed",
@@ -740,40 +849,8 @@ public class StandardSession
             }
             setValid(false);
 
-            /*
-             * Compute how long this session has been alive, and update
-             * session manager's related properties accordingly
-             */
-            long timeNow = System.currentTimeMillis();
-            int timeAlive = (int) ((timeNow - creationTime)/1000);
-            synchronized (manager) {
-                if (timeAlive > manager.getSessionMaxAliveTime()) {
-                    manager.setSessionMaxAliveTime(timeAlive);
-                }
-                int numExpired = manager.getExpiredSessions();
-                if (numExpired < Integer.MAX_VALUE) {
-                    numExpired++;
-                    manager.setExpiredSessions(numExpired);
-                }
-
-                int average = manager.getSessionAverageAliveTime();
-                // Using long, as otherwise (average * numExpired) might overflow 
-                average = (int) (((((long) average) * (numExpired - 1)) + timeAlive)
-                        / numExpired);
-                manager.setSessionAverageAliveTime(average);
-            }
-
-            if (manager instanceof ManagerBase) {
-                ManagerBase mb = (ManagerBase) manager;
-                SessionTiming timing = new SessionTiming(timeNow, timeAlive);
-                synchronized (mb.sessionExpirationTiming) {
-                    mb.sessionExpirationTiming.add(timing);
-                    mb.sessionExpirationTiming.poll();
-                }
-            }
-
             // Remove this session from our manager's active sessions
-            manager.remove(this);
+            manager.remove(this, true);
 
             // Notify interested session event listeners
             if (notify) {
@@ -826,6 +903,7 @@ public class StandardSession
                     ((HttpSessionActivationListener)attribute)
                         .sessionWillPassivate(event);
                 } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
                     manager.getContainer().getLogger().error
                         (sm.getString("standardSession.attributeEvent"), t);
                 }
@@ -861,6 +939,7 @@ public class StandardSession
                     ((HttpSessionActivationListener)attribute)
                         .sessionDidActivate(event);
                 } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
                     manager.getContainer().getLogger().error
                         (sm.getString("standardSession.attributeEvent"), t);
                 }
@@ -876,6 +955,7 @@ public class StandardSession
      *
      * @param name Name of the note to be returned
      */
+    @Override
     public Object getNote(String name) {
 
         return (notes.get(name));
@@ -887,7 +967,8 @@ public class StandardSession
      * Return an Iterator containing the String names of all notes bindings
      * that exist for this session.
      */
-    public Iterator getNoteNames() {
+    @Override
+    public Iterator<String> getNoteNames() {
 
         return (notes.keySet().iterator());
 
@@ -898,6 +979,7 @@ public class StandardSession
      * Release all object references, and initialize instance variables, in
      * preparation for reuse of this object.
      */
+    @Override
     public void recycle() {
 
         // Reset the instance variables associated with this Session
@@ -923,6 +1005,7 @@ public class StandardSession
      *
      * @param name Name of the note to be removed
      */
+    @Override
     public void removeNote(String name) {
 
         notes.remove(name);
@@ -933,6 +1016,7 @@ public class StandardSession
     /**
      * Remove a session event listener from this component.
      */
+    @Override
     public void removeSessionListener(SessionListener listener) {
 
         listeners.remove(listener);
@@ -947,6 +1031,7 @@ public class StandardSession
      * @param name Name to which the object should be bound
      * @param value Object to be bound to the specified name
      */
+    @Override
     public void setNote(String name, Object value) {
 
         notes.put(name, value);
@@ -957,9 +1042,10 @@ public class StandardSession
     /**
      * Return a string representation of this object.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer();
+        StringBuilder sb = new StringBuilder();
         sb.append("StandardSession[");
         sb.append(id);
         sb.append("]");
@@ -1016,6 +1102,7 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on an
      *  invalidated session
      */
+    @Override
     public long getCreationTime() {
 
         if (!isValidInternal())
@@ -1027,9 +1114,20 @@ public class StandardSession
     }
 
 
+    /**
+     * Return the time when this session was created, in milliseconds since
+     * midnight, January 1, 1970 GMT, bypassing the session validation checks.
+     */
+    @Override
+    public long getCreationTimeInternal() {
+        return this.creationTime;
+    }
+
+
     /**
      * Return the ServletContext to which this session belongs.
      */
+    @Override
     public ServletContext getServletContext() {
 
         if (manager == null)
@@ -1050,7 +1148,9 @@ public class StandardSession
      *  replacement.  It will be removed in a future version of the
      *  Java Servlet API.
      */
-    public HttpSessionContext getSessionContext() {
+    @Override
+    @Deprecated
+    public javax.servlet.http.HttpSessionContext getSessionContext() {
 
         if (sessionContext == null)
             sessionContext = new StandardSessionContext();
@@ -1071,6 +1171,7 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on an
      *  invalidated session
      */
+    @Override
     public Object getAttribute(String name) {
 
         if (!isValidInternal())
@@ -1091,13 +1192,14 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on an
      *  invalidated session
      */
-    public Enumeration getAttributeNames() {
+    @Override
+    public Enumeration<String> getAttributeNames() {
 
         if (!isValidInternal())
             throw new IllegalStateException
                 (sm.getString("standardSession.getAttributeNames.ise"));
 
-        return (new Enumerator(attributes.keySet(), true));
+        return (new Enumerator<String>(attributes.keySet(), true));
 
     }
 
@@ -1114,6 +1216,8 @@ public class StandardSession
      * @deprecated As of Version 2.2, this method is replaced by
      *  <code>getAttribute()</code>
      */
+    @Override
+    @Deprecated
     public Object getValue(String name) {
 
         return (getAttribute(name));
@@ -1131,6 +1235,8 @@ public class StandardSession
      * @deprecated As of Version 2.2, this method is replaced by
      *  <code>getAttributeNames()</code>
      */
+    @Override
+    @Deprecated
     public String[] getValueNames() {
 
         if (!isValidInternal())
@@ -1148,6 +1254,7 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on
      *  an invalidated session
      */
+    @Override
     public void invalidate() {
 
         if (!isValidInternal())
@@ -1170,6 +1277,7 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on an
      *  invalidated session
      */
+    @Override
     public boolean isNew() {
 
         if (!isValidInternal())
@@ -1181,7 +1289,6 @@ public class StandardSession
     }
 
 
-
     /**
      * Bind an object to this session, using the specified name.  If an object
      * of the same name is already bound to this session, the object is
@@ -1200,6 +1307,8 @@ public class StandardSession
      * @deprecated As of Version 2.2, this method is replaced by
      *  <code>setAttribute()</code>
      */
+    @Override
+    @Deprecated
     public void putValue(String name, Object value) {
 
         setAttribute(name, value);
@@ -1221,6 +1330,7 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on an
      *  invalidated session
      */
+    @Override
     public void removeAttribute(String name) {
 
         removeAttribute(name, true);
@@ -1273,6 +1383,8 @@ public class StandardSession
      * @deprecated As of Version 2.2, this method is replaced by
      *  <code>removeAttribute()</code>
      */
+    @Override
+    @Deprecated
     public void removeValue(String name) {
 
         removeAttribute(name);
@@ -1297,6 +1409,7 @@ public class StandardSession
      * @exception IllegalStateException if this method is called on an
      *  invalidated session
      */
+    @Override
     public void setAttribute(String name, Object value) {
         setAttribute(name,value,true);
     }
@@ -1333,8 +1446,8 @@ public class StandardSession
 
         // Validate our current state
         if (!isValidInternal())
-            throw new IllegalStateException
-                (sm.getString("standardSession.setAttribute.ise"));
+            throw new IllegalStateException(sm.getString(
+                    "standardSession.setAttribute.ise", getIdInternal()));
         if ((manager != null) && manager.getDistributable() &&
           !(value instanceof Serializable))
             throw new IllegalArgumentException
@@ -1367,6 +1480,7 @@ public class StandardSession
                 ((HttpSessionBindingListener) unbound).valueUnbound
                     (new HttpSessionBindingEvent(getSession(), name));
             } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
                 manager.getContainer().getLogger().error
                     (sm.getString("standardSession.bindingEvent"), t);
             }
@@ -1411,6 +1525,7 @@ public class StandardSession
                                        listener);
                 }
             } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
                 try {
                     if (unbound != null) {
                         fireContainerEvent(context,
@@ -1422,7 +1537,7 @@ public class StandardSession
                                            listener);
                     }
                 } catch (Exception e) {
-                    ;
+                    // Ignore
                 }
                 manager.getContainer().getLogger().error
                     (sm.getString("standardSession.attributeEvent"), t);
@@ -1476,13 +1591,13 @@ public class StandardSession
 
         // Deserialize the attribute count and attribute values
         if (attributes == null)
-            attributes = new Hashtable();
+            attributes = new ConcurrentHashMap<String, Object>();
         int n = ((Integer) stream.readObject()).intValue();
         boolean isValidSave = isValid;
         isValid = true;
         for (int i = 0; i < n; i++) {
             String name = (String) stream.readObject();
-            Object value = (Object) stream.readObject();
+            Object value = stream.readObject();
             if ((value instanceof String) && (value.equals(NOT_SERIALIZED)))
                 continue;
             if (manager.getContainer().getLogger().isDebugEnabled())
@@ -1493,11 +1608,11 @@ public class StandardSession
         isValid = isValidSave;
 
         if (listeners == null) {
-            listeners = new ArrayList();
+            listeners = new ArrayList<SessionListener>();
         }
 
         if (notes == null) {
-            notes = new Hashtable();
+            notes = new Hashtable<String, Object>();
         }
     }
 
@@ -1524,12 +1639,12 @@ public class StandardSession
     protected void writeObject(ObjectOutputStream stream) throws IOException {
 
         // Write the scalar instance variables (except Manager)
-        stream.writeObject(new Long(creationTime));
-        stream.writeObject(new Long(lastAccessedTime));
-        stream.writeObject(new Integer(maxInactiveInterval));
-        stream.writeObject(new Boolean(isNew));
-        stream.writeObject(new Boolean(isValid));
-        stream.writeObject(new Long(thisAccessedTime));
+        stream.writeObject(Long.valueOf(creationTime));
+        stream.writeObject(Long.valueOf(lastAccessedTime));
+        stream.writeObject(Integer.valueOf(maxInactiveInterval));
+        stream.writeObject(Boolean.valueOf(isNew));
+        stream.writeObject(Boolean.valueOf(isValid));
+        stream.writeObject(Long.valueOf(thisAccessedTime));
         stream.writeObject(id);
         if (manager.getContainer().getLogger().isDebugEnabled())
             manager.getContainer().getLogger().debug
@@ -1537,8 +1652,8 @@ public class StandardSession
 
         // Accumulate the names of serializable and non-serializable attributes
         String keys[] = keys();
-        ArrayList saveNames = new ArrayList();
-        ArrayList saveValues = new ArrayList();
+        ArrayList<String> saveNames = new ArrayList<String>();
+        ArrayList<Object> saveValues = new ArrayList<Object>();
         for (int i = 0; i < keys.length; i++) {
             Object value = attributes.get(keys[i]);
             if (value == null)
@@ -1554,9 +1669,9 @@ public class StandardSession
 
         // Serialize the attribute count and the Serializable attributes
         int n = saveNames.size();
-        stream.writeObject(new Integer(n));
+        stream.writeObject(Integer.valueOf(n));
         for (int i = 0; i < n; i++) {
-            stream.writeObject((String) saveNames.get(i));
+            stream.writeObject(saveNames.get(i));
             try {
                 stream.writeObject(saveValues.get(i));
                 if (manager.getContainer().getLogger().isDebugEnabled())
@@ -1614,7 +1729,6 @@ public class StandardSession
             ((StandardContext) context).fireContainerEvent(type, data);
         }
     }
-                                      
 
 
     /**
@@ -1631,11 +1745,11 @@ public class StandardSession
         SessionEvent event = new SessionEvent(this, type, data);
         SessionListener list[] = new SessionListener[0];
         synchronized (listeners) {
-            list = (SessionListener[]) listeners.toArray(list);
+            list = listeners.toArray(list);
         }
 
         for (int i = 0; i < list.length; i++){
-            ((SessionListener) list[i]).sessionEvent(event);
+            (list[i]).sessionEvent(event);
         }
 
     }
@@ -1648,7 +1762,7 @@ public class StandardSession
      */
     protected String[] keys() {
 
-        return ((String[]) attributes.keySet().toArray(EMPTY_ARRAY));
+        return attributes.keySet().toArray(EMPTY_ARRAY);
 
     }
 
@@ -1709,12 +1823,13 @@ public class StandardSession
                                    "afterSessionAttributeRemoved",
                                    listener);
             } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
                 try {
                     fireContainerEvent(context,
                                        "afterSessionAttributeRemoved",
                                        listener);
                 } catch (Exception e) {
-                    ;
+                    // Ignore
                 }
                 manager.getContainer().getLogger().error
                     (sm.getString("standardSession.attributeEvent"), t);
@@ -1733,6 +1848,7 @@ public class StandardSession
             this.cl = cl;
         }
 
+        @Override
         public Void run() {
             Thread.currentThread().setContextClassLoader(cl);
             return null;
@@ -1756,10 +1872,12 @@ public class StandardSession
  *  interface will be removed in a future version of this API.
  */
 
-final class StandardSessionContext implements HttpSessionContext {
+@Deprecated
+final class StandardSessionContext
+        implements javax.servlet.http.HttpSessionContext {
 
 
-    protected HashMap dummy = new HashMap();
+    protected HashMap<?,String> dummy = new HashMap<String,String>();
 
     /**
      * Return the session identifiers of all sessions defined
@@ -1769,9 +1887,11 @@ final class StandardSessionContext implements HttpSessionContext {
      *  This method must return an empty <code>Enumeration</code>
      *  and will be removed in a future version of the API.
      */
-    public Enumeration getIds() {
+    @Override
+    @Deprecated
+    public Enumeration<String> getIds() {
 
-        return (new Enumerator(dummy));
+        return (new Enumerator<String>(dummy));
 
     }
 
@@ -1786,6 +1906,8 @@ final class StandardSessionContext implements HttpSessionContext {
      *  This method must return null and will be removed in a
      *  future version of the API.
      */
+    @Override
+    @Deprecated
     public HttpSession getSession(String id) {
 
         return (null);
@@ -1793,5 +1915,4 @@ final class StandardSessionContext implements HttpSessionContext {
     }
 
 
-
 }
