@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -24,7 +24,6 @@ import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.net.InetAddress;
 import java.net.Socket;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -32,14 +31,23 @@ import java.util.List;
 import java.util.logging.LogManager;
 
 import org.apache.catalina.Container;
-import org.apache.catalina.Lifecycle;
+import org.apache.catalina.Globals;
 import org.apache.catalina.LifecycleException;
+import org.apache.catalina.LifecycleState;
+import org.apache.catalina.Server;
 import org.apache.catalina.core.StandardServer;
+import org.apache.catalina.ha.ClusterRuleSet;
+import org.apache.catalina.security.SecurityConfig;
 import org.apache.juli.ClassLoaderLogManager;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.digester.Digester;
 import org.apache.tomcat.util.digester.Rule;
+import org.apache.tomcat.util.log.SystemLogHandler;
+import org.apache.tomcat.util.res.StringManager;
 import org.xml.sax.Attributes;
 import org.xml.sax.InputSource;
+import org.xml.sax.SAXParseException;
 
 
 /**
@@ -52,6 +60,7 @@ import org.xml.sax.InputSource;
  *     "catalina.base" system property.   [conf/server.xml]
  * <li><b>-help</b>      - Display usage information.
  * <li><b>-nonaming</b>  - Disable naming support.
+ * <li><b>configtest</b> - Try to test the config
  * <li><b>start</b>      - Start an instance of Catalina.
  * <li><b>stop</b>       - Stop the currently running instance of Catalina.
  * </u>
@@ -60,14 +69,25 @@ import org.xml.sax.InputSource;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: Catalina.java 946849 2010-05-21 01:38:20Z kkolinko $
+ * @version $Id: Catalina.java 1144690 2011-07-09 16:05:42Z markt $
  */
 
-public class Catalina extends Embedded {
+public class Catalina {
+
+
+    /**
+     * The string manager for this package.
+     */
+    protected static final StringManager sm =
+        StringManager.getManager(Constants.Package);
 
 
     // ----------------------------------------------------- Instance Variables
 
+    /**
+     * Use await.
+     */
+    protected boolean await = false;
 
     /**
      * Pathname to the server configuration file.
@@ -82,6 +102,12 @@ public class Catalina extends Embedded {
         Catalina.class.getClassLoader();
 
 
+    /**
+     * The server component we are starting or stopping.
+     */
+    protected Server server = null;
+
+
     /**
      * Are we starting a new server?
      */
@@ -106,6 +132,19 @@ public class Catalina extends Embedded {
     protected Thread shutdownHook = null;
 
 
+    /**
+     * Is naming enabled ?
+     */
+    protected boolean useNaming = true;
+
+
+    // ----------------------------------------------------------- Constructors
+
+    public Catalina() {
+        setSecurityProtection();
+    }
+
+
     // ------------------------------------------------------------- Properties
 
 
@@ -140,49 +179,48 @@ public class Catalina extends Embedded {
      * @param parentClassLoader The shared extensions class loader.
      */
     public void setParentClassLoader(ClassLoader parentClassLoader) {
-
         this.parentClassLoader = parentClassLoader;
+    }
 
+    public ClassLoader getParentClassLoader() {
+        return parentClassLoader;
+    }
+
+    public void setServer(Server server) {
+        this.server = server;
     }
 
 
-    // ----------------------------------------------------------- Main Program
-
-    /**
-     * The application main program.
-     *
-     * @param args Command line arguments
-     */
-    public static void main(String args[]) {
-        (new Catalina()).process(args);
+    public Server getServer() {
+        return server;
     }
 
 
     /**
-     * The instance main program.
-     *
-     * @param args Command line arguments
+     * Return true if naming is enabled.
      */
-    public void process(String args[]) {
-
-        setAwait(true);
-        setCatalinaHome();
-        setCatalinaBase();
-        try {
-            if (arguments(args)) {
-                if (starting) {
-                    load(args);
-                    start();
-                } else if (stopping) {
-                    stopServer();
-                }
-            }
-        } catch (Exception e) {
-            e.printStackTrace(System.out);
-        }
+    public boolean isUseNaming() {
+        return (this.useNaming);
     }
 
 
+    /**
+     * Enables or disables naming support.
+     *
+     * @param useNaming The new use naming value
+     */
+    public void setUseNaming(boolean useNaming) {
+        this.useNaming = useNaming;
+    }
+
+    public void setAwait(boolean b) {
+        await = b;
+    }
+
+    public boolean isAwait() {
+        return await;
+    }
+
     // ------------------------------------------------------ Protected Methods
 
 
@@ -216,6 +254,9 @@ public class Catalina extends Embedded {
             } else if (args[i].equals("start")) {
                 starting = true;
                 stopping = false;
+            } else if (args[i].equals("configtest")) {
+                starting = true;
+                stopping = false;
             } else if (args[i].equals("stop")) {
                 starting = false;
                 stopping = true;
@@ -237,7 +278,7 @@ public class Catalina extends Embedded {
 
         File file = new File(configFile);
         if (!file.isAbsolute())
-            file = new File(System.getProperty("catalina.base"), configFile);
+            file = new File(System.getProperty(Globals.CATALINA_BASE_PROP), configFile);
         return (file);
 
     }
@@ -252,7 +293,8 @@ public class Catalina extends Embedded {
         Digester digester = new Digester();
         digester.setValidating(false);
         digester.setRulesValidation(true);
-        HashMap<Class, List<String>> fakeAttributes = new HashMap<Class, List<String>>();
+        HashMap<Class<?>, List<String>> fakeAttributes =
+            new HashMap<Class<?>, List<String>>();
         ArrayList<String> attrs = new ArrayList<String>();
         attrs.add("className");
         fakeAttributes.put(Object.class, attrs);
@@ -309,16 +351,14 @@ public class Catalina extends Embedded {
                             "addExecutor",
                             "org.apache.catalina.Executor");
 
-        
+
         digester.addRule("Server/Service/Connector",
                          new ConnectorCreateRule());
-        digester.addRule("Server/Service/Connector", 
+        digester.addRule("Server/Service/Connector",
                          new SetAllPropertiesRule(new String[]{"executor"}));
         digester.addSetNext("Server/Service/Connector",
                             "addConnector",
                             "org.apache.catalina.connector.Connector");
-        
-        
 
 
         digester.addObjectCreate("Server/Service/Connector/Listener",
@@ -334,13 +374,13 @@ public class Catalina extends Embedded {
         digester.addRuleSet(new EngineRuleSet("Server/Service/"));
         digester.addRuleSet(new HostRuleSet("Server/Service/Engine/"));
         digester.addRuleSet(new ContextRuleSet("Server/Service/Engine/Host/"));
-        digester.addRuleSet(ClusterRuleSetFactory.getClusterRuleSet("Server/Service/Engine/Host/Cluster/"));
+        digester.addRuleSet(new ClusterRuleSet("Server/Service/Engine/Host/Cluster/"));
         digester.addRuleSet(new NamingRuleSet("Server/Service/Engine/Host/Context/"));
 
         // When the 'engine' is found, set the parentClassLoader.
         digester.addRule("Server/Service/Engine",
                          new SetParentClassLoaderRule(parentClassLoader));
-        digester.addRuleSet(ClusterRuleSetFactory.getClusterRuleSet("Server/Service/Engine/Cluster/"));
+        digester.addRuleSet(new ClusterRuleSet("Server/Service/Engine/Cluster/"));
 
         long t2=System.currentTimeMillis();
         if (log.isDebugEnabled())
@@ -382,7 +422,8 @@ public class Catalina extends Embedded {
             arguments(arguments);
         }
 
-        if( getServer() == null ) {
+        Server s = getServer();
+        if( s == null ) {
             // Create and execute our Digester
             Digester digester = createStopDigester();
             digester.setClassLoader(Thread.currentThread().getContextClassLoader());
@@ -399,15 +440,23 @@ public class Catalina extends Embedded {
                 log.error("Catalina.stop: ", e);
                 System.exit(1);
             }
+        } else {
+            // Server object already present. Must be running as a service
+            try {
+                s.stop();
+            } catch (LifecycleException e) {
+                log.error("Catalina.stop: ", e);
+            }
+            return;
         }
 
         // Stop the existing server
+        s = getServer();
         try {
-            if (getServer().getPort()>0) { 
-                String hostAddress = InetAddress.getByName("localhost").getHostAddress();
-                Socket socket = new Socket(hostAddress, getServer().getPort());
+            if (s.getPort()>0) {
+                Socket socket = new Socket(s.getAddress(), s.getPort());
                 OutputStream stream = socket.getOutputStream();
-                String shutdown = getServer().getShutdown();
+                String shutdown = s.getShutdown();
                 for (int i = 0; i < shutdown.length(); i++)
                     stream.write(shutdown.charAt(i));
                 stream.flush();
@@ -425,24 +474,6 @@ public class Catalina extends Embedded {
     }
 
 
-    /**
-     * Set the <code>catalina.base</code> System property to the current
-     * working directory if it has not been set.
-     * @deprecated Use initDirs()
-     */
-    public void setCatalinaBase() {
-        initDirs();
-    }
-
-    /**
-     * Set the <code>catalina.home</code> System property to the current
-     * working directory if it has not been set.
-     * @deprecated Use initDirs()
-     */
-    public void setCatalinaHome() {
-        initDirs();
-    }
-
     /**
      * Start a new server instance.
      */
@@ -467,7 +498,9 @@ public class Catalina extends Embedded {
             inputStream = new FileInputStream(file);
             inputSource = new InputSource("file://" + file.getAbsolutePath());
         } catch (Exception e) {
-            ;
+            if (log.isDebugEnabled()) {
+                log.debug(sm.getString("catalina.configFail", file), e);
+            }
         }
         if (inputStream == null) {
             try {
@@ -477,7 +510,10 @@ public class Catalina extends Embedded {
                     (getClass().getClassLoader()
                      .getResource(getConfigFile()).toString());
             } catch (Exception e) {
-                ;
+                if (log.isDebugEnabled()) {
+                    log.debug(sm.getString("catalina.configFail",
+                            getConfigFile()), e);
+                }
             }
         }
 
@@ -486,20 +522,29 @@ public class Catalina extends Embedded {
         if( inputStream==null ) {
             try {
                 inputStream = getClass().getClassLoader()
-                .getResourceAsStream("server-embed.xml");
+                        .getResourceAsStream("server-embed.xml");
                 inputSource = new InputSource
                 (getClass().getClassLoader()
                         .getResource("server-embed.xml").toString());
             } catch (Exception e) {
-                ;
+                if (log.isDebugEnabled()) {
+                    log.debug(sm.getString("catalina.configFail",
+                            "server-embed.xml"), e);
+                }
             }
         }
-        
 
-        if ((inputStream == null) && (file != null)) {
-            log.warn("Can't load server.xml from " + file.getAbsolutePath());
-            if (file.exists() && !file.canRead()) {
-                log.warn("Permissions incorrect, read permission is not allowed on the file.");
+
+        if (inputStream == null || inputSource == null) {
+            if  (file == null) {
+                log.warn(sm.getString("catalina.configFail",
+                        getConfigFile() + "] or [server-embed.xml]"));
+            } else {
+                log.warn(sm.getString("catalina.configFail",
+                        file.getAbsolutePath()));
+                if (file.exists() && !file.canRead()) {
+                    log.warn("Permissions incorrect, read permission is not allowed on the file.");
+                }
             }
             return;
         }
@@ -509,26 +554,29 @@ public class Catalina extends Embedded {
             digester.push(this);
             digester.parse(inputSource);
             inputStream.close();
+        } catch (SAXParseException spe) {
+            log.warn("Catalina.start using " + getConfigFile() + ": " +
+                    spe.getMessage());
+            return;
         } catch (Exception e) {
-            log.warn("Catalina.start using "
-                               + getConfigFile() + ": " , e);
+            log.warn("Catalina.start using " + getConfigFile() + ": " , e);
             return;
         }
 
+        getServer().setCatalina(this);
+
         // Stream redirection
         initStreams();
 
         // Start the new server
-        if (getServer() instanceof Lifecycle) {
-            try {
-                getServer().initialize();
-            } catch (LifecycleException e) {
-                if (Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"))
-                    throw new java.lang.Error(e);
-                else   
-                    log.error("Catalina.start", e);
-                
-            }
+        try {
+            getServer().init();
+        } catch (LifecycleException e) {
+            if (Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"))
+                throw new java.lang.Error(e);
+            else
+                log.error("Catalina.start", e);
+
         }
 
         long t2 = System.nanoTime();
@@ -538,7 +586,7 @@ public class Catalina extends Embedded {
     }
 
 
-    /* 
+    /*
      * Load using arguments
      */
     public void load(String args[]) {
@@ -551,13 +599,6 @@ public class Catalina extends Embedded {
         }
     }
 
-    public void create() {
-
-    }
-
-    public void destroy() {
-
-    }
 
     /**
      * Start a new server instance.
@@ -574,14 +615,12 @@ public class Catalina extends Embedded {
         }
 
         long t1 = System.nanoTime();
-        
+
         // Start the new server
-        if (getServer() instanceof Lifecycle) {
-            try {
-                ((Lifecycle) getServer()).start();
-            } catch (LifecycleException e) {
-                log.error("Catalina.start: ", e);
-            }
+        try {
+            getServer().start();
+        } catch (LifecycleException e) {
+            log.error("Catalina.start: ", e);
         }
 
         long t2 = System.nanoTime();
@@ -595,7 +634,7 @@ public class Catalina extends Embedded {
                     shutdownHook = new CatalinaShutdownHook();
                 }
                 Runtime.getRuntime().addShutdownHook(shutdownHook);
-                
+
                 // If JULI is being used, disable JULI's shutdown hook since
                 // shutdown hooks run in parallel and log messages may be lost
                 // if JULI's hook completes before the CatalinaShutdownHook()
@@ -606,6 +645,7 @@ public class Catalina extends Embedded {
                 }
             }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             // This will fail on JDK 1.2. Ignoring, as Tomcat can run
             // fine without the shutdown hook.
         }
@@ -624,7 +664,7 @@ public class Catalina extends Embedded {
     public void stop() {
 
         try {
-            // Remove the ShutdownHook first so that server.stop() 
+            // Remove the ShutdownHook first so that server.stop()
             // doesn't get invoked twice
             if (useShutdownHook) {
                 Runtime.getRuntime().removeShutdownHook(shutdownHook);
@@ -638,17 +678,24 @@ public class Catalina extends Embedded {
                 }
             }
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             // This will fail on JDK 1.2. Ignoring, as Tomcat can run
             // fine without the shutdown hook.
         }
 
         // Shut down the server
-        if (getServer() instanceof Lifecycle) {
-            try {
-                ((Lifecycle) getServer()).stop();
-            } catch (LifecycleException e) {
-                log.error("Catalina.stop", e);
+        try {
+            Server s = getServer();
+            LifecycleState state = s.getState();
+            if (LifecycleState.STOPPING_PREP.compareTo(state) <= 0
+                    && LifecycleState.DESTROYED.compareTo(state) >= 0) {
+                // Nothing to do. stop() was already called
+            } else {
+                s.stop();
+                s.destroy();
             }
+        } catch (LifecycleException e) {
+            log.error("Catalina.stop", e);
         }
 
     }
@@ -678,6 +725,113 @@ public class Catalina extends Embedded {
     }
 
 
+    protected void initDirs() {
+
+        String catalinaHome = System.getProperty(Globals.CATALINA_HOME_PROP);
+        if (catalinaHome == null) {
+            // Backwards compatibility patch for J2EE RI 1.3
+            String j2eeHome = System.getProperty("com.sun.enterprise.home");
+            if (j2eeHome != null) {
+                catalinaHome=System.getProperty("com.sun.enterprise.home");
+            } else if (System.getProperty(Globals.CATALINA_BASE_PROP) != null) {
+                catalinaHome = System.getProperty(Globals.CATALINA_BASE_PROP);
+            } else {
+                // Use IntrospectionUtils and guess the dir
+                catalinaHome = IntrospectionUtils.guessInstall
+                    (Globals.CATALINA_HOME_PROP, Globals.CATALINA_BASE_PROP, "catalina.jar");
+                if (catalinaHome == null) {
+                    catalinaHome = IntrospectionUtils.guessInstall
+                        ("tomcat.install", Globals.CATALINA_HOME_PROP, "tomcat.jar");
+                }
+            }
+        }
+        // last resort - for minimal/embedded cases.
+        if(catalinaHome==null) {
+            catalinaHome=System.getProperty("user.dir");
+        }
+        if (catalinaHome != null) {
+            File home = new File(catalinaHome);
+            if (!home.isAbsolute()) {
+                try {
+                    catalinaHome = home.getCanonicalPath();
+                } catch (IOException e) {
+                    catalinaHome = home.getAbsolutePath();
+                }
+            }
+            System.setProperty(Globals.CATALINA_HOME_PROP, catalinaHome);
+        }
+
+        if (System.getProperty(Globals.CATALINA_BASE_PROP) == null) {
+            System.setProperty(Globals.CATALINA_BASE_PROP,
+                               catalinaHome);
+        } else {
+            String catalinaBase = System.getProperty(Globals.CATALINA_BASE_PROP);
+            File base = new File(catalinaBase);
+            if (!base.isAbsolute()) {
+                try {
+                    catalinaBase = base.getCanonicalPath();
+                } catch (IOException e) {
+                    catalinaBase = base.getAbsolutePath();
+                }
+            }
+            System.setProperty(Globals.CATALINA_BASE_PROP, catalinaBase);
+        }
+
+        String temp = System.getProperty("java.io.tmpdir");
+        if (temp == null || (!(new File(temp)).exists())
+                || (!(new File(temp)).isDirectory())) {
+            log.error(sm.getString("embedded.notmp", temp));
+        }
+
+    }
+
+
+    protected void initStreams() {
+        // Replace System.out and System.err with a custom PrintStream
+        System.setOut(new SystemLogHandler(System.out));
+        System.setErr(new SystemLogHandler(System.err));
+    }
+
+
+    protected void initNaming() {
+        // Setting additional variables
+        if (!useNaming) {
+            log.info( "Catalina naming disabled");
+            System.setProperty("catalina.useNaming", "false");
+        } else {
+            System.setProperty("catalina.useNaming", "true");
+            String value = "org.apache.naming";
+            String oldValue =
+                System.getProperty(javax.naming.Context.URL_PKG_PREFIXES);
+            if (oldValue != null) {
+                value = value + ":" + oldValue;
+            }
+            System.setProperty(javax.naming.Context.URL_PKG_PREFIXES, value);
+            if( log.isDebugEnabled() )
+                log.debug("Setting naming prefix=" + value);
+            value = System.getProperty
+                (javax.naming.Context.INITIAL_CONTEXT_FACTORY);
+            if (value == null) {
+                System.setProperty
+                    (javax.naming.Context.INITIAL_CONTEXT_FACTORY,
+                     "org.apache.naming.java.javaURLContextFactory");
+            } else {
+                log.debug( "INITIAL_CONTEXT_FACTORY alread set " + value );
+            }
+        }
+    }
+
+
+    /**
+     * Set the security package access/protection.
+     */
+    protected void setSecurityProtection(){
+        SecurityConfig securityConfig = SecurityConfig.newInstance();
+        securityConfig.setPackageDefinition();
+        securityConfig.setPackageAccess();
+    }
+
+
     // --------------------------------------- CatalinaShutdownHook Inner Class
 
     // XXX Should be moved to embedded !
@@ -686,12 +840,14 @@ public class Catalina extends Embedded {
      */
     protected class CatalinaShutdownHook extends Thread {
 
+        @Override
         public void run() {
             try {
                 if (getServer() != null) {
                     Catalina.this.stop();
                 }
             } catch (Throwable ex) {
+                ExceptionUtils.handleThrowable(ex);
                 log.error(sm.getString("catalina.shutdownHookFail"), ex);
             } finally {
                 // If JULI is used, shut JULI down *after* the server shuts down
@@ -701,13 +857,11 @@ public class Catalina extends Embedded {
                     ((ClassLoaderLogManager) logManager).shutdown();
                 }
             }
-
         }
-
     }
-    
-    
-    private static org.apache.juli.logging.Log log=
+
+
+    private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog( Catalina.class );
 
 }
@@ -731,6 +885,7 @@ final class SetParentClassLoaderRule extends Rule {
 
     ClassLoader parentClassLoader = null;
 
+    @Override
     public void begin(String namespace, String name, Attributes attributes)
         throws Exception {
 
