@@ -28,10 +28,11 @@ import org.apache.catalina.Authenticator;
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
 import org.apache.catalina.Engine;
+import org.apache.catalina.Globals;
 import org.apache.catalina.Host;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Loader;
 import org.apache.catalina.Realm;
 import org.apache.catalina.Valve;
@@ -42,12 +43,11 @@ import org.apache.catalina.core.StandardHost;
 import org.apache.catalina.core.StandardService;
 import org.apache.catalina.loader.WebappLoader;
 import org.apache.catalina.security.SecurityConfig;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.log.SystemLogHandler;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -94,15 +94,18 @@ import org.apache.tomcat.util.log.SystemLogHandler;
  * To initiate a normal shutdown, call the <code>stop()</code> method of
  * this object.
  * <p>
- * @see org.apache.catalina.startup.Catalina#main For a complete example
+ * @see org.apache.catalina.startup.Bootstrap#main For a complete example
  * of how Tomcat is set up and launched as an Embedded application.
  *
  * @author Craig R. McClanahan
- * @version $Id: Embedded.java 1070204 2011-02-13 10:25:08Z markt $
+ * @version $Id: Embedded.java 1134042 2011-06-09 19:04:37Z markt $
+ * 
+ * @deprecated Use {@link Tomcat} instead.
  */
 
-public class Embedded  extends StandardService implements Lifecycle {
-    private static Log log = LogFactory.getLog(Embedded.class);
+@Deprecated
+public class Embedded  extends StandardService {
+    private static final Log log = LogFactory.getLog(Embedded.class);
 
     // ----------------------------------------------------------- Constructors
 
@@ -157,7 +160,7 @@ public class Embedded  extends StandardService implements Lifecycle {
     /**
      * Custom mappings of login methods to authenticators
      */
-    protected HashMap authenticators;
+    protected HashMap<String,Authenticator> authenticators;
 
 
     /**
@@ -167,15 +170,9 @@ public class Embedded  extends StandardService implements Lifecycle {
         "org.apache.catalina.startup.Embedded/1.0";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
     /**
      * The default realm to be used by all containers associated with
-     * this compoennt.
+     * this component.
      */
     protected Realm realm = null;
 
@@ -183,15 +180,10 @@ public class Embedded  extends StandardService implements Lifecycle {
     /**
      * The string manager for this package.
      */
-    protected static StringManager sm =
+    protected static final StringManager sm =
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * Has this component been started yet?
-     */
-    protected boolean started = false;
-
     /**
      * Use await.
      */
@@ -220,14 +212,14 @@ public class Embedded  extends StandardService implements Lifecycle {
 
         boolean oldUseNaming = this.useNaming;
         this.useNaming = useNaming;
-        support.firePropertyChange("useNaming", new Boolean(oldUseNaming),
-                                   new Boolean(this.useNaming));
+        support.firePropertyChange("useNaming", Boolean.valueOf(oldUseNaming),
+                                   Boolean.valueOf(this.useNaming));
 
     }
 
 
     /**
-     * Return true if redirction of standard streams is enabled.
+     * Return true if redirection of standard streams is enabled.
      */
     public boolean isRedirectStreams() {
 
@@ -237,16 +229,17 @@ public class Embedded  extends StandardService implements Lifecycle {
 
 
     /**
-     * Enables or disables naming support.
+     * Enables or disables redirection.
      *
-     * @param useNaming The new use naming value
+     * @param redirectStreams The new redirection value
      */
     public void setRedirectStreams(boolean redirectStreams) {
 
         boolean oldRedirectStreams = this.redirectStreams;
         this.redirectStreams = redirectStreams;
-        support.firePropertyChange("redirectStreams", new Boolean(oldRedirectStreams),
-                                   new Boolean(this.redirectStreams));
+        support.firePropertyChange("redirectStreams",
+                                   Boolean.valueOf(oldRedirectStreams),
+                                   Boolean.valueOf(this.redirectStreams));
 
     }
 
@@ -282,20 +275,20 @@ public class Embedded  extends StandardService implements Lifecycle {
         return await;
     }
 
-    public void setCatalinaHome( String s ) {
-        System.setProperty( "catalina.home", s);
+    public void setCatalinaHome(String s) {
+        System.setProperty(Globals.CATALINA_HOME_PROP, s);
     }
 
-    public void setCatalinaBase( String s ) {
-        System.setProperty( "catalina.base", s);
+    public void setCatalinaBase(String s) {
+        System.setProperty(Globals.CATALINA_BASE_PROP, s);
     }
 
     public String getCatalinaHome() {
-        return System.getProperty("catalina.home");
+        return System.getProperty(Globals.CATALINA_HOME_PROP);
     }
 
     public String getCatalinaBase() {
-        return System.getProperty("catalina.base");
+        return System.getProperty(Globals.CATALINA_BASE_PROP);
     }
 
 
@@ -309,6 +302,7 @@ public class Embedded  extends StandardService implements Lifecycle {
      *
      * @exception IllegalStateException if no engines have been added yet
      */
+    @Override
     public synchronized void addConnector(Connector connector) {
 
         if( log.isDebugEnabled() ) {
@@ -346,9 +340,9 @@ public class Embedded  extends StandardService implements Lifecycle {
         engines = results;
 
         // Start this Engine if necessary
-        if (started && (engine instanceof Lifecycle)) {
+        if (getState().isAvailable()) {
             try {
-                ((Lifecycle) engine).start();
+                engine.start();
             } catch (LifecycleException e) {
                 log.error("Engine.start", e);
             }
@@ -370,8 +364,8 @@ public class Embedded  extends StandardService implements Lifecycle {
      */
     public Connector createConnector(InetAddress address, int port,
                                      boolean secure) {
-	return createConnector(address != null? address.toString() : null,
-			       port, secure);
+        return createConnector(address != null? address.toString() : null,
+                port, secure);
     }
 
     public Connector createConnector(String address, int port,
@@ -387,38 +381,38 @@ public class Embedded  extends StandardService implements Lifecycle {
 
     public Connector createConnector(InetAddress address, int port,
                                      String protocol) {
-	return createConnector(address != null? address.toString() : null,
-			       port, protocol);
+        return createConnector(address != null? address.toString() : null,
+                port, protocol);
     }
 
     public Connector createConnector(String address, int port,
-				     String protocol) {
+            String protocol) {
 
         Connector connector = null;
 
-	if (address != null) {
-	    /*
-	     * InetAddress.toString() returns a string of the form
-	     * "<hostname>/<literal_IP>". Get the latter part, so that the
-	     * address can be parsed (back) into an InetAddress using
-	     * InetAddress.getByName().
-	     */
-	    int index = address.indexOf('/');
-	    if (index != -1) {
-		address = address.substring(index + 1);
-	    }
-	}
+        if (address != null) {
+            /*
+             * InetAddress.toString() returns a string of the form
+             * "<hostname>/<literal_IP>". Get the latter part, so that the
+             * address can be parsed (back) into an InetAddress using
+             * InetAddress.getByName().
+             */
+            int index = address.indexOf('/');
+            if (index != -1) {
+                address = address.substring(index + 1);
+            }
+        }
 
-	if (log.isDebugEnabled()) {
+        if (log.isDebugEnabled()) {
             log.debug("Creating connector for address='" +
-		      ((address == null) ? "ALL" : address) +
-		      "' port='" + port + "' protocol='" + protocol + "'");
-	}
+                      ((address == null) ? "ALL" : address) +
+                      "' port='" + port + "' protocol='" + protocol + "'");
+        }
 
         try {
 
             if (protocol.equals("ajp")) {
-                connector = new Connector("org.apache.jk.server.JkCoyoteHandler");
+                connector = new Connector("org.apache.coyote.ajp.AjpProtocol");
             } else if (protocol.equals("memory")) {
                 connector = new Connector("org.apache.coyote.memory.MemoryProtocolHandler");
             } else if (protocol.equals("http")) {
@@ -483,7 +477,7 @@ public class Embedded  extends StandardService implements Lifecycle {
 
         ContextConfig config = new ContextConfig();
         config.setCustomAuthenticators(authenticators);
-        ((Lifecycle) context).addLifecycleListener(config);
+        context.addLifecycleListener(config);
 
         return (context);
 
@@ -576,6 +570,7 @@ public class Embedded  extends StandardService implements Lifecycle {
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -593,7 +588,7 @@ public class Embedded  extends StandardService implements Lifecycle {
     public synchronized void removeContext(Context context) {
 
         if( log.isDebugEnabled() )
-            log.debug("Removing context[" + context.getPath() + "]");
+            log.debug("Removing context[" + context.getName() + "]");
 
         // Is this Context actually among those that are defined?
         boolean found = false;
@@ -653,7 +648,7 @@ public class Embedded  extends StandardService implements Lifecycle {
         while (true) {
             int n = -1;
             for (int i = 0; i < connectors.length; i++) {
-                if (connectors[i].getContainer() == (Container) engine) {
+                if (connectors[i].getService().getContainer() == engine) {
                     n = i;
                     break;
                 }
@@ -664,14 +659,12 @@ public class Embedded  extends StandardService implements Lifecycle {
         }
 
         // Stop this Engine if necessary
-        if (engine instanceof Lifecycle) {
-            if( log.isDebugEnabled() )
-                log.debug(" Stopping this Engine");
-            try {
-                ((Lifecycle) engine).stop();
-            } catch (LifecycleException e) {
-                log.error("Engine.stop", e);
-            }
+        if( log.isDebugEnabled() )
+            log.debug(" Stopping this Engine");
+        try {
+            engine.stop();
+        } catch (LifecycleException e) {
+            log.error("Engine.stop", e);
         }
 
         // Remove this Engine from our set of defined Engines
@@ -746,7 +739,7 @@ public class Embedded  extends StandardService implements Lifecycle {
         if (authenticators == null) {
             synchronized (this) {
                 if (authenticators == null) {
-                    authenticators = new HashMap();
+                    authenticators = new HashMap<String,Authenticator>();
                 }
             }
         }
@@ -758,49 +751,15 @@ public class Embedded  extends StandardService implements Lifecycle {
 
 
     /**
-     * Add a lifecycle event listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this 
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called after <code>configure()</code>,
-     * and before any of the public methods of the component are utilized.
+     * Start nested components ({@link Connector}s and {@link Engine}s) and
+     * implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
+    @Override
+    protected void startInternal() throws LifecycleException {
 
         if( log.isInfoEnabled() )
             log.info("Starting tomcat server");
@@ -811,60 +770,45 @@ public class Embedded  extends StandardService implements Lifecycle {
         // Initialize some naming specific properties
         initNaming();
 
-        // Validate and update our current component state
-        if (started)
-            throw new LifecycleException
-                (sm.getString("embedded.alreadyStarted"));
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
-        initialized = true;
+        setState(LifecycleState.STARTING);
 
         // Start our defined Engines first
         for (int i = 0; i < engines.length; i++) {
-            if (engines[i] instanceof Lifecycle)
-                ((Lifecycle) engines[i]).start();
+            engines[i].start();
         }
 
         // Start our defined Connectors second
         for (int i = 0; i < connectors.length; i++) {
-            connectors[i].initialize();
-            if (connectors[i] instanceof Lifecycle)
-                ((Lifecycle) connectors[i]).start();
+            ((Lifecycle) connectors[i]).start();
         }
 
     }
 
 
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.
+     * Stop nested components ({@link Connector}s and {@link Engine}s) and
+     * implement the requirements of
+     * {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that needs to be reported
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
         if( log.isDebugEnabled() )
             log.debug("Stopping embedded server");
 
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException
-                (sm.getString("embedded.notStarted"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
+        setState(LifecycleState.STOPPING);
 
         // Stop our defined Connectors first
         for (int i = 0; i < connectors.length; i++) {
-            if (connectors[i] instanceof Lifecycle)
-                ((Lifecycle) connectors[i]).stop();
+            ((Lifecycle) connectors[i]).stop();
         }
 
         // Stop our defined Engines second
         for (int i = 0; i < engines.length; i++) {
-            if (engines[i] instanceof Lifecycle)
-                ((Lifecycle) engines[i]).stop();
+            engines[i].stop();
         }
 
     }
@@ -874,7 +818,7 @@ public class Embedded  extends StandardService implements Lifecycle {
 
 
     /** Initialize naming - this should only enable java:env and root naming.
-     * If tomcat is embeded in an application that already defines those -
+     * If tomcat is embedded in an application that already defines those -
      * it shouldn't do it.
      *
      * XXX The 2 should be separated, you may want to enable java: but not
@@ -914,21 +858,21 @@ public class Embedded  extends StandardService implements Lifecycle {
 
     protected void initDirs() {
 
-        String catalinaHome = System.getProperty("catalina.home");
+        String catalinaHome = System.getProperty(Globals.CATALINA_HOME_PROP);
         if (catalinaHome == null) {
             // Backwards compatibility patch for J2EE RI 1.3
             String j2eeHome = System.getProperty("com.sun.enterprise.home");
             if (j2eeHome != null) {
                 catalinaHome=System.getProperty("com.sun.enterprise.home");
-            } else if (System.getProperty("catalina.base") != null) {
-                catalinaHome = System.getProperty("catalina.base");
+            } else if (System.getProperty(Globals.CATALINA_BASE_PROP) != null) {
+                catalinaHome = System.getProperty(Globals.CATALINA_BASE_PROP);
             } else {
                 // Use IntrospectionUtils and guess the dir
                 catalinaHome = IntrospectionUtils.guessInstall
-                    ("catalina.home", "catalina.base", "catalina.jar");
+                    (Globals.CATALINA_HOME_PROP, Globals.CATALINA_BASE_PROP, "catalina.jar");
                 if (catalinaHome == null) {
                     catalinaHome = IntrospectionUtils.guessInstall
-                        ("tomcat.install", "catalina.home", "tomcat.jar");
+                        ("tomcat.install", Globals.CATALINA_HOME_PROP, "tomcat.jar");
                 }
             }
         }
@@ -945,14 +889,14 @@ public class Embedded  extends StandardService implements Lifecycle {
                     catalinaHome = home.getAbsolutePath();
                 }
             }
-            System.setProperty("catalina.home", catalinaHome);
+            System.setProperty(Globals.CATALINA_HOME_PROP, catalinaHome);
         }
 
-        if (System.getProperty("catalina.base") == null) {
-            System.setProperty("catalina.base",
+        if (System.getProperty(Globals.CATALINA_BASE_PROP) == null) {
+            System.setProperty(Globals.CATALINA_BASE_PROP,
                                catalinaHome);
         } else {
-            String catalinaBase = System.getProperty("catalina.base");
+            String catalinaBase = System.getProperty(Globals.CATALINA_BASE_PROP);
             File base = new File(catalinaBase);
             if (!base.isAbsolute()) {
                 try {
@@ -961,7 +905,7 @@ public class Embedded  extends StandardService implements Lifecycle {
                     catalinaBase = base.getAbsolutePath();
                 }
             }
-            System.setProperty("catalina.base", catalinaBase);
+            System.setProperty(Globals.CATALINA_BASE_PROP, catalinaBase);
         }
         
         String temp = System.getProperty("java.io.tmpdir");
@@ -976,9 +920,8 @@ public class Embedded  extends StandardService implements Lifecycle {
     protected void initStreams() {
         if (redirectStreams) {
             // Replace System.out and System.err with a custom PrintStream
-            SystemLogHandler systemlog = new SystemLogHandler(System.out);
-            System.setOut(systemlog);
-            System.setErr(systemlog);
+            System.setOut(new SystemLogHandler(System.out));
+            System.setErr(new SystemLogHandler(System.err));
         }
     }
     
