@@ -21,7 +21,7 @@ import java.io.IOException;
 import java.io.ObjectInput;
 import java.io.ObjectOutput;
 import java.io.Serializable;
-import java.io.UnsupportedEncodingException;
+import java.nio.charset.Charset;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -30,6 +30,7 @@ import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
 
 import org.apache.catalina.tribes.Channel;
 import org.apache.catalina.tribes.ChannelException;
@@ -46,7 +47,6 @@ import org.apache.catalina.tribes.membership.MemberImpl;
 import org.apache.catalina.tribes.util.Arrays;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
-import java.util.concurrent.ConcurrentHashMap;
 
 /**
  *
@@ -54,7 +54,9 @@ import java.util.concurrent.ConcurrentHashMap;
  * @version 1.0
  */
 public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements RpcCallback, ChannelListener, MembershipListener, Heartbeat {
-    protected static Log log = LogFactory.getLog(AbstractReplicatedMap.class);
+    private static final long serialVersionUID = 1L;
+
+    private static final Log log = LogFactory.getLog(AbstractReplicatedMap.class);
 
     /**
      * The default initial capacity - MUST be a power of two.
@@ -69,7 +71,8 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
     /**
      * Used to identify the map
      */
-    final String chset = "ISO-8859-1";
+    private static final Charset CHARSET_ISO_8859_1 =
+        Charset.forName("ISO-8859-1");
 
 //------------------------------------------------------------------------------
 //              INSTANCE VARIABLES
@@ -106,7 +109,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
     /**
      * A list of members in our map
      */
-    protected transient HashMap mapMembers = new HashMap();
+    protected transient HashMap<Member, Long> mapMembers = new HashMap<Member, Long>();
     /**
      * Our default send options
      */
@@ -144,6 +147,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
 //------------------------------------------------------------------------------
     
     public static interface MapOwner {
+        // a typo, should have been "objectMadePrimary"
         public void objectMadePrimay(Object key, Object value);
     }
 
@@ -201,14 +205,9 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         this.channel = channel;
         this.rpcTimeout = timeout;
 
-        try {
-            this.mapname = mapContextName;
-            //unique context is more efficient if it is stored as bytes
-            this.mapContextName = mapContextName.getBytes(chset);
-        } catch (UnsupportedEncodingException x) {
-            log.warn("Unable to encode mapContextName[" + mapContextName + "] using getBytes(" + chset +") using default getBytes()", x);
-            this.mapContextName = mapContextName.getBytes();
-        }
+        this.mapname = mapContextName;
+        //unique context is more efficient if it is stored as bytes
+        this.mapContextName = mapContextName.getBytes(CHARSET_ISO_8859_1);
         if ( log.isTraceEnabled() ) log.trace("Created Lazy Map with name:"+mapContextName+", bytes:"+Arrays.toString(this.mapContextName));
 
         //create an rpc channel and add the map as a listener
@@ -255,13 +254,13 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
                                         null);
         if ( channel.getMembers().length > 0 ) {
             try {
-            //send a ping, wait for all nodes to reply
-            Response[] resp = rpcChannel.send(channel.getMembers(), 
-                                              msg, rpcChannel.ALL_REPLY, 
-                                              (channelSendOptions),
-                                              (int) accessTimeout);
-            for (int i = 0; i < resp.length; i++) {
-                memberAlive(resp[i].getSource());
+                //send a ping, wait for all nodes to reply
+                Response[] resp = rpcChannel.send(channel.getMembers(), 
+                                                  msg, RpcChannel.ALL_REPLY, 
+                                                  (channelSendOptions),
+                                                  (int) accessTimeout);
+                for (int i = 0; i < resp.length; i++) {
+                    memberAlive(resp[i].getSource());
                 }
             } catch (ChannelException ce) {
                 // Handle known failed members
@@ -273,14 +272,14 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         }
         //update our map of members, expire some if we didn't receive a ping back
         synchronized (mapMembers) {
-            Iterator it = mapMembers.entrySet().iterator();
+            Iterator<Map.Entry<Member, Long>> it = mapMembers.entrySet().iterator();
             long now = System.currentTimeMillis();
             while ( it.hasNext() ) {
-                Map.Entry entry = (Map.Entry)it.next();
-                long access = ((Long)entry.getValue()).longValue(); 
+                Map.Entry<Member,Long> entry = it.next();
+                long access = entry.getValue().longValue(); 
                 if ( (now - access) > timeout ) {
                     it.remove();
-                    memberDisappeared( (Member) entry.getKey());
+                    memberDisappeared(entry.getKey());
                 }
             }
         }//synch
@@ -314,7 +313,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
                                         false, null, null, null, channel.getLocalMember(false), null);
         if ( rpc) {
             Response[] resp = rpcChannel.send(members, msg,
-                    rpcChannel.FIRST_REPLY, (channelSendOptions), rpcTimeout);
+                    RpcChannel.FIRST_REPLY, (channelSendOptions), rpcTimeout);
             if (resp.length > 0) {
                 for (int i = 0; i < resp.length; i++) {
                     mapMemberAdded(resp[i].getSource());
@@ -332,6 +331,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         finalize();
     }
 
+    @Override
     public void finalize() {
         try {broadcast(MapMessage.MSG_STOP,false); }catch ( Exception ignore){}
         //cleanup
@@ -350,12 +350,13 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         this.externalLoaders = null;
     }
     
+    @Override
     public int hashCode() {
         return Arrays.hashCode(this.mapContextName);
     }
     
+    @Override
     public boolean equals(Object o) {
-        if ( o == null ) return false;
         if ( !(o instanceof AbstractReplicatedMap)) return false;
         if ( !(o.getClass().equals(this.getClass())) ) return false;
         AbstractReplicatedMap other = (AbstractReplicatedMap)o;
@@ -365,7 +366,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
 //------------------------------------------------------------------------------
 //              GROUP COM INTERFACES
 //------------------------------------------------------------------------------
-    public Member[] getMapMembers(HashMap members) {
+    public Member[] getMapMembers(HashMap<Member, Long> members) {
         synchronized (members) {
             Member[] result = new Member[members.size()];
             members.keySet().toArray(result);
@@ -378,7 +379,8 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
     
     public Member[] getMapMembersExcl(Member[] exclude) {
         synchronized (mapMembers) {
-            HashMap list = (HashMap)mapMembers.clone();
+            @SuppressWarnings("unchecked") // mapMembers has the correct type
+            HashMap<Member, Long> list = (HashMap<Member, Long>)mapMembers.clone();
             for (int i=0; i<exclude.length;i++) list.remove(exclude[i]);
             return getMapMembers(list);
         }
@@ -398,7 +400,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         MapEntry entry = (MapEntry)super.get(key);
         if ( entry == null ) return;
         if ( !entry.isSerializable() ) return;
-        if (entry != null && entry.isPrimary() && entry.getBackupNodes()!= null && entry.getBackupNodes().length > 0) {
+        if (entry.isPrimary() && entry.getBackupNodes()!= null && entry.getBackupNodes().length > 0) {
             Object value = entry.getValue();
             //check to see if we need to replicate this object isDirty()||complete
             boolean repl = complete || ( (value instanceof ReplicatedMapEntry) && ( (ReplicatedMapEntry) value).isDirty());
@@ -456,9 +458,9 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
      * @param complete boolean
      */
     public void replicate(boolean complete) {
-        Iterator i = super.entrySet().iterator();
+        Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
         while (i.hasNext()) {
-            Map.Entry e = (Map.Entry) i.next();
+            Map.Entry<?,?> e = i.next();
             replicate(e.getKey(), complete);
         } //while
 
@@ -471,12 +473,12 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             if (backup != null) {
                 MapMessage msg = new MapMessage(mapContextName, getStateMessageType(), false,
                                                 null, null, null, null, null);
-                Response[] resp = rpcChannel.send(new Member[] {backup}, msg, rpcChannel.FIRST_REPLY, channelSendOptions, rpcTimeout);
+                Response[] resp = rpcChannel.send(new Member[] {backup}, msg, RpcChannel.FIRST_REPLY, channelSendOptions, rpcTimeout);
                 if (resp.length > 0) {
                     synchronized (stateMutex) {
                         msg = (MapMessage) resp[0].getMessage();
                         msg.deserialize(getExternalLoaders());
-                        ArrayList list = (ArrayList) msg.getValue();
+                        ArrayList<?> list = (ArrayList<?>) msg.getValue();
                         for (int i = 0; i < list.size(); i++) {
                             messageReceived( (Serializable) list.get(i), resp[0].getSource());
                         } //for
@@ -496,29 +498,30 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
     }
 
     /**
-     * @todo implement state transfer
+     * TODO implement state transfer
      * @param msg Serializable
      * @return Serializable - null if no reply should be sent
      */
+    @Override
     public Serializable replyRequest(Serializable msg, final Member sender) {
         if (! (msg instanceof MapMessage))return null;
         MapMessage mapmsg = (MapMessage) msg;
 
         //map init request
-        if (mapmsg.getMsgType() == mapmsg.MSG_INIT) {
+        if (mapmsg.getMsgType() == MapMessage.MSG_INIT) {
             mapmsg.setPrimary(channel.getLocalMember(false));
             return mapmsg;
         }
         
         //map start request
-        if (mapmsg.getMsgType() == mapmsg.MSG_START) {
+        if (mapmsg.getMsgType() == MapMessage.MSG_START) {
             mapmsg.setPrimary(channel.getLocalMember(false));
             mapMemberAdded(sender);
             return mapmsg;
         }
 
         //backup request
-        if (mapmsg.getMsgType() == mapmsg.MSG_RETRIEVE_BACKUP) {
+        if (mapmsg.getMsgType() == MapMessage.MSG_RETRIEVE_BACKUP) {
             MapEntry entry = (MapEntry)super.get(mapmsg.getKey());
             if (entry == null || (!entry.isSerializable()) )return null;
             mapmsg.setValue( (Serializable) entry.getValue());
@@ -526,15 +529,15 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         }
 
         //state transfer request
-        if (mapmsg.getMsgType() == mapmsg.MSG_STATE || mapmsg.getMsgType() == mapmsg.MSG_STATE_COPY) {
+        if (mapmsg.getMsgType() == MapMessage.MSG_STATE || mapmsg.getMsgType() == MapMessage.MSG_STATE_COPY) {
             synchronized (stateMutex) { //make sure we dont do two things at the same time
-                ArrayList list = new ArrayList();
-                Iterator i = super.entrySet().iterator();
+                ArrayList<MapMessage> list = new ArrayList<MapMessage>();
+                Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
                 while (i.hasNext()) {
-                    Map.Entry e = (Map.Entry) i.next();
+                    Map.Entry<?,?> e = i.next();
                     MapEntry entry = (MapEntry) super.get(e.getKey());
                     if ( entry != null && entry.isSerializable() ) {
-                        boolean copy = (mapmsg.getMsgType() == mapmsg.MSG_STATE_COPY);
+                        boolean copy = (mapmsg.getMsgType() == MapMessage.MSG_STATE_COPY);
                         MapMessage me = new MapMessage(mapContextName, 
                                                        copy?MapMessage.MSG_COPY:MapMessage.MSG_PROXY,
                             false, (Serializable) entry.getKey(), copy?(Serializable) entry.getValue():null, null, entry.getPrimary(),entry.getBackupNodes());
@@ -557,6 +560,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
      * @param msg Serializable
      * @param sender Member
      */
+    @Override
     public void leftOver(Serializable msg, Member sender) {
         //left over membership messages
         if (! (msg instanceof MapMessage))return;
@@ -576,6 +580,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         }
     }
 
+    @Override
     public void messageReceived(Serializable msg, Member sender) {
         if (! (msg instanceof MapMessage)) return;
 
@@ -667,6 +672,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         } //end if
     }
 
+    @Override
     public boolean accept(Serializable msg, Member sender) {
         boolean result = false;
         if (msg instanceof MapMessage) {
@@ -689,9 +695,9 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         }
         if ( memberAdded ) {
             synchronized (stateMutex) {
-                Iterator i = super.entrySet().iterator();
+                Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
                 while (i.hasNext()) {
-                    Map.Entry e = (Map.Entry) i.next();
+                    Map.Entry<?,?> e = i.next();
                     MapEntry entry = (MapEntry) super.get(e.getKey());
                     if ( entry == null ) continue;
                     if (entry.isPrimary() && (entry.getBackupNodes() == null || entry.getBackupNodes().length == 0)) {
@@ -717,20 +723,22 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
     }
 
     public Member[] excludeFromSet(Member[] mbrs, Member[] set) {
-        ArrayList result = new ArrayList();
+        ArrayList<Member> result = new ArrayList<Member>();
         for (int i=0; i<set.length; i++ ) {
             boolean include = true;
             for (int j=0; j<mbrs.length; j++ ) 
                 if ( mbrs[j].equals(set[i]) ) include = false;
             if ( include ) result.add(set[i]);
         }
-        return (Member[])result.toArray(new Member[result.size()]);
+        return result.toArray(new Member[result.size()]);
     }
 
+    @Override
     public void memberAdded(Member member) {
         //do nothing
     }
 
+    @Override
     public void memberDisappeared(Member member) {
         boolean removed = false;
         synchronized (mapMembers) {
@@ -741,9 +749,9 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             }
         }
         
-        Iterator i = super.entrySet().iterator();
+        Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
         while (i.hasNext()) {
-            Map.Entry e = (Map.Entry) i.next();
+            Map.Entry<?,?> e = i.next();
             MapEntry entry = (MapEntry) super.get(e.getKey());
             if (entry==null) continue;
             if (entry.isPrimary() && inSet(member,entry.getBackupNodes())) {
@@ -810,6 +818,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
 
     protected abstract Member[] publishEntryInfo(Object key, Object value) throws ChannelException;
     
+    @Override
     public void heartbeat() {
         try {
             ping(accessTimeout);
@@ -828,6 +837,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
      * @param key Object
      * @return Object
      */
+    @Override
     public Object remove(Object key) {
         return remove(key,true);
     }
@@ -849,6 +859,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         return (MapEntry)super.get(key);
     }
     
+    @Override
     public Object get(Object key) {
         MapEntry entry = (MapEntry)super.get(key);
         if (log.isTraceEnabled()) log.trace("Requesting id:"+key+" entry:"+entry);
@@ -862,7 +873,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
                     //make sure we don't retrieve from ourselves
                     msg = new MapMessage(getMapContextName(), MapMessage.MSG_RETRIEVE_BACKUP, false,
                                          (Serializable) key, null, null, null,null);
-                    Response[] resp = getRpcChannel().send(entry.getBackupNodes(),msg, this.getRpcChannel().FIRST_REPLY, Channel.SEND_OPTIONS_DEFAULT, getRpcTimeout());
+                    Response[] resp = getRpcChannel().send(entry.getBackupNodes(),msg, RpcChannel.FIRST_REPLY, Channel.SEND_OPTIONS_DEFAULT, getRpcTimeout());
                     if (resp == null || resp.length == 0) {
                         //no responses
                         log.warn("Unable to retrieve remote object for key:" + key);
@@ -911,16 +922,18 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
     protected void printMap(String header) {
         try {
             System.out.println("\nDEBUG MAP:"+header);
-            System.out.println("Map["+ new String(mapContextName, chset) + ", Map Size:" + super.size());
+            System.out.println("Map[" +
+                    new String(mapContextName, CHARSET_ISO_8859_1) +
+                    ", Map Size:" + super.size());
             Member[] mbrs = getMapMembers();
             for ( int i=0; i<mbrs.length;i++ ) {
                 System.out.println("Mbr["+(i+1)+"="+mbrs[i].getName());
             }
-            Iterator i = super.entrySet().iterator();
+            Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
             int cnt = 0;
 
             while (i.hasNext()) {
-                Map.Entry e = (Map.Entry) i.next();
+                Map.Entry<?,?> e = i.next();
                 System.out.println( (++cnt) + ". " + super.get(e.getKey()));
             }
             System.out.println("EndMap]\n\n");
@@ -936,10 +949,12 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
          * @param key Object
          * @return boolean
          */
+        @Override
         public boolean containsKey(Object key) {
             return super.containsKey(key);
         }
     
+        @Override
         public Object put(Object key, Object value) {
             return put(key,value,true);
         }
@@ -971,14 +986,16 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
          * Copies all values from one map to this instance
          * @param m Map
          */
+        @Override
         public void putAll(Map m) {
-            Iterator i = m.entrySet().iterator();
+            Iterator<Map.Entry<?,?>> i = m.entrySet().iterator();
             while ( i.hasNext() ) {
-                Map.Entry entry = (Map.Entry)i.next();
+                Map.Entry<?,?> entry = i.next();
                 put(entry.getKey(),entry.getValue());
             }
         }
         
+        @Override
         public void clear() {
             clear(true);
         }
@@ -986,7 +1003,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         public void clear(boolean notify) {
             if ( notify ) {
                 //only delete active keys
-                Iterator keys = keySet().iterator();
+                Iterator<Object> keys = keySet().iterator();
                 while (keys.hasNext())
                     remove(keys.next());
             } else {
@@ -994,20 +1011,22 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             }
         }
     
+        @Override
         public boolean containsValue(Object value) {
             if ( value == null ) {
                 return super.containsValue(value);
             } else {
-                Iterator i = super.entrySet().iterator();
+                Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
                 while (i.hasNext()) {
-                    Map.Entry e = (Map.Entry) i.next();
+                    Map.Entry<?,?> e = i.next();
                     MapEntry entry = (MapEntry) super.get(e.getKey());
-                    if (entry!=null && entry.isPrimary() && value.equals(entry.getValue())) return true;
+                    if (entry!=null && entry.isActive() && value.equals(entry.getValue())) return true;
                 }//while
                 return false;
             }//end if
         }
     
+        @Override
         public Object clone() {
             throw new UnsupportedOperationException("This operation is not valid on a replicated map");
         }
@@ -1030,66 +1049,67 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             return super.size();
         }
     
-        public Set entrySet() {
-            LinkedHashSet set = new LinkedHashSet(super.size());
-            Iterator i = super.entrySet().iterator();
+        @Override
+        public Set<MapEntry> entrySet() {
+            LinkedHashSet<MapEntry> set = new LinkedHashSet<MapEntry>(super.size());
+            Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
             while ( i.hasNext() ) {
-                Map.Entry e = (Map.Entry)i.next();
+                Map.Entry<?,?> e = i.next();
                 Object key = e.getKey();
                 MapEntry entry = (MapEntry)super.get(key);
-                if ( entry != null && entry.isPrimary() ) {
+                if ( entry != null && entry.isActive() ) {
                     set.add(new MapEntry(key, entry.getValue()));
                 }
             }
             return Collections.unmodifiableSet(set);
         }
     
-        public Set keySet() {
+        @Override
+        public Set<Object> keySet() {
             //todo implement
             //should only return keys where this is active.
-            LinkedHashSet set = new LinkedHashSet(super.size());
-            Iterator i = super.entrySet().iterator();
+            LinkedHashSet<Object> set = new LinkedHashSet<Object>(super.size());
+            Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
             while ( i.hasNext() ) {
-                Map.Entry e = (Map.Entry)i.next();
+                Map.Entry<?,?> e = i.next();
                 Object key = e.getKey();
                 MapEntry entry = (MapEntry)super.get(key);
-                if ( entry!=null && entry.isPrimary() ) set.add(key);
+                if ( entry!=null && entry.isActive() ) set.add(key);
             }
             return Collections.unmodifiableSet(set);
 
         }
     
     
+        @Override
         public int size() {
             //todo, implement a counter variable instead
             //only count active members in this node
             int counter = 0;
-            Iterator it = super.entrySet().iterator();
+            Iterator<Map.Entry<?,?>> it = super.entrySet().iterator();
             while (it!=null && it.hasNext() ) {
-                Map.Entry e = (Map.Entry) it.next();
+                Map.Entry<?,?> e = it.next();
                 if ( e != null ) {
                     MapEntry entry = (MapEntry) super.get(e.getKey());
-                    if (entry!=null && entry.isPrimary() && entry.getValue() != null) counter++;
+                    if (entry!=null && entry.isActive() && entry.getValue() != null) counter++;
                 }
             }
             return counter;
         }
     
-        protected boolean removeEldestEntry(Map.Entry eldest) {
-            return false;
-        }
-    
+        @Override
         public boolean isEmpty() {
             return size()==0;
         }
     
-        public Collection values() {
-            ArrayList values = new ArrayList();
-            Iterator i = super.entrySet().iterator();
+        @Override
+        public Collection<Object> values() {
+            ArrayList<Object> values = new ArrayList<Object>();
+            Iterator<Map.Entry<?,?>> i = super.entrySet().iterator();
             while ( i.hasNext() ) {
-                Map.Entry e = (Map.Entry)i.next();
+                Map.Entry<?,?> e = i.next();
                 MapEntry entry = (MapEntry)super.get(e.getKey());
-                if (entry!=null && entry.isPrimary() && entry.getValue()!=null) values.add(entry.getValue());
+                if (entry!=null && entry.isActive() && entry.getValue()!=null) values.add(entry.getValue());
             }
             return Collections.unmodifiableCollection(values);
         }
@@ -1098,7 +1118,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
 //------------------------------------------------------------------------------
 //                Map Entry class
 //------------------------------------------------------------------------------
-    public static class MapEntry implements Map.Entry {
+    public static class MapEntry implements Map.Entry<Object,Object> {
         private boolean backup;
         private boolean proxy;
         private Member[] backupNodes;
@@ -1137,9 +1157,13 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         }
 
         public boolean isPrimary() {
-            return ( (!proxy) && (!backup));
+            return (!proxy && !backup);
         }
 
+        public boolean isActive() {
+            return !proxy;
+        }
+        
         public void setProxy(boolean proxy) {
             this.proxy = proxy;
         }
@@ -1165,16 +1189,19 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             return primary;
         }
 
+        @Override
         public Object getValue() {
             return value;
         }
 
+        @Override
         public Object setValue(Object value) {
             Object old = this.value;
             this.value = value;
             return old;
         }
 
+        @Override
         public Object getKey() {
             return key;
         }
@@ -1185,10 +1212,12 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             return old;
         }
 
+        @Override
         public int hashCode() {
             return key.hashCode();
         }
 
+        @Override
         public boolean equals(Object o) {
             return key.equals(o);
         }
@@ -1219,8 +1248,9 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             }
         }
         
+        @Override
         public String toString() {
-            StringBuffer buf = new StringBuffer("MapEntry[key:");
+            StringBuilder buf = new StringBuilder("MapEntry[key:");
             buf.append(getKey()).append("; ");
             buf.append("value:").append(getValue()).append("; ");
             buf.append("primary:").append(isPrimary()).append("; ");
@@ -1236,6 +1266,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
 //------------------------------------------------------------------------------
 
     public static class MapMessage implements Serializable {
+        private static final long serialVersionUID = 1L;
         public static final int MSG_BACKUP = 1;
         public static final int MSG_RETRIEVE_BACKUP = 2;
         public static final int MSG_PROXY = 3;
@@ -1258,8 +1289,9 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
         private Member[] nodes;
         private Member primary;
         
+        @Override
         public String toString() {
-            StringBuffer buf = new StringBuffer("MapMessage[context=");
+            StringBuilder buf = new StringBuilder("MapMessage[context=");
             buf.append(new String(mapId));
             buf.append("; type=");
             buf.append(getTypeDesc());
@@ -1350,7 +1382,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             if ( value!=null ) return value;
             if ( valuedata == null || valuedata.length == 0 ) return null;
             value = XByteBuffer.deserialize(valuedata,0,valuedata.length,cls);
-            valuedata = null;;
+            valuedata = null;
             return value;
         }
         
@@ -1366,10 +1398,6 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             return nodes;
         }
 
-        private void setBackUpNodes(Member[] nodes) {
-            this.nodes = nodes;
-        }
-        
         public Member getPrimary() {
             return primary;
         }
@@ -1405,7 +1433,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
             Member[] members = new Member[nodecount];
             for ( int i=0; i<members.length; i++ ) {
                 byte[] d = new byte[in.readInt()];
-                in.read(d);
+                in.readFully(d);
                 if (d.length > 0) members[i] = MemberImpl.getMember(d);
             }
             return members;
@@ -1428,6 +1456,7 @@ public abstract class AbstractReplicatedMap extends ConcurrentHashMap implements
          * shallow clone
          * @return Object
          */
+        @Override
         public Object clone() {
             MapMessage msg = new MapMessage(this.mapId, this.msgtype, this.diff, this.key, this.value, this.diffvalue, this.primary, this.nodes);
             msg.keydata = this.keydata;
