@@ -19,37 +19,39 @@ package org.apache.catalina.tribes.transport.nio;
 
 import java.io.IOException;
 import java.net.ServerSocket;
+import java.nio.channels.CancelledKeyException;
+import java.nio.channels.ClosedSelectorException;
+import java.nio.channels.DatagramChannel;
 import java.nio.channels.SelectableChannel;
 import java.nio.channels.SelectionKey;
 import java.nio.channels.Selector;
 import java.nio.channels.ServerSocketChannel;
 import java.nio.channels.SocketChannel;
 import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.Set;
 
-import org.apache.catalina.tribes.ChannelReceiver;
-import org.apache.catalina.tribes.io.ListenCallback;
 import org.apache.catalina.tribes.io.ObjectReader;
+import org.apache.catalina.tribes.transport.AbstractRxTask;
 import org.apache.catalina.tribes.transport.Constants;
 import org.apache.catalina.tribes.transport.ReceiverBase;
 import org.apache.catalina.tribes.transport.RxTaskPool;
-import org.apache.catalina.tribes.transport.AbstractRxTask;
 import org.apache.catalina.tribes.util.StringManager;
-import java.util.LinkedList;
-import java.util.Set;
-import java.nio.channels.CancelledKeyException;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
 
 /**
  * @author Filip Hanik
- * @version $Id: NioReceiver.java 939515 2010-04-29 23:59:49Z kkolinko $
+ * @version $Id: NioReceiver.java 1142666 2011-07-04 13:54:52Z kkolinko $
  */
-public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiver, ListenCallback {
+public class NioReceiver extends ReceiverBase implements Runnable {
 
-    protected static org.apache.juli.logging.Log log = org.apache.juli.logging.LogFactory.getLog(NioReceiver.class);
+    private static final Log log = LogFactory.getLog(NioReceiver.class);
 
     /**
      * The string manager for this package.
      */
-    protected StringManager sm = StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(Constants.Package);
 
     /**
      * The descriptive information about this implementation.
@@ -58,8 +60,9 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
 
     private Selector selector = null;
     private ServerSocketChannel serverChannel = null;
+    private DatagramChannel datagramChannel = null;
 
-    protected LinkedList events = new LinkedList();
+    protected LinkedList<Runnable> events = new LinkedList<Runnable>();
 //    private Object interestOpsMutex = new Object();
 
     public NioReceiver() {
@@ -78,6 +81,7 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
 //        return interestOpsMutex;
 //    }
 
+    @Override
     public void stop() {
         this.stopListening();
         super.stop();
@@ -85,9 +89,10 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
 
     /**
      * start cluster receiver
-     * @throws Exception
-     * @see org.apache.catalina.tribes.ClusterReceiver#start()
+     * @throws IOException
+     * @see org.apache.catalina.tribes.ChannelReceiver#start()
      */
+    @Override
     public void start() throws IOException {
         super.start();
         try {
@@ -109,7 +114,8 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
             else throw new IOException(x.getMessage());
         }
     }
-    
+
+    @Override
     public AbstractRxTask createRxTask() {
         NioReplicationTask thread = new NioReplicationTask(this,this);
         thread.setUseBufferPool(this.getUseBufferPool());
@@ -117,9 +123,9 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
         thread.setOptions(getWorkerThreadOptions());
         return thread;
     }
-    
-    
-    
+
+
+
     protected void bind() throws IOException {
         // allocate an unbound server socket channel
         serverChannel = ServerSocketChannel.open();
@@ -129,14 +135,24 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
         selector = Selector.open();
         // set the port the server channel will listen to
         //serverSocket.bind(new InetSocketAddress(getBind(), getTcpListenPort()));
-        bind(serverSocket,getTcpListenPort(),getAutoBind());
+        bind(serverSocket,getPort(),getAutoBind());
         // set non-blocking mode for the listening socket
         serverChannel.configureBlocking(false);
         // register the ServerSocketChannel with the Selector
         serverChannel.register(selector, SelectionKey.OP_ACCEPT);
-        
+
+        //set up the datagram channel
+        if (this.getUdpPort()>0) {
+            datagramChannel = DatagramChannel.open();
+            datagramChannel.configureBlocking(false);
+            //bind to the address to avoid security checks
+            bindUdp(datagramChannel.socket(),getUdpPort(),getAutoBind());
+        }
+
+
+
     }
-    
+
     public void addEvent(Runnable event) {
         if ( selector != null ) {
             synchronized (events) {
@@ -151,7 +167,7 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
         if ( events.size() == 0 ) return;
         synchronized (events) {
             Runnable r = null;
-            while ( (events.size() > 0) && (r = (Runnable)events.removeFirst()) != null ) {
+            while ( (events.size() > 0) && (r = events.removeFirst()) != null ) {
                 try {
                     if ( log.isTraceEnabled() ) log.trace("Processing event in selector:"+r);
                     r.run();
@@ -162,18 +178,21 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
             events.clear();
         }
     }
-    
+
     public static void cancelledKey(SelectionKey key) {
         ObjectReader reader = (ObjectReader)key.attachment();
         if ( reader != null ) {
             reader.setCancelled(true);
             reader.finish();
         }
-        key.cancel(); 
+        key.cancel();
         key.attach(null);
-        try { ((SocketChannel)key.channel()).socket().close(); } catch (IOException e) { if (log.isDebugEnabled()) log.debug("", e); }
+        if (key.channel() instanceof SocketChannel)
+            try { ((SocketChannel)key.channel()).socket().close(); } catch (IOException e) { if (log.isDebugEnabled()) log.debug("", e); }
+        if (key.channel() instanceof DatagramChannel)
+            try { ((DatagramChannel)key.channel()).socket().close(); } catch (Exception e) { if (log.isDebugEnabled()) log.debug("", e); }
         try { key.channel().close(); } catch (IOException e) { if (log.isDebugEnabled()) log.debug("", e); }
-        
+
     }
     protected long lastCheck = System.currentTimeMillis();
     protected void socketTimeouts() {
@@ -181,10 +200,10 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
         if ( (now-lastCheck) < getSelectorTimeout() ) return;
         //timeout
         Selector tmpsel = selector;
-        Set keys =  (isListening()&&tmpsel!=null)?tmpsel.keys():null;
+        Set<SelectionKey> keys =  (isListening()&&tmpsel!=null)?tmpsel.keys():null;
         if ( keys == null ) return;
-        for (Iterator iter = keys.iterator(); iter.hasNext(); ) {
-            SelectionKey key = (SelectionKey) iter.next();
+        for (Iterator<SelectionKey> iter = keys.iterator(); iter.hasNext();) {
+            SelectionKey key = iter.next();
             try {
 //                if (key.interestOps() == SelectionKey.OP_READ) {
 //                    //only timeout sockets that we are waiting for a read from
@@ -200,11 +219,9 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
                     ObjectReader ka = (ObjectReader) key.attachment();
                     if ( ka != null ) {
                         long delta = now - ka.getLastAccess();
-                        if (delta > (long) getTimeout() && (!ka.isAccessed())) {
-                            log.warn("Channel key is registered, but has had no interest ops for the last "+getTimeout()+" ms. (cancelled:"+ka.isCancelled()+"):"+key+" last access:"+new java.sql.Timestamp(ka.getLastAccess()));
-//                            System.out.println("Interest:"+key.interestOps());
-//                            System.out.println("Ready Ops:"+key.readyOps());
-//                            System.out.println("Valid:"+key.isValid());
+                        if (delta > getTimeout() && (!ka.isAccessed())) {
+                            if (log.isWarnEnabled())
+                                log.warn("Channel key is registered, but has had no interest ops for the last "+getTimeout()+" ms. (cancelled:"+ka.isCancelled()+"):"+key+" last access:"+new java.sql.Timestamp(ka.getLastAccess())+" Possible cause: all threads used, perform thread dump");
                             ka.setLastAccess(now);
                             //key.interestOps(SelectionKey.OP_READ);
                         }//end if
@@ -231,8 +248,17 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
             log.warn("ServerSocketChannel already started");
             return;
         }
-        
+
         setListen(true);
+        if (selector!=null && datagramChannel!=null) {
+            ObjectReader oreader = new ObjectReader(MAX_UDP_SIZE); //max size for a datagram packet
+            datagramChannel.socket().setSendBufferSize(getUdpTxBufSize());
+            datagramChannel.socket().setReceiveBufferSize(getUdpRxBufSize());
+            datagramChannel.socket().setReuseAddress(getSoReuseAddress());
+            datagramChannel.socket().setSoTimeout(getTimeout());
+            datagramChannel.socket().setTrafficClass(getSoTrafficClass());
+            registerChannel(selector,datagramChannel,SelectionKey.OP_READ,oreader);
+        }
 
         while (doListen() && selector != null) {
             // this may block for a long time, upon return the
@@ -240,7 +266,7 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
             try {
                 events();
                 socketTimeouts();
-                int n = selector.select(getTcpSelectorTimeout());
+                int n = selector.select(getSelectorTimeout());
                 if (n == 0) {
                     //there is a good chance that we got here
                     //because the TcpReplicationThread called
@@ -255,10 +281,10 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
                     continue; // nothing to do
                 }
                 // get an iterator over the set of selected keys
-                Iterator it = selector.selectedKeys().iterator();
+                Iterator<SelectionKey> it = (selector!=null)?selector.selectedKeys().iterator():null;
                 // look at each key in the selected set
-                while (it.hasNext()) {
-                    SelectionKey key = (SelectionKey) it.next();
+                while (selector!=null && it.hasNext()) {
+                    SelectionKey key = it.next();
                     // Is a new connection coming in?
                     if (key.isAcceptable()) {
                         ServerSocketChannel server = (ServerSocketChannel) key.channel();
@@ -286,40 +312,48 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
                     }
 
                     // remove key from selected set, it's been handled
-                    it.remove();
+                    if (selector!=null) it.remove();
                 }
             } catch (java.nio.channels.ClosedSelectorException cse) {
                 // ignore is normal at shutdown or stop listen socket
             } catch (java.nio.channels.CancelledKeyException nx) {
                 log.warn("Replication client disconnected, error when polling key. Ignoring client.");
-            } catch (Throwable x) {
-                try {
-                    log.error("Unable to process request in NioReceiver", x);
-                }catch ( Throwable tx ) {
-                    //in case an out of memory error, will affect the logging framework as well
-                    tx.printStackTrace();
+            } catch (Throwable t) {
+                if (t instanceof ThreadDeath) {
+                    throw (ThreadDeath) t;
                 }
+                if (t instanceof VirtualMachineError) {
+                    throw (VirtualMachineError) t;
+                }
+                log.error("Unable to process request in NioReceiver", t);
             }
 
         }
         serverChannel.close();
-        if (selector != null)
-            selector.close();
+        if (datagramChannel!=null) {
+            try {
+                datagramChannel.close();
+            }catch (Exception iox) {
+                if (log.isDebugEnabled()) log.debug("Unable to close datagram channel.",iox);
+            }
+            datagramChannel=null;
+        }
+        closeSelector();
     }
 
-    
+
 
     /**
      * Close Selector.
      *
-     * @see org.apache.catalina.tribes.transport.ClusterReceiverBase#stopListening()
+     * @see org.apache.catalina.tribes.transport.ReceiverBase#stop()
      */
     protected void stopListening() {
         setListen(false);
         if (selector != null) {
             try {
                 selector.wakeup();
-                selector.close();
+                closeSelector();
             } catch (Exception x) {
                 log.error("Unable to close cluster receiver selector.", x);
             } finally {
@@ -328,6 +362,27 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
         }
     }
 
+    private void closeSelector() throws IOException {
+        Selector selector = this.selector;
+        this.selector = null;
+        if (selector==null) return;
+        try {
+            Iterator<SelectionKey> it = selector.keys().iterator();
+            // look at each key in the selected set
+            while (it.hasNext()) {
+                SelectionKey key = it.next();
+                key.channel().close();
+                key.attach(null);
+                key.cancel();
+            }
+        }catch ( IOException ignore ){
+            if (log.isWarnEnabled()) {
+                log.warn("Unable to cleanup on selector close.",ignore);
+            }
+        }catch ( ClosedSelectorException ignore){}
+        selector.close();
+    }
+
     // ----------------------------------------------------------
 
     /**
@@ -348,6 +403,7 @@ public class NioReceiver extends ReceiverBase implements Runnable, ChannelReceiv
     /**
      * Start thread and listen
      */
+    @Override
     public void run() {
         try {
             listen();
