@@ -26,18 +26,20 @@ import java.net.URLEncoder;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Date;
+import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 import java.util.TimeZone;
 
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpSession;
 
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 
 
 
@@ -47,7 +49,8 @@ import org.apache.juli.logging.LogFactory;
  *
  * The following fields are supported:
  * <ul>
- * <li><code>c-dns</code>:  Client hostname</li>
+ * <li><code>c-dns</code>:  Client hostname (or ip address if
+ * <code>enableLookups</code> for the connector is false)</li>
  * <li><code>c-ip</code>:  Client ip address</li>
  * <li><code>bytes</code>:  bytes served</li>
  * <li><code>cs-method</code>:  request method</li>
@@ -62,6 +65,7 @@ import org.apache.juli.logging.LogFactory;
  * <li><code>sc-status</code>:  The status code</li>
  * <li><code>time</code>:  Time the request was served</li>
  * <li><code>time-taken</code>:  Time (in seconds) taken to serve the request</li>
+ * <li><code>x-threadname</code>: Current request thread name (can compare later with stacktraces)</li>
  * <li><code>x-A(XXX)</code>: Pull XXX attribute from the servlet context </li>
  * <li><code>x-C(XXX)</code>: Pull the first cookie of the name XXX </li>
  * <li><code>x-O(XXX)</code>: Pull the all response header values XXX </li>
@@ -72,14 +76,14 @@ import org.apache.juli.logging.LogFactory;
  *                             certain POST parameters.
  * </li>
  * <li>For any of the x-H(...) the following method will be called from the
- *                HttpServletRequestObject </li>
+ *                HttpServletRequest object </li>
  * <li><code>x-H(authType)</code>: getAuthType </li>
  * <li><code>x-H(characterEncoding)</code>: getCharacterEncoding </li>
  * <li><code>x-H(contentLength)</code>: getContentLength </li>
  * <li><code>x-H(locale)</code>:  getLocale</li>
  * <li><code>x-H(protocol)</code>: getProtocol </li>
  * <li><code>x-H(remoteUser)</code>:  getRemoteUser</li>
- * <li><code>x-H(requestedSessionId)</code>: getGequestedSessionId</li>
+ * <li><code>x-H(requestedSessionId)</code>: getRequestedSessionId</li>
  * <li><code>x-H(requestedSessionIdFromCookie)</code>:
  *                  isRequestedSessionIdFromCookie </li>
  * <li><code>x-H(requestedSessionIdValid)</code>:
@@ -96,7 +100,7 @@ import org.apache.juli.logging.LogFactory;
  * </p>
  *
  * <p>
- * For UvNIX users, another field called <code>checkExists</code>is also
+ * For UvNIX users, another field called <code>checkExists</code> is also
  * available. If set to true, the log file's existence will be checked before
  * each logging. This way an external log rotator can move the file
  * somewhere and tomcat will start with a new file.
@@ -105,14 +109,14 @@ import org.apache.juli.logging.LogFactory;
  * <p>
  * For JMX junkies, a public method called </code>rotate</code> has
  * been made available to allow you to tell this instance to move
- * the existing log file to somewhere else start writing a new log file.
+ * the existing log file to somewhere else and start writing a new log file.
  * </p>
  *
  * <p>
  * Conditional logging is also supported. This can be done with the
  * <code>condition</code> property.
  * If the value returned from ServletRequest.getAttribute(condition)
- * yields a non-null value. The logging will be skipped.
+ * yields a non-null value, the logging will be skipped.
  * </p>
  *
  * <p>
@@ -125,14 +129,12 @@ import org.apache.juli.logging.LogFactory;
  * @author Tim Funk
  * @author Peter Rossbach
  * 
- * @version $Revision: 832123 $ $Date: 2009-11-02 22:57:21 +0100 (Mo, 02. Nov 2009) $
+ * @version $Id: ExtendedAccessLogValve.java 1152716 2011-08-01 09:41:53Z kkolinko $
  */
 
-public class ExtendedAccessLogValve
-    extends AccessLogValve
-    implements Lifecycle {
+public class ExtendedAccessLogValve extends AccessLogValve {
 
-    private static Log log = LogFactory.getLog(ExtendedAccessLogValve.class);
+    private static final Log log = LogFactory.getLog(ExtendedAccessLogValve.class);
 
     // ----------------------------------------------------- Instance Variables
 
@@ -150,6 +152,7 @@ public class ExtendedAccessLogValve
     /**
      * Return descriptive information about this implementation.
      */
+    @Override
     public String getInfo() {
         return (extendedAccessLogInfo);
     }
@@ -181,12 +184,13 @@ public class ExtendedAccessLogValve
             if ("".equals(svalue))
                 return "-";
         } catch (Throwable e) {
+            ExceptionUtils.handleThrowable(e);
             /* Log error */
             return "-";
         }
 
         /* Wrap all quotes in double quotes. */
-        StringBuffer buffer = new StringBuffer(svalue.length() + 2);
+        StringBuilder buffer = new StringBuilder(svalue.length() + 2);
         buffer.append('\'');
         int i = 0;
         while (i < svalue.length()) {
@@ -208,6 +212,7 @@ public class ExtendedAccessLogValve
     /**
      * Open the new log file for the date specified by <code>dateStamp</code>.
      */
+    @Override
     protected synchronized void open() {
         super.open();
         if (currentLogFile.length()==0) {
@@ -227,12 +232,14 @@ public class ExtendedAccessLogValve
         
         private static final ThreadLocal<ElementTimestampStruct> currentDate =
                 new ThreadLocal<ElementTimestampStruct>() {
+            @Override
             protected ElementTimestampStruct initialValue() {
                 return new ElementTimestampStruct("yyyy-MM-dd");
             }
         };
                 
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             ElementTimestampStruct eds = currentDate.get();
             long millis = eds.currentTimestamp.getTime();
@@ -253,12 +260,14 @@ public class ExtendedAccessLogValve
         
         private static final ThreadLocal<ElementTimestampStruct> currentTime =
                 new ThreadLocal<ElementTimestampStruct>() {
+            @Override
             protected ElementTimestampStruct initialValue() {
                 return new ElementTimestampStruct("HH:mm:ss");
             }
         };
             
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             ElementTimestampStruct eds = currentTime.get();
             long millis = eds.currentTimestamp.getTime();
@@ -279,7 +288,8 @@ public class ExtendedAccessLogValve
         public RequestHeaderElement(String header) {
             this.header = header;
         }
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             buf.append(wrap(request.getHeader(header)));
         }
@@ -292,7 +302,8 @@ public class ExtendedAccessLogValve
             this.header = header;
         }
         
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             buf.append(wrap(response.getHeader(header)));
         }
@@ -304,7 +315,8 @@ public class ExtendedAccessLogValve
         public ServletContextElement(String attribute) {
             this.attribute = attribute;
         }
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             buf.append(wrap(request.getContext().getServletContext()
                     .getAttribute(attribute)));
@@ -317,7 +329,8 @@ public class ExtendedAccessLogValve
         public CookieElement(String name) {
             this.name = name;
         }
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             Cookie[] c = request.getCookies();
             for (int i = 0; c != null && i < c.length; i++) {
@@ -338,21 +351,23 @@ public class ExtendedAccessLogValve
             this.header = header;
         }
         
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
-           if (null != response) {
-                String[] values = response.getHeaderValues(header);
-                if(values.length > 0) {
-                    StringBuffer buffer = new StringBuffer();
-                    for (int i = 0; i < values.length; i++) {
-                        String string = values[i];
-                        buffer.append(string) ;
-                        if(i+1<values.length)
+            if (null != response) {
+                Iterator<String> iter = response.getHeaders(header).iterator();
+                if (iter.hasNext()) {
+                    StringBuilder buffer = new StringBuilder();
+                    boolean first = true;
+                    while (iter.hasNext()) {
+                        if (!first) {
                             buffer.append(",");
+                        }
+                        buffer.append(iter.next());
                     }
                     buf.append(wrap(buffer.toString()));
-                    return ;
                 }
+                return ;
             }
             buf.append("-");
         }
@@ -365,7 +380,8 @@ public class ExtendedAccessLogValve
             this.attribute = attribute;
         }
         
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             buf.append(wrap(request.getAttribute(attribute)));
         }        
@@ -377,7 +393,8 @@ public class ExtendedAccessLogValve
         public SessionAttributeElement(String attribute) {
             this.attribute = attribute;
         }
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             HttpSession session = null;
             if (request != null) {
@@ -404,15 +421,16 @@ public class ExtendedAccessLogValve
             return URLEncoder.encode(value);
         }   
         
-        public void addElement(StringBuffer buf, Date date, Request request,
+        @Override
+        public void addElement(StringBuilder buf, Date date, Request request,
                 Response response, long time) {
             buf.append(wrap(urlEncode(request.getParameter(parameter))));
         }
     }
     
-    protected class PatternTokenizer {
+    protected static class PatternTokenizer {
         private StringReader sr = null;
-        private StringBuffer buf = new StringBuffer();
+        private StringBuilder buf = new StringBuilder();
         private boolean ended = false;
         private boolean subToken;
         private boolean parameter;
@@ -442,22 +460,22 @@ public class ExtendedAccessLogValve
                 switch (c) {
                 case ' ':
                     result = buf.toString();
-                    buf = new StringBuffer();
+                    buf = new StringBuilder();
                     buf.append((char) c);
                     return result;
                 case '-':
                     result = buf.toString();
-                    buf = new StringBuffer();
+                    buf = new StringBuilder();
                     subToken = true;
                     return result;
                 case '(':
                     result = buf.toString();
-                    buf = new StringBuffer();
+                    buf = new StringBuilder();
                     parameter = true;
                     return result;
                 case ')':
                     result = buf.toString();
-                    buf = new StringBuffer();
+                    buf = new StringBuilder();
                     break;
                 default:
                     buf.append((char) c);
@@ -482,7 +500,7 @@ public class ExtendedAccessLogValve
             while (c != -1) {
                 if (c == ')') {
                     result = buf.toString();
-                    buf = new StringBuffer();
+                    buf = new StringBuilder();
                     return result;
                 }
                 buf.append((char) c);
@@ -494,10 +512,10 @@ public class ExtendedAccessLogValve
         public String getWhiteSpaces() throws IOException {
             if(isEnded())
                 return "" ;
-            StringBuffer whiteSpaces = new StringBuffer();
+            StringBuilder whiteSpaces = new StringBuilder();
             if (buf.length() > 0) {
                 whiteSpaces.append(buf);
-                buf = new StringBuffer();
+                buf = new StringBuilder();
             }
             int c = sr.read();
             while (Character.isWhitespace((char) c)) {
@@ -517,7 +535,7 @@ public class ExtendedAccessLogValve
         }
         
         public String getRemains() throws IOException {
-            StringBuffer remains = new StringBuffer();
+            StringBuilder remains = new StringBuilder();
             for(int c = sr.read(); c != -1; c = sr.read()) {
                 remains.append((char) c);
             }
@@ -526,6 +544,7 @@ public class ExtendedAccessLogValve
         
     }
     
+    @Override
     protected AccessLogElement[] createLogElements() {
         if (log.isDebugEnabled()) {
             log.debug("decodePattern, pattern =" + pattern);
@@ -602,12 +621,14 @@ public class ExtendedAccessLogValve
                 return new LocalAddrElement();
             } else if ("dns".equals(nextToken)) {
                 return new AccessLogElement() {
-                    public void addElement(StringBuffer buf, Date date,
+                    @Override
+                    public void addElement(StringBuilder buf, Date date,
                             Request request, Response response, long time) {
                         String value;
                         try {
                             value = InetAddress.getLocalHost().getHostName();
                         } catch (Throwable e) {
+                            ExceptionUtils.handleThrowable(e);
                             value = "localhost";
                         }
                         buf.append(value);
@@ -640,7 +661,8 @@ public class ExtendedAccessLogValve
                         return new RequestURIElement();
                     } else if ("query".equals(token)) {
                         return new AccessLogElement() {
-                            public void addElement(StringBuffer buf, Date date,
+                            @Override
+                            public void addElement(StringBuilder buf, Date date,
                                     Request request, Response response,
                                     long time) {
                                 String query = request.getQueryString();
@@ -654,7 +676,8 @@ public class ExtendedAccessLogValve
                     }
                 } else {
                     return new AccessLogElement() {
-                        public void addElement(StringBuffer buf, Date date,
+                        @Override
+                        public void addElement(StringBuilder buf, Date date,
                                 Request request, Response response, long time) {
                             String query = request.getQueryString();
                             if (query == null) {
@@ -707,7 +730,7 @@ public class ExtendedAccessLogValve
         throws IOException {
         String token = null;
         if (tokenizer.hasSubToken()) {
-            token = tokenizer.getToken();
+            tokenizer.getToken();
             return new StringElement("-");
         } else if (tokenizer.hasParameter()) {
             tokenizer.getParameter();
@@ -724,6 +747,10 @@ public class ExtendedAccessLogValve
             return null;
         }
         String token = tokenizer.getToken();
+        if ("threadname".equals(token)) {
+            return new ThreadNameElement();
+        }
+
         if (!tokenizer.hasParameter()) {
             log.error("x param in wrong format. Needs to be 'x-#(...)' read the docs!");
             return null;
@@ -756,28 +783,32 @@ public class ExtendedAccessLogValve
     protected AccessLogElement getServletRequestElement(String parameter) {
         if ("authType".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(request.getAuthType()));
                 }
             };
         } else if ("remoteUser".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(request.getRemoteUser()));
                 }
             };
         } else if ("requestedSessionId".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(request.getRequestedSessionId()));
                 }
             };
         } else if ("requestedSessionIdFromCookie".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(""
                             + request.isRequestedSessionIdFromCookie()));
@@ -785,49 +816,56 @@ public class ExtendedAccessLogValve
             };
         } else if ("requestedSessionIdValid".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap("" + request.isRequestedSessionIdValid()));
                 }
             };
         } else if ("contentLength".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap("" + request.getContentLength()));
                 }
             };
         } else if ("characterEncoding".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(request.getCharacterEncoding()));
                 }
             };
         } else if ("locale".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(request.getLocale()));
                 }
             };
         } else if ("protocol".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap(request.getProtocol()));
                 }
             };
         } else if ("scheme".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(request.getScheme());
                 }
             };
         } else if ("secure".equals(parameter)) {
             return new AccessLogElement() {
-                public void addElement(StringBuffer buf, Date date,
+                @Override
+                public void addElement(StringBuilder buf, Date date,
                         Request request, Response response, long time) {
                     buf.append(wrap("" + request.isSecure()));
                 }
@@ -844,7 +882,7 @@ public class ExtendedAccessLogValve
         private String currentTimestampString;
         
         ElementTimestampStruct(String format) {
-            currentTimestampFormat = new SimpleDateFormat(format);
+            currentTimestampFormat = new SimpleDateFormat(format, Locale.US);
             currentTimestampFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
         }
     }
