@@ -31,7 +31,6 @@ import org.apache.catalina.Store;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.session.PersistentManager;
-import org.apache.catalina.util.StringManager;
 
 
 /**
@@ -44,12 +43,15 @@ import org.apache.catalina.util.StringManager;
  *                              per session at any one time.
  *
  * @author Jean-Frederic Clere
- * @version $Id: PersistentValve.java 939353 2010-04-29 15:50:43Z kkolinko $
+ * @version $Id: PersistentValve.java 1133193 2011-06-07 23:12:18Z markt $
  */
 
-public class PersistentValve
-    extends ValveBase {
+public class PersistentValve extends ValveBase {
 
+    //------------------------------------------------------ Constructor
+    public PersistentValve() {
+        super(true);
+    }
 
     // ----------------------------------------------------- Instance Variables
 
@@ -61,19 +63,13 @@ public class PersistentValve
         "org.apache.catalina.valves.PersistentValve/1.0";
 
 
-    /**
-     * The string manager for this package.
-     */
-    private static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-
     // ------------------------------------------------------------- Properties
 
 
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -95,6 +91,7 @@ public class PersistentValve
      * @exception IOException if an input/output error occurred
      * @exception ServletException if a servlet error occurred
      */
+    @Override
     public void invoke(Request request, Response response)
         throws IOException, ServletException {
 
@@ -149,43 +146,52 @@ public class PersistentValve
         // Ask the next valve to process the request.
         getNext().invoke(request, response);
 
-        // Read the sessionid after the response.
-        // HttpSession hsess = hreq.getSession(false);
-        Session hsess;
-        try {
-            hsess = request.getSessionInternal();
-        } catch (Exception ex) {
-            hsess = null;
-        }
-        String newsessionId = null;
-        if (hsess!=null)
-            newsessionId = hsess.getIdInternal();
-
-        if (container.getLogger().isDebugEnabled())
-            container.getLogger().debug("newsessionId: " + newsessionId);
-        if (newsessionId!=null) {
-            /* store the session in the store and remove it from the manager */
-            if (manager instanceof PersistentManager) {
-                Session session = manager.findSession(newsessionId);
-                Store store = ((PersistentManager) manager).getStore();
-                if (store != null && session!=null &&
-                    session.isValid() &&
-                    !isSessionStale(session, System.currentTimeMillis())) {
-                    // ((StandardSession)session).passivate();
-                    store.save(session);
-                    ((PersistentManager) manager).removeSuper(session);
-                    session.recycle();
+        // If still processing async, don't try to store the session
+        // TODO: Are there some async states where it is would be safe to store
+        // the session?
+        if (!request.isAsync()) {
+            // Read the sessionid after the response.
+            // HttpSession hsess = hreq.getSession(false);
+            Session hsess;
+            try {
+                hsess = request.getSessionInternal();
+            } catch (Exception ex) {
+                hsess = null;
+            }
+            String newsessionId = null;
+            if (hsess!=null)
+                newsessionId = hsess.getIdInternal();
+    
+            if (container.getLogger().isDebugEnabled())
+                container.getLogger().debug("newsessionId: " + newsessionId);
+            if (newsessionId!=null) {
+                /* store the session and remove it from the manager */
+                if (manager instanceof PersistentManager) {
+                    Session session = manager.findSession(newsessionId);
+                    Store store = ((PersistentManager) manager).getStore();
+                    if (store != null && session!=null &&
+                        session.isValid() &&
+                        !isSessionStale(session, System.currentTimeMillis())) {
+                        // ((StandardSession)session).passivate();
+                        store.save(session);
+                        ((PersistentManager) manager).removeSuper(session);
+                        session.recycle();
+                    } else {
+                        if (container.getLogger().isDebugEnabled())
+                            container.getLogger().debug("newsessionId store: " +
+                                    store + " session: " + session +
+                                    " valid: " +
+                                    (session == null ? "N/A" : Boolean.toString(
+                                            session.isValid())) +
+                                    " stale: " + isSessionStale(session,
+                                            System.currentTimeMillis()));
+    
+                    }
                 } else {
                     if (container.getLogger().isDebugEnabled())
-                        container.getLogger().debug("newsessionId store: " + store + " session: " +
-                                session + " valid: " + session.isValid() +
-                                " Staled: " +
-                                isSessionStale(session, System.currentTimeMillis()));
-
+                        container.getLogger().debug("newsessionId Manager: " +
+                                manager);
                 }
-            } else {
-                if (container.getLogger().isDebugEnabled())
-                    container.getLogger().debug("newsessionId Manager: " + manager);
             }
         }
     }
@@ -198,12 +204,14 @@ public class PersistentValve
      */
     protected boolean isSessionStale(Session session, long timeNow) {
  
-        int maxInactiveInterval = session.getMaxInactiveInterval();
-        if (maxInactiveInterval >= 0) {
-            int timeIdle = // Truncate, do not round up
-                (int) ((timeNow - session.getLastAccessedTime()) / 1000L);
-            if (timeIdle >= maxInactiveInterval)
-                return true;
+        if (session != null) {
+            int maxInactiveInterval = session.getMaxInactiveInterval();
+            if (maxInactiveInterval >= 0) {
+                int timeIdle = // Truncate, do not round up
+                    (int) ((timeNow - session.getThisAccessedTime()) / 1000L);
+                if (timeIdle >= maxInactiveInterval)
+                    return true;
+            }
         }
  
         return false;
