@@ -18,22 +18,17 @@
 package org.apache.catalina.valves;
 
 import java.io.IOException;
-import java.util.ArrayList;
+import java.util.Enumeration;
 import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.regex.Pattern;
-import java.util.regex.PatternSyntaxException;
 
 import javax.servlet.ServletException;
-import javax.servlet.ServletRequest;
 
-import org.apache.tomcat.util.res.StringManager;
+import org.apache.catalina.AccessLog;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
-import org.apache.catalina.valves.Constants;
-import org.apache.catalina.valves.RequestFilterValve;
-import org.apache.catalina.valves.ValveBase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 
@@ -87,11 +82,14 @@ import org.apache.juli.logging.LogFactory;
  * </tr>
  * <tr>
  * <td>internalProxies</td>
- * <td>List of internal proxies ip adress. If they appear in the <code>remoteIpHeader</code> value, they will be trusted and will not appear
+ * <td>Regular expression that matches the IP addresses of internal proxies.
+ * If they appear in the <code>remoteIpHeader</code> value, they will be
+ * trusted and will not appear
  * in the <code>proxiesHeader</code> value</td>
  * <td>RemoteIPInternalProxy</td>
- * <td>Comma delimited list of regular expressions (in the syntax supported by the {@link java.util.regex.Pattern} library)</td>
- * <td>10\.\d{1,3}\.\d{1,3}\.\d{1,3}, 192\.168\.\d{1,3}\.\d{1,3}, 169\.254\.\d{1,3}\.\d{1,3}, 127\.\d{1,3}\.\d{1,3}\.\d{1,3} <br/>
+ * <td>Regular expression (in the syntax supported by
+ * {@link java.util.regex.Pattern java.util.regex})</td>
+ * <td>10\.\d{1,3}\.\d{1,3}\.\d{1,3}|192\.168\.\d{1,3}\.\d{1,3}|169\.254\.\d{1,3}\.\d{1,3}|127\.\d{1,3}\.\d{1,3}\.\d{1,3}<br/>
  * By default, 10/8, 192.168/16, 169.254/16 and 127/8 are allowed ; 172.16/12 has not been enabled by default because it is complex to
  * describe with regular expressions</td>
  * </tr>
@@ -106,10 +104,12 @@ import org.apache.juli.logging.LogFactory;
  * </tr>
  * <tr>
  * <td>trustedProxies</td>
- * <td>List of trusted proxies ip adress. If they appear in the <code>remoteIpHeader</code> value, they will be trusted and will appear
- * in the <code>proxiesHeader</code> value</td>
+ * <td>Regular expression that matches the IP addresses of trusted proxies.
+ * If they appear in the <code>remoteIpHeader</code> value, they will be
+ * trusted and will appear in the <code>proxiesHeader</code> value</td>
  * <td>RemoteIPTrustedProxy</td>
- * <td>Comma delimited list of regular expressions (in the syntax supported by the {@link java.util.regex.Pattern} library)</td>
+ * <td>Regular expression (in the syntax supported by
+ * {@link java.util.regex.Pattern java.util.regex})</td>
  * <td>&nbsp;</td>
  * </tr>
  * <tr>
@@ -128,14 +128,14 @@ import org.apache.juli.logging.LogFactory;
  * </tr>
  * <tr>
  * <td>httpServerPort</td>
- * <td>Value returned by {@link ServletRequest#getServerPort()} when the <code>protocolHeader</code> indicates <code>http</code> protocol</td>
+ * <td>Value returned by {@link javax.servlet.ServletRequest#getServerPort()} when the <code>protocolHeader</code> indicates <code>http</code> protocol</td>
  * <td>N/A</td>
  * <td>integer</td>
  * <td>80</td>
  * </tr>
  * <tr>
  * <td>httpsServerPort</td>
- * <td>Value returned by {@link ServletRequest#getServerPort()} when the <code>protocolHeader</code> indicates <code>https</code> protocol</td>
+ * <td>Value returned by {@link javax.servlet.ServletRequest#getServerPort()} when the <code>protocolHeader</code> indicates <code>https</code> protocol</td>
  * <td>N/A</td>
  * <td>integer</td>
  * <td>443</td>
@@ -164,7 +164,7 @@ import org.apache.juli.logging.LogFactory;
  * <code><pre>
  * &lt;Valve 
  *   className="org.apache.catalina.valves.RemoteIpValve"
- *   internalProxies="192\.168\.0\.10, 192\.168\.0\.11"
+ *   internalProxies="192\.168\.0\.10|192\.168\.0\.11"
  *   remoteIpHeader="x-forwarded-for"
  *   remoteIpProxiesHeader="x-forwarded-by"
  *   protocolHeader="x-forwarded-proto"
@@ -226,10 +226,10 @@ import org.apache.juli.logging.LogFactory;
  * <code><pre>
  * &lt;Valve 
  *   className="org.apache.catalina.valves.RemoteIpValve"
- *   internalProxies="192\.168\.0\.10, 192\.168\.0\.11"
+ *   internalProxies="192\.168\.0\.10|192\.168\.0\.11"
  *   remoteIpHeader="x-forwarded-for"
  *   remoteIpProxiesHeader="x-forwarded-by"
- *   trustedProxies="proxy1, proxy2"
+ *   trustedProxies="proxy1|proxy2"
  *   /&gt;</pre></code>
  * <p>
  * Request values:
@@ -268,10 +268,10 @@ import org.apache.juli.logging.LogFactory;
  * <code><pre>
  * &lt;Valve 
  *   className="org.apache.catalina.valves.RemoteIpValve"
- *   internalProxies="192\.168\.0\.10, 192\.168\.0\.11"
+ *   internalProxies="192\.168\.0\.10|192\.168\.0\.11"
  *   remoteIpHeader="x-forwarded-for"
  *   remoteIpProxiesHeader="x-forwarded-by"
- *   trustedProxies="proxy1, proxy2"
+ *   trustedProxies="proxy1|proxy2"
  *   /&gt;</pre></code>
  * <p>
  * Request values:
@@ -311,10 +311,10 @@ import org.apache.juli.logging.LogFactory;
  * <code><pre>
  * &lt;Valve 
  *   className="org.apache.catalina.valves.RemoteIpValve"
- *   internalProxies="192\.168\.0\.10, 192\.168\.0\.11"
+ *   internalProxies="192\.168\.0\.10|192\.168\.0\.11"
  *   remoteIpHeader="x-forwarded-for"
  *   remoteIpProxiesHeader="x-forwarded-by"
- *   trustedProxies="proxy1, proxy2"
+ *   trustedProxies="proxy1|proxy2"
  *   /&gt;</pre></code>
  * <p>
  * Request values:
@@ -361,35 +361,12 @@ public class RemoteIpValve extends ValveBase {
     /**
      * Logger
      */
-    private static Log log = LogFactory.getLog(RemoteIpValve.class);
+    private static final Log log = LogFactory.getLog(RemoteIpValve.class);
     
     /**
-     * The StringManager for this package.
-     */
-    protected static StringManager sm = StringManager.getManager(Constants.Package);
-    
-    /**
-     * Convert a given comma delimited list of regular expressions into an array of compiled {@link Pattern}
+     * Convert a given comma delimited String into an array of String
      * 
-     * @return array of patterns (not <code>null</code>)
-     */
-    protected static Pattern[] commaDelimitedListToPatternArray(String commaDelimitedPatterns) {
-        String[] patterns = commaDelimitedListToStringArray(commaDelimitedPatterns);
-        List<Pattern> patternsList = new ArrayList<Pattern>();
-        for (String pattern : patterns) {
-            try {
-                patternsList.add(Pattern.compile(pattern));
-            } catch (PatternSyntaxException e) {
-                throw new IllegalArgumentException(sm.getString("remoteIpValve.syntax", pattern), e);
-            }
-        }
-        return patternsList.toArray(new Pattern[0]);
-    }
-    
-    /**
-     * Convert a given comma delimited list of regular expressions into an array of String
-     * 
-     * @return array of patterns (non <code>null</code>)
+     * @return array of String (non <code>null</code>)
      */
     protected static String[] commaDelimitedListToStringArray(String commaDelimitedStrings) {
         return (commaDelimitedStrings == null || commaDelimitedStrings.length() == 0) ? new String[0] : commaSeparatedValuesPattern
@@ -416,18 +393,6 @@ public class RemoteIpValve extends ValveBase {
         return result.toString();
     }
     
-    /**
-     * Return <code>true</code> if the given <code>str</code> matches at least one of the given <code>patterns</code>.
-     */
-    protected static boolean matchesOne(String str, Pattern... patterns) {
-        for (Pattern pattern : patterns) {
-            if (pattern.matcher(str).matches()) {
-                return true;
-            }
-        }
-        return false;
-    }
-    
     /**
      * @see #setHttpServerPort(int)
      */
@@ -438,13 +403,16 @@ public class RemoteIpValve extends ValveBase {
      */
     private int httpsServerPort = 443;
     
+    private boolean changeLocalPort = false;
+
     /**
      * @see #setInternalProxies(String)
      */
-    private Pattern[] internalProxies = new Pattern[] {
-        Pattern.compile("10\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}"), Pattern.compile("192\\.168\\.\\d{1,3}\\.\\d{1,3}"),
-        Pattern.compile("169\\.254\\.\\d{1,3}\\.\\d{1,3}"), Pattern.compile("127\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}")
-    };
+    private Pattern internalProxies = Pattern.compile(
+            "10\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}|" +
+            "192\\.168\\.\\d{1,3}\\.\\d{1,3}|" +
+            "169\\.254\\.\\d{1,3}\\.\\d{1,3}|" +
+            "127\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}");
     
     /**
      * @see #setProtocolHeader(String)
@@ -456,6 +424,8 @@ public class RemoteIpValve extends ValveBase {
      */
     private String protocolHeaderHttpsValue = "https";
     
+    private String portHeader = null;
+
     /**
      * @see #setProxiesHeader(String)
      */
@@ -465,11 +435,27 @@ public class RemoteIpValve extends ValveBase {
      * @see #setRemoteIpHeader(String)
      */
     private String remoteIpHeader = "X-Forwarded-For";
-    
+
+    /**
+     * @see #setRequestAttributesEnabled(boolean)
+     */
+    private boolean requestAttributesEnabled = true;
+
     /**
      * @see RemoteIpValve#setTrustedProxies(String)
      */
-    private Pattern[] trustedProxies = new Pattern[0];
+    private Pattern trustedProxies = null;
+    
+
+    /**
+     * Default constructor that ensures {@link ValveBase#ValveBase(boolean)} is
+     * called with <code>true</code>.
+     */
+    public RemoteIpValve() {
+        // Async requests are supported with this valve
+        super(true);
+    }
+
     
     public int getHttpsServerPort() {
         return httpsServerPort;
@@ -479,6 +465,36 @@ public class RemoteIpValve extends ValveBase {
         return httpServerPort;
     }
     
+    public boolean isChangeLocalPort() {
+        return changeLocalPort;
+    }
+
+    public void setChangeLocalPort(boolean changeLocalPort) {
+        this.changeLocalPort = changeLocalPort;
+    }
+
+    /**
+     * Obtain the name of the HTTP header used to override the value returned
+     * by {@link Request#getServerPort()} and (optionally depending on {link
+     * {@link #isChangeLocalPort()} {@link Request#getLocalPort()}.
+     * 
+     * @return  The HTTP header name
+     */
+    public String getPortHeader() {
+        return portHeader;
+    }
+
+    /**
+     * Set the name of the HTTP header used to override the value returned
+     * by {@link Request#getServerPort()} and (optionally depending on {link
+     * {@link #isChangeLocalPort()} {@link Request#getLocalPort()}.
+     * 
+     * @param   portHeader  The HTTP header name
+     */
+    public void setPortHeader(String portHeader) {
+        this.portHeader = portHeader;
+    }
+
     /**
      * Return descriptive information about this Valve implementation.
      */
@@ -489,14 +505,13 @@ public class RemoteIpValve extends ValveBase {
     
     /**
      * @see #setInternalProxies(String)
-     * @return comma delimited list of internal proxies
+     * @return Regular expression that defines the internal proxies
      */
     public String getInternalProxies() {
-        List<String> internalProxiesAsStringList = new ArrayList<String>();
-        for (Pattern internalProxyPattern : internalProxies) {
-            internalProxiesAsStringList.add(String.valueOf(internalProxyPattern));
+        if (internalProxies == null) {
+            return null;
         }
-        return listToCommaDelimitedString(internalProxiesAsStringList);
+        return internalProxies.toString();
     }
     
     /**
@@ -531,16 +546,24 @@ public class RemoteIpValve extends ValveBase {
         return remoteIpHeader;
     }
 
+    /**
+     * @see #setRequestAttributesEnabled(boolean)
+     * @return <code>true</code> if the attributes will be logged, otherwise
+     *         <code>false</code>
+     */
+    public boolean getRequestAttributesEnabled() {
+        return requestAttributesEnabled;
+    }
+
     /**
      * @see #setTrustedProxies(String)
-     * @return comma delimited list of trusted proxies
+     * @return Regular expression that defines the trusted proxies
      */
     public String getTrustedProxies() {
-        List<String> trustedProxiesAsStringList = new ArrayList<String>();
-        for (Pattern trustedProxy : trustedProxies) {
-            trustedProxiesAsStringList.add(String.valueOf(trustedProxy));
+        if (trustedProxies == null) {
+            return null;
         }
-        return listToCommaDelimitedString(trustedProxiesAsStringList);
+        return trustedProxies.toString();
     }
     
     /**
@@ -554,20 +577,31 @@ public class RemoteIpValve extends ValveBase {
         final boolean originalSecure = request.isSecure();
         final int originalServerPort = request.getServerPort();
         
-        if (matchesOne(originalRemoteAddr, internalProxies)) {
+        if (internalProxies !=null &&
+                internalProxies.matcher(originalRemoteAddr).matches()) {
             String remoteIp = null;
             // In java 6, proxiesHeaderValue should be declared as a java.util.Deque
             LinkedList<String> proxiesHeaderValue = new LinkedList<String>();
+            StringBuilder concatRemoteIpHeaderValue = new StringBuilder();
             
-            String[] remoteIpHeaderValue = commaDelimitedListToStringArray(request.getHeader(remoteIpHeader));
+            for (Enumeration<String> e = request.getHeaders(remoteIpHeader); e.hasMoreElements();) {
+                if (concatRemoteIpHeaderValue.length() > 0) {
+                    concatRemoteIpHeaderValue.append(", ");
+                }
+
+                concatRemoteIpHeaderValue.append(e.nextElement());
+            }
+
+            String[] remoteIpHeaderValue = commaDelimitedListToStringArray(concatRemoteIpHeaderValue.toString());
             int idx;
             // loop on remoteIpHeaderValue to find the first trusted remote ip and to build the proxies chain
             for (idx = remoteIpHeaderValue.length - 1; idx >= 0; idx--) {
                 String currentRemoteIp = remoteIpHeaderValue[idx];
                 remoteIp = currentRemoteIp;
-                if (matchesOne(currentRemoteIp, internalProxies)) {
+                if (internalProxies.matcher(currentRemoteIp).matches()) {
                     // do nothing, internalProxies IPs are not appended to the
-                } else if (matchesOne(currentRemoteIp, trustedProxies)) {
+                } else if (trustedProxies != null &&
+                        trustedProxies.matcher(currentRemoteIp).matches()) {
                     proxiesHeaderValue.addFirst(currentRemoteIp);
                 } else {
                     idx--; // decrement idx because break statement doesn't do it
@@ -611,13 +645,13 @@ public class RemoteIpValve extends ValveBase {
                     // use request.coyoteRequest.scheme instead of request.setScheme() because request.setScheme() is no-op in Tomcat 6.0
                     request.getCoyoteRequest().scheme().setString("https");
                     
-                    request.setServerPort(httpsServerPort);
+                    setPorts(request, httpsServerPort);
                 } else {
                     request.setSecure(false);
                     // use request.coyoteRequest.scheme instead of request.setScheme() because request.setScheme() is no-op in Tomcat 6.0
                     request.getCoyoteRequest().scheme().setString("http");
                     
-                    request.setServerPort(httpServerPort);
+                    setPorts(request, httpServerPort);
                 }
             }
             
@@ -633,6 +667,16 @@ public class RemoteIpValve extends ValveBase {
                         + request.getRemoteAddr() + "'");
             }
         }
+        if (requestAttributesEnabled) {
+            request.setAttribute(AccessLog.REMOTE_ADDR_ATTRIBUTE,
+                    request.getRemoteAddr());
+            request.setAttribute(AccessLog.REMOTE_HOST_ATTRIBUTE,
+                    request.getRemoteHost());
+            request.setAttribute(AccessLog.PROTOCOL_ATTRIBUTE,
+                    request.getProtocol());
+            request.setAttribute(AccessLog.SERVER_PORT_ATTRIBUTE,
+                    Integer.valueOf(request.getServerPort()));
+        }
         try {
             getNext().invoke(request, response);
         } finally {
@@ -647,6 +691,28 @@ public class RemoteIpValve extends ValveBase {
             request.setServerPort(originalServerPort);
         }
     }
+
+    private void setPorts(Request request, int defaultPort) {
+        int port = defaultPort;
+        if (portHeader != null) {
+            String portHeaderValue = request.getHeader(portHeader);
+            if (portHeaderValue != null) {
+                try {
+                    port = Integer.parseInt(portHeaderValue);
+                } catch (NumberFormatException nfe) {
+                    if (log.isDebugEnabled()) {
+                        log.debug(sm.getString(
+                                "remoteIpValve.invalidPortHeader",
+                                portHeaderValue, portHeader), nfe);
+                    }
+                }
+            }
+        }
+        request.setServerPort(port);
+        if (changeLocalPort) {
+            request.setLocalPort(port);
+        }
+    }
     
     /**
      * <p>
@@ -674,14 +740,18 @@ public class RemoteIpValve extends ValveBase {
     
     /**
      * <p>
-     * Comma delimited list of internal proxies. Can be expressed with regular expressions.
+     * Regular expression that defines the internal proxies.
      * </p>
      * <p>
-     * Default value : 10\.\d{1,3}\.\d{1,3}\.\d{1,3}, 192\.168\.\d{1,3}\.\d{1,3}, 127\.\d{1,3}\.\d{1,3}\.\d{1,3}
+     * Default value : 10\.\d{1,3}\.\d{1,3}\.\d{1,3}|192\.168\.\d{1,3}\.\d{1,3}|169\.254.\d{1,3}.\d{1,3}|127\.\d{1,3}\.\d{1,3}\.\d{1,3}
      * </p>
      */
-    public void setInternalProxies(String commaDelimitedInternalProxies) {
-        this.internalProxies = commaDelimitedListToPatternArray(commaDelimitedInternalProxies);
+    public void setInternalProxies(String internalProxies) {
+        if (internalProxies == null || internalProxies.length() == 0) {
+            this.internalProxies = null;
+        } else {
+            this.internalProxies = Pattern.compile(internalProxies);
+        }
     }
     
     /**
@@ -746,16 +816,42 @@ public class RemoteIpValve extends ValveBase {
         this.remoteIpHeader = remoteIpHeader;
     }
     
+    /**
+     * Should this valve set request attributes for IP address, Hostname,
+     * protocol and port used for the request? This are typically used in
+     * conjunction with the {@link AccessLog} which will otherwise log the
+     * original values. Default is <code>true</code>.
+     * 
+     * The attributes set are:
+     * <ul>
+     * <li>org.apache.catalina.RemoteAddr</li>
+     * <li>org.apache.catalina.RemoteHost</li>
+     * <li>org.apache.catalina.Protocol</li>
+     * <li>org.apache.catalina.ServerPost</li>
+     * </ul>
+     * 
+     * @param requestAttributesEnabled  <code>true</code> causes the attributes
+     *                                  to be set, <code>false</code> disables
+     *                                  the setting of the attributes. 
+     */
+    public void setRequestAttributesEnabled(boolean requestAttributesEnabled) {
+        this.requestAttributesEnabled = requestAttributesEnabled;
+    }
+
     /**
      * <p>
-     * Comma delimited list of proxies that are trusted when they appear in the {@link #remoteIpHeader} header. Can be expressed as a
-     * regular expression.
+     * Regular expression defining proxies that are trusted when they appear in
+     * the {@link #remoteIpHeader} header.
      * </p>
      * <p>
      * Default value : empty list, no external proxy is trusted.
      * </p>
      */
-    public void setTrustedProxies(String commaDelimitedTrustedProxies) {
-        this.trustedProxies = commaDelimitedListToPatternArray(commaDelimitedTrustedProxies);
+    public void setTrustedProxies(String trustedProxies) {
+        if (trustedProxies == null || trustedProxies.length() == 0) {
+            this.trustedProxies = null;
+        } else {
+            this.trustedProxies = Pattern.compile(trustedProxies);
+        }
     }
 }
