@@ -295,7 +295,7 @@ public class JspCompilationContext {
                 // May not be in a JAR in some IDE environments
                 result = context.getResource(canonicalURI(res));
             }
-        } else if (res.startsWith("jar:file:")) {
+        } else if (res.startsWith("jar:jndi:")) {
                 // This is a tag file packaged in a jar that is being checked
                 // for a dependency
                 result = new URL(res);
@@ -385,6 +385,11 @@ public class JspCompilationContext {
         return jspUri;
     }
 
+    /**
+     * @deprecated Will be removed in Tomcat 8.0.x. Use
+     * {@link #getLastModified(String)} instead.
+     */
+    @Deprecated
     public long getJspLastModified() {
         long result = -1;
         URLConnection uc = null;
@@ -422,6 +427,44 @@ public class JspCompilationContext {
         return result;
     }
 
+
+    public Long getLastModified(String resource) {
+        long result = -1;
+        URLConnection uc = null;
+        try {
+            URL jspUrl = getResource(resource);
+            if (jspUrl == null) {
+                incrementRemoved();
+                return Long.valueOf(result);
+            }
+            uc = jspUrl.openConnection();
+            if (uc instanceof JarURLConnection) {
+                result = ((JarURLConnection) uc).getJarEntry().getTime();
+            } else {
+                result = uc.getLastModified();
+            }
+        } catch (IOException e) {
+            if (log.isDebugEnabled()) {
+                log.debug(Localizer.getMessage(
+                        "jsp.error.lastModified", getJspFile()), e);
+            }
+            result = -1;
+        } finally {
+            if (uc != null) {
+                try {
+                    uc.getInputStream().close();
+                } catch (IOException e) {
+                    if (log.isDebugEnabled()) {
+                        log.debug(Localizer.getMessage(
+                                "jsp.error.lastModified", getJspFile()), e);
+                    }
+                    result = -1;
+                }
+            }
+        }
+        return Long.valueOf(result);
+    }
+
     public boolean isTagFile() {
         return isTagFile;
     }
@@ -666,7 +709,7 @@ public class JspCompilationContext {
     protected boolean makeOutputDir() {
         synchronized(outputDirLock) {
             File outDirFile = new File(outputDir);
-            return (outDirFile.exists() || outDirFile.mkdirs());
+            return (outDirFile.mkdirs() || outDirFile.isDirectory());
         }
     }
 
