@@ -22,12 +22,12 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Hashtable;
 import java.util.Iterator;
+import java.util.Locale;
 
 import javax.el.ELException;
 import javax.el.ExpressionFactory;
 import javax.el.FunctionMapper;
 import javax.servlet.jsp.tagext.FunctionInfo;
-import javax.servlet.jsp.tagext.JspFragment;
 import javax.servlet.jsp.tagext.PageData;
 import javax.servlet.jsp.tagext.TagAttributeInfo;
 import javax.servlet.jsp.tagext.TagData;
@@ -36,7 +36,6 @@ import javax.servlet.jsp.tagext.TagInfo;
 import javax.servlet.jsp.tagext.TagLibraryInfo;
 import javax.servlet.jsp.tagext.ValidationMessage;
 
-import org.apache.el.lang.ELSupport;
 import org.apache.jasper.JasperException;
 import org.apache.jasper.el.ELContextImpl;
 import org.xml.sax.Attributes;
@@ -86,11 +85,12 @@ class Validator {
         /*
          * Constructor
          */
-        DirectiveVisitor(Compiler compiler) throws JasperException {
+        DirectiveVisitor(Compiler compiler) {
             this.pageInfo = compiler.getPageInfo();
             this.err = compiler.getErrorDispatcher();
         }
 
+        @Override
         public void visit(Node.IncludeDirective n) throws JasperException {
             // Since pageDirectiveSeen flag only applies to the Current page
             // save it here and restore it after the file is included.
@@ -100,6 +100,7 @@ class Validator {
             pageEncodingSeen = pageEncodingSeenSave;
         }
 
+        @Override
         public void visit(Node.PageDirective n) throws JasperException {
 
             JspUtil.checkAttributes("Page directive", n, pageDirectiveAttrs,
@@ -234,6 +235,7 @@ class Validator {
             pageInfo.addImports(n.getImports());
         }
 
+        @Override
         public void visit(Node.TagDirective n) throws JasperException {
             // Note: Most of the validation is done in TagFileProcessor
             // when it created a TagInfo object from the
@@ -301,12 +303,14 @@ class Validator {
             pageInfo.addImports(n.getImports());
         }
 
+        @Override
         public void visit(Node.AttributeDirective n) throws JasperException {
             // Do nothing, since this attribute directive has already been
             // validated by TagFileProcessor when it created a TagInfo object
             // from the tag file in which the directive appeared
         }
 
+        @Override
         public void visit(Node.VariableDirective n) throws JasperException {
             // Do nothing, since this variable directive has already been
             // validated by TagFileProcessor when it created a TagInfo object
@@ -327,7 +331,7 @@ class Validator {
 
             Node.Root root = pageDir.getRoot();
             String configEnc = root.getJspConfigPageEncoding();
-            String pageDirEnc = thePageDirEnc.toUpperCase();
+            String pageDirEnc = thePageDirEnc.toUpperCase(Locale.ENGLISH);
 
             /*
              * Compare the 'pageEncoding' attribute of the page directive with
@@ -336,7 +340,7 @@ class Validator {
              * "UTF-16LE" as identical.
              */
             if (configEnc != null) {
-                configEnc = configEnc.toUpperCase();
+                configEnc = configEnc.toUpperCase(Locale.ENGLISH);
                 if (!pageDirEnc.equals(configEnc)
                         && (!pageDirEnc.startsWith("UTF-16") || !configEnc
                                 .startsWith("UTF-16"))) {
@@ -356,7 +360,7 @@ class Validator {
              * identical.
              */
             if ((root.isXmlSyntax() && root.isEncodingSpecifiedInProlog()) || root.isBomPresent()) {
-                String pageEnc = root.getPageEncoding().toUpperCase();
+                String pageEnc = root.getPageEncoding().toUpperCase(Locale.ENGLISH);
                 if (!pageDirEnc.equals(pageEnc)
                         && (!pageDirEnc.startsWith("UTF-16") || !pageEnc
                                 .startsWith("UTF-16"))) {
@@ -384,7 +388,7 @@ class Validator {
                 Node.TagDirective pageDir) throws JasperException {
 
             Node.Root root = pageDir.getRoot();
-            String pageDirEnc = thePageDirEnc.toUpperCase();
+            String pageDirEnc = thePageDirEnc.toUpperCase(Locale.ENGLISH);
             /*
              * Compare the 'pageEncoding' attribute of the page directive with
              * the encoding specified in the XML prolog (only for XML syntax,
@@ -393,7 +397,7 @@ class Validator {
              * identical.
              */
             if ((root.isXmlSyntax() && root.isEncodingSpecifiedInProlog()) || root.isBomPresent()) {
-                String pageEnc = root.getPageEncoding().toUpperCase();
+                String pageEnc = root.getPageEncoding().toUpperCase(Locale.ENGLISH);
                 if (!pageDirEnc.equals(pageEnc)
                         && (!pageDirEnc.startsWith("UTF-16") || !pageEnc
                                 .startsWith("UTF-16"))) {
@@ -417,7 +421,7 @@ class Validator {
 
         private ClassLoader loader;
 
-        private final StringBuffer buf = new StringBuffer(32);
+        private final StringBuilder buf = new StringBuilder(32);
 
         private static final JspUtil.ValidAttribute[] jspRootAttrs = {
                 new JspUtil.ValidAttribute("xsi:schemaLocation"),
@@ -432,15 +436,15 @@ class Validator {
                 new JspUtil.ValidAttribute("prefix", true) };
 
         private static final JspUtil.ValidAttribute[] includeActionAttrs = {
-                new JspUtil.ValidAttribute("page", true, true),
+                new JspUtil.ValidAttribute("page", true),
                 new JspUtil.ValidAttribute("flush") };
 
         private static final JspUtil.ValidAttribute[] paramActionAttrs = {
                 new JspUtil.ValidAttribute("name", true),
-                new JspUtil.ValidAttribute("value", true, true) };
+                new JspUtil.ValidAttribute("value", true) };
 
-        private static final JspUtil.ValidAttribute[] forwardActionAttrs = { new JspUtil.ValidAttribute(
-                "page", true, true) };
+        private static final JspUtil.ValidAttribute[] forwardActionAttrs = {
+                new JspUtil.ValidAttribute("page", true) };
 
         private static final JspUtil.ValidAttribute[] getPropertyAttrs = {
                 new JspUtil.ValidAttribute("name", true),
@@ -449,7 +453,7 @@ class Validator {
         private static final JspUtil.ValidAttribute[] setPropertyAttrs = {
                 new JspUtil.ValidAttribute("name", true),
                 new JspUtil.ValidAttribute("property", true),
-                new JspUtil.ValidAttribute("value", false, true),
+                new JspUtil.ValidAttribute("value", false),
                 new JspUtil.ValidAttribute("param") };
 
         private static final JspUtil.ValidAttribute[] useBeanAttrs = {
@@ -457,7 +461,7 @@ class Validator {
                 new JspUtil.ValidAttribute("scope"),
                 new JspUtil.ValidAttribute("class"),
                 new JspUtil.ValidAttribute("type"),
-                new JspUtil.ValidAttribute("beanName", false, true) };
+                new JspUtil.ValidAttribute("beanName", false) };
 
         private static final JspUtil.ValidAttribute[] plugInAttrs = {
                 new JspUtil.ValidAttribute("type", true),
@@ -465,18 +469,19 @@ class Validator {
                 new JspUtil.ValidAttribute("codebase"),
                 new JspUtil.ValidAttribute("align"),
                 new JspUtil.ValidAttribute("archive"),
-                new JspUtil.ValidAttribute("height", false, true),
+                new JspUtil.ValidAttribute("height", false),
                 new JspUtil.ValidAttribute("hspace"),
                 new JspUtil.ValidAttribute("jreversion"),
                 new JspUtil.ValidAttribute("name"),
                 new JspUtil.ValidAttribute("vspace"),
-                new JspUtil.ValidAttribute("width", false, true),
+                new JspUtil.ValidAttribute("width", false),
                 new JspUtil.ValidAttribute("nspluginurl"),
                 new JspUtil.ValidAttribute("iepluginurl") };
 
         private static final JspUtil.ValidAttribute[] attributeAttrs = {
                 new JspUtil.ValidAttribute("name", true),
-                new JspUtil.ValidAttribute("trim") };
+                new JspUtil.ValidAttribute("trim"),
+                new JspUtil.ValidAttribute("omit")};
 
         private static final JspUtil.ValidAttribute[] invokeAttrs = {
                 new JspUtil.ValidAttribute("fragment", true),
@@ -495,6 +500,9 @@ class Validator {
                 new JspUtil.ValidAttribute("doctype-public"),
                 new JspUtil.ValidAttribute("doctype-system") };
 
+        private static final ExpressionFactory EXPRESSION_FACTORY =
+            ExpressionFactory.newInstance();
+
         /*
          * Constructor
          */
@@ -504,21 +512,25 @@ class Validator {
             this.loader = compiler.getCompilationContext().getClassLoader();
         }
 
+        @Override
         public void visit(Node.JspRoot n) throws JasperException {
             JspUtil.checkAttributes("Jsp:root", n, jspRootAttrs, err);
             String version = n.getTextAttribute("version");
-            if (!version.equals("1.2") && !version.equals("2.0") && !version.equals("2.1")) {
+            if (!version.equals("1.2") && !version.equals("2.0") &&
+                    !version.equals("2.1") && !version.equals("2.2")) {
                 err.jspError(n, "jsp.error.jsproot.version.invalid", version);
             }
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.IncludeDirective n) throws JasperException {
             JspUtil.checkAttributes("Include directive", n,
                     includeDirectiveAttrs, err);
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.TaglibDirective n) throws JasperException {
             JspUtil.checkAttributes("Taglib directive", n,
                     taglibDirectiveAttrs, err);
@@ -535,17 +547,18 @@ class Validator {
             }
         }
 
+        @Override
         public void visit(Node.ParamAction n) throws JasperException {
             JspUtil.checkAttributes("Param action", n, paramActionAttrs, err);
             // make sure the value of the 'name' attribute is not a
             // request-time expression
             throwErrorIfExpression(n, "name", "jsp:param");
             n.setValue(getJspAttribute(null, "value", null, null, n
-                    .getAttributeValue("value"), java.lang.String.class, n,
-                    false));
+                    .getAttributeValue("value"), n, false));
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.ParamsAction n) throws JasperException {
             // Make sure we've got at least one nested jsp:param
             Node.Nodes subElems = n.getBody();
@@ -555,27 +568,29 @@ class Validator {
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.IncludeAction n) throws JasperException {
             JspUtil.checkAttributes("Include action", n, includeActionAttrs,
                     err);
             n.setPage(getJspAttribute(null, "page", null, null, n
-                    .getAttributeValue("page"), java.lang.String.class, n,
-                    false));
-            visitBody(n);
-        };
-
-        public void visit(Node.ForwardAction n) throws JasperException {
-            JspUtil.checkAttributes("Forward", n, forwardActionAttrs, err);
-            n.setPage(getJspAttribute(null, "page", null, null, n
-                    .getAttributeValue("page"), java.lang.String.class, n,
-                    false));
+                    .getAttributeValue("page"), n, false));
             visitBody(n);
         }
 
+        @Override
+        public void visit(Node.ForwardAction n) throws JasperException {
+            JspUtil.checkAttributes("Forward", n, forwardActionAttrs, err);
+            n.setPage(getJspAttribute(null, "page", null, null, n
+                    .getAttributeValue("page"), n, false));
+            visitBody(n);
+        }
+
+        @Override
         public void visit(Node.GetProperty n) throws JasperException {
             JspUtil.checkAttributes("GetProperty", n, getPropertyAttrs, err);
         }
 
+        @Override
         public void visit(Node.SetProperty n) throws JasperException {
             JspUtil.checkAttributes("SetProperty", n, setPropertyAttrs, err);
             String property = n.getTextAttribute("property");
@@ -583,7 +598,7 @@ class Validator {
             String value = n.getAttributeValue("value");
 
             n.setValue(getJspAttribute(null, "value", null, null, value,
-                    java.lang.Object.class, n, false));
+                    n, false));
 
             boolean valueSpecified = n.getValue() != null;
 
@@ -598,6 +613,7 @@ class Validator {
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.UseBean n) throws JasperException {
             JspUtil.checkAttributes("UseBean", n, useBeanAttrs, err);
 
@@ -618,8 +634,7 @@ class Validator {
                 err.jspError(n, "jsp.error.usebean.noSession");
 
             Node.JspAttribute jattr = getJspAttribute(null, "beanName", null,
-                    null, n.getAttributeValue("beanName"),
-                    java.lang.String.class, n, false);
+                    null, n.getAttributeValue("beanName"), n, false);
             n.setBeanName(jattr);
             if (className != null && jattr != null)
                 err.jspError(n, "jsp.error.usebean.notBoth");
@@ -632,6 +647,7 @@ class Validator {
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.PlugIn n) throws JasperException {
             JspUtil.checkAttributes("Plugin", n, plugInAttrs, err);
 
@@ -656,45 +672,51 @@ class Validator {
                 err.jspError(n, "jsp.error.plugin.nocode");
 
             Node.JspAttribute width = getJspAttribute(null, "width", null,
-                    null, n.getAttributeValue("width"), java.lang.String.class,
-                    n, false);
+                    null, n.getAttributeValue("width"), n, false);
             n.setWidth(width);
 
             Node.JspAttribute height = getJspAttribute(null, "height", null,
-                    null, n.getAttributeValue("height"),
-                    java.lang.String.class, n, false);
+                    null, n.getAttributeValue("height"), n, false);
             n.setHeight(height);
 
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.NamedAttribute n) throws JasperException {
             JspUtil.checkAttributes("Attribute", n, attributeAttrs, err);
+            n.setOmit(getJspAttribute(null, "omit", null, null, n
+                    .getAttributeValue("omit"), n, true));
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.JspBody n) throws JasperException {
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.Declaration n) throws JasperException {
             if (pageInfo.isScriptingInvalid()) {
                 err.jspError(n.getStart(), "jsp.error.no.scriptlets");
             }
         }
 
+        @Override
         public void visit(Node.Expression n) throws JasperException {
             if (pageInfo.isScriptingInvalid()) {
                 err.jspError(n.getStart(), "jsp.error.no.scriptlets");
             }
         }
 
+        @Override
         public void visit(Node.Scriptlet n) throws JasperException {
             if (pageInfo.isScriptingInvalid()) {
                 err.jspError(n.getStart(), "jsp.error.no.scriptlets");
             }
         }
 
+        @Override
         public void visit(Node.ELExpression n) throws JasperException {
             // exit if we are ignoring EL all together
             if (pageInfo.isELIgnored())
@@ -710,7 +732,7 @@ class Validator {
             }
 
             // build expression
-            StringBuffer expr = this.getBuffer();
+            StringBuilder expr = this.getBuffer();
             expr.append(n.getType()).append('{').append(n.getText())
                     .append('}');
             ELNode.Nodes el = ELParser.parse(expr.toString(), pageInfo
@@ -723,6 +745,7 @@ class Validator {
             n.setEL(el);
         }
 
+        @Override
         public void visit(Node.UninterpretedTag n) throws JasperException {
             if (n.getNamedAttributeNodes().size() != 0) {
                 err.jspError(n, "jsp.error.namedAttribute.invalidUse");
@@ -741,8 +764,8 @@ class Validator {
                         }
                     }
                     jspAttrs[i] = getJspAttribute(null, attrs.getQName(i),
-                            attrs.getURI(i), attrs.getLocalName(i), value,
-                            java.lang.Object.class, n, false);
+                            attrs.getURI(i), attrs.getLocalName(i), value, n,
+                            false);
                 }
                 n.setJspAttributes(jspAttrs);
             }
@@ -775,6 +798,7 @@ class Validator {
             return false;
         }
 
+        @Override
         public void visit(Node.CustomTag n) throws JasperException {
 
             TagInfo tagInfo = n.getTagInfo();
@@ -783,7 +807,7 @@ class Validator {
             }
 
             /*
-             * The bodyconet of a SimpleTag cannot be JSP.
+             * The bodycontent of a SimpleTag cannot be JSP.
              */
             if (n.implementsSimpleTag()
                     && tagInfo.getBodyContent().equalsIgnoreCase(
@@ -865,12 +889,14 @@ class Validator {
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.JspElement n) throws JasperException {
 
             Attributes attrs = n.getAttributes();
             if (attrs == null) {
                 err.jspError(n, "jsp.error.jspelement.missing.name");
             }
+            @SuppressWarnings("null") // Exception will have been thrown above
             int xmlAttrLen = attrs.getLength();
 
             Node.Nodes namedAttrs = n.getNamedAttributeNodes();
@@ -887,14 +913,12 @@ class Validator {
                 if ("name".equals(attrs.getLocalName(i))) {
                     n.setNameAttribute(getJspAttribute(null, attrs.getQName(i),
                             attrs.getURI(i), attrs.getLocalName(i), attrs
-                                    .getValue(i), java.lang.String.class, n,
-                            false));
+                                    .getValue(i), n, false));
                 } else {
                     if (jspAttrIndex < jspAttrSize) {
                         jspAttrs[jspAttrIndex++] = getJspAttribute(null, attrs
                                 .getQName(i), attrs.getURI(i), attrs
-                                .getLocalName(i), attrs.getValue(i),
-                                java.lang.Object.class, n, false);
+                                .getLocalName(i), attrs.getValue(i), n, false);
                     }
                 }
             }
@@ -915,6 +939,7 @@ class Validator {
             visitBody(n);
         }
 
+        @Override
         public void visit(Node.JspOutput n) throws JasperException {
             JspUtil.checkAttributes("jsp:output", n, jspOutputAttrs, err);
 
@@ -979,6 +1004,7 @@ class Validator {
             }
         }
 
+        @Override
         public void visit(Node.InvokeAction n) throws JasperException {
 
             JspUtil.checkAttributes("Invoke", n, invokeAttrs, err);
@@ -996,6 +1022,7 @@ class Validator {
             }
         }
 
+        @Override
         public void visit(Node.DoBodyAction n) throws JasperException {
 
             JspUtil.checkAttributes("DoBody", n, doBodyAttrs, err);
@@ -1122,12 +1149,12 @@ class Validator {
                                     }
                                 }
                                 if (tldAttr.isDeferredValue()) {
-                                    // The String litteral must be castable to what is declared as type
+                                    // The String literal must be castable to what is declared as type
                                     // for the attribute
                                     expectedType = tldAttr.getExpectedTypeName();
                                 }
                                 if (expectedType != null) {
-                                    Class expectedClass = String.class;
+                                    Class<?> expectedClass = String.class;
                                     try {
                                         expectedClass = JspUtil.toClass(expectedType, loader);
                                     } catch (ClassNotFoundException e) {
@@ -1135,13 +1162,27 @@ class Validator {
                                             (n, "jsp.error.unknown_attribute_type",
                                              tldAttr.getName(), expectedType);
                                     }
-                                    // Check casting
-                                    try {
-                                        ELSupport.checkType(attrs.getValue(i), expectedClass);
-                                    } catch (Exception e) {
-                                        err.jspError
-                                            (n, "jsp.error.coerce_to_type",
-                                             tldAttr.getName(), expectedType, attrs.getValue(i));
+                                    // Check casting - not possible for all types
+                                    if (String.class.equals(expectedClass) ||
+                                            expectedClass == Long.TYPE ||
+                                            expectedClass == Double.TYPE ||
+                                            expectedClass == Byte.TYPE ||
+                                            expectedClass == Short.TYPE ||
+                                            expectedClass == Integer.TYPE ||
+                                            expectedClass == Float.TYPE ||
+                                            Number.class.isAssignableFrom(expectedClass) ||
+                                            Character.class.equals(expectedClass) ||
+                                            Character.TYPE == expectedClass ||
+                                            Boolean.class.equals(expectedClass) ||
+                                            Boolean.TYPE == expectedClass ||
+                                            expectedClass.isEnum()) {
+                                        try {
+                                            EXPRESSION_FACTORY.coerceToType(attrs.getValue(i), expectedClass);
+                                        } catch (Exception e) {
+                                            err.jspError
+                                                (n, "jsp.error.coerce_to_type",
+                                                 tldAttr.getName(), expectedType, attrs.getValue(i));
+                                        }
                                     }
                                 }
 
@@ -1162,43 +1203,28 @@ class Validator {
                                             tldAttr.getName());
                                 }
                                 
-                                Class expectedType = String.class;
-                                try {
-                                    String typeStr = tldAttr.getTypeName();
-                                    if (tldAttr.isFragment()) {
-                                        expectedType = JspFragment.class;
-                                    } else if (typeStr != null) {
-                                        expectedType = JspUtil.toClass(typeStr,
-                                                loader);
+                                if (elExpression) {
+                                    // El expression
+                                    validateFunctions(el, n);
+                                    jspAttrs[i] = new Node.JspAttribute(tldAttr,
+                                            attrs.getQName(i), attrs.getURI(i), 
+                                            attrs.getLocalName(i),
+                                            attrs.getValue(i), false, el, false);
+                                    ELContextImpl ctx = new ELContextImpl();
+                                    ctx.setFunctionMapper(getFunctionMapper(el));
+                                    try {
+                                        jspAttrs[i].validateEL(this.pageInfo.getExpressionFactory(), ctx);
+                                    } catch (ELException e) {
+                                        this.err.jspError(n.getStart(),
+                                                "jsp.error.invalid.expression", 
+                                                attrs.getValue(i), e.toString());
                                     }
-                                    if (elExpression) {
-                                        // El expression
-                                        validateFunctions(el, n);
-                                        jspAttrs[i] = new Node.JspAttribute(tldAttr,
-                                                attrs.getQName(i), attrs.getURI(i), 
-                                                attrs.getLocalName(i),
-                                                attrs.getValue(i), false, el, false);
-                                        ELContextImpl ctx = new ELContextImpl();
-                                        ctx.setFunctionMapper(getFunctionMapper(el));
-                                        try {
-                                            jspAttrs[i].validateEL(this.pageInfo.getExpressionFactory(), ctx);
-                                        } catch (ELException e) {
-                                            this.err.jspError(n.getStart(),
-                                                    "jsp.error.invalid.expression", 
-                                                    attrs.getValue(i), e.toString());
-                                        }
-                                    } else {
-                                        // Runtime expression
-                                        jspAttrs[i] = getJspAttribute(tldAttr,
-                                                attrs.getQName(i), attrs.getURI(i),
-                                                attrs.getLocalName(i), attrs
-                                                .getValue(i), expectedType, n,
-                                                false);
-                                    }
-                                } catch (ClassNotFoundException e) {
-                                    err.jspError
-                                        (n, "jsp.error.unknown_attribute_type",
-                                         tldAttr.getName(), tldAttr.getTypeName());
+                                } else {
+                                    // Runtime expression
+                                    jspAttrs[i] = getJspAttribute(tldAttr,
+                                            attrs.getQName(i), attrs.getURI(i),
+                                            attrs.getLocalName(i), attrs
+                                            .getValue(i), n, false);
                                 }
                             }
                             
@@ -1229,8 +1255,7 @@ class Validator {
                     if (tagInfo.hasDynamicAttributes()) {
                         jspAttrs[i] = getJspAttribute(null, attrs.getQName(i),
                                 attrs.getURI(i), attrs.getLocalName(i), attrs
-                                        .getValue(i), java.lang.Object.class,
-                                n, true);
+                                        .getValue(i), n, true);
                     } else {
                         err.jspError(n, "jsp.error.bad_attribute", attrs
                                 .getQName(i), n.getLocalName());
@@ -1311,7 +1336,7 @@ class Validator {
          */
         private Node.JspAttribute getJspAttribute(TagAttributeInfo tai,
                 String qName, String uri, String localName, String value,
-                Class expectedType, Node n, boolean dynamic)
+                Node n, boolean dynamic)
                 throws JasperException {
 
             Node.JspAttribute result = null;
@@ -1384,9 +1409,9 @@ class Validator {
         }
 
         /*
-         * Return an empty StringBuffer [not thread-safe]
+         * Return an empty StringBuilder [not thread-safe]
          */
-        private StringBuffer getBuffer() {
+        private StringBuilder getBuffer() {
             this.buf.setLength(0);
             return this.buf;
         }
@@ -1440,6 +1465,7 @@ class Validator {
         private static class NamedAttributeVisitor extends Node.Visitor {
             private boolean hasDynamicContent;
 
+            @Override
             public void doVisit(Node n) throws JasperException {
                 if (!(n instanceof Node.JspText)
                         && !(n instanceof Node.TemplateText)) {
@@ -1490,6 +1516,7 @@ class Validator {
                     this.n = n;
                 }
 
+                @Override
                 public void visit(ELNode.Function func) throws JasperException {
                     String prefix = func.getPrefix();
                     String function = func.getName();
@@ -1506,11 +1533,8 @@ class Validator {
                             err.jspError(n, "jsp.error.noFunctionPrefix",
                                     function);
                         } else {
-                            err
-                                    .jspError(
-                                            n,
-                                            "jsp.error.attribute.invalidPrefix",
-                                            prefix);
+                            err.jspError(n, "jsp.error.attribute.invalidPrefix",
+                                    prefix);
                         }
                     }
                     TagLibraryInfo taglib = pageInfo.getTaglib(uri);
@@ -1607,7 +1631,7 @@ class Validator {
                 }
                 start = p + 1;
             }
-            return (String[]) params.toArray(new String[params.size()]);
+            return params.toArray(new String[params.size()]);
         }
 
         private FunctionMapper getFunctionMapper(ELNode.Nodes el)
@@ -1621,6 +1645,7 @@ class Validator {
                     fnmap.put(fnQName, method);
                 }
 
+                @Override
                 public Method resolveFunction(String prefix, String localName) {
                     return this.fnmap.get(prefix + ":" + localName);
                 }
@@ -1633,9 +1658,10 @@ class Validator {
                     this.fmapper = fmapper;
                 }
 
+                @Override
                 public void visit(ELNode.Function n) throws JasperException {
 
-                    Class c = null;
+                    Class<?> c = null;
                     Method method = null;
                     try {
                         c = loader.loadClass(n.getFunctionInfo()
@@ -1648,7 +1674,7 @@ class Validator {
                     }
                     String paramTypes[] = n.getParameters();
                     int size = paramTypes.length;
-                    Class params[] = new Class[size];
+                    Class<?> params[] = new Class[size];
                     int i = 0;
                     try {
                         for (i = 0; i < size; i++) {
@@ -1688,6 +1714,7 @@ class Validator {
             this.err = compiler.getErrorDispatcher();
         }
 
+        @Override
         public void visit(Node.CustomTag n) throws JasperException {
             TagInfo tagInfo = n.getTagInfo();
             if (tagInfo == null) {
@@ -1696,7 +1723,7 @@ class Validator {
 
             ValidationMessage[] errors = tagInfo.validate(n.getTagData());
             if (errors != null && errors.length != 0) {
-                StringBuffer errMsg = new StringBuffer();
+                StringBuilder errMsg = new StringBuilder();
                 errMsg.append("<h3>");
                 errMsg.append(Localizer.getMessage(
                         "jsp.error.tei.invalid.attributes", n.getQName()));
@@ -1786,11 +1813,11 @@ class Validator {
     private static void validateXmlView(PageData xmlView, Compiler compiler)
             throws JasperException {
 
-        StringBuffer errMsg = null;
+        StringBuilder errMsg = null;
         ErrorDispatcher errDisp = compiler.getErrorDispatcher();
 
-        for (Iterator iter = compiler.getPageInfo().getTaglibs().iterator(); iter
-                .hasNext();) {
+        for (Iterator<TagLibraryInfo> iter =
+            compiler.getPageInfo().getTaglibs().iterator(); iter.hasNext();) {
 
             Object o = iter.next();
             if (!(o instanceof TagLibraryInfoImpl))
@@ -1800,7 +1827,7 @@ class Validator {
             ValidationMessage[] errors = tli.validate(xmlView);
             if ((errors != null) && (errors.length != 0)) {
                 if (errMsg == null) {
-                    errMsg = new StringBuffer();
+                    errMsg = new StringBuilder();
                 }
                 errMsg.append("<h3>");
                 errMsg.append(Localizer.getMessage(
