@@ -20,7 +20,7 @@ package org.apache.tomcat.jni;
 /** SSL
  *
  * @author Mladen Turk
- * @version $Id: SSL.java 1461021 2013-03-26 08:48:14Z kkolinko $
+ * @version $Id: SSL.java 1149092 2011-07-21 09:40:03Z jfclere $
  */
 
 public final class SSL {
@@ -73,7 +73,7 @@ public final class SSL {
     public static final int SSL_PROTOCOL_SSLV2 = (1<<0);
     public static final int SSL_PROTOCOL_SSLV3 = (1<<1);
     public static final int SSL_PROTOCOL_TLSV1 = (1<<2);
-    public static final int SSL_PROTOCOL_ALL   = (SSL_PROTOCOL_SSLV2|SSL_PROTOCOL_SSLV3|SSL_PROTOCOL_TLSV1);
+    public static final int SSL_PROTOCOL_ALL   = (SSL_PROTOCOL_SSLV3|SSL_PROTOCOL_TLSV1);
 
     /*
      * Define the SSL verify levels
@@ -113,11 +113,12 @@ public final class SSL {
     /* SSL_OP_ALL: various bug workarounds that should be rather harmless.
      *             This used to be 0x000FFFFFL before 0.9.7. */
     public static final int SSL_OP_ALL                              = 0x00000FFF;
-
     /* As server, disallow session resumption on renegotiation */
     public static final int SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION = 0x00010000;
-    /* Don't use compression even if supported */
-    public static final int SSL_OP_NO_COMPRESSION                         = 0x00020000;
+    /* Permit unsafe legacy renegotiation */
+    public static final int SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION      = 0x00040000;
+    /* If set, always create a new key when using tmp_eddh parameters */
+    public static final int SSL_OP_SINGLE_ECDH_USE                  = 0x00080000;
     /* If set, always create a new key when using tmp_dh parameters */
     public static final int SSL_OP_SINGLE_DH_USE                    = 0x00100000;
     /* Set to always use the tmp_rsa key when doing RSA operations,
@@ -223,26 +224,11 @@ public final class SSL {
      * lifetime of JVM. Library.init() has to be called before.
      * @param engine Support for external a Crypto Device ("engine"),
      *                usually
-     * a hardware accellerator card for crypto operations.
+     * a hardware accelerator card for crypto operations.
      * @return APR status code
      */
     public static native int initialize(String engine);
 
-    /**
-     * Enable/Disable FIPS Mode.
-     *
-     * @param mode 1 - enable, 0 - disable
-     *
-     * @return FIPS_mode_set return code
-     */
-    public static native int fipsModeSet(int mode);
-
-    /**
-      * Set source of entropy to use in SSL
-      *  @param filename Filename containing random data
-      */
-    public static native boolean randSet(String filename);
-
     /**
      * Add content of the file to the PRNG
      * @param filename Filename containing random data.
@@ -271,6 +257,14 @@ public final class SSL {
     public static native boolean randMake(String filename, int len,
                                           boolean base64);
 
+    /**
+     * Sets global random filename.
+     * @param filename Filename to use.
+     *        If set it will be used for SSL initialization
+     *        and all contexts where explicitly not set.
+     */
+    public static native void randSet(String filename);
+
     /**
      * Initialize new BIO
      * @param pool The pool to use.
@@ -281,7 +275,7 @@ public final class SSL {
             throws Exception;
 
     /**
-     * Close BIO and derefrence callback object
+     * Close BIO and dereference callback object
      * @param bio BIO to close and destroy.
      * @return APR Status code
      */
@@ -326,7 +320,7 @@ public final class SSL {
      * SSL_TMP_KEY_DH_4096
      * </PRE>
      * @param idx temporary key index.
-     * @param file File contatining DH params.
+     * @param file File containing DH params.
      */
     public static native boolean loadDSATempKey(int idx, String file);
 
@@ -336,16 +330,15 @@ public final class SSL {
     public static native String getLastError();
 
     /**
-     * Return true if all the requested SSL_OP_* are supported by OpenSSL.
-     * 
-     * <i>Note that for versions of tcnative &lt; 1.1.25, this method will
-     * return <code>true</code> if and only if <code>op</code>=
-     * {@link #SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION} and tcnative
-     * supports that flag.</i>
-     *
-     * @param op Bitwise-OR of all SSL_OP_* to test.
-     * 
-     * @return true if all SSL_OP_* are supported by OpenSSL library.
+     * Return true if SSL_OP_ if defined.
+     * <p>
+     * Currently used for testing weather the
+     * SSL_OP_ALLOW_UNSAFE_LEGACY_RENEGOTIATION is supported by OpenSSL.
+     * <p>
+     * @param op SSL_OP to test.
+     * @return true if SSL_OP is supported by OpenSSL library.
      */
     public static native boolean hasOp(int op);
+
 }
+
