@@ -27,9 +27,11 @@ import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
 
+import org.apache.tomcat.util.res.StringManager;
+
 /** Efficient conversion of bytes  to character .
  *  
- *  This uses the standard JDK mechansim - a reader - but provides mechanisms
+ *  This uses the standard JDK mechanism - a reader - but provides mechanisms
  *  to recycle all the objects that are used. It is compatible with JDK1.1
  *  and up,
  *  ( nio is better, but it's not available even in 1.2 or 1.3 )
@@ -41,12 +43,15 @@ import java.util.Map;
 public class B2CConverter {
     
     
-    private static org.apache.juli.logging.Log log=
+    private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog( B2CConverter.class );
+    
+    private static final StringManager sm =
+        StringManager.getManager(Constants.Package);
 
     private static final Map<String, Charset> encodingToCharsetCache =
         new HashMap<String, Charset>();
-    
+
     static {
         for (Charset charset: Charset.availableCharsets().values()) {
             encodingToCharsetCache.put(
@@ -59,20 +64,21 @@ public class B2CConverter {
     }
 
     public static Charset getCharset(String enc)
-            throws UnsupportedEncodingException {
+            throws UnsupportedEncodingException{
 
         // Encoding names should all be ASCII
         String lowerCaseEnc = enc.toLowerCase(Locale.US);
-        
-        Charset charset = encodingToCharsetCache.get(lowerCaseEnc);
 
+        Charset charset = encodingToCharsetCache.get(lowerCaseEnc);
+        
         if (charset == null) {
             // Pre-population of the cache means this must be invalid
-            throw new UnsupportedEncodingException(enc);
+            throw new UnsupportedEncodingException(
+                    sm.getString("b2cConvertor.unknownEncoding", enc));
         }
         return charset;
     }
-
+    
     private IntermediateInputStream iis;
     private ReadConvertor conv;
     private String encoding;
@@ -100,49 +106,32 @@ public class B2CConverter {
     static final int BUFFER_SIZE=8192;
     char result[]=new char[BUFFER_SIZE];
 
-    /** Convert a buffer of bytes into a chars
-     * @deprecated
-     */
-    public  void convert( ByteChunk bb, CharChunk cb )
-        throws IOException
-    {
-        // Set the ByteChunk as input to the Intermediate reader
-        convert(bb, cb, cb.getBuffer().length - cb.getEnd());
-    }
-
-    /**
-     * Convert a buffer of bytes into a chars.
-     *
-     * @param bb    Input byte buffer
-     * @param cb    Output char buffer
-     * @param limit Number of bytes to convert
-     * @throws IOException
-     */    
     public void convert( ByteChunk bb, CharChunk cb, int limit) 
         throws IOException
     {
         iis.setByteChunk( bb );
         try {
             // read from the reader
-            int bbLengthBeforeRead  = 0;
-            while( limit > 0 ) {
+            int bbLengthBeforeRead = 0;
+            while( limit > 0 ) { // conv.ready() ) {
                 int size = limit < BUFFER_SIZE ? limit : BUFFER_SIZE;
                 bbLengthBeforeRead = bb.getLength();
                 int cnt=conv.read( result, 0, size );
                 if( cnt <= 0 ) {
                     // End of stream ! - we may be in a bad state
-                    if( debug>0)
-                        log( "EOF" );
+                    if(log.isDebugEnabled())
+                        log.debug("B2CConverter: EOF");
                     return;
                 }
-                if( debug > 1 )
-                    log("Converted: " + new String( result, 0, cnt ));
+                if(log.isDebugEnabled())
+                    log.debug("B2CConverter: Converted: " +
+                            new String(result, 0, cnt));
                 cb.append( result, 0, cnt );
                 limit = limit - (bbLengthBeforeRead - bb.getLength());
             }
         } catch( IOException ex) {
-            if( debug>0)
-                log( "Reseting the converter " + ex.toString() );
+            if(log.isDebugEnabled())
+                log.debug("B2CConverter: Reseting the converter " + ex.toString());
             reset();
             throw ex;
         }
@@ -154,69 +143,9 @@ public class B2CConverter {
     {
         // destroy the reader/iis
         iis=new IntermediateInputStream();
-        conv=new ReadConvertor( iis, getCharset(encoding) );
+        conv = new ReadConvertor(iis, getCharset(encoding));
     }
 
-    private final int debug=0;
-    void log( String s ) {
-        if (log.isDebugEnabled())
-            log.debug("B2CConverter: " + s );
-    }
-
-    // -------------------- Not used - the speed improvemnt is quite small
-
-    /*
-    private Hashtable decoders;
-    public static final boolean useNewString=false;
-    public static final boolean useSpecialDecoders=true;
-    private UTF8Decoder utfD;
-    // private char[] conversionBuff;
-    CharChunk conversionBuf;
-
-
-    private  static String decodeString(ByteChunk mb, String enc)
-        throws IOException
-    {
-        byte buff=mb.getBuffer();
-        int start=mb.getStart();
-        int end=mb.getEnd();
-        if( useNewString ) {
-            if( enc==null) enc="UTF8";
-            return new String( buff, start, end-start, enc );
-        }
-        B2CConverter b2c=null;
-        if( useSpecialDecoders &&
-            (enc==null || "UTF8".equalsIgnoreCase(enc))) {
-            if( utfD==null ) utfD=new UTF8Decoder();
-            b2c=utfD;
-        }
-        if(decoders == null ) decoders=new Hashtable();
-        if( enc==null ) enc="UTF8";
-        b2c=(B2CConverter)decoders.get( enc );
-        if( b2c==null ) {
-            if( useSpecialDecoders ) {
-                if( "UTF8".equalsIgnoreCase( enc ) ) {
-                    b2c=new UTF8Decoder();
-                }
-            }
-            if( b2c==null )
-                b2c=new B2CConverter( enc );
-            decoders.put( enc, b2c );
-        }
-        if( conversionBuf==null ) conversionBuf=new CharChunk(1024);
-
-        try {
-            conversionBuf.recycle();
-            b2c.convert( this, conversionBuf );
-            //System.out.println("XXX 1 " + conversionBuf );
-            return conversionBuf.toString();
-        } catch( IOException ex ) {
-            ex.printStackTrace();
-            return null;
-        }
-    }
-
-    */
 }
 
 // -------------------- Private implementation --------------------
@@ -228,22 +157,21 @@ public class B2CConverter {
  */
 final class  ReadConvertor extends InputStreamReader {
     
-    // Has a private, internal byte[8192]
-    
     /** Create a converter.
      */
-    public ReadConvertor( IntermediateInputStream in, Charset charset )
-    {
-        super( in, charset );
+    public ReadConvertor(IntermediateInputStream in, Charset charset) {
+        super(in, charset);
     }
     
-    /** Overriden - will do nothing but reset internal state.
+    /** Overridden - will do nothing but reset internal state.
      */
+    @Override
     public  final void close() throws IOException {
         // NOTHING
         // Calling super.close() would reset out and cb.
     }
     
+    @Override
     public  final int read(char cbuf[], int off, int len)
         throws IOException
     {
@@ -266,7 +194,7 @@ final class  ReadConvertor extends InputStreamReader {
 }
 
 
-/** Special output stream where close() is overriden, so super.close()
+/** Special output stream where close() is overridden, so super.close()
     is never called.
     
     This allows recycling. It can also be disabled, so callbacks will
@@ -278,15 +206,18 @@ final class IntermediateInputStream extends InputStream {
     public IntermediateInputStream() {
     }
     
+    @Override
     public  final void close() throws IOException {
         // shouldn't be called - we filter it out in writer
         throw new IOException("close() called - shouldn't happen ");
     }
     
+    @Override
     public  final  int read(byte cbuf[], int off, int len) throws IOException {
         return bc.substract(cbuf, off, len);
     }
     
+    @Override
     public  final int read() throws IOException {
         return bc.substract();
     }
