@@ -34,8 +34,8 @@ import java.io.Serializable;
  * to be able to parse the request without converting to string.
  */
 
-// TODO: This class could either extend ByteBuffer, or better a ByteBuffer inside
-// this way it could provide the search/etc on ByteBuffer, as a helper.
+// TODO: This class could either extend ByteBuffer, or better a ByteBuffer
+// inside this way it could provide the search/etc on ByteBuffer, as a helper.
 
 /**
  * This class is used to represent a chunk of bytes, and
@@ -43,18 +43,19 @@ import java.io.Serializable;
  *
  * The buffer can be modified and used for both input and output.
  *
- * There are 2 modes: The chunk can be associated with a sink - ByteInputChannel or ByteOutputChannel,
- * which will be used when the buffer is empty ( on input ) or filled ( on output ).
- * For output, it can also grow. This operating mode is selected by calling setLimit() or
- * allocate(initial, limit) with limit != -1.
+ * There are 2 modes: The chunk can be associated with a sink - ByteInputChannel
+ * or ByteOutputChannel, which will be used when the buffer is empty (on input)
+ * or filled (on output).
+ * For output, it can also grow. This operating mode is selected by calling
+ * setLimit() or allocate(initial, limit) with limit != -1.
  *
- * Various search and append method are defined - similar with String and StringBuffer, but
- * operating on bytes.
- *
- * This is important because it allows processing the http headers directly on the received bytes,
- * without converting to chars and Strings until the strings are needed. In addition, the charset
- * is determined later, from headers or user code.
+ * Various search and append method are defined - similar with String and
+ * StringBuffer, but operating on bytes.
  *
+ * This is important because it allows processing the http headers directly on
+ * the received bytes, without converting to chars and Strings until the strings
+ * are needed. In addition, the charset is determined later, from headers or
+ * user code.
  *
  * @author dac@sun.com
  * @author James Todd [gonzo@sun.com]
@@ -63,7 +64,9 @@ import java.io.Serializable;
  */
 public final class ByteChunk implements Cloneable, Serializable {
 
-    /** Input interface, used when the buffer is emptiy
+    private static final long serialVersionUID = 1L;
+
+    /** Input interface, used when the buffer is empty
      *
      * Same as java.nio.channel.ReadableByteChannel
      */
@@ -118,6 +121,7 @@ public final class ByteChunk implements Cloneable, Serializable {
      * Creates a new, uninitialized ByteChunk object.
      */
     public ByteChunk() {
+        // NO-OP
     }
 
     public ByteChunk( int initial ) {
@@ -232,7 +236,7 @@ public final class ByteChunk implements Cloneable, Serializable {
 
     /** Maximum amount of data in this buffer.
      *
-     *  If -1 or not set, the buffer will grow undefinitely.
+     *  If -1 or not set, the buffer will grow indefinitely.
      *  Can be smaller than the current buffer size ( which will not shrink ).
      *  When the limit is reached, the buffer will be flushed ( if out is set )
      *  or throw exception.
@@ -416,8 +420,9 @@ public final class ByteChunk implements Cloneable, Serializable {
     }
 
 
-    /** Send the buffer to the sink. Called by append() when the limit is reached.
-     *  You can also call it explicitely to force the data to be written.
+    /**
+     * Send the buffer to the sink. Called by append() when the limit is
+     * reached. You can also call it explicitly to force the data to be written.
      *
      * @throws IOException
      */
@@ -481,6 +486,7 @@ public final class ByteChunk implements Cloneable, Serializable {
     
     // -------------------- Conversion and getters --------------------
 
+    @Override
     public String toString() {
         if (null == buff) {
             return null;
@@ -493,11 +499,14 @@ public final class ByteChunk implements Cloneable, Serializable {
     public String toStringInternal() {
         String strValue=null;
         try {
-            if( enc==null ) enc=DEFAULT_CHARACTER_ENCODING;
-            strValue = new String( buff, start, end-start, enc );
+            if (enc == null) {
+                enc = DEFAULT_CHARACTER_ENCODING;
+            }
+            strValue = new String(buff, start, end-start,
+                    B2CConverter.getCharset(enc));
             /*
              Does not improve the speed too much on most systems,
-             it's safer to use the "clasical" new String().
+             it's safer to use the "classical" new String().
              
              Most overhead is in creating char[] and copying,
              the internal implementation of new String() is very close to
@@ -646,7 +655,7 @@ public final class ByteChunk implements Cloneable, Serializable {
         if (b1 == null || b2 == null || b2.length > len) {
             return false;
         }
-        for (int i = start, j = 0; i < end && j < b2.length; ) {
+        for (int i = start, j = 0; i < end && j < b2.length;) {
             if (b1[i++] != b2[j++]) 
                 return false;
         }
@@ -684,7 +693,7 @@ public final class ByteChunk implements Cloneable, Serializable {
             if( buff[i] != first ) continue;
             // found first char, now look for a match
             int myPos=i+1;
-            for( int srcPos=srcOff + 1; srcPos< srcEnd; ) {
+            for( int srcPos=srcOff + 1; srcPos< srcEnd;) {
                 if( buff[myPos++] != src.charAt( srcPos++ ))
                     continue mainLoop;
             }
@@ -728,35 +737,62 @@ public final class ByteChunk implements Cloneable, Serializable {
     }
 
     /**
-     * Returns true if the message bytes starts with the specified string.
-     * @param c the character
-     * @param starting The start position
+     * Returns the first instance of the given character in this ByteChunk
+     * starting at the specified byte. If the character is not found, -1 is
+     * returned.
+     * <br/>
+     * NOTE: This only works for characters in the range 0-127.
+     * 
+     * @param c         The character
+     * @param starting  The start position
+     * @return          The position of the first instance of the character or
+     *                      -1 if the character is not found.
      */
     public int indexOf(char c, int starting) {
-        int ret = indexOf( buff, start+starting, end, c);
+        int ret = indexOf(buff, start + starting, end, c);
         return (ret >= start) ? ret - start : -1;
     }
 
-    public static int  indexOf( byte bytes[], int off, int end, char qq )
-    {
-        // Works only for UTF 
-        while( off < end ) {
-            byte b=bytes[off];
-            if( b==qq )
-                return off;
-            off++;
+    /**
+     * Returns the first instance of the given character in the given byte array
+     * between the specified start and end.
+     * <br/>
+     * NOTE: This only works for characters in the range 0-127.
+     * 
+     * @param bytes The byte array to search
+     * @param start The point to start searching from in the byte array
+     * @param end   The point to stop searching in the byte array
+     * @param c     The character to search for 
+     * @return      The position of the first instance of the character or -1
+     *                  if the character is not found.
+     */
+    public static int indexOf(byte bytes[], int start, int end, char c) {
+        int offset = start;
+        
+        while (offset < end) {
+            byte b=bytes[offset];
+            if (b == c)
+                return offset;
+            offset++;
         }
         return -1;
     }
 
-    /** Find a character, no side effects.
-     *  @return index of char if found, -1 if not
+    /**
+     * Returns the first instance of the given byte in the byte array between
+     * the specified start and end.
+     * 
+     * @param bytes The byte array to search
+     * @param start The point to start searching from in the byte array
+     * @param end   The point to stop searching in the byte array
+     * @param b     The byte to search for 
+     * @return      The position of the first instance of the byte or -1 if the
+     *                  byte is not found.
      */
-    public static int findChar( byte buf[], int start, int end, char c ) {
-        byte b=(byte)c;
+    public static int findByte(byte bytes[], int start, int end, byte b) {
         int offset = start;
         while (offset < end) {
-            if (buf[offset] == b) {
+            if (bytes[offset] == b) {
                 return offset;
             }
             offset++;
@@ -764,15 +800,23 @@ public final class ByteChunk implements Cloneable, Serializable {
         return -1;
     }
 
-    /** Find a character, no side effects.
-     *  @return index of char if found, -1 if not
+    /**
+     * Returns the first instance of any of the given bytes in the byte array
+     * between the specified start and end.
+     * 
+     * @param bytes The byte array to search
+     * @param start The point to start searching from in the byte array
+     * @param end   The point to stop searching in the byte array
+     * @param b     The array of bytes to search for 
+     * @return      The position of the first instance of the byte or -1 if the
+     *                  byte is not found.
      */
-    public static int findChars( byte buf[], int start, int end, byte c[] ) {
-        int clen=c.length;
+    public static int findBytes(byte bytes[], int start, int end, byte b[]) {
+        int blen = b.length;
         int offset = start;
         while (offset < end) {
-            for( int i=0; i<clen; i++ ) 
-                if (buf[offset] == c[i]) {
+            for (int i = 0;  i < blen; i++) 
+                if (bytes[offset] == b[i]) {
                     return offset;
                 }
             offset++;
@@ -780,24 +824,32 @@ public final class ByteChunk implements Cloneable, Serializable {
         return -1;
     }
 
-    /** Find the first character != c 
-     *  @return index of char if found, -1 if not
+    /**
+     * Returns the first instance of any byte that is not one of the given bytes
+     * in the byte array between the specified start and end.
+     * 
+     * @param bytes The byte array to search
+     * @param start The point to start searching from in the byte array
+     * @param end   The point to stop searching in the byte array
+     * @param b     The list of bytes to search for 
+     * @return      The position of the first instance a byte that is not
+     *                  in the list of bytes to search for or -1 if no such byte
+     *                  is found.
      */
-    public static int findNotChars( byte buf[], int start, int end, byte c[] )
-    {
-        int clen=c.length;
+    public static int findNotBytes(byte bytes[], int start, int end, byte b[]) {
+        int blen = b.length;
         int offset = start;
         boolean found;
                 
         while (offset < end) {
-            found=true;
-            for( int i=0; i<clen; i++ ) {
-                if (buf[offset] == c[i]) {
+            found = true;
+            for (int i = 0; i < blen; i++) {
+                if (bytes[offset] == b[i]) {
                     found=false;
                     break;
                 }
             }
-            if( found ) { // buf[offset] != c[0..len]
+            if (found) {
                 return offset;
             }
             offset++;
@@ -820,6 +872,4 @@ public final class ByteChunk implements Cloneable, Serializable {
         }
         return result;
     }
-    
-    
 }
