@@ -1,5 +1,4 @@
-/* $Id: Digester.java 1173608 2011-09-21 12:56:11Z kkolinko $
- *
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
  * contributor license agreements.  See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
@@ -30,7 +29,6 @@ import java.util.HashMap;
 import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Properties;
 
 import javax.xml.parsers.ParserConfigurationException;
 import javax.xml.parsers.SAXParser;
@@ -38,6 +36,7 @@ import javax.xml.parsers.SAXParserFactory;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.xml.sax.Attributes;
 import org.xml.sax.EntityResolver;
@@ -53,8 +52,6 @@ import org.xml.sax.helpers.AttributesImpl;
 import org.xml.sax.helpers.DefaultHandler;
 
 
-
-
 /**
  * <p>A <strong>Digester</strong> processes an XML input stream by matching a
  * series of element nesting patterns to execute Rules that have been added
@@ -77,12 +74,11 @@ import org.xml.sax.helpers.DefaultHandler;
 
 public class Digester extends DefaultHandler {
 
-
-    // ---------------------------------------------------------- Static Fields
-
-
+    
+    // ---------------------------------------------------------- Static Fields    
     private static class SystemPropertySource 
         implements IntrospectionUtils.PropertySource {
+        @Override
         public String getProperty( String key ) {
             return System.getProperty(key);
         }
@@ -90,6 +86,29 @@ public class Digester extends DefaultHandler {
 
     protected static IntrospectionUtils.PropertySource source[] = 
         new IntrospectionUtils.PropertySource[] { new SystemPropertySource() };
+    
+    static {
+        String className = System.getProperty("org.apache.tomcat.util.digester.PROPERTY_SOURCE");
+        if (className!=null) {
+            IntrospectionUtils.PropertySource[] sources = new IntrospectionUtils.PropertySource[2];
+            sources[1] = source[0];
+            ClassLoader[] cls = new ClassLoader[] {Digester.class.getClassLoader(),Thread.currentThread().getContextClassLoader()};
+            boolean initialized = false;
+            for (int i=0; i<cls.length && (!initialized); i++) {
+                try {
+                    Class<?> clazz = Class.forName(className,true,cls[i]);
+                    IntrospectionUtils.PropertySource src = (IntrospectionUtils.PropertySource)clazz.newInstance();
+                    sources[0] = src;
+                    initialized = true;
+                } catch (Throwable t) {
+                    ExceptionUtils.handleThrowable(t);
+                    LogFactory.getLog("org.apache.commons.digester.Digester").
+                        error("Unable to load property source["+className+"].",t);
+                }
+            }
+            if (initialized) source = sources;
+        }
+    }
 
 
     // --------------------------------------------------------- Constructors
@@ -126,7 +145,7 @@ public class Digester extends DefaultHandler {
      * allows Digester to be used in environments which are unfriendly to
      * JAXP1.1 (such as WebLogic 6.0).  Note that if you use this option you
      * have to configure namespace and validation support yourself, as these
-     * properties only affect the SAXParser and emtpy constructor.
+     * properties only affect the SAXParser and empty constructor.
      */
     public Digester(XMLReader reader) {
 
@@ -143,13 +162,14 @@ public class Digester extends DefaultHandler {
     /**
      * The body text of the current element.
      */
-    protected StringBuffer bodyText = new StringBuffer();
+    protected StringBuilder bodyText = new StringBuilder();
 
 
     /**
      * The stack of body text string buffers for surrounding elements.
      */
-    protected ArrayStack bodyTexts = new ArrayStack();
+    protected ArrayStack<StringBuilder> bodyTexts =
+        new ArrayStack<StringBuilder>();
 
 
     /**
@@ -162,7 +182,7 @@ public class Digester extends DefaultHandler {
      *
      * @since 1.6
      */
-    protected ArrayStack matches = new ArrayStack(10);
+    protected ArrayStack<List<Rule>> matches = new ArrayStack<List<Rule>>(10);
     
     /**
      * The class loader to use for instantiating application objects.
@@ -188,7 +208,8 @@ public class Digester extends DefaultHandler {
      * The URLs of entityValidator that have been registered, keyed by the public
      * identifier that corresponds.
      */
-    protected HashMap entityValidator = new HashMap();
+    protected HashMap<String,String> entityValidator =
+        new HashMap<String,String>();
 
 
     /**
@@ -203,13 +224,6 @@ public class Digester extends DefaultHandler {
      */
     protected SAXParserFactory factory = null;
 
-    /**
-     * @deprecated This is now managed by {@link ParserFeatureSetterFactory}
-     */
-    protected String JAXP_SCHEMA_LANGUAGE =
-        "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
-    
-    
     /**
      * The Locator associated with our parser.
      */
@@ -236,14 +250,15 @@ public class Digester extends DefaultHandler {
      * is required because documents can declare nested uses of the same
      * prefix for different Namespace URIs).
      */
-    protected HashMap namespaces = new HashMap();
+    protected HashMap<String,ArrayStack<String>> namespaces =
+        new HashMap<String,ArrayStack<String>>();
 
 
     /**
      * The parameters stack being utilized by CallMethodRule and
      * CallParamRule rules.
      */
-    protected ArrayStack params = new ArrayStack();
+    protected ArrayStack<Object> params = new ArrayStack<Object>();
 
 
     /**
@@ -280,23 +295,10 @@ public class Digester extends DefaultHandler {
      */
     protected Rules rules = null;
 
-   /**
-     * The XML schema language to use for validating an XML instance. By
-     * default this value is set to <code>W3C_XML_SCHEMA</code>
-     */
-    protected String schemaLanguage = W3C_XML_SCHEMA;
-    
-        
-    /**
-     * The XML schema to use for validating an XML instance.
-     */
-    protected String schemaLocation = null;
-    
-    
     /**
      * The object stack being constructed.
      */
-    protected ArrayStack stack = new ArrayStack();
+    protected ArrayStack<Object> stack = new ArrayStack<Object>();
 
 
     /**
@@ -312,7 +314,6 @@ public class Digester extends DefaultHandler {
     protected boolean validating = false;
 
 
-    
     /**
      * Warn on missing attributes and elements.
      */
@@ -322,31 +323,26 @@ public class Digester extends DefaultHandler {
     /**
      * Fake attributes map (attributes are often used for object creation).
      */
-    protected Map<Class, List<String>> fakeAttributes = null;
+    protected Map<Class<?>, List<String>> fakeAttributes = null;
 
 
     /**
      * The Log to which most logging calls will be made.
      */
     protected Log log =
-        LogFactory.getLog("org.apache.tomcat.util.digester.Digester");
+        LogFactory.getLog("org.apache.commons.digester.Digester");
 
 
     /**
      * The Log to which all SAX event related logging calls will be made.
      */
     protected Log saxLog =
-        LogFactory.getLog("org.apache.tomcat.util.digester.Digester.sax");
+        LogFactory.getLog("org.apache.commons.digester.Digester.sax");
     
         
-    /**
-     * The schema language supported. By default, we use this one.
-     */
-    protected static final String W3C_XML_SCHEMA =
-        "http://www.w3.org/2001/XMLSchema";
-    
     /** Stacks used for interrule communication, indexed by name String */
-    private HashMap stacksByName = new HashMap();
+    private HashMap<String,ArrayStack<Object>> stacksByName =
+        new HashMap<String,ArrayStack<Object>>();
     
     // ------------------------------------------------------------- Properties
 
@@ -359,12 +355,12 @@ public class Digester extends DefaultHandler {
      */
     public String findNamespaceURI(String prefix) {
         
-        ArrayStack stack = (ArrayStack) namespaces.get(prefix);
+        ArrayStack<String> stack = namespaces.get(prefix);
         if (stack == null) {
             return (null);
         }
         try {
-            return ((String) stack.peek());
+            return stack.peek();
         } catch (EmptyStackException e) {
             return (null);
         }
@@ -438,35 +434,6 @@ public class Digester extends DefaultHandler {
     }
 
 
-    /**
-     * Return the debugging detail level of our currently enabled logger.
-     *
-     * @deprecated This method now always returns 0. This copy of Digester uses
-     * JULI; see the documentation for that library for more information.
-     */
-    public int getDebug() {
-
-        return (0);
-
-    }
-
-
-    /**
-     * Set the debugging detail level of our currently enabled logger.
-     *
-     * @param debug New debugging detail level (0=off, increasing integers
-     *  for more detail)
-     *
-     * @deprecated This method now has no effect at all. This copy of Digester
-     * uses JULI; see the documentation for that library for more information.
-     */
-    public void setDebug(int debug) {
-
-        ; // No action is taken
-
-    }
-
-
     /**
      * Return the error handler for this Digester.
      */
@@ -491,13 +458,28 @@ public class Digester extends DefaultHandler {
 
     /**
      * Return the SAXParserFactory we will use, creating one if necessary.
+     * @throws ParserConfigurationException 
+     * @throws SAXNotSupportedException 
+     * @throws SAXNotRecognizedException 
      */
-    public SAXParserFactory getFactory() {
+    public SAXParserFactory getFactory()
+    throws SAXNotRecognizedException, SAXNotSupportedException,
+    ParserConfigurationException {
 
         if (factory == null) {
             factory = SAXParserFactory.newInstance();
             factory.setNamespaceAware(namespaceAware);
             factory.setValidating(validating);
+            if (validating) {
+                // Enable DTD validation
+                factory.setFeature(
+                        "http://xml.org/sax/features/validation",
+                        true);
+                // Enable schema validation
+                factory.setFeature(
+                        "http://apache.org/xml/features/validation/schema",
+                        true);
+            }
         }
         return (factory);
 
@@ -634,7 +616,7 @@ public class Digester extends DefaultHandler {
 
     
     /**
-     * Set the publid id of the current file being parse.
+     * Set the public id of the current file being parse.
      * @param publicId the DTD/Schema public's id.
      */
     public void setPublicId(String publicId){
@@ -692,16 +674,7 @@ public class Digester extends DefaultHandler {
 
         // Create a new parser
         try {
-            if (validating) {
-                Properties properties = new Properties();
-                properties.put("SAXParserFactory", getFactory());
-                if (schemaLocation != null) {
-                    properties.put("schemaLocation", schemaLocation);
-                    properties.put("schemaLanguage", schemaLanguage);
-                }
-                parser = ParserFeatureSetterFactory.newSAXParser(properties);               } else {
-                parser = getFactory().newSAXParser();
-            }
+            parser = getFactory().newSAXParser();
         } catch (Exception e) {
             log.error("Digester.getParser: ", e);
             return (null);
@@ -757,25 +730,6 @@ public class Digester extends DefaultHandler {
     }
 
 
-    /**
-     * By setting the reader in the constructor, you can bypass JAXP and
-     * be able to use digester in Weblogic 6.0.  
-     *
-     * @deprecated Use getXMLReader() instead, which can throw a
-     *  SAXException if the reader cannot be instantiated
-     */
-    public XMLReader getReader() {
-
-        try {
-            return (getXMLReader());
-        } catch (SAXException e) {
-            log.error("Cannot get XMLReader", e);
-            return (null);
-        }
-
-    }
-
-
     /**
      * Return the <code>Rules</code> implementation object containing our
      * rules collection and associated matching policy.  If none has been
@@ -806,50 +760,6 @@ public class Digester extends DefaultHandler {
     }
 
 
-    /**
-     * Return the XML Schema URI used for validating an XML instance.
-     */
-    public String getSchema() {
-
-        return (this.schemaLocation);
-
-    }
-
-
-    /**
-     * Set the XML Schema URI used for validating a XML Instance.
-     *
-     * @param schemaLocation a URI to the schema.
-     */
-    public void setSchema(String schemaLocation){
-
-        this.schemaLocation = schemaLocation;
-
-    }   
-    
-
-    /**
-     * Return the XML Schema language used when parsing.
-     */
-    public String getSchemaLanguage() {
-
-        return (this.schemaLanguage);
-
-    }
-
-
-    /**
-     * Set the XML Schema language used when parsing. By default, we use W3C.
-     *
-     * @param schemaLanguage a URI to the schema language.
-     */
-    public void setSchemaLanguage(String schemaLanguage){
-
-        this.schemaLanguage = schemaLanguage;
-
-    }   
-
-
     /**
      * Return the boolean as to whether the context classloader should be used.
      */
@@ -925,7 +835,7 @@ public class Digester extends DefaultHandler {
     /**
      * Return the fake attributes list.
      */
-    public Map<Class, List<String>> getFakeAttributes() {
+    public Map<Class<?>, List<String>> getFakeAttributes() {
 
         return (this.fakeAttributes);
 
@@ -958,7 +868,7 @@ public class Digester extends DefaultHandler {
      *
      * @param fakeAttributes The new fake attributes.
      */
-    public void setFakeAttributes(Map<Class, List<String>> fakeAttributes) {
+    public void setFakeAttributes(Map<Class<?>, List<String>> fakeAttributes) {
 
         this.fakeAttributes = fakeAttributes;
 
@@ -1003,6 +913,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void characters(char buffer[], int start, int length)
             throws SAXException {
 
@@ -1020,6 +931,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void endDocument() throws SAXException {
 
         if (saxLog.isDebugEnabled()) {
@@ -1036,9 +948,9 @@ public class Digester extends DefaultHandler {
         }
 
         // Fire "finish" events for all defined rules
-        Iterator rules = getRules().rules().iterator();
+        Iterator<Rule> rules = getRules().rules().iterator();
         while (rules.hasNext()) {
-            Rule rule = (Rule) rules.next();
+            Rule rule = rules.next();
             try {
                 rule.finish();
             } catch (Exception e) {
@@ -1068,6 +980,7 @@ public class Digester extends DefaultHandler {
      *   empty string if qualified names are not available.
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void endElement(String namespaceURI, String localName,
                            String qName) throws SAXException {
 
@@ -1093,12 +1006,12 @@ public class Digester extends DefaultHandler {
         }
 
         // Fire "body" events for all relevant rules
-        List rules = (List) matches.pop();
+        List<Rule> rules = matches.pop();
         if ((rules != null) && (rules.size() > 0)) {
             String bodyText = this.bodyText.toString();
             for (int i = 0; i < rules.size(); i++) {
                 try {
-                    Rule rule = (Rule) rules.get(i);
+                    Rule rule = rules.get(i);
                     if (debug) {
                         log.debug("  Fire body() for " + rule);
                     }
@@ -1121,7 +1034,7 @@ public class Digester extends DefaultHandler {
         }
 
         // Recover the body text from the surrounding element
-        bodyText = (StringBuffer) bodyTexts.pop();
+        bodyText = bodyTexts.pop();
         if (debug) {
             log.debug("  Popping body text '" + bodyText.toString() + "'");
         }
@@ -1131,7 +1044,7 @@ public class Digester extends DefaultHandler {
             for (int i = 0; i < rules.size(); i++) {
                 int j = (rules.size() - i) - 1;
                 try {
-                    Rule rule = (Rule) rules.get(j);
+                    Rule rule = rules.get(j);
                     if (debug) {
                         log.debug("  Fire end() for " + rule);
                     }
@@ -1164,6 +1077,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void endPrefixMapping(String prefix) throws SAXException {
 
         if (saxLog.isDebugEnabled()) {
@@ -1171,7 +1085,7 @@ public class Digester extends DefaultHandler {
         }
 
         // Deregister this prefix mapping
-        ArrayStack stack = (ArrayStack) namespaces.get(prefix);
+        ArrayStack<String> stack = namespaces.get(prefix);
         if (stack == null) {
             return;
         }
@@ -1196,6 +1110,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void ignorableWhitespace(char buffer[], int start, int len)
             throws SAXException {
 
@@ -1204,7 +1119,7 @@ public class Digester extends DefaultHandler {
                     new String(buffer, start, len) + ")");
         }
 
-        ;   // No processing required
+        // No processing required
 
     }
 
@@ -1217,6 +1132,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void processingInstruction(String target, String data)
             throws SAXException {
 
@@ -1224,7 +1140,7 @@ public class Digester extends DefaultHandler {
             saxLog.debug("processingInstruction('" + target + "','" + data + "')");
         }
 
-        ;   // No processing is required
+        // No processing is required
 
     }
 
@@ -1245,6 +1161,7 @@ public class Digester extends DefaultHandler {
      *
      * @param locator The new locator
      */
+    @Override
     public void setDocumentLocator(Locator locator) {
 
         if (saxLog.isDebugEnabled()) {
@@ -1263,13 +1180,14 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void skippedEntity(String name) throws SAXException {
 
         if (saxLog.isDebugEnabled()) {
             saxLog.debug("skippedEntity(" + name + ")");
         }
 
-        ; // No processing required
+        // No processing required
 
     }
 
@@ -1279,6 +1197,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void startDocument() throws SAXException {
 
         if (saxLog.isDebugEnabled()) {
@@ -1305,6 +1224,7 @@ public class Digester extends DefaultHandler {
      *   no attributes, it shall be an empty Attributes object. 
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void startElement(String namespaceURI, String localName,
                              String qName, Attributes list)
             throws SAXException {
@@ -1323,7 +1243,7 @@ public class Digester extends DefaultHandler {
         if (debug) {
             log.debug("  Pushing body text '" + bodyText.toString() + "'");
         }
-        bodyText = new StringBuffer();
+        bodyText = new StringBuilder();
 
         // the actual element name is either in localName or qName, depending 
         // on whether the parser is namespace aware
@@ -1333,7 +1253,7 @@ public class Digester extends DefaultHandler {
         }
 
         // Compute the current matching rule
-        StringBuffer sb = new StringBuffer(match);
+        StringBuilder sb = new StringBuilder(match);
         if (match.length() > 0) {
             sb.append('/');
         }
@@ -1344,12 +1264,12 @@ public class Digester extends DefaultHandler {
         }
 
         // Fire "begin" events for all relevant rules
-        List rules = getRules().match(namespaceURI, match);
+        List<Rule> rules = getRules().match(namespaceURI, match);
         matches.push(rules);
         if ((rules != null) && (rules.size() > 0)) {
             for (int i = 0; i < rules.size(); i++) {
                 try {
-                    Rule rule = (Rule) rules.get(i);
+                    Rule rule = rules.get(i);
                     if (debug) {
                         log.debug("  Fire begin() for " + rule);
                     }
@@ -1379,6 +1299,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing error is to be reported
      */
+    @Override
     public void startPrefixMapping(String prefix, String namespaceURI)
             throws SAXException {
 
@@ -1387,9 +1308,9 @@ public class Digester extends DefaultHandler {
         }
 
         // Register this prefix mapping
-        ArrayStack stack = (ArrayStack) namespaces.get(prefix);
+        ArrayStack<String> stack = namespaces.get(prefix);
         if (stack == null) {
-            stack = new ArrayStack();
+            stack = new ArrayStack<String>();
             namespaces.put(prefix, stack);
         }
         stack.push(namespaceURI);
@@ -1407,6 +1328,7 @@ public class Digester extends DefaultHandler {
      * @param publicId The public identifier (if any)
      * @param systemId The system identifier (if any)
      */
+    @Override
     public void notationDecl(String name, String publicId, String systemId) {
 
         if (saxLog.isDebugEnabled()) {
@@ -1425,6 +1347,7 @@ public class Digester extends DefaultHandler {
      * @param systemId The system identifier (if any)
      * @param notation The name of the associated notation
      */
+    @Override
     public void unparsedEntityDecl(String name, String publicId,
                                    String systemId, String notation) {
 
@@ -1466,6 +1389,7 @@ public class Digester extends DefaultHandler {
      * @exception SAXException if a parsing exception occurs
      * 
      */
+    @Override
     public InputSource resolveEntity(String publicId, String systemId)
             throws SAXException {     
                 
@@ -1479,19 +1403,14 @@ public class Digester extends DefaultHandler {
         // Has this system identifier been registered?
         String entityURL = null;
         if (publicId != null) {
-            entityURL = (String) entityValidator.get(publicId);
+            entityURL = entityValidator.get(publicId);
         }
          
-        // Redirect the schema location to a local destination
-        if (schemaLocation != null && entityURL == null && systemId != null){
-            entityURL = (String)entityValidator.get(systemId);
-        } 
-
         if (entityURL == null) { 
             if (systemId == null) {
                 // cannot resolve
                 if (log.isDebugEnabled()) {
-                    log.debug(" Cannot resolve entity: '" + entityURL + "'");
+                    log.debug(" Cannot resolve entity: '" + publicId + "'");
                 }
                 return (null);
                 
@@ -1528,6 +1447,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing exception occurs
      */
+    @Override
     public void error(SAXParseException exception) throws SAXException {
 
         log.error("Parse Error at line " + exception.getLineNumber() +
@@ -1548,6 +1468,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing exception occurs
      */
+    @Override
     public void fatalError(SAXParseException exception) throws SAXException {
 
         log.error("Parse Fatal Error at line " + exception.getLineNumber() +
@@ -1568,6 +1489,7 @@ public class Digester extends DefaultHandler {
      *
      * @exception SAXException if a parsing exception occurs
      */
+    @Override
     public void warning(SAXParseException exception) throws SAXException {
          if (errorHandler != null) {
             log.warn("Parse Warning Error at line " + exception.getLineNumber() +
@@ -1582,33 +1504,6 @@ public class Digester extends DefaultHandler {
 
     // ------------------------------------------------------- Public Methods
 
-
-    /**
-     * Log a message to our associated logger.
-     *
-     * @param message The message to be logged
-     * @deprecated Call getLogger() and use it's logging methods
-     */
-    public void log(String message) {
-
-        log.info(message);
-
-    }
-
-
-    /**
-     * Log a message and exception to our associated logger.
-     *
-     * @param message The message to be logged
-     * @deprecated Call getLogger() and use it's logging methods
-     */
-    public void log(String message, Throwable exception) {
-
-        log.error(message, exception);
-
-    }
-
-
     /**
      * Parse the content of the specified file using this Digester.  Returns
      * the root element from the object stack (if any).
@@ -1820,7 +1715,7 @@ public class Digester extends DefaultHandler {
      *  for a single parameter from the body of this element)
      * @param paramTypes Set of Java class names for the types
      *  of the expected parameters
-     *  (if you wish to use a primitive type, specify the corresonding
+     *  (if you wish to use a primitive type, specify the corresponding
      *  Java wrapper class instead, such as <code>java.lang.Boolean</code>
      *  for a <code>boolean</code> parameter)
      * @see CallMethodRule
@@ -1849,13 +1744,13 @@ public class Digester extends DefaultHandler {
      * @param paramCount Number of expected parameters (or zero
      *  for a single parameter from the body of this element)
      * @param paramTypes The Java class names of the arguments
-     *  (if you wish to use a primitive type, specify the corresonding
+     *  (if you wish to use a primitive type, specify the corresponding
      *  Java wrapper class instead, such as <code>java.lang.Boolean</code>
      *  for a <code>boolean</code> parameter)
      * @see CallMethodRule
      */
     public void addCallMethod(String pattern, String methodName,
-                              int paramCount, Class paramTypes[]) {
+                              int paramCount, Class<?> paramTypes[]) {
 
         addRule(pattern,
                 new CallMethodRule(
@@ -1997,7 +1892,7 @@ public class Digester extends DefaultHandler {
      * @param clazz Java class of the object creation factory class
      * @see FactoryCreateRule
      */
-    public void addFactoryCreate(String pattern, Class clazz) {
+    public void addFactoryCreate(String pattern, Class<?> clazz) {
 
         addFactoryCreate(pattern, clazz, false);
 
@@ -2032,7 +1927,7 @@ public class Digester extends DefaultHandler {
      *  value specified by <code>className</code>
      * @see FactoryCreateRule
      */
-    public void addFactoryCreate(String pattern, Class clazz,
+    public void addFactoryCreate(String pattern, Class<?> clazz,
                                  String attributeName) {
 
         addFactoryCreate(pattern, clazz, attributeName, false);
@@ -2088,7 +1983,7 @@ public class Digester extends DefaultHandler {
      */
     public void addFactoryCreate(
                                     String pattern, 
-                                    Class clazz,
+                                    Class<?> clazz,
                                     boolean ignoreCreateExceptions) {
 
         addRule(
@@ -2135,7 +2030,7 @@ public class Digester extends DefaultHandler {
      */
     public void addFactoryCreate(
                                     String pattern, 
-                                    Class clazz,
+                                    Class<?> clazz,
                                     String attributeName,
                                     boolean ignoreCreateExceptions) {
 
@@ -2188,7 +2083,7 @@ public class Digester extends DefaultHandler {
      * @param clazz Java class to be created
      * @see ObjectCreateRule
      */
-    public void addObjectCreate(String pattern, Class clazz) {
+    public void addObjectCreate(String pattern, Class<?> clazz) {
 
         addRule(pattern,
                 new ObjectCreateRule(clazz));
@@ -2225,7 +2120,7 @@ public class Digester extends DefaultHandler {
      */
     public void addObjectCreate(String pattern,
                                 String attributeName,
-                                Class clazz) {
+                                Class<?> clazz) {
 
         addRule(pattern,
                 new ObjectCreateRule(attributeName, clazz));
@@ -2253,7 +2148,7 @@ public class Digester extends DefaultHandler {
      * @param pattern Element matching pattern
      * @param methodName Method name to call on the parent element
      * @param paramType Java class name of the expected parameter type
-     *  (if you wish to use a primitive type, specify the corresonding
+     *  (if you wish to use a primitive type, specify the corresponding
      *  Java wrapper class instead, such as <code>java.lang.Boolean</code>
      *  for a <code>boolean</code> parameter)
      * @see SetNextRule
@@ -2387,7 +2282,7 @@ public class Digester extends DefaultHandler {
      * @param pattern Element matching pattern
      * @param methodName Method name to call on the parent element
      * @param paramType Java class name of the expected parameter type
-     *  (if you wish to use a primitive type, specify the corresonding
+     *  (if you wish to use a primitive type, specify the corresponding
      *  Java wrapper class instead, such as <code>java.lang.Boolean</code>
      *  for a <code>boolean</code> parameter)
      * @see SetTopRule
@@ -2509,9 +2404,9 @@ public class Digester extends DefaultHandler {
      * @since 1.6
      */
     public void push(String stackName, Object value) {
-        ArrayStack namedStack = (ArrayStack) stacksByName.get(stackName);
+        ArrayStack<Object> namedStack = stacksByName.get(stackName);
         if (namedStack == null) {
-            namedStack = new ArrayStack();
+            namedStack = new ArrayStack<Object>();
             stacksByName.put(stackName, namedStack);
         }
         namedStack.push(value);
@@ -2532,7 +2427,7 @@ public class Digester extends DefaultHandler {
      */
     public Object pop(String stackName) {
         Object result = null;
-        ArrayStack namedStack = (ArrayStack) stacksByName.get(stackName);
+        ArrayStack<Object> namedStack = stacksByName.get(stackName);
         if (namedStack == null) {
             if (log.isDebugEnabled()) {
                 log.debug("Stack '" + stackName + "' is empty");
@@ -2562,7 +2457,7 @@ public class Digester extends DefaultHandler {
      */
     public Object peek(String stackName) {
         Object result = null;
-        ArrayStack namedStack = (ArrayStack) stacksByName.get(stackName);
+        ArrayStack<Object> namedStack = stacksByName.get(stackName);
         if (namedStack == null ) {
             if (log.isDebugEnabled()) {
                 log.debug("Stack '" + stackName + "' is empty");
@@ -2588,7 +2483,7 @@ public class Digester extends DefaultHandler {
      */
     public boolean isEmpty(String stackName) {
         boolean result = true;
-        ArrayStack namedStack = (ArrayStack) stacksByName.get(stackName);
+        ArrayStack<Object> namedStack = stacksByName.get(stackName);
         if (namedStack != null ) {
             result = namedStack.isEmpty();
         }
@@ -2635,8 +2530,8 @@ public class Digester extends DefaultHandler {
             return;
         }
 
-        log = LogFactory.getLog("org.apache.tomcat.util.digester.Digester");
-        saxLog = LogFactory.getLog("org.apache.tomcat.util.digester.Digester.sax");
+        log = LogFactory.getLog("org.apache.commons.digester.Digester");
+        saxLog = LogFactory.getLog("org.apache.commons.digester.Digester.sax");
 
         // Perform lazy configuration as needed
         initialize(); // call hook method for subclasses that want to be initialized once only
@@ -2668,7 +2563,7 @@ public class Digester extends DefaultHandler {
     protected void initialize() {
 
         // Perform lazy initialization as needed
-        ; // Nothing required by default
+        // Nothing required by default
 
     }    
 
@@ -2678,33 +2573,13 @@ public class Digester extends DefaultHandler {
     /**
      * Return the set of DTD URL registrations, keyed by public identifier.
      */
-    Map getRegistrations() {
+    Map<String,String> getRegistrations() {
 
         return (entityValidator);
 
     }
 
 
-    /**
-     * Return the set of rules that apply to the specified match position.
-     * The selected rules are those that match exactly, or those rules
-     * that specify a suffix match and the tail of the rule matches the
-     * current match position.  Exact matches have precedence over
-     * suffix matches, then (among suffix matches) the longest match
-     * is preferred.
-     *
-     * @param match The current match position
-     *
-     * @deprecated Call <code>match()</code> on the <code>Rules</code>
-     *  implementation returned by <code>getRules()</code>
-     */
-    List getRules(String match) {
-
-        return (getRules().match(match));
-
-    }
-
-
     /**
      * <p>Return the top object on the parameters stack without removing it.  If there are
      * no objects on the stack, return <code>null</code>.</p>
@@ -2801,7 +2676,7 @@ public class Digester extends DefaultHandler {
         }
         if (locator != null) {
             String error = "Error at (" + locator.getLineNumber() + ", " +
-                    locator.getColumnNumber() + ": " + message;
+                    locator.getColumnNumber() + ") : " + message;
             if (e != null) {
                 return new SAXParseException(error, locator, e);
             } else {
@@ -2879,11 +2754,11 @@ public class Digester extends DefaultHandler {
 
 
     /**
-     * Return a new StringBuffer containing the same contents as the
+     * Return a new StringBuilder containing the same contents as the
      * input buffer, except that data of form ${varname} have been
      * replaced by the value of that var as defined in the system property.
      */
-    private StringBuffer updateBodyText(StringBuffer bodyText) {
+    private StringBuilder updateBodyText(StringBuilder bodyText) {
         String in = bodyText.toString();
         String out;
         try {
@@ -2897,7 +2772,7 @@ public class Digester extends DefaultHandler {
             // a new buffer
             return bodyText;
         } else {
-            return new StringBuffer(out);
+            return new StringBuilder(out);
         }
     }
 
