@@ -97,26 +97,17 @@ public class JIoEndpoint extends AbstractEndpoint {
     public ServerSocketFactory getServerSocketFactory() { return serverSocketFactory; }
 
 
-    /**
-     * Is sendfile available
+    /*
+     * Optional feature support.
      */
     @Override
-    public boolean getUseSendfile() {
-        // Not supported
-        return false;
-    }
-
-
-    /**
-     * Is deferAccept supported?
-     */
+    public boolean getUseSendfile() { return false; } // Not supported
     @Override
-    public boolean getDeferAccept() {
-        // Not supported
-        return false;
-    }
-    
-    
+    public boolean getUseComet() { return false; } // Not supported
+    @Override
+    public boolean getUseCometTimeout() { return false; } // Not supported
+    @Override
+    public boolean getDeferAccept() { return false; } // Not supported
 
 
     // ------------------------------------------------ Handler Inner Interface
@@ -127,7 +118,6 @@ public class JIoEndpoint extends AbstractEndpoint {
      * thread local fields.
      */
     public interface Handler extends AbstractEndpoint.Handler {
-        public SocketState process(SocketWrapper<Socket> socket);
         public SocketState process(SocketWrapper<Socket> socket,
                 SocketStatus status);
         public SSLImplementation getSslImplementation(); 
@@ -305,8 +295,12 @@ public class JIoEndpoint extends AbstractEndpoint {
                         state = SocketState.CLOSED;
                     }
                         
-                    if ( (state != SocketState.CLOSED) ) {
-                        state = (status==null)?handler.process(socket):handler.process(socket,status);
+                    if ((state != SocketState.CLOSED)) {
+                        if (status == null) {
+                            state = handler.process(socket, SocketStatus.OPEN);
+                        } else {
+                            state = handler.process(socket,status);
+                        }
                     }
                     if (state == SocketState.CLOSED) {
                         // Close socket
@@ -319,8 +313,7 @@ public class JIoEndpoint extends AbstractEndpoint {
                         } catch (IOException e) {
                             // Ignore
                         }
-                    } else if (state == SocketState.ASYNC_END ||
-                            state == SocketState.OPEN){
+                    } else if (state == SocketState.OPEN){
                         socket.setKeptAlive(true);
                         socket.access();
                         launch = true;
@@ -539,30 +532,32 @@ public class JIoEndpoint extends AbstractEndpoint {
     public boolean processSocketAsync(SocketWrapper<Socket> socket,
             SocketStatus status) {
         try {
-            if (waitingRequests.remove(socket)) {
-                SocketProcessor proc = new SocketProcessor(socket,status);
-                ClassLoader loader = Thread.currentThread().getContextClassLoader();
-                try {
-                    //threads should not be created by the webapp classloader
-                    if (Constants.IS_SECURITY_ENABLED) {
-                        PrivilegedAction<Void> pa = new PrivilegedSetTccl(
-                                getClass().getClassLoader());
-                        AccessController.doPrivileged(pa);
-                    } else {
-                        Thread.currentThread().setContextClassLoader(
-                                getClass().getClassLoader());
-                    }
-                    // During shutdown, executor may be null - avoid NPE
-                    if (!running) {
-                        return false;
-                    }
-                    getExecutor().execute(proc);
-                }finally {
-                    if (Constants.IS_SECURITY_ENABLED) {
-                        PrivilegedAction<Void> pa = new PrivilegedSetTccl(loader);
-                        AccessController.doPrivileged(pa);
-                    } else {
-                        Thread.currentThread().setContextClassLoader(loader);
+            synchronized (socket) {
+                if (waitingRequests.remove(socket)) {
+                    SocketProcessor proc = new SocketProcessor(socket,status);
+                    ClassLoader loader = Thread.currentThread().getContextClassLoader();
+                    try {
+                        //threads should not be created by the webapp classloader
+                        if (Constants.IS_SECURITY_ENABLED) {
+                            PrivilegedAction<Void> pa = new PrivilegedSetTccl(
+                                    getClass().getClassLoader());
+                            AccessController.doPrivileged(pa);
+                        } else {
+                            Thread.currentThread().setContextClassLoader(
+                                    getClass().getClassLoader());
+                        }
+                        // During shutdown, executor may be null - avoid NPE
+                        if (!running) {
+                            return false;
+                        }
+                        getExecutor().execute(proc);
+                    } finally {
+                        if (Constants.IS_SECURITY_ENABLED) {
+                            PrivilegedAction<Void> pa = new PrivilegedSetTccl(loader);
+                            AccessController.doPrivileged(pa);
+                        } else {
+                            Thread.currentThread().setContextClassLoader(loader);
+                        }
                     }
                 }
             }
