@@ -29,16 +29,17 @@ import java.util.Iterator;
 import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.MutableInteger;
 import org.apache.tomcat.util.net.NioEndpoint.KeyAttachment;
-import java.util.concurrent.atomic.AtomicInteger;
 
 public class NioBlockingSelector {
     
-    protected static Log log = LogFactory.getLog(NioBlockingSelector.class);
+    private static final Log log = LogFactory.getLog(NioBlockingSelector.class);
     
     private static int threadCounter = 0;
     
@@ -91,7 +92,7 @@ public class NioBlockingSelector {
             while ( (!timedout) && buf.hasRemaining()) {
                 if (keycount > 0) { //only write if we were registered for a write
                     int cnt = socket.write(buf); //write the data
-                    lastWrite.set(cnt);
+                    if (lastWrite != null) lastWrite.set(cnt);
                     if (cnt == -1)
                         throw new EOFException();
                     written += cnt;
@@ -137,7 +138,6 @@ public class NioBlockingSelector {
      * take up a lot of CPU cycles.
      * @param buf ByteBuffer - the buffer containing the data, we will read as until we have read at least one byte or we timed out
      * @param socket SocketChannel - the socket to write data to
-     * @param selector Selector - the selector to use for blocking, if null then a busy read will be initiated
      * @param readTimeout long - the timeout for this read operation in milliseconds, -1 means no timeout
      * @return int - returns the number of bytes read
      * @throws EOFException if read returns -1
@@ -166,7 +166,11 @@ public class NioBlockingSelector {
                 try {
                     if ( att.getReadLatch()==null || att.getReadLatch().getCount()==0) att.startReadLatch(1);
                     poller.add(att,SelectionKey.OP_READ, reference);
-                    att.awaitReadLatch(readTimeout,TimeUnit.MILLISECONDS);
+                    if (readTimeout < 0) {
+                        att.awaitReadLatch(Long.MAX_VALUE, TimeUnit.MILLISECONDS);
+                    } else {
+                        att.awaitReadLatch(readTimeout, TimeUnit.MILLISECONDS);
+                    }
                 }catch (InterruptedException ignore) {
                     Thread.interrupted();
                 }
@@ -178,7 +182,7 @@ public class NioBlockingSelector {
                     keycount = 1;
                     att.resetReadLatch();
                 }
-                if (readTimeout > 0 && (keycount == 0))
+                if (readTimeout >= 0 && (keycount == 0))
                     timedout = (System.currentTimeMillis() - time) >= readTimeout;
             } //while
             if (timedout)
@@ -194,14 +198,15 @@ public class NioBlockingSelector {
     }
 
     
-    protected class BlockPoller extends Thread {
-        protected boolean run = true;
+    protected static class BlockPoller extends Thread {
+        protected volatile boolean run = true;
         protected Selector selector = null;
         protected ConcurrentLinkedQueue<Runnable> events = new ConcurrentLinkedQueue<Runnable>();
         public void disable() { run = false; selector.wakeup();}
         protected AtomicInteger wakeupCounter = new AtomicInteger(0);
         public void cancelKey(final SelectionKey key) {
             Runnable r = new Runnable() {
+                @Override
                 public void run() {
                     key.cancel();
                 }
@@ -225,6 +230,7 @@ public class NioBlockingSelector {
         
         public void add(final KeyAttachment key, final int ops, final KeyReference ref) {
             Runnable r = new Runnable() {
+                @Override
                 public void run() {
                     if ( key == null ) return;
                     NioChannel nch = key.getChannel();
@@ -254,6 +260,7 @@ public class NioBlockingSelector {
         
         public void remove(final KeyAttachment key, final int ops) {
             Runnable r = new Runnable() {
+                @Override
                 public void run() {
                     if ( key == null ) return;
                     NioChannel nch = key.getChannel();
@@ -267,13 +274,13 @@ public class NioBlockingSelector {
                             if (SelectionKey.OP_READ==(ops&SelectionKey.OP_READ))countDown(key.getReadLatch());
                         } else {
                             if (sk.isValid()) {
-                            	sk.interestOps(sk.interestOps() & (~ops));
-                            	if (SelectionKey.OP_WRITE==(ops&SelectionKey.OP_WRITE)) countDown(key.getWriteLatch());
-                            	if (SelectionKey.OP_READ==(ops&SelectionKey.OP_READ))countDown(key.getReadLatch());
-                            	if (sk.interestOps()==0) {
-                            	    sk.cancel();
-                            	    sk.attach(null);
-                            	}
+                                sk.interestOps(sk.interestOps() & (~ops));
+                                if (SelectionKey.OP_WRITE==(ops&SelectionKey.OP_WRITE)) countDown(key.getWriteLatch());
+                                if (SelectionKey.OP_READ==(ops&SelectionKey.OP_READ))countDown(key.getReadLatch());
+                                if (sk.interestOps()==0) {
+                                    sk.cancel();
+                                    sk.attach(null);
+                                }
                             }else {
                                 sk.cancel();
                                 sk.attach(null);
@@ -303,6 +310,7 @@ public class NioBlockingSelector {
             return result;
         }
 
+        @Override
         public void run() {
             while (run) {
                 try {
@@ -328,6 +336,7 @@ public class NioBlockingSelector {
                         if ( log.isDebugEnabled() ) log.debug("Possibly encountered sun bug 5076772 on windows JDK 1.5",x);
                         continue;
                     } catch (Throwable x) {
+                        ExceptionUtils.handleThrowable(x);
                         log.error("",x);
                         continue;
                     }
@@ -341,7 +350,7 @@ public class NioBlockingSelector {
                         KeyAttachment attachment = (KeyAttachment)sk.attachment();
                         try {
                             attachment.access();
-                            iterator.remove(); ;
+                            iterator.remove();
                             sk.interestOps(sk.interestOps() & (~sk.readyOps()));
                             if ( sk.isReadable() ) {
                                 countDown(attachment.getReadLatch());
@@ -350,7 +359,7 @@ public class NioBlockingSelector {
                                 countDown(attachment.getWriteLatch());
                             }
                         }catch (CancelledKeyException ckx) {
-                            if (sk!=null) sk.cancel();
+                            sk.cancel();
                             countDown(attachment.getReadLatch());
                             countDown(attachment.getWriteLatch());
                         }
@@ -378,7 +387,7 @@ public class NioBlockingSelector {
         }
     }
     
-    public class KeyReference {
+    public static class KeyReference {
         SelectionKey key = null;
         
         @Override
@@ -391,4 +400,4 @@ public class NioBlockingSelector {
         }
     }
 
-}
\ No newline at end of file
+}
