@@ -34,6 +34,13 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+
+import org.junit.Test;
+
 import org.apache.catalina.Context;
 import org.apache.catalina.Wrapper;
 import org.apache.catalina.connector.Request;
@@ -46,12 +53,15 @@ import org.apache.tomcat.util.buf.ByteChunk;
 public class TestAsyncContextImpl extends TomcatBaseTest {
 
     // Time for a request to process (need to allow for threads to start etc.)
-    private static final long REQUEST_TIME = 500;
+    private static final long REQUEST_TIME = 1000;
     // Timeout thread (where used) checks for timeout every second
     private static final long TIMEOUT_MARGIN = 1000;
+    // Timing tests need a small error margin to prevent failures
+    private static final long ERROR_MARGIN = 10;
     // Default timeout for these tests
     private static final long TIMEOUT = 3000;
 
+    @Test
     public void testBug49528() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -88,7 +98,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         validateAccessLog(alv, 1, 200, Bug49528Servlet.THREAD_SLEEP_TIME,
                 Bug49528Servlet.THREAD_SLEEP_TIME + REQUEST_TIME);
     }
-    
+
+    @Test
     public void testBug49567() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -125,7 +136,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         validateAccessLog(alv, 1, 200, Bug49567Servlet.THREAD_SLEEP_TIME,
                 Bug49567Servlet.THREAD_SLEEP_TIME + REQUEST_TIME);
     }
-    
+
+    @Test
     public void testAsyncStartNoComplete() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -166,7 +178,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
                 AsyncStartNoCompleteServlet.ASYNC_TIMEOUT + TIMEOUT_MARGIN +
                         REQUEST_TIME);
     }
-    
+
+    @Test
     public void testAsyncStartWithComplete() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -363,18 +376,30 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         }
     }
 
-    public void testTimeoutListenerComplete() throws Exception {
+    @Test
+    public void testTimeoutListenerCompleteNoDispatch() throws Exception {
+        // Should work
         doTestTimeout(true, null);
     }
-    
-    public void testTimeoutListenerNoComplete() throws Exception {
+
+    @Test
+    public void testTimeoutListenerNoCompleteNoDispatch() throws Exception {
+        // Should trigger an error - must do one or other
         doTestTimeout(false, null);
     }
 
-    public void testTimeoutListenerDispatch() throws Exception {
+    @Test
+    public void testTimeoutListenerCompleteDispatch() throws Exception {
+        // Should trigger an error - can't do both
         doTestTimeout(true, "/nonasync");
     }
-    
+
+    @Test
+    public void testTimeoutListenerNoCompleteDispatch() throws Exception {
+        // Should work
+        doTestTimeout(false, "/nonasync");
+    }
+
 
     private void doTestTimeout(boolean completeOnTimeout, String dispatchUrl)
     throws Exception {
@@ -386,7 +411,7 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         
         // Create the folder that will trigger the redirect
         File foo = new File(docBase, "async");
-        if (!foo.exists() && !foo.mkdirs()) {
+        if (!foo.mkdirs() && !foo.isDirectory()) {
             fail("Unable to create async directory in docBase");
         }
         
@@ -411,23 +436,46 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         ctx.getPipeline().addValve(alv);
         
         tomcat.start();
-        ByteChunk res = getUrl("http://localhost:" + getPort() + "/async");
+        ByteChunk res = new ByteChunk();
+        try {
+            getUrl("http://localhost:" + getPort() + "/async", res, null);
+        } catch (IOException ioe) {
+            // Ignore - expected for some error conditions
+        }
         StringBuilder expected = new StringBuilder("requestInitialized-");
         expected.append("TimeoutServletGet-onTimeout-");
-        if (!completeOnTimeout) {
-            expected.append("onError-");
-        }
-        if (dispatchUrl == null) {
-            expected.append("onComplete-");
+        if (completeOnTimeout) {
+            if (dispatchUrl == null) {
+                expected.append("onComplete-");
+                expected.append("requestDestroyed");
+            } else {
+                // Error - no further output
+                // There is no onComplete- since the complete event would be
+                // fired during post processing but since there is an error that
+                // never happens.
+            }
         } else {
-            expected.append("NonAsyncServletGet-");
+            if (dispatchUrl == null) {
+                expected.append("onError-");
+            } else {
+                expected.append("NonAsyncServletGet-");
+            }
+            expected.append("onComplete-");
+            expected.append("requestDestroyed");
         }
-        expected.append("requestDestroyed");
         assertEquals(expected.toString(), res.toString());
 
         // Check the access log
-        validateAccessLog(alv, 1, 200, TimeoutServlet.ASYNC_TIMEOUT,
-                TimeoutServlet.ASYNC_TIMEOUT + TIMEOUT_MARGIN + REQUEST_TIME);
+        if (completeOnTimeout && dispatchUrl != null) {
+            if (!isAccessLogEnabled()) {
+                validateAccessLog(alv, 1, 500, 0, TimeoutServlet.ASYNC_TIMEOUT +
+                        TIMEOUT_MARGIN + REQUEST_TIME);
+            }
+        } else {
+            validateAccessLog(alv, 1, 200, TimeoutServlet.ASYNC_TIMEOUT,
+                    TimeoutServlet.ASYNC_TIMEOUT + TIMEOUT_MARGIN +
+                    REQUEST_TIME);
+        }
     }
     
     private static class TimeoutServlet extends HttpServlet {
@@ -458,26 +506,32 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         }
     }
 
+    @Test
     public void testDispatchSingle() throws Exception {
         doTestDispatch(1, false);
     }
-    
+
+    @Test
     public void testDispatchDouble() throws Exception {
         doTestDispatch(2, false);
     }
-    
+
+    @Test
     public void testDispatchMultiple() throws Exception {
         doTestDispatch(5, false);
     }
-    
+
+    @Test
     public void testDispatchWithThreadSingle() throws Exception {
         doTestDispatch(1, true);
     }
-    
+
+    @Test
     public void testDispatchWithThreadDouble() throws Exception {
         doTestDispatch(2, true);
     }
-    
+
+    @Test
     public void testDispatchWithThreadMultiple() throws Exception {
         doTestDispatch(5, true);
     }
@@ -587,7 +641,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
             resp.flushBuffer();
         }
     }
-    
+
+    @Test
     public void testListeners() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -693,11 +748,10 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
             resp.getWriter().write("onTimeout-");
             resp.flushBuffer();
             if (completeOnTimeout){
-                if (dispatchUrl == null) {
-                    event.getAsyncContext().complete();
-                } else {
-                    event.getAsyncContext().dispatch(dispatchUrl);
-                }
+                event.getAsyncContext().complete();
+            }
+            if (dispatchUrl != null) {
+                event.getAsyncContext().dispatch(dispatchUrl);
             }
         }
 
@@ -747,52 +801,64 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         }
     }
 
+    @Test
     public void testDispatchErrorSingle() throws Exception {
         doTestDispatchError(1, false, false);
     }
-    
+
+    @Test
     public void testDispatchErrorDouble() throws Exception {
         doTestDispatchError(2, false, false);
     }
-    
+
+    @Test
     public void testDispatchErrorMultiple() throws Exception {
         doTestDispatchError(5, false, false);
     }
-    
+
+    @Test
     public void testDispatchErrorWithThreadSingle() throws Exception {
         doTestDispatchError(1, true, false);
     }
-    
+
+    @Test
     public void testDispatchErrorWithThreadDouble() throws Exception {
         doTestDispatchError(2, true, false);
     }
-    
+
+    @Test
     public void testDispatchErrorWithThreadMultiple() throws Exception {
         doTestDispatchError(5, true, false);
     }
-    
+
+    @Test
     public void testDispatchErrorSingleThenComplete() throws Exception {
         doTestDispatchError(1, false, true);
     }
-    
+
+    @Test
     public void testDispatchErrorDoubleThenComplete() throws Exception {
         doTestDispatchError(2, false, true);
     }
-    
+
+    @Test
     public void testDispatchErrorMultipleThenComplete() throws Exception {
         doTestDispatchError(5, false, true);
     }
-    
+
+    @Test
     public void testDispatchErrorWithThreadSingleThenComplete()
             throws Exception {
         doTestDispatchError(1, true, true);
     }
-    
+
+    @Test
     public void testDispatchErrorWithThreadDoubleThenComplete()
             throws Exception {
         doTestDispatchError(2, true, true);
     }
-    
+
+    @Test
     public void testDispatchErrorWithThreadMultipleThenComplete()
             throws Exception {
         doTestDispatchError(5, true, true);
@@ -872,7 +938,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
             throw new ServletException("Opps.");
         }
     }
-    
+
+    @Test
     public void testBug50352() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -933,7 +1000,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
             });
         }
     }
-    
+
+    @Test
     public void testBug50753() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -999,6 +1067,7 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         }
     }
 
+    @Test
     public void testErrorHandling() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -1048,11 +1117,14 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
         for (int j = 0; j < count; j++) {
             Entry entry = entries.get(j);
             assertEquals(status, entry.getStatus());
-            assertTrue(entry.toString(), entry.getTime() >= minTime);
-            assertTrue(entry.toString(), entry.getTime() < maxTime);
+            assertTrue(entry.toString(),
+                    entry.getTime() >= minTime - ERROR_MARGIN);
+            assertTrue(entry.toString(),
+                    entry.getTime() < maxTime + ERROR_MARGIN);
         }
     }
-    
+
+    @Test
     public void testCommitOnComplete() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
@@ -1112,8 +1184,8 @@ public class TestAsyncContextImpl extends TomcatBaseTest {
             actxt.complete();
         }
     }
-    
 
+    @Test
     public void testBug51197() throws Exception {
         // Setup Tomcat instance
         Tomcat tomcat = getTomcatInstance();
