@@ -29,15 +29,22 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
+import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertTrue;
+
+import org.junit.Test;
+
 import org.apache.catalina.core.StandardContext;
 import org.apache.tomcat.util.buf.ByteChunk;
 
 public class TestContextConfig extends TomcatBaseTest {
 
+    @Test
     public void testOverrideWithSCIDefaultName() throws Exception {
         doTestOverrideDefaultServletWithSCI("default");
     }
 
+    @Test
     public void testOverrideWithSCIDefaultMapping() throws Exception {
         doTestOverrideDefaultServletWithSCI("anything");
     }
@@ -69,6 +76,24 @@ public class TestContextConfig extends TomcatBaseTest {
         assertEquals("OK - Custom default Servlet", res.toString());
     }
 
+    @Test
+    public void testBug51396() throws Exception {
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir =  new File("test/webapp-3.0-fragments");
+        // app dir is relative to server home
+        tomcat.addWebapp(null, "/test", appDir.getAbsolutePath());
+        
+        tomcat.start();
+
+        ByteChunk bc = new ByteChunk();
+        int rc = getUrl("http://localhost:" + getPort() +
+                "/test/bug51396.jsp", bc, null);
+        
+        assertEquals(HttpServletResponse.SC_OK, rc);
+        assertTrue(bc.toString().contains("<p>OK</p>"));
+    }
+
     private static class CustomDefaultServletSCI
             implements ServletContainerInitializer {
 
